
/* niff to gfx converter v1.2 */

/* ============================================================
          Header Area
 ============================================================ */

#ifndef __ND_COMMONS__
#define __ND_COMMONS__

/* ------------------ macros */

#define SEGTOP(sno) ((u32)(sno)<<24)
#define SEGADRS(sno,adrs) (SEGTOP(sno)|((u32)(adrs)&0x00ffffff))

/* ------------------ geometric animation */

enum{
  /* NDR_ANIM_CHANNEL.kind */
  NDC_ANIMKIND_TRS_X  = 0x00,
  NDC_ANIMKIND_TRS_Y  = 0x01,
  NDC_ANIMKIND_TRS_Z  = 0x02,
  NDC_ANIMKIND_ROT_X  = 0x03,
  NDC_ANIMKIND_ROT_Y  = 0x04,
  NDC_ANIMKIND_ROT_Z  = 0x05,
  NDC_ANIMKIND_SCL_X  = 0x06,
  NDC_ANIMKIND_SCL_Y  = 0x07,
  NDC_ANIMKIND_SCL_Z  = 0x08
};

typedef struct {
  u16 kind;
  s16 start;
  s16 end;
  s16 nkey;
  s16 *key;
  float *value;
} NDR_ANIM_CHANNEL;

typedef struct {
  NDR_ANIM_CHANNEL *chlist;
  u16 nch;
  u16 total;
} NDR_ANIM;

/* ------------------ shape */

typedef struct{
  s16 *key;
  s16 *tblno;
  u16 nkey;
  u16 nelem;
} NDR_DEFORM;

typedef struct{
  Gfx *gfx;
  Vtx *vtx;
  u16 nvtx;
  u16 vtxsegno;

  /* using texture segment */
  u8 *img;
  u16 *tlut;
  u16 imgsegno, tlutsegno;

  /* shape deform data link */
  s16 *shaperef;
  NDR_DEFORM *vtx_deform;
  NDR_DEFORM *vtxcol_deform;
  NDR_DEFORM *vtxnv_deform;

} NDR_SHAPE;

/* ------------------ object */

enum{
  /* object types */
  NDC_TYPE_NULL = 0x01,
  NDC_TYPE_3D = 0x02,
  NDC_TYPE_BILLBOARD = 0x03,

  /* ColorCombine Mode */
  NDC_CC_COMBINED = 0x00,
  NDC_CC_TEX0 = 0x01,
  NDC_CC_TEX1 = 0x02,
  NDC_CC_PRIMITIVE = 0x03,
  NDC_CC_SHADE = 0x04,
  NDC_CC_ENV = 0x05,
  NDC_CC_KEY_CENTER = 0x10,
  NDC_CC_KEY_SCALE = 0x11,
  NDC_CC_COMBINED_ALPHA = 0x06,
  NDC_CC_TEX0_ALPHA = 0x07,
  NDC_CC_TEX1_ALPHA = 0x08,
  NDC_CC_PRIMITIVE_ALPHA = 0x09,
  NDC_CC_SHADE_ALPHA = 0x0a,
  NDC_CC_ENV_ALPHA = 0x0b,
  NDC_CC_LOD = 0x0c,
  NDC_CC_PRIM_LOD = 0x0d,
  NDC_CC_NOISE = 0x12,
  NDC_CC_K4 = 0x14,
  NDC_CC_K5 = 0x15,
  NDC_CC_1 = 0x0e,
  NDC_CC_0 = 0x0f,

  /* Render Mode(complexed) */
  NDC_RM_AA_EN                     = 0x80000000,
  NDC_RM_Z_CMP                     = 0x40000000,
  NDC_RM_Z_UPD                     = 0x20000000,
  NDC_RM_IM_RD                     = 0x10000000,
  NDC_RM_CVG_DST                   = 0x0c000000,
    NDC_RM_CVG_DST_CLAMP           = 0x00000000,
    NDC_RM_CVG_DST_WRAP            = 0x04000000,
    NDC_RM_CVG_DST_FULL            = 0x08000000,
    NDC_RM_CVG_DST_SAVE            = 0x0c000000,
  NDC_RM_CLR_ON_CVG                = 0x02000000,
  NDC_RM_CVG_X_ALPHA               = 0x01000000,
  NDC_RM_ALPHA_CVG_SEL             = 0x00800000,
  NDC_RM_FORCE_BL                  = 0x00400000,
  NDC_RM_ZMODE                     = 0x00300000,
    NDC_RM_ZMODE_OPA               = 0x00000000,
    NDC_RM_ZMODE_INTER             = 0x00100000,
    NDC_RM_ZMODE_XLU               = 0x00200000,
    NDC_RM_ZMODE_DEC               = 0x00300000,
  NDC_RM_ALPHA_COMPARE             = 0x000c0000,
    NDC_RM_ALPHA_COMPARE_NONE      = 0x00000000,
    NDC_RM_ALPHA_COMPARE_THRESHOLD = 0x00040000,
    NDC_RM_ALPHA_COMPARE_DITHER    = 0x00080000,
  NDC_RM_DITHER_ALPHA              = 0x00030000,
    NDC_RM_DITHER_ALPHA_PATTERN    = 0x00000000,
    NDC_RM_DITHER_ALPHA_NOPATTERN  = 0x00010000,
    NDC_RM_DITHER_ALPHA_NOISE      = 0x00020000,
    NDC_RM_DITHER_ALPHA_DISABLE    = 0x00030000,
  NDC_RM_BLENDER_MUX0              = 0x0000ff00,
  NDC_RM_BLENDER_MUX1              = 0x000000ff,

  /* NDR_OBJ.regflag(bit assign) */
  NDC_REGF_2CYC        = 0x4000
};

typedef struct{
  float tx, ty, tz;
  float rx, ry, rz;
  float sx, sy, sz;
} TRX;

typedef struct ndr_objTag{

  /* object propaties */
  u16 type, gid;
  u32 prio;
  TRX trx;

  /* resourse index */
  s16 shape;

  /* RSP, RDP status register setting */
  u16 regflag;
  u32 geommode;
  u32 rdmode;
  u32 primcol;
  u8 cc1[8], cc2[8];

  /* geometric animation setting */
  s16 geom_anim;
  u16 loop;
  u16 framerate;
  u32 order;

  /* children links */
  s16 bb_obj[2];
  u16 nchild;
  u16 *child;

} NDR_OBJ;

/* ------------------ scene */

typedef struct{
  TRX trx;
  u16 objno;
} NDR_SCN_INST_OBJ;

typedef struct{
  int type;
  u32 color;
  float x, y, z;
  float a1,a2;
} NDR_SCN_LIGHT;

typedef struct{
  struct{
    u16 near, far;
    u8 r,g,b,a;
  }fog;
  struct{
    u32 mode;
    float fovy, aspect, scale;
    u16 top, bottom, left, right, near, far;
    u16 eye_inst, upper_inst, lookat_inst;
  }cam;
  struct{
    u32 amb_color;
    int nlight;
    NDR_SCN_LIGHT *light;
  }light;
  u16 fill_col, fill_depth;
  NDR_SCN_INST_OBJ *entryobj;
  u16 nentobj;
} NDR_SCENE;

typedef struct{
  NDR_SCENE *scene;
  float convert_scale;

  NDR_OBJ **obj;
  NDR_SHAPE **shape;
  NDR_ANIM **anim;
  s16 **rawvtx;
  u32 **rawvtxcol;
  s8 **rawvtxnv;

  u16 nobj;
  u16 nanim;
  u16 nshape;
  u16 nrawvtx;
  u16 nrawvtxcol;
  u16 nrawvtxnv;
} NDR_RESINFO;

#endif    /* #ifndef __ND_COMMONS__ */

#ifdef PREVIEW_TYPE
#undef PREVIEW_TYPE
#endif  /* #ifdef PREVIEW_TYPE */

#ifdef RESINFO_NAME
#undef RESINFO_NAME
#endif  /* #ifdef RESINFO_NAME */

#define PREVIEW_TYPE 28
extern NDR_RESINFO resinfo_frake_eat;
#define RESINFO_NAME resinfo_frake_eat

static Gfx _frake_eat_aligner0[] = { gsSPEndDisplayList() };

/* ============================================================
          Bitmap
 ============================================================ */
static u8 bmp_frake_eat[] = {
/* name:t_shirt3  type:RGBA  pixel/bit: 16 bits
  offset:0x00000000  length:2048 bytes. */
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xa7,0xb4,
0xaf,0x6e,0xaf,0x6e,0xaf,0x6e,0xaf,0x6e,
0xa7,0xb4,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xd0,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xd0,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xd0,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xd0,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xd0,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xd0,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x42,0xcd,0xd0,0xcd,0xd0,0x9f,0xfa,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xa7,0xb6,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xa7,0xb6,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xa7,0xb6,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xa7,0xb6,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xd5,0x88,0xbe,0x9e,0xbe,0x9e,0x9f,0xfc,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xce,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xa7,0x72,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xaf,0x6e,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xaf,0x6e,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xaf,0x6e,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xaf,0x6e,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfa,
0xa7,0xb6,0xa7,0xb6,0xa7,0xb6,0xa7,0xb6,
0xa7,0xb6,0xa7,0xb6,0xa7,0xb6,0xa7,0xb6,
0xa7,0xb6,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x5c,0x1e,0x39,0x88,0x52,0x0a,0x6a,0x8e,
0x6a,0x8e,0x5a,0x4c,0x39,0x88,0x31,0xcc,
0x65,0x6a,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x21,0x8c,0x8b,0x92,
0xf6,0x20,0xfe,0x62,0xfe,0x62,0xfe,0x62,
0xfe,0x62,0xfe,0x62,0xfe,0x62,0xf6,0x20,
0x8b,0x92,0x18,0x82,0x7e,0xb4,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x6d,0xac,0xa4,0x16,0xfe,0x62,
0xfe,0x62,0xfe,0x62,0xfe,0x62,0xfe,0x62,
0xfe,0x62,0xfe,0x62,0xfe,0x62,0xfe,0x62,
0xfe,0x62,0xd5,0x5c,0x10,0xc6,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
/* name:belt2  type:RGBA  pixel/bit: 16 bits
  offset:0x00000800  length:2048 bytes. */
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x24,0x48,0x24,0x48,0x24,0x48,0x26,
0x50,0x66,0x50,0x66,0x48,0x26,0x48,0x24,
0x48,0x24,0x48,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x48,0x24,0x48,0x26,0x48,0x24,0x40,0x24,
0x08,0x02,0x18,0x88,0x49,0xd6,0x73,0x24,
0x8b,0xaa,0x8b,0xaa,0x73,0x24,0x41,0xd6,
0x18,0x88,0x08,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x28,0xcc,0x6a,0xe2,0x10,0x86,0x00,0x02,
0x00,0x00,0x10,0x86,0x4a,0x16,0x7b,0xa4,
0x8c,0x2c,0x8c,0x2c,0x7b,0xa4,0x42,0x14,
0x10,0x84,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x18,0xc8,0x73,0x22,0x18,0xc6,0x00,0x00,
0x08,0x02,0x18,0x88,0x49,0xd8,0x7b,0x66,
0x8b,0xaa,0x8b,0xaa,0x73,0x24,0x41,0xd6,
0x18,0x88,0x08,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x18,0x8a,0x6a,0xe2,0x20,0xca,0x00,0x02,
0x48,0x24,0x48,0x24,0x48,0x24,0x48,0x26,
0x50,0x66,0x50,0x66,0x48,0x26,0x48,0x24,
0x48,0x24,0x48,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x48,0x24,0x48,0x26,0x48,0x24,0x40,0x24,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
/* name:mouth2_32  type:RGBA  pixel/bit: 16 bits
  offset:0x00001000  length:2048 bytes. */
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x5e,0xed,0x5e,0xed,0x5e,0xed,0x5e,
0xe5,0x1c,0xdd,0x1c,0xed,0x5e,0xed,0x5e,
0xf5,0x5e,0xf5,0x5e,0xf5,0x9e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x9e,0xed,0x5e,0xed,0x5e,0xd4,0xdc,
0xb4,0x16,0x9b,0x54,0x8b,0x12,0x72,0x8e,
0x62,0x0c,0x62,0x0c,0x6a,0x8e,0x7a,0xd0,
0x93,0x52,0xa3,0xd4,0xbc,0x18,0xd4,0xda,
0xed,0x5e,0xf5,0x9e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0x9e,0xed,0x5e,
0xcc,0x9a,0x8b,0x52,0x6a,0x4e,0x5a,0x0c,
0x41,0x48,0x31,0x06,0x29,0x06,0x20,0xc4,
0x20,0x84,0x28,0xc4,0x20,0xc4,0x20,0xc4,
0x31,0x06,0x39,0x48,0x41,0x88,0x5a,0x0c,
0x83,0x10,0xc4,0x58,0xed,0x5e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x5e,0xe5,0x1e,0xc4,0x58,0x8b,0x52,
0x51,0xca,0x29,0x06,0x20,0x84,0x20,0xc4,
0x31,0x06,0x39,0x46,0x51,0xca,0x62,0x0c,
0x6a,0x4e,0x7a,0x8e,0x6a,0x4e,0x49,0xca,
0x41,0x88,0x39,0x46,0x28,0xc4,0x28,0xc4,
0x28,0xc4,0x51,0xca,0x9b,0x54,0xdc,0xdc,
0xf5,0x5e,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xed,0x5e,
0xc4,0x58,0x8b,0x10,0x49,0x8a,0x31,0x06,
0x20,0x84,0x39,0x46,0x5a,0x0a,0x7a,0xd0,
0x93,0x52,0xa3,0xd4,0xc4,0x58,0xdc,0xdc,
0xed,0x5e,0xed,0x5e,0xe5,0x1c,0xcc,0x9a,
0xb4,0x16,0xa3,0x94,0x83,0x10,0x6a,0x4c,
0x31,0x06,0x28,0xc4,0x31,0x06,0x6a,0x4c,
0xbc,0x58,0xed,0x5e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xed,0x5e,0xa3,0xd4,
0x49,0xca,0x29,0x06,0x20,0x84,0x39,0x48,
0x6a,0x4c,0xa3,0x94,0xd4,0xdc,0xed,0x5e,
0xf5,0x5e,0xf5,0x5e,0xf5,0x9e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x9e,0xf5,0x5e,0xed,0x5e,0xd4,0xdc,
0x9b,0x94,0x51,0xca,0x31,0x06,0x20,0xc4,
0x51,0xca,0xbc,0x18,0xed,0x5e,0xf5,0xa0,
0xf5,0x9e,0xe5,0x1c,0xa3,0x94,0x41,0x88,
0x20,0x84,0x39,0x48,0x62,0x4c,0xab,0xd6,
0xdc,0xdc,0xed,0x5e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xed,0x5e,0xcc,0x9a,0x82,0xd0,0x39,0x46,
0x28,0xc4,0x5a,0x0a,0xc4,0x58,0xed,0x5e,
0xdc,0xdc,0x8b,0x52,0x39,0x48,0x20,0x84,
0x41,0x88,0x9b,0x94,0xdc,0xdc,0xed,0x5e,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0x5e,0xe5,0x1c,0x8b,0x12,
0x31,0x46,0x28,0xc4,0x62,0x4c,0xcc,0x9a,
0x9b,0x94,0x39,0x46,0x20,0xc4,0x51,0xca,
0xab,0xd6,0xed,0x5e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0x9e,0xdd,0x1c,
0x82,0xd0,0x31,0x06,0x31,0x06,0x8b,0x12,
0x6a,0x8e,0x28,0xc4,0x5a,0x0a,0xbc,0x18,
0xed,0x5e,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0x9e,
0xd4,0xda,0x72,0x8e,0x39,0x48,0x82,0xd0,
0xc4,0x58,0x9b,0x94,0xc4,0x9a,0xed,0x5e,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x5e,0xd4,0xda,0xb4,0x16,0xd4,0x9a,
0xf5,0x9e,0xf5,0x5e,0xf5,0x9e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0x5e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
/* name:eye_32b  type:RGBA  pixel/bit: 16 bits
  offset:0x00001800  length:2048 bytes. */
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xe2,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x9e,0xdc,0xdc,0x8b,0x12,0x72,0x8e,
0x83,0x10,0x6a,0x4c,0x31,0x06,0x39,0x46,
0x7a,0xd0,0x7a,0xd0,0xbc,0x18,0xed,0x5e,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xed,0x5e,0xa3,0x94,
0x5a,0x0c,0x49,0xca,0x6a,0xd4,0x73,0x16,
0xac,0xe4,0xde,0xb2,0xe6,0xf4,0xe6,0xf4,
0xad,0x26,0x6b,0x14,0x6a,0x90,0x39,0x46,
0xab,0xd6,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0x9b,0x54,0x31,0x46,0x31,0x8a,
0x21,0xd0,0x11,0x90,0x00,0x86,0x09,0x0c,
0x22,0x56,0x5b,0xe2,0xef,0xbe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xcd,0xec,
0x31,0x06,0x9b,0x54,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0x8b,0x52,0x31,0x4a,0x21,0x8e,0x01,0x0c,
0x02,0x16,0x03,0x24,0x03,0x64,0x03,0x64,
0x02,0x5a,0x01,0x0c,0x11,0x4c,0xa5,0xf0,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0xd6,0x72,0x31,0x48,0x93,0x52,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0x93,0x54,
0x31,0x4a,0x10,0xc8,0x02,0x16,0x03,0xa8,
0x02,0xe0,0x01,0x4e,0x00,0xc8,0x01,0x90,
0x03,0x22,0x03,0xa8,0x02,0x5a,0x00,0x86,
0xa5,0xf0,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xde,0xf4,0x39,0x48,0x83,0x10,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xc4,0x58,0x39,0x8a,
0x21,0x8c,0x02,0x5a,0x03,0xa8,0x03,0x22,
0x00,0x86,0x00,0x00,0x00,0x00,0x00,0x00,
0x01,0x4e,0x03,0xa8,0x03,0xa8,0x02,0x5a,
0x09,0x4c,0xef,0x7c,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xde,0xb2,0x41,0x48,
0xc4,0x58,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x7a,0xd0,0x31,0x8e,
0x01,0x90,0x03,0xa8,0x03,0xa8,0x01,0x92,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x86,0x03,0xa8,0x03,0xa8,0x03,0xa8,
0x01,0x4e,0x53,0x9e,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0x73,0x14,
0x82,0xd0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x7b,0x14,0x21,0x4c,
0x03,0x22,0x03,0xa8,0x03,0x64,0x21,0x8e,
0x21,0x08,0x00,0x42,0x00,0x00,0x00,0x00,
0x00,0x86,0x03,0xa8,0x03,0xa8,0x03,0xa8,
0x02,0x9c,0x22,0x14,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xce,0x30,
0x51,0xca,0xf5,0x9e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x39,0x8a,0x11,0x08,
0x03,0x66,0x03,0x22,0x84,0xa4,0xef,0x7a,
0xef,0x7a,0x84,0x20,0x00,0x00,0x00,0x00,
0x00,0xca,0x03,0xa8,0x03,0xa8,0x03,0xa8,
0x03,0x64,0x01,0x0a,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xf7,0xbc,
0x52,0x0e,0xd4,0x9a,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x29,0x48,0x11,0x0a,
0x03,0xa6,0x33,0x60,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0x18,0xc6,0x00,0x02,
0x02,0xde,0x03,0xa8,0x03,0xa8,0x03,0xa8,
0x03,0x66,0x00,0x88,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0x6a,0xd4,0x82,0xd0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x5a,0x50,0x19,0x4a,
0x03,0x64,0x74,0x64,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0x5b,0x18,0x02,0x5a,
0x03,0xa8,0x03,0xa8,0x03,0xa8,0x03,0xa8,
0x03,0x22,0x09,0x4c,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0xbd,0x68,0x72,0x8e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x83,0x54,0x21,0x4a,
0x02,0x16,0x5c,0x24,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0x4b,0x5c,0x03,0xa8,
0x03,0xa8,0x03,0xa8,0x03,0xa8,0x03,0xa8,
0x01,0xd6,0x22,0x56,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0x9c,0xa0,0x7a,0x8e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xdc,0xdc,0x29,0x48,
0x01,0x0a,0x13,0x20,0xe7,0x38,0xff,0xfe,
0xff,0xfe,0xef,0x7a,0x0a,0x9a,0x03,0xa8,
0x03,0xa8,0x03,0xa8,0x03,0xa8,0x03,0x66,
0x00,0xca,0x85,0x2a,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0x62,0x92,0x7a,0xd0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0x8b,0x96,
0x10,0xc6,0x01,0x4e,0x23,0x1e,0xa5,0x6c,
0xa5,0x6a,0x2b,0x1e,0x03,0x66,0x03,0xa8,
0x03,0xa8,0x03,0xa8,0x03,0x20,0x01,0x0a,
0x3b,0x1c,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xef,0x7a,
0x41,0x8a,0xc4,0x98,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xed,0x5e,
0x6a,0xd2,0x08,0x86,0x00,0x84,0x01,0xd4,
0x03,0x22,0x03,0x64,0x03,0x64,0x03,0x20,
0x01,0xd6,0x01,0x4e,0x00,0xc8,0x53,0xe2,
0xf7,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0x62,0xd4,
0x72,0x8e,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xd5,0x1e,0x52,0x94,0x08,0x86,
0x00,0x86,0x00,0x84,0x00,0xca,0x1a,0x56,
0x2a,0x56,0x64,0x66,0xce,0xf8,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xe7,0x38,0x62,0xd6,0x5a,0x0c,
0xed,0x5e,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xac,0x5a,0x62,0x92,0x4a,0x10,0x7b,0xde,
0x6b,0x5a,0x73,0x9c,0xad,0x6a,0xe7,0x38,
0xe7,0x38,0xe7,0x38,0xa5,0x28,0x6b,0x5a,
0x73,0x5a,0x31,0x48,0x7a,0xd0,0xf5,0x9e,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0x9e,0xe5,0x1e,0xb4,0x18,
0x83,0x14,0x7b,0x14,0x83,0x98,0x5a,0x92,
0x29,0x48,0x4a,0x0e,0x7b,0x16,0x7b,0x14,
0xcc,0x9a,0xed,0x5e,0xf5,0x9e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x6a,0x4e,0x5a,0x0c,
0x62,0x0c,0x9b,0x94,0xed,0x5e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x51,0xca,0x00,0x00,
0x08,0x40,0x08,0x00,0x49,0xca,0xb4,0x16,
0xd4,0xda,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x8b,0x12,0x18,0x82,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x18,0x82,0x51,0xca,0xb4,0x16,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0x9e,0xbc,0x58,
0x51,0xca,0x08,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x51,0xca,
0xb4,0x16,0xed,0x5e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x5e,0xb4,0x16,0x5a,0x0a,0x08,0x42,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x08,0x00,0x31,0x46,0x7a,0x8e,0xbc,0x58,
0xe5,0x1e,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xc4,0x58,
0x5a,0x0c,0x10,0x42,0x00,0x00,0x08,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x41,0x88,0xab,0xd6,0xf5,0x5e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xcc,0x9a,0x7a,0xd0,0x9b,0x94,
0x8b,0x52,0x7a,0xd0,0x39,0x48,0x20,0xc4,
0x7a,0xd0,0x7a,0xd0,0x83,0x12,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xe2,
/* name:refmap  type:RGBA  pixel/bit: 16 bits
  offset:0x00002000  length:2048 bytes. */
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x18,0xc9,0x10,0x87,0x08,0x43,0x08,0x01,
0x08,0x43,0x08,0x43,0x08,0x43,0x21,0x09,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x10,0x45,0x20,0xc7,
0x10,0x85,0x10,0x85,0x10,0x85,0x10,0x85,
0x10,0x85,0x10,0x85,0x10,0x85,0x10,0x45,
0x10,0x85,0x10,0x43,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x08,0x43,0x18,0xc7,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x10,0x85,0x21,0x09,
0x10,0x85,0x10,0x85,0x10,0x45,0x10,0x43,
0x10,0x45,0x10,0x43,0x10,0x85,0x10,0x43,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x21,0x09,0x18,0xc7,
0x21,0x09,0x21,0x07,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x20,0xc7,0x10,0x85,0x10,0x45,0x10,0x43,
0x18,0x85,0x08,0x01,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x08,0x43,0x18,0xc7,0x21,0x09,
0x10,0x85,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x20,0xc7,0x20,0xc7,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x18,0xc7,0x08,0x03,
0x10,0x85,0x18,0x85,0x08,0x43,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x08,0x43,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x18,0xc7,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x18,0xc7,
0x18,0x87,0x10,0x43,0x10,0x43,0x10,0x85,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x20,0xc7,
0x08,0x43,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x18,0x87,0x20,0xc7,
0x18,0xc7,0x18,0x85,0x10,0x45,0x20,0xc9,
0x10,0x85,0x10,0x85,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x10,0x85,0x10,0x43,0x10,0x43,
0x20,0xc7,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x43,
0x18,0xc5,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x10,0x85,0x08,0x43,0x08,0x43,
0x10,0x43,0x10,0x43,0x10,0x45,0x10,0x45,
0x10,0x85,0x18,0x87,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x08,0x43,0x10,0x43,
0x10,0x43,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x10,0x87,0x10,0x43,
0x20,0xc5,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x10,0x85,0x08,0x43,0x08,0x43,
0x08,0x43,0x08,0x43,0x10,0x45,0x10,0x85,
0x10,0x85,0x21,0x09,0x10,0x85,0x21,0x09,
0x21,0x09,0x21,0x07,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x10,0x85,
0x10,0x43,0x18,0xc7,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x10,0x85,0x10,0x43,
0x20,0xc5,0x20,0xc5,0x21,0x09,0x21,0x09,
0x08,0x01,0x10,0x43,0x10,0x43,0x18,0x85,
0x10,0x43,0x08,0x43,0x10,0x45,0x21,0x09,
0x10,0x85,0x10,0x85,0x10,0x85,0x21,0x09,
0x21,0x0b,0x21,0x0b,0x08,0x43,0x21,0x0b,
0x21,0x0b,0x21,0x09,0x21,0x07,0x21,0x09,
0x08,0x43,0x18,0x85,0x00,0x00,0x00,0x00,
0x00,0x00,0x18,0xc7,0x08,0x43,0x10,0x43,
0x20,0xc5,0x20,0xc5,0x21,0x09,0x10,0x43,
0x10,0x43,0x10,0x83,0x18,0x83,0x18,0x83,
0x18,0xc7,0x18,0xc7,0x10,0x85,0x10,0x85,
0x10,0x85,0x10,0x85,0x10,0x85,0x21,0x0b,
0x21,0x0b,0x21,0x0b,0x21,0x0b,0x21,0x09,
0x21,0x09,0x20,0xc5,0x20,0xc5,0x21,0x09,
0x18,0xc7,0x10,0x43,0x08,0x01,0x00,0x00,
0x00,0x00,0x10,0x87,0x08,0x43,0x10,0x85,
0x18,0x85,0x21,0x09,0x21,0x09,0x08,0x41,
0x18,0x85,0x20,0xc7,0x20,0xc7,0x10,0x83,
0x10,0x43,0x10,0x43,0x10,0x85,0x10,0x85,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x21,0x09,
0x21,0x0b,0x21,0x09,0x20,0xc5,0x20,0xc5,
0x20,0xc5,0x20,0xc5,0x18,0x85,0x21,0x07,
0x10,0x85,0x10,0x43,0x08,0x43,0x00,0x00,
0x00,0x00,0x10,0x85,0x08,0x01,0x21,0x09,
0x10,0x83,0x18,0x85,0x10,0x43,0x08,0x03,
0x18,0x85,0x18,0x85,0x18,0xc5,0x18,0xc7,
0x10,0x85,0x10,0x85,0x08,0x43,0x21,0x09,
0x18,0x85,0x10,0x85,0x10,0x85,0x10,0x85,
0x08,0x43,0x08,0x43,0x10,0x83,0x10,0x83,
0x08,0x43,0x08,0x43,0x21,0x09,0x10,0x83,
0x10,0x85,0x08,0x43,0x10,0x43,0x00,0x00,
0x18,0x87,0x08,0x43,0x08,0x01,0x21,0x09,
0x18,0x85,0x08,0x01,0x08,0x43,0x08,0x01,
0x08,0x01,0x08,0x01,0x08,0x01,0x08,0x03,
0x08,0x03,0x10,0x43,0x08,0x01,0x08,0x43,
0x08,0x43,0x08,0x43,0x08,0x43,0x08,0x43,
0x08,0x43,0x08,0x43,0x10,0x43,0x10,0x43,
0x08,0x43,0x08,0x43,0x21,0x09,0x10,0x85,
0x10,0x85,0x08,0x43,0x10,0x85,0x18,0x85,
0x10,0x87,0x08,0x43,0x08,0x43,0x10,0x85,
0x18,0x85,0x08,0x43,0x08,0x43,0x08,0x43,
0x08,0x43,0x10,0x83,0x21,0x09,0x21,0x07,
0x08,0x43,0x08,0x43,0x08,0x03,0x08,0x43,
0x08,0x43,0x10,0x43,0x10,0x85,0x08,0x45,
0x10,0x85,0x10,0x85,0x08,0x43,0x08,0x43,
0x08,0x43,0x08,0x03,0x21,0x09,0x18,0xc7,
0x18,0xc7,0x08,0x43,0x10,0x85,0x21,0x07,
0x10,0x85,0x08,0x43,0x10,0x43,0x08,0x43,
0x10,0x43,0x10,0x43,0x20,0xc7,0x21,0x07,
0x10,0x85,0x10,0x87,0x21,0x09,0x10,0x85,
0x10,0x85,0x08,0x43,0x08,0x03,0x10,0x45,
0x10,0x87,0x10,0x85,0x10,0x87,0x10,0x45,
0x08,0x43,0x10,0x87,0x08,0x43,0x10,0x45,
0x08,0x43,0x08,0x03,0x18,0xc7,0x18,0x85,
0x10,0x85,0x10,0x43,0x10,0x85,0x10,0x85,
0x10,0x85,0x08,0x03,0x08,0x43,0x08,0x43,
0x08,0x01,0x10,0x43,0x10,0x85,0x10,0x85,
0x10,0x85,0x10,0x87,0x10,0x85,0x10,0x43,
0x08,0x43,0x08,0x43,0x08,0x43,0x10,0x45,
0x10,0x87,0x10,0xc7,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x19,0x09,0x10,0x85,0x08,0x45,
0x08,0x45,0x08,0x03,0x08,0x43,0x10,0x85,
0x10,0x85,0x10,0x45,0x10,0x85,0x10,0x85,
0x10,0x85,0x08,0x43,0x10,0x85,0x18,0x87,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x18,0xc7,
0x18,0xc7,0x18,0x87,0x10,0x85,0x10,0x43,
0x10,0x85,0x08,0x43,0x08,0x43,0x18,0xc7,
0x18,0xc9,0x10,0xc7,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x10,0x85,0x10,0x85,
0x08,0x43,0x18,0x87,0x18,0xc7,0x18,0xc7,
0x18,0xc7,0x18,0xc7,0x18,0x85,0x10,0x43,
0x08,0x43,0x10,0x43,0x18,0xc7,0x18,0xc7,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x18,0xc7,
0x18,0xc7,0x20,0xc9,0x20,0xc9,0x21,0x09,
0x21,0x09,0x10,0x87,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x20,0xc9,0x10,0x85,
0x10,0x85,0x10,0x85,0x10,0x43,0x10,0x85,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x18,0xc7,
0x18,0xc7,0x20,0xc7,0x18,0xc7,0x18,0xc7,
0x00,0x00,0x10,0x85,0x10,0xc7,0x18,0xc7,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x18,0xc7,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x19,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x29,0x09,0x29,0x09,
0x21,0x09,0x21,0x09,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x00,0x00,
0x00,0x00,0x10,0x85,0x18,0xc7,0x18,0xc7,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x18,0xc7,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x21,0x09,
0x21,0x09,0x10,0x45,0x21,0x0b,0x21,0x0b,
0x29,0x4b,0x29,0x4b,0x29,0x4d,0x29,0x4b,
0x29,0x4b,0x29,0x0b,0x21,0x09,0x21,0x0b,
0x21,0x09,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x18,0xc7,0x18,0xc7,0x18,0xc9,0x00,0x00,
0x00,0x00,0x10,0x87,0x18,0xc7,0x18,0xc9,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x18,0xc7,
0x18,0xc9,0x18,0xc9,0x21,0x09,0x21,0x09,
0x21,0x09,0x29,0x0b,0x21,0x0b,0x29,0x4b,
0x29,0x4d,0x31,0x8d,0x31,0x8f,0x39,0xcf,
0x31,0x8f,0x31,0x8d,0x29,0x4b,0x21,0x0b,
0x21,0x0b,0x21,0x09,0x18,0xc9,0x18,0xc7,
0x18,0xc7,0x18,0xc9,0x18,0xc9,0x00,0x00,
0x00,0x00,0x00,0x00,0x18,0xc7,0x18,0xc9,
0x18,0xc9,0x18,0xc7,0x18,0xc7,0x18,0xc9,
0x20,0xc9,0x21,0x09,0x19,0x09,0x21,0x09,
0x21,0x0b,0x21,0x0b,0x29,0x4d,0x29,0x4d,
0x29,0x8f,0x39,0xd1,0x42,0x13,0x4a,0x53,
0x42,0x13,0x39,0xd1,0x31,0x8f,0x21,0x4b,
0x21,0x0b,0x21,0x09,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x20,0xc7,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x18,0xc7,0x18,0xc7,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x19,0x09,
0x19,0x0b,0x29,0x4b,0x21,0x4d,0x29,0x8f,
0x39,0xd1,0x52,0x97,0x6b,0x9f,0x73,0x9f,
0x6b,0x5d,0x52,0x95,0x31,0xd1,0x29,0x4d,
0x21,0x0b,0x21,0x09,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x18,0xc7,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x19,0x0b,0x21,0x0b,0x21,0x4d,0x31,0x8f,
0x42,0x55,0x7c,0x23,0xb5,0xf1,0xde,0xf9,
0xb5,0xaf,0x73,0x9f,0x42,0x13,0x31,0x8d,
0x21,0x0b,0x18,0xc9,0x18,0xc9,0x21,0x09,
0x18,0xc9,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x21,0x09,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x18,0xc7,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x19,0x0b,0x21,0x0b,0x29,0x4d,0x31,0xd1,
0x52,0x99,0x9c,0xe9,0xff,0xff,0xff,0xff,
0xe7,0x3b,0x84,0x23,0x4a,0x55,0x31,0x8f,
0x29,0x4b,0x18,0xc9,0x18,0xc9,0x21,0x09,
0x21,0x09,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x21,0x09,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xc7,0x10,0xc9,0x18,0xc9,
0x18,0xc9,0x19,0x0b,0x21,0x4d,0x31,0xd1,
0x52,0xd9,0x94,0xe9,0xf7,0xbf,0xff,0xff,
0xc6,0x75,0x6b,0x5d,0x42,0x13,0x29,0x4d,
0x21,0x0b,0x19,0x09,0x18,0xc9,0x21,0x09,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x21,0x09,0x21,0x09,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x10,0xc9,0x10,0xc9,
0x18,0xc9,0x18,0xcb,0x21,0x0d,0x29,0x8f,
0x42,0x13,0x5b,0x1b,0x84,0x23,0x84,0x65,
0x6b,0x5d,0x42,0x13,0x29,0x4d,0x21,0x0b,
0x21,0x0b,0x19,0x0b,0x19,0x0b,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x21,0x09,0x21,0x0b,
0x19,0x09,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x18,0xcb,0x21,0x0b,
0x29,0x8f,0x39,0xd1,0x42,0x13,0x42,0x13,
0x31,0xd1,0x29,0x8f,0x21,0x0d,0x21,0x0b,
0x19,0x0b,0x19,0x0b,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x21,0x0b,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xcb,0x19,0x0b,0x19,0x0b,
0x21,0x0d,0x21,0x4d,0x29,0x4d,0x29,0x4d,
0x21,0x4d,0x21,0x0d,0x21,0x0b,0x19,0x0b,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x21,0x0b,0x19,0x0b,0x18,0xcb,
0x18,0xcb,0x19,0x0b,0x19,0x0b,0x21,0x0b,
0x21,0x0b,0x21,0x0b,0x21,0x0b,0x21,0x0b,
0x21,0x0b,0x19,0x0b,0x19,0x0b,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x19,0x0b,0x19,0x0b,0x19,0x0b,0x19,0x0b,
0x19,0x0b,0x19,0x0b,0x19,0x0b,0x21,0x0b,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
};

static Gfx _frake_eat_aligner1[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_41_[] = {
  {    44,     5,   -40,0, 0x0000,0x0000, 0x2a,0x76,0x76,0xff }, /*     0 */
  {   -44,     5,   -40,0, 0x0000,0x0000, 0x35,0x95,0x95,0xff }, /*     1 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x47,0x77,0x77,0xff }, /*     2 */
  {   -11,   107,    -2,0, 0x0000,0x0000, 0x91,0xf1,0xf1,0xff }, /*     3 */
  {    11,   107,   -25,0, 0x0000,0x0000, 0x90,0xf0,0xf0,0xff }, /*     4 */
  {   -11,   107,   -25,0, 0x0000,0x0000, 0x8e,0xed,0xed,0xff }, /*     5 */
  {   -11,   107,    -2,0, 0x0000,0x0000, 0x91,0xf1,0xf1,0xff }, /*     6 */
  {    11,   107,    -2,0, 0x0000,0x0000, 0x94,0xf7,0xf7,0xff }, /*     7 */
  {    11,   107,   -25,0, 0x0000,0x0000, 0x90,0xf0,0xf0,0xff }, /*     8 */
  {    41,    31,    28,0, 0x0000,0x0000, 0x82,0xd9,0xd9,0xff }, /*     9 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x47,0x77,0x77,0xff }, /*    10 */
  {    47,    98,     6,0, 0x0000,0x0000, 0x88,0xe3,0xe3,0xff }, /*    11 */
  {   -40,    31,   -38,0, 0x0000,0x0000, 0x82,0xd9,0xd9,0xff }, /*    12 */
  {   -47,    98,     6,0, 0x0000,0x0000, 0x8b,0xe8,0xe8,0xff }, /*    13 */
  {   -46,    98,   -33,0, 0x0000,0x0000, 0x7a,0xcb,0xcb,0xff }, /*    14 */
  {   -47,    98,     6,0, 0x0000,0x0000, 0x8b,0xe8,0xe8,0xff }, /*    15 */
  {   -11,   107,   -25,0, 0x0000,0x0000, 0x8e,0xed,0xed,0xff }, /*    16 */
  {   -46,    98,   -33,0, 0x0000,0x0000, 0x7a,0xcb,0xcb,0xff }, /*    17 */
  {   -46,    98,   -33,0, 0x0000,0x0000, 0x7a,0xcb,0xcb,0xff }, /*    18 */
  {    11,   107,   -25,0, 0x0000,0x0000, 0x90,0xf0,0xf0,0xff }, /*    19 */
  {    46,    98,   -33,0, 0x0000,0x0000, 0x68,0xae,0xae,0xff }, /*    20 */
  {    46,    98,   -33,0, 0x0000,0x0000, 0x68,0xae,0xae,0xff }, /*    21 */
  {    11,   107,   -25,0, 0x0000,0x0000, 0x90,0xf0,0xf0,0xff }, /*    22 */
  {    47,    98,     6,0, 0x0000,0x0000, 0x88,0xe3,0xe3,0xff }, /*    23 */
  {    11,   107,   -25,0, 0x0000,0x0000, 0x90,0xf0,0xf0,0xff }, /*    24 */
  {    11,   107,    -2,0, 0x0000,0x0000, 0x94,0xf7,0xf7,0xff }, /*    25 */
  {    47,    98,     6,0, 0x0000,0x0000, 0x88,0xe3,0xe3,0xff }, /*    26 */
  {    41,    31,    28,0, 0x0000,0x0000, 0x82,0xd9,0xd9,0xff }, /*    27 */
  {    44,     5,   -40,0, 0x0000,0x0000, 0x2a,0x76,0x76,0xff }, /*    28 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x47,0x77,0x77,0xff }, /*    29 */
  {   -41,    31,    28,0, 0x0000,0x0000, 0x6f,0xb9,0xb9,0xff }, /*    30 */
  {    44,     5,    31,0, 0x0000,0x0000, 0x2d,0x7d,0x7d,0xff }, /*    31 */
  {    41,    31,    28,0, 0x0000,0x0000, 0x82,0xd9,0xd9,0xff }, /*    32 */
  {   -41,    31,    28,0, 0x0000,0x0000, 0x6f,0xb9,0xb9,0xff }, /*    33 */
  {   -44,     5,    31,0, 0x0000,0x0000, 0x2b,0x79,0x79,0xff }, /*    34 */
  {    44,     5,    31,0, 0x0000,0x0000, 0x2d,0x7d,0x7d,0xff }, /*    35 */
  {   -40,    31,   -38,0, 0x0000,0x0000, 0x82,0xd9,0xd9,0xff }, /*    36 */
  {   -44,     5,   -40,0, 0x0000,0x0000, 0x35,0x95,0x95,0xff }, /*    37 */
  {   -41,    31,    28,0, 0x0000,0x0000, 0x6f,0xb9,0xb9,0xff }, /*    38 */
  {   -40,    31,   -38,0, 0x0000,0x0000, 0x82,0xd9,0xd9,0xff }, /*    39 */
  {   -46,    98,   -33,0, 0x0000,0x0000, 0x7a,0xcb,0xcb,0xff }, /*    40 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x47,0x77,0x77,0xff }, /*    41 */
  {   -46,    98,   -33,0, 0x0000,0x0000, 0x7a,0xcb,0xcb,0xff }, /*    42 */
  {    46,    98,   -33,0, 0x0000,0x0000, 0x68,0xae,0xae,0xff }, /*    43 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x47,0x77,0x77,0xff }, /*    44 */
  {   -44,     5,   -40,0, 0x0000,0x0000, 0x35,0x95,0x95,0xff }, /*    45 */
  {   -44,     5,    31,0, 0x0000,0x0000, 0x2b,0x79,0x79,0xff }, /*    46 */
  {   -41,    31,    28,0, 0x0000,0x0000, 0x6f,0xb9,0xb9,0xff }, /*    47 */
  {    41,    31,    28,0, 0x0000,0x0000, 0x82,0xd9,0xd9,0xff }, /*    48 */
  {    44,     5,    31,0, 0x0000,0x0000, 0x2d,0x7d,0x7d,0xff }, /*    49 */
  {    44,     5,   -40,0, 0x0000,0x0000, 0x2a,0x76,0x76,0xff }, /*    50 */
  {   -46,    98,   -33,0, 0x0000,0x0000, 0x7a,0xcb,0xcb,0xff }, /*    51 */
  {   -11,   107,   -25,0, 0x0000,0x0000, 0x8e,0xed,0xed,0xff }, /*    52 */
  {    11,   107,   -25,0, 0x0000,0x0000, 0x90,0xf0,0xf0,0xff }, /*    53 */
  {   -47,    98,     6,0, 0x0000,0x0000, 0x8b,0xe8,0xe8,0xff }, /*    54 */
  {   -11,   107,    -2,0, 0x0000,0x0000, 0x91,0xf1,0xf1,0xff }, /*    55 */
  {   -11,   107,   -25,0, 0x0000,0x0000, 0x8e,0xed,0xed,0xff }, /*    56 */
  {   -40,    31,   -38,0, 0x0000,0x0000, 0x82,0xd9,0xd9,0xff }, /*    57 */
  {   -41,    31,    28,0, 0x0000,0x0000, 0x6f,0xb9,0xb9,0xff }, /*    58 */
  {   -47,    98,     6,0, 0x0000,0x0000, 0x8b,0xe8,0xe8,0xff }, /*    59 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x47,0x77,0x77,0xff }, /*    60 */
  {    46,    98,   -33,0, 0x0000,0x0000, 0x68,0xae,0xae,0xff }, /*    61 */
  {    47,    98,     6,0, 0x0000,0x0000, 0x88,0xe3,0xe3,0xff }, /*    62 */
  {   -44,     5,   -40,0, 0x0000,0x0000, 0x35,0x95,0x95,0xff }, /*    63 */
  {   -40,    31,   -38,0, 0x0000,0x0000, 0x82,0xd9,0xd9,0xff }, /*    64 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x47,0x77,0x77,0xff }, /*    65 */
  {    41,    31,    28,0, 0x0779,0x0000, 0xd9,0xd9,0xd9,0xff }, /*    66 */
  {    47,    98,     6,0, 0x07ff,0x070b, 0xe3,0xe3,0xe3,0xff }, /*    67 */
  {   -41,    31,    28,0, 0x0086,0x0000, 0xb9,0xb9,0xb9,0xff }, /*    68 */
  {    47,    98,     6,0, 0x07ff,0x070b, 0xe3,0xe3,0xe3,0xff }, /*    69 */
  {   -11,   107,    -2,0, 0x0302,0x0800, 0xf1,0xf1,0xf1,0xff }, /*    70 */
  {   -47,    98,     6,0, 0x0000,0x070b, 0xe8,0xe8,0xe8,0xff }, /*    71 */
  {    47,    98,     6,0, 0x07ff,0x070b, 0xe3,0xe3,0xe3,0xff }, /*    72 */
  {    11,   107,    -2,0, 0x04fd,0x0800, 0xf7,0xf7,0xf7,0xff }, /*    73 */
  {   -11,   107,    -2,0, 0x0302,0x0800, 0xf1,0xf1,0xf1,0xff }, /*    74 */
  {    47,    98,     6,0, 0x07ff,0x070b, 0xe3,0xe3,0xe3,0xff }, /*    75 */
  {   -47,    98,     6,0, 0x0000,0x070b, 0xe8,0xe8,0xe8,0xff }, /*    76 */
  {   -41,    31,    28,0, 0x0086,0x0000, 0xb9,0xb9,0xb9,0xff }, /*    77 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_41_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP2Triangles(  3,  4,  5,  0,  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP2Triangles( 21, 22, 23,  0, 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 30, 0 ),
  gsSP2Triangles( 30, 31,  0,  0,  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 992), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 1024), 14, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsDPPipeSync(),
  gsDPSetCombineLERP(TEXEL0, 0, SHADE, 0, SHADE, 0, TEXEL0, 0, TEXEL0, 0, SHADE, 0, SHADE, 0, TEXEL0, 0),
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_ON),
  gsDPSetTextureLUT( G_TT_NONE ),
  gsDPLoadMultiBlock( SEGADRS(6,0), 0, 0,
    G_IM_FMT_RGBA, G_IM_SIZ_16b, 32, 32, 0,
    0, 0, 5, 5, 0, 0 ),
  gsSP1Triangle(  2,  3,  4,  0),
  gsSP1Triangle(  5,  6,  7,  0),
  gsSP1Triangle(  8,  9, 10,  0),
  gsSP1Triangle( 11, 12, 13,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_41_ = {
  gfx_Shape_of___POLYHEDRON_41_,
  vtx_Shape_of___POLYHEDRON_41_, 78, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner2[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_55_[] = {
  {   -92,    45,     4,0, 0x0000,0x0000, 0xea,0xa9,0x7a,0xff }, /*     0 */
  {   -52,    54,     5,0, 0x0000,0x0000, 0xa7,0x78,0x56,0xff }, /*     1 */
  {   -75,    71,     5,0, 0x0000,0x0000, 0xcc,0x93,0x6a,0xff }, /*     2 */
  {   -92,    45,     4,0, 0x0000,0x0000, 0xea,0xa9,0x7a,0xff }, /*     3 */
  {   -71,    30,     4,0, 0x0000,0x0000, 0x84,0x60,0x45,0xff }, /*     4 */
  {   -52,    54,     5,0, 0x0000,0x0000, 0xa7,0x78,0x56,0xff }, /*     5 */
  {   -71,    30,   -20,0, 0x0000,0x0000, 0xb3,0x81,0x5d,0xff }, /*     6 */
  {   -92,    45,   -20,0, 0x0000,0x0000, 0xca,0x92,0x69,0xff }, /*     7 */
  {   -52,    54,   -22,0, 0x0000,0x0000, 0xc3,0x8d,0x65,0xff }, /*     8 */
  {   -92,    45,   -20,0, 0x0000,0x0000, 0xca,0x92,0x69,0xff }, /*     9 */
  {   -75,    71,   -22,0, 0x0000,0x0000, 0xaa,0x7b,0x58,0xff }, /*    10 */
  {   -52,    54,   -22,0, 0x0000,0x0000, 0xc3,0x8d,0x65,0xff }, /*    11 */
  {   -71,    30,     4,0, 0x0000,0x0000, 0x84,0x60,0x45,0xff }, /*    12 */
  {   -71,    30,   -20,0, 0x0000,0x0000, 0xb3,0x81,0x5d,0xff }, /*    13 */
  {   -52,    54,     5,0, 0x0000,0x0000, 0xa7,0x78,0x56,0xff }, /*    14 */
  {   -71,    30,   -20,0, 0x0000,0x0000, 0xb3,0x81,0x5d,0xff }, /*    15 */
  {   -52,    54,   -22,0, 0x0000,0x0000, 0xc3,0x8d,0x65,0xff }, /*    16 */
  {   -52,    54,     5,0, 0x0000,0x0000, 0xa7,0x78,0x56,0xff }, /*    17 */
  {   -92,    45,   -20,0, 0x0000,0x0000, 0xca,0x92,0x69,0xff }, /*    18 */
  {   -75,    71,     5,0, 0x0000,0x0000, 0xcc,0x93,0x6a,0xff }, /*    19 */
  {   -75,    71,   -22,0, 0x0000,0x0000, 0xaa,0x7b,0x58,0xff }, /*    20 */
  {   -52,    54,     5,0, 0x0000,0x0000, 0xa7,0x78,0x56,0xff }, /*    21 */
  {   -56,    73,    -8,0, 0x0000,0x0000, 0xab,0x7b,0x59,0xff }, /*    22 */
  {   -75,    71,     5,0, 0x0000,0x0000, 0xcc,0x93,0x6a,0xff }, /*    23 */
  {   -75,    71,     5,0, 0x0000,0x0000, 0xcc,0x93,0x6a,0xff }, /*    24 */
  {   -56,    73,    -8,0, 0x0000,0x0000, 0xab,0x7b,0x59,0xff }, /*    25 */
  {   -75,    71,   -22,0, 0x0000,0x0000, 0xaa,0x7b,0x58,0xff }, /*    26 */
  {   -75,    71,   -22,0, 0x0000,0x0000, 0xaa,0x7b,0x58,0xff }, /*    27 */
  {   -56,    73,    -8,0, 0x0000,0x0000, 0xab,0x7b,0x59,0xff }, /*    28 */
  {   -52,    54,   -22,0, 0x0000,0x0000, 0xc3,0x8d,0x65,0xff }, /*    29 */
  {   -52,    54,   -22,0, 0x0000,0x0000, 0xc3,0x8d,0x65,0xff }, /*    30 */
  {   -56,    73,    -8,0, 0x0000,0x0000, 0xab,0x7b,0x59,0xff }, /*    31 */
  {   -52,    54,     5,0, 0x0000,0x0000, 0xa7,0x78,0x56,0xff }, /*    32 */
  {   -92,    45,   -20,0, 0x0000,0x0000, 0xca,0x92,0x69,0xff }, /*    33 */
  {   -92,    45,     4,0, 0x0000,0x0000, 0xea,0xa9,0x7a,0xff }, /*    34 */
  {   -75,    71,     5,0, 0x0000,0x0000, 0xcc,0x93,0x6a,0xff }, /*    35 */
  {   -71,    30,   -20,0, 0x0000,0x0000, 0xb3,0x81,0x5d,0xff }, /*    36 */
  {   -88,    29,    -8,0, 0x0000,0x0000, 0xbd,0x52,0x3b,0xff }, /*    37 */
  {   -92,    45,   -20,0, 0x0000,0x0000, 0xca,0x92,0x69,0xff }, /*    38 */
  {   -92,    45,   -20,0, 0x0000,0x0000, 0xca,0x92,0x69,0xff }, /*    39 */
  {   -88,    29,    -8,0, 0x0000,0x0000, 0xbd,0x52,0x3b,0xff }, /*    40 */
  {   -92,    45,     4,0, 0x0000,0x0000, 0xea,0xa9,0x7a,0xff }, /*    41 */
  {   -92,    45,     4,0, 0x0000,0x0000, 0xea,0xa9,0x7a,0xff }, /*    42 */
  {   -88,    29,    -8,0, 0x0000,0x0000, 0xbd,0x52,0x3b,0xff }, /*    43 */
  {   -71,    30,     4,0, 0x0000,0x0000, 0x84,0x60,0x45,0xff }, /*    44 */
  {   -71,    30,     4,0, 0x0000,0x0000, 0x84,0x60,0x45,0xff }, /*    45 */
  {   -88,    29,    -8,0, 0x0000,0x0000, 0xbd,0x52,0x3b,0xff }, /*    46 */
  {   -71,    30,   -20,0, 0x0000,0x0000, 0xb3,0x81,0x5d,0xff }, /*    47 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_55_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP2Triangles( 12, 13, 14,  0, 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 16, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_55_ = {
  gfx_Shape_of___POLYHEDRON_55_,
  vtx_Shape_of___POLYHEDRON_55_, 48, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner3[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_57_[] = {
  {  -110,    15,    -1,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*     0 */
  {   -74,    29,     3,0, 0x0000,0x0000, 0xc3,0x8d,0x65,0xff }, /*     1 */
  {   -92,    42,     3,0, 0x0000,0x0000, 0xaf,0x7f,0x5b,0xff }, /*     2 */
  {  -110,    15,    -1,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*     3 */
  {   -94,     3,    -1,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*     4 */
  {   -74,    29,     3,0, 0x0000,0x0000, 0xc3,0x8d,0x65,0xff }, /*     5 */
  {   -94,     3,   -15,0, 0x0000,0x0000, 0x8b,0x64,0x48,0xff }, /*     6 */
  {  -110,    15,   -15,0, 0x0000,0x0000, 0xde,0xa0,0x73,0xff }, /*     7 */
  {   -74,    29,   -19,0, 0x0000,0x0000, 0xa4,0x76,0x55,0xff }, /*     8 */
  {   -94,     3,    -1,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*     9 */
  {   -94,     3,   -15,0, 0x0000,0x0000, 0x8b,0x64,0x48,0xff }, /*    10 */
  {   -74,    29,     3,0, 0x0000,0x0000, 0xc3,0x8d,0x65,0xff }, /*    11 */
  {   -94,     3,   -15,0, 0x0000,0x0000, 0x8b,0x64,0x48,0xff }, /*    12 */
  {   -74,    29,   -19,0, 0x0000,0x0000, 0xa4,0x76,0x55,0xff }, /*    13 */
  {   -74,    29,     3,0, 0x0000,0x0000, 0xc3,0x8d,0x65,0xff }, /*    14 */
  {  -110,    15,   -15,0, 0x0000,0x0000, 0xde,0xa0,0x73,0xff }, /*    15 */
  {   -92,    42,     3,0, 0x0000,0x0000, 0xaf,0x7f,0x5b,0xff }, /*    16 */
  {   -92,    42,   -19,0, 0x0000,0x0000, 0xcc,0x94,0x6a,0xff }, /*    17 */
  {  -110,    15,   -15,0, 0x0000,0x0000, 0xde,0xa0,0x73,0xff }, /*    18 */
  {  -110,    15,    -1,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*    19 */
  {   -92,    42,     3,0, 0x0000,0x0000, 0xaf,0x7f,0x5b,0xff }, /*    20 */
  {   -74,    29,     3,0, 0x0000,0x0000, 0xc3,0x8d,0x65,0xff }, /*    21 */
  {   -77,    44,    -8,0, 0x0000,0x0000, 0xdd,0xa0,0x73,0xff }, /*    22 */
  {   -92,    42,     3,0, 0x0000,0x0000, 0xaf,0x7f,0x5b,0xff }, /*    23 */
  {   -92,    42,     3,0, 0x0000,0x0000, 0xaf,0x7f,0x5b,0xff }, /*    24 */
  {   -77,    44,    -8,0, 0x0000,0x0000, 0xdd,0xa0,0x73,0xff }, /*    25 */
  {   -92,    42,   -19,0, 0x0000,0x0000, 0xcc,0x94,0x6a,0xff }, /*    26 */
  {   -92,    42,   -19,0, 0x0000,0x0000, 0xcc,0x94,0x6a,0xff }, /*    27 */
  {   -77,    44,    -8,0, 0x0000,0x0000, 0xdd,0xa0,0x73,0xff }, /*    28 */
  {   -74,    29,   -19,0, 0x0000,0x0000, 0xa4,0x76,0x55,0xff }, /*    29 */
  {   -74,    29,   -19,0, 0x0000,0x0000, 0xa4,0x76,0x55,0xff }, /*    30 */
  {   -77,    44,    -8,0, 0x0000,0x0000, 0xdd,0xa0,0x73,0xff }, /*    31 */
  {   -74,    29,     3,0, 0x0000,0x0000, 0xc3,0x8d,0x65,0xff }, /*    32 */
  {  -110,    15,   -15,0, 0x0000,0x0000, 0xde,0xa0,0x73,0xff }, /*    33 */
  {   -92,    42,   -19,0, 0x0000,0x0000, 0xcc,0x94,0x6a,0xff }, /*    34 */
  {   -74,    29,   -19,0, 0x0000,0x0000, 0xa4,0x76,0x55,0xff }, /*    35 */
  {   -94,     3,   -15,0, 0x0000,0x0000, 0x8b,0x64,0x48,0xff }, /*    36 */
  {  -107,     2,    -8,0, 0x0000,0x0000, 0xd0,0x96,0x6c,0xff }, /*    37 */
  {  -110,    15,   -15,0, 0x0000,0x0000, 0xde,0xa0,0x73,0xff }, /*    38 */
  {  -110,    15,   -15,0, 0x0000,0x0000, 0xde,0xa0,0x73,0xff }, /*    39 */
  {  -107,     2,    -8,0, 0x0000,0x0000, 0xd0,0x96,0x6c,0xff }, /*    40 */
  {  -110,    15,    -1,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*    41 */
  {  -110,    15,    -1,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*    42 */
  {  -107,     2,    -8,0, 0x0000,0x0000, 0xd0,0x96,0x6c,0xff }, /*    43 */
  {   -94,     3,    -1,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*    44 */
  {   -94,     3,    -1,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*    45 */
  {  -107,     2,    -8,0, 0x0000,0x0000, 0xd0,0x96,0x6c,0xff }, /*    46 */
  {   -94,     3,   -15,0, 0x0000,0x0000, 0x8b,0x64,0x48,0xff }, /*    47 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_57_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP2Triangles(  9, 10, 11,  0, 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 16, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_57_ = {
  gfx_Shape_of___POLYHEDRON_57_,
  vtx_Shape_of___POLYHEDRON_57_, 48, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner4[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_59_[] = {
  {  -127,    -9,     2,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*     0 */
  {  -103,    -5,     3,0, 0x0000,0x0000, 0xd7,0x9b,0x70,0xff }, /*     1 */
  {  -116,     3,     3,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*     2 */
  {  -127,    -9,     2,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*     3 */
  {  -113,   -19,     2,0, 0x0000,0x0000, 0xc6,0x8f,0x67,0xff }, /*     4 */
  {  -103,    -5,     3,0, 0x0000,0x0000, 0xd7,0x9b,0x70,0xff }, /*     5 */
  {  -113,   -19,   -24,0, 0x0000,0x0000, 0x7b,0x59,0x40,0xff }, /*     6 */
  {  -127,    -9,   -24,0, 0x0000,0x0000, 0xe8,0xa8,0x78,0xff }, /*     7 */
  {  -103,    -5,   -19,0, 0x0000,0x0000, 0x8c,0x65,0x49,0xff }, /*     8 */
  {  -127,    -9,   -24,0, 0x0000,0x0000, 0xe8,0xa8,0x78,0xff }, /*     9 */
  {  -116,     3,   -19,0, 0x0000,0x0000, 0xe7,0xa7,0x78,0xff }, /*    10 */
  {  -103,    -5,   -19,0, 0x0000,0x0000, 0x8c,0x65,0x49,0xff }, /*    11 */
  {  -113,   -19,     2,0, 0x0000,0x0000, 0xc6,0x8f,0x67,0xff }, /*    12 */
  {  -113,   -19,   -24,0, 0x0000,0x0000, 0x7b,0x59,0x40,0xff }, /*    13 */
  {  -103,    -5,     3,0, 0x0000,0x0000, 0xd7,0x9b,0x70,0xff }, /*    14 */
  {  -113,   -19,   -24,0, 0x0000,0x0000, 0x7b,0x59,0x40,0xff }, /*    15 */
  {  -103,    -5,   -19,0, 0x0000,0x0000, 0x8c,0x65,0x49,0xff }, /*    16 */
  {  -103,    -5,     3,0, 0x0000,0x0000, 0xd7,0x9b,0x70,0xff }, /*    17 */
  {  -127,    -9,   -24,0, 0x0000,0x0000, 0xe8,0xa8,0x78,0xff }, /*    18 */
  {  -116,     3,     3,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    19 */
  {  -116,     3,   -19,0, 0x0000,0x0000, 0xe7,0xa7,0x78,0xff }, /*    20 */
  {  -127,    -9,   -24,0, 0x0000,0x0000, 0xe8,0xa8,0x78,0xff }, /*    21 */
  {  -127,    -9,     2,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    22 */
  {  -116,     3,     3,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    23 */
  {  -103,    -5,     3,0, 0x0000,0x0000, 0xd7,0x9b,0x70,0xff }, /*    24 */
  {  -103,     7,    -8,0, 0x0000,0x0000, 0xac,0x7c,0x59,0xff }, /*    25 */
  {  -116,     3,     3,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    26 */
  {  -116,     3,     3,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    27 */
  {  -103,     7,    -8,0, 0x0000,0x0000, 0xac,0x7c,0x59,0xff }, /*    28 */
  {  -116,     3,   -19,0, 0x0000,0x0000, 0xe7,0xa7,0x78,0xff }, /*    29 */
  {  -116,     3,   -19,0, 0x0000,0x0000, 0xe7,0xa7,0x78,0xff }, /*    30 */
  {  -103,     7,    -8,0, 0x0000,0x0000, 0xac,0x7c,0x59,0xff }, /*    31 */
  {  -103,    -5,   -19,0, 0x0000,0x0000, 0x8c,0x65,0x49,0xff }, /*    32 */
  {  -103,    -5,   -19,0, 0x0000,0x0000, 0x8c,0x65,0x49,0xff }, /*    33 */
  {  -103,     7,    -8,0, 0x0000,0x0000, 0xac,0x7c,0x59,0xff }, /*    34 */
  {  -103,    -5,     3,0, 0x0000,0x0000, 0xd7,0x9b,0x70,0xff }, /*    35 */
  {  -113,   -19,   -24,0, 0x0000,0x0000, 0x7b,0x59,0x40,0xff }, /*    36 */
  {  -124,   -20,    -8,0, 0x0000,0x0000, 0x8d,0x66,0x49,0xff }, /*    37 */
  {  -127,    -9,   -24,0, 0x0000,0x0000, 0xe8,0xa8,0x78,0xff }, /*    38 */
  {  -127,    -9,   -24,0, 0x0000,0x0000, 0xe8,0xa8,0x78,0xff }, /*    39 */
  {  -124,   -20,    -8,0, 0x0000,0x0000, 0x8d,0x66,0x49,0xff }, /*    40 */
  {  -127,    -9,     2,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    41 */
  {  -127,    -9,     2,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    42 */
  {  -124,   -20,    -8,0, 0x0000,0x0000, 0x8d,0x66,0x49,0xff }, /*    43 */
  {  -113,   -19,     2,0, 0x0000,0x0000, 0xc6,0x8f,0x67,0xff }, /*    44 */
  {  -113,   -19,     2,0, 0x0000,0x0000, 0xc6,0x8f,0x67,0xff }, /*    45 */
  {  -124,   -20,    -8,0, 0x0000,0x0000, 0x8d,0x66,0x49,0xff }, /*    46 */
  {  -113,   -19,   -24,0, 0x0000,0x0000, 0x7b,0x59,0x40,0xff }, /*    47 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_59_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP2Triangles( 12, 13, 14,  0, 15, 16, 17,  0),
  gsSP2Triangles( 18, 19, 20,  0, 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 16, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP2Triangles(  7,  8,  9,  0, 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_59_ = {
  gfx_Shape_of___POLYHEDRON_59_,
  vtx_Shape_of___POLYHEDRON_59_, 48, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner5[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_53_[] = {
  {   -81,    69,     9,0, 0x0000,0x0000, 0x8e,0xee,0xee,0xff }, /*     0 */
  {   -34,    80,     5,0, 0x0000,0x0000, 0x69,0xb0,0xb0,0xff }, /*     1 */
  {   -57,    97,     5,0, 0x0000,0x0000, 0x79,0xc9,0xc9,0xff }, /*     2 */
  {   -81,    69,     9,0, 0x0000,0x0000, 0x8e,0xee,0xee,0xff }, /*     3 */
  {   -51,    48,     9,0, 0x0000,0x0000, 0x52,0x8a,0x8a,0xff }, /*     4 */
  {   -34,    80,     5,0, 0x0000,0x0000, 0x69,0xb0,0xb0,0xff }, /*     5 */
  {   -51,    48,   -26,0, 0x0000,0x0000, 0x6d,0xb5,0xb5,0xff }, /*     6 */
  {   -81,    69,   -26,0, 0x0000,0x0000, 0x7f,0xd3,0xd3,0xff }, /*     7 */
  {   -34,    80,   -22,0, 0x0000,0x0000, 0x72,0xbe,0xbe,0xff }, /*     8 */
  {   -81,    69,   -26,0, 0x0000,0x0000, 0x7f,0xd3,0xd3,0xff }, /*     9 */
  {   -57,    97,   -22,0, 0x0000,0x0000, 0x66,0xaa,0xaa,0xff }, /*    10 */
  {   -34,    80,   -22,0, 0x0000,0x0000, 0x72,0xbe,0xbe,0xff }, /*    11 */
  {   -51,    48,     9,0, 0x0000,0x0000, 0x52,0x8a,0x8a,0xff }, /*    12 */
  {   -51,    48,   -26,0, 0x0000,0x0000, 0x6d,0xb5,0xb5,0xff }, /*    13 */
  {   -34,    80,     5,0, 0x0000,0x0000, 0x69,0xb0,0xb0,0xff }, /*    14 */
  {   -81,    69,   -26,0, 0x0000,0x0000, 0x7f,0xd3,0xd3,0xff }, /*    15 */
  {   -57,    97,     5,0, 0x0000,0x0000, 0x79,0xc9,0xc9,0xff }, /*    16 */
  {   -57,    97,   -22,0, 0x0000,0x0000, 0x66,0xaa,0xaa,0xff }, /*    17 */
  {   -81,    69,   -26,0, 0x0000,0x0000, 0x7f,0xd3,0xd3,0xff }, /*    18 */
  {   -81,    69,     9,0, 0x0000,0x0000, 0x8e,0xee,0xee,0xff }, /*    19 */
  {   -57,    97,     5,0, 0x0000,0x0000, 0x79,0xc9,0xc9,0xff }, /*    20 */
  {   -34,    80,     5,0, 0x0000,0x0000, 0x69,0xb0,0xb0,0xff }, /*    21 */
  {   -38,    99,    -8,0, 0x0000,0x0000, 0x65,0xa8,0xa8,0xff }, /*    22 */
  {   -57,    97,     5,0, 0x0000,0x0000, 0x79,0xc9,0xc9,0xff }, /*    23 */
  {   -57,    97,     5,0, 0x0000,0x0000, 0x79,0xc9,0xc9,0xff }, /*    24 */
  {   -38,    99,    -8,0, 0x0000,0x0000, 0x65,0xa8,0xa8,0xff }, /*    25 */
  {   -57,    97,   -22,0, 0x0000,0x0000, 0x66,0xaa,0xaa,0xff }, /*    26 */
  {   -51,    48,   -26,0, 0x0000,0x0000, 0x6d,0xb5,0xb5,0xff }, /*    27 */
  {   -34,    80,   -22,0, 0x0000,0x0000, 0x72,0xbe,0xbe,0xff }, /*    28 */
  {   -34,    80,     5,0, 0x0000,0x0000, 0x69,0xb0,0xb0,0xff }, /*    29 */
  {   -57,    97,   -22,0, 0x0000,0x0000, 0x66,0xaa,0xaa,0xff }, /*    30 */
  {   -38,    99,    -8,0, 0x0000,0x0000, 0x65,0xa8,0xa8,0xff }, /*    31 */
  {   -34,    80,   -22,0, 0x0000,0x0000, 0x72,0xbe,0xbe,0xff }, /*    32 */
  {   -34,    80,   -22,0, 0x0000,0x0000, 0x72,0xbe,0xbe,0xff }, /*    33 */
  {   -38,    99,    -8,0, 0x0000,0x0000, 0x65,0xa8,0xa8,0xff }, /*    34 */
  {   -34,    80,     5,0, 0x0000,0x0000, 0x69,0xb0,0xb0,0xff }, /*    35 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_53_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP2Triangles( 24, 25, 26,  0, 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 4, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_53_ = {
  gfx_Shape_of___POLYHEDRON_53_,
  vtx_Shape_of___POLYHEDRON_53_, 36, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner6[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_47_[] = {
  {    75,    71,     5,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*     0 */
  {    52,    54,     5,0, 0x0000,0x0000, 0xb9,0x86,0x60,0xff }, /*     1 */
  {    92,    45,     4,0, 0x0000,0x0000, 0xee,0xac,0x7c,0xff }, /*     2 */
  {    52,    54,     5,0, 0x0000,0x0000, 0xb9,0x86,0x60,0xff }, /*     3 */
  {    71,    30,     4,0, 0x0000,0x0000, 0x8a,0x64,0x47,0xff }, /*     4 */
  {    92,    45,     4,0, 0x0000,0x0000, 0xee,0xac,0x7c,0xff }, /*     5 */
  {    52,    54,   -22,0, 0x0000,0x0000, 0xb9,0x86,0x60,0xff }, /*     6 */
  {    92,    45,   -20,0, 0x0000,0x0000, 0xb2,0x81,0x5d,0xff }, /*     7 */
  {    71,    30,   -20,0, 0x0000,0x0000, 0x99,0x6e,0x4f,0xff }, /*     8 */
  {    52,    54,     5,0, 0x0000,0x0000, 0xb9,0x86,0x60,0xff }, /*     9 */
  {    71,    30,   -20,0, 0x0000,0x0000, 0x99,0x6e,0x4f,0xff }, /*    10 */
  {    71,    30,     4,0, 0x0000,0x0000, 0x8a,0x64,0x47,0xff }, /*    11 */
  {    75,    71,   -22,0, 0x0000,0x0000, 0xa7,0x79,0x57,0xff }, /*    12 */
  {    75,    71,     5,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*    13 */
  {    92,    45,   -20,0, 0x0000,0x0000, 0xb2,0x81,0x5d,0xff }, /*    14 */
  {    75,    71,     5,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*    15 */
  {    56,    73,    -8,0, 0x0000,0x0000, 0xe6,0xa7,0x78,0xff }, /*    16 */
  {    52,    54,     5,0, 0x0000,0x0000, 0xb9,0x86,0x60,0xff }, /*    17 */
  {    75,    71,   -22,0, 0x0000,0x0000, 0xa7,0x79,0x57,0xff }, /*    18 */
  {    56,    73,    -8,0, 0x0000,0x0000, 0xe6,0xa7,0x78,0xff }, /*    19 */
  {    75,    71,     5,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*    20 */
  {    52,    54,   -22,0, 0x0000,0x0000, 0xb9,0x86,0x60,0xff }, /*    21 */
  {    56,    73,    -8,0, 0x0000,0x0000, 0xe6,0xa7,0x78,0xff }, /*    22 */
  {    75,    71,   -22,0, 0x0000,0x0000, 0xa7,0x79,0x57,0xff }, /*    23 */
  {    52,    54,     5,0, 0x0000,0x0000, 0xb9,0x86,0x60,0xff }, /*    24 */
  {    56,    73,    -8,0, 0x0000,0x0000, 0xe6,0xa7,0x78,0xff }, /*    25 */
  {    52,    54,   -22,0, 0x0000,0x0000, 0xb9,0x86,0x60,0xff }, /*    26 */
  {    75,    71,     5,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*    27 */
  {    92,    45,     4,0, 0x0000,0x0000, 0xee,0xac,0x7c,0xff }, /*    28 */
  {    92,    45,   -20,0, 0x0000,0x0000, 0xb2,0x81,0x5d,0xff }, /*    29 */
  {    52,    54,     5,0, 0x0000,0x0000, 0xb9,0x86,0x60,0xff }, /*    30 */
  {    52,    54,   -22,0, 0x0000,0x0000, 0xb9,0x86,0x60,0xff }, /*    31 */
  {    71,    30,   -20,0, 0x0000,0x0000, 0x99,0x6e,0x4f,0xff }, /*    32 */
  {    52,    54,   -22,0, 0x0000,0x0000, 0xb9,0x86,0x60,0xff }, /*    33 */
  {    75,    71,   -22,0, 0x0000,0x0000, 0xa7,0x79,0x57,0xff }, /*    34 */
  {    92,    45,   -20,0, 0x0000,0x0000, 0xb2,0x81,0x5d,0xff }, /*    35 */
  {    92,    45,   -20,0, 0x0000,0x0000, 0xb2,0x81,0x5d,0xff }, /*    36 */
  {    88,    29,    -8,0, 0x0000,0x0000, 0xca,0x57,0x3f,0xff }, /*    37 */
  {    71,    30,   -20,0, 0x0000,0x0000, 0x99,0x6e,0x4f,0xff }, /*    38 */
  {    92,    45,     4,0, 0x0000,0x0000, 0xee,0xac,0x7c,0xff }, /*    39 */
  {    88,    29,    -8,0, 0x0000,0x0000, 0xca,0x57,0x3f,0xff }, /*    40 */
  {    92,    45,   -20,0, 0x0000,0x0000, 0xb2,0x81,0x5d,0xff }, /*    41 */
  {    71,    30,     4,0, 0x0000,0x0000, 0x8a,0x64,0x47,0xff }, /*    42 */
  {    88,    29,    -8,0, 0x0000,0x0000, 0xca,0x57,0x3f,0xff }, /*    43 */
  {    92,    45,     4,0, 0x0000,0x0000, 0xee,0xac,0x7c,0xff }, /*    44 */
  {    71,    30,   -20,0, 0x0000,0x0000, 0x99,0x6e,0x4f,0xff }, /*    45 */
  {    88,    29,    -8,0, 0x0000,0x0000, 0xca,0x57,0x3f,0xff }, /*    46 */
  {    71,    30,     4,0, 0x0000,0x0000, 0x8a,0x64,0x47,0xff }, /*    47 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_47_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 16, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_47_ = {
  gfx_Shape_of___POLYHEDRON_47_,
  vtx_Shape_of___POLYHEDRON_47_, 48, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner7[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_49_[] = {
  {    92,    42,     3,0, 0x0000,0x0000, 0x9d,0x72,0x52,0xff }, /*     0 */
  {    74,    29,     3,0, 0x0000,0x0000, 0xc9,0x91,0x68,0xff }, /*     1 */
  {   110,    15,    -1,0, 0x0000,0x0000, 0xd7,0x9c,0x70,0xff }, /*     2 */
  {    74,    29,     3,0, 0x0000,0x0000, 0xc9,0x91,0x68,0xff }, /*     3 */
  {    94,     3,    -1,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*     4 */
  {   110,    15,    -1,0, 0x0000,0x0000, 0xd7,0x9c,0x70,0xff }, /*     5 */
  {    74,    29,   -19,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*     6 */
  {   110,    15,   -15,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*     7 */
  {    94,     3,   -15,0, 0x0000,0x0000, 0x6f,0x50,0x3a,0xff }, /*     8 */
  {    74,    29,   -19,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*     9 */
  {    92,    42,   -19,0, 0x0000,0x0000, 0xe2,0xa3,0x75,0xff }, /*    10 */
  {   110,    15,   -15,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*    11 */
  {    74,    29,     3,0, 0x0000,0x0000, 0xc9,0x91,0x68,0xff }, /*    12 */
  {    94,     3,   -15,0, 0x0000,0x0000, 0x6f,0x50,0x3a,0xff }, /*    13 */
  {    94,     3,    -1,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*    14 */
  {    92,    42,   -19,0, 0x0000,0x0000, 0xe2,0xa3,0x75,0xff }, /*    15 */
  {    92,    42,     3,0, 0x0000,0x0000, 0x9d,0x72,0x52,0xff }, /*    16 */
  {   110,    15,   -15,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*    17 */
  {    92,    42,     3,0, 0x0000,0x0000, 0x9d,0x72,0x52,0xff }, /*    18 */
  {   110,    15,    -1,0, 0x0000,0x0000, 0xd7,0x9c,0x70,0xff }, /*    19 */
  {   110,    15,   -15,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*    20 */
  {    92,    42,     3,0, 0x0000,0x0000, 0x9d,0x72,0x52,0xff }, /*    21 */
  {    77,    44,    -8,0, 0x0000,0x0000, 0x73,0x53,0x3c,0xff }, /*    22 */
  {    74,    29,     3,0, 0x0000,0x0000, 0xc9,0x91,0x68,0xff }, /*    23 */
  {    92,    42,   -19,0, 0x0000,0x0000, 0xe2,0xa3,0x75,0xff }, /*    24 */
  {    77,    44,    -8,0, 0x0000,0x0000, 0x73,0x53,0x3c,0xff }, /*    25 */
  {    92,    42,     3,0, 0x0000,0x0000, 0x9d,0x72,0x52,0xff }, /*    26 */
  {    74,    29,   -19,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*    27 */
  {    77,    44,    -8,0, 0x0000,0x0000, 0x73,0x53,0x3c,0xff }, /*    28 */
  {    92,    42,   -19,0, 0x0000,0x0000, 0xe2,0xa3,0x75,0xff }, /*    29 */
  {    74,    29,     3,0, 0x0000,0x0000, 0xc9,0x91,0x68,0xff }, /*    30 */
  {    77,    44,    -8,0, 0x0000,0x0000, 0x73,0x53,0x3c,0xff }, /*    31 */
  {    74,    29,   -19,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*    32 */
  {    74,    29,     3,0, 0x0000,0x0000, 0xc9,0x91,0x68,0xff }, /*    33 */
  {    74,    29,   -19,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*    34 */
  {    94,     3,   -15,0, 0x0000,0x0000, 0x6f,0x50,0x3a,0xff }, /*    35 */
  {   110,    15,   -15,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*    36 */
  {   107,     2,    -8,0, 0x0000,0x0000, 0x9d,0x72,0x52,0xff }, /*    37 */
  {    94,     3,   -15,0, 0x0000,0x0000, 0x6f,0x50,0x3a,0xff }, /*    38 */
  {   110,    15,    -1,0, 0x0000,0x0000, 0xd7,0x9c,0x70,0xff }, /*    39 */
  {   107,     2,    -8,0, 0x0000,0x0000, 0x9d,0x72,0x52,0xff }, /*    40 */
  {   110,    15,   -15,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*    41 */
  {    94,     3,    -1,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*    42 */
  {   107,     2,    -8,0, 0x0000,0x0000, 0x9d,0x72,0x52,0xff }, /*    43 */
  {   110,    15,    -1,0, 0x0000,0x0000, 0xd7,0x9c,0x70,0xff }, /*    44 */
  {    94,     3,   -15,0, 0x0000,0x0000, 0x6f,0x50,0x3a,0xff }, /*    45 */
  {   107,     2,    -8,0, 0x0000,0x0000, 0x9d,0x72,0x52,0xff }, /*    46 */
  {    94,     3,    -1,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*    47 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_49_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 16, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_49_ = {
  gfx_Shape_of___POLYHEDRON_49_,
  vtx_Shape_of___POLYHEDRON_49_, 48, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner8[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_51_[] = {
  {   116,     3,     3,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*     0 */
  {   103,    -5,     3,0, 0x0000,0x0000, 0xa8,0x7a,0x57,0xff }, /*     1 */
  {   127,    -9,     2,0, 0x0000,0x0000, 0xa3,0x76,0x55,0xff }, /*     2 */
  {   103,    -5,     3,0, 0x0000,0x0000, 0xa8,0x7a,0x57,0xff }, /*     3 */
  {   113,   -19,     2,0, 0x0000,0x0000, 0xd3,0x99,0x6e,0xff }, /*     4 */
  {   127,    -9,     2,0, 0x0000,0x0000, 0xa3,0x76,0x55,0xff }, /*     5 */
  {   103,    -5,   -19,0, 0x0000,0x0000, 0x77,0x56,0x3e,0xff }, /*     6 */
  {   127,    -9,   -24,0, 0x0000,0x0000, 0xdd,0xa0,0x73,0xff }, /*     7 */
  {   113,   -19,   -24,0, 0x0000,0x0000, 0x9e,0x72,0x52,0xff }, /*     8 */
  {   103,    -5,   -19,0, 0x0000,0x0000, 0x77,0x56,0x3e,0xff }, /*     9 */
  {   116,     3,   -19,0, 0x0000,0x0000, 0xf0,0xae,0x7d,0xff }, /*    10 */
  {   127,    -9,   -24,0, 0x0000,0x0000, 0xdd,0xa0,0x73,0xff }, /*    11 */
  {   103,    -5,     3,0, 0x0000,0x0000, 0xa8,0x7a,0x57,0xff }, /*    12 */
  {   113,   -19,   -24,0, 0x0000,0x0000, 0x9e,0x72,0x52,0xff }, /*    13 */
  {   113,   -19,     2,0, 0x0000,0x0000, 0xd3,0x99,0x6e,0xff }, /*    14 */
  {   116,     3,   -19,0, 0x0000,0x0000, 0xf0,0xae,0x7d,0xff }, /*    15 */
  {   116,     3,     3,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*    16 */
  {   127,    -9,   -24,0, 0x0000,0x0000, 0xdd,0xa0,0x73,0xff }, /*    17 */
  {   116,     3,     3,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*    18 */
  {   103,     7,    -8,0, 0x0000,0x0000, 0xca,0x92,0x69,0xff }, /*    19 */
  {   103,    -5,     3,0, 0x0000,0x0000, 0xa8,0x7a,0x57,0xff }, /*    20 */
  {   116,     3,   -19,0, 0x0000,0x0000, 0xf0,0xae,0x7d,0xff }, /*    21 */
  {   103,     7,    -8,0, 0x0000,0x0000, 0xca,0x92,0x69,0xff }, /*    22 */
  {   116,     3,     3,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*    23 */
  {   103,    -5,   -19,0, 0x0000,0x0000, 0x77,0x56,0x3e,0xff }, /*    24 */
  {   103,     7,    -8,0, 0x0000,0x0000, 0xca,0x92,0x69,0xff }, /*    25 */
  {   116,     3,   -19,0, 0x0000,0x0000, 0xf0,0xae,0x7d,0xff }, /*    26 */
  {   103,    -5,     3,0, 0x0000,0x0000, 0xa8,0x7a,0x57,0xff }, /*    27 */
  {   103,     7,    -8,0, 0x0000,0x0000, 0xca,0x92,0x69,0xff }, /*    28 */
  {   103,    -5,   -19,0, 0x0000,0x0000, 0x77,0x56,0x3e,0xff }, /*    29 */
  {   116,     3,     3,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*    30 */
  {   127,    -9,     2,0, 0x0000,0x0000, 0xa3,0x76,0x55,0xff }, /*    31 */
  {   127,    -9,   -24,0, 0x0000,0x0000, 0xdd,0xa0,0x73,0xff }, /*    32 */
  {   103,    -5,     3,0, 0x0000,0x0000, 0xa8,0x7a,0x57,0xff }, /*    33 */
  {   103,    -5,   -19,0, 0x0000,0x0000, 0x77,0x56,0x3e,0xff }, /*    34 */
  {   113,   -19,   -24,0, 0x0000,0x0000, 0x9e,0x72,0x52,0xff }, /*    35 */
  {   127,    -9,   -24,0, 0x0000,0x0000, 0xdd,0xa0,0x73,0xff }, /*    36 */
  {   124,   -20,    -8,0, 0x0000,0x0000, 0xf3,0xb0,0x7e,0xff }, /*    37 */
  {   113,   -19,   -24,0, 0x0000,0x0000, 0x9e,0x72,0x52,0xff }, /*    38 */
  {   127,    -9,     2,0, 0x0000,0x0000, 0xa3,0x76,0x55,0xff }, /*    39 */
  {   124,   -20,    -8,0, 0x0000,0x0000, 0xf3,0xb0,0x7e,0xff }, /*    40 */
  {   127,    -9,   -24,0, 0x0000,0x0000, 0xdd,0xa0,0x73,0xff }, /*    41 */
  {   113,   -19,     2,0, 0x0000,0x0000, 0xd3,0x99,0x6e,0xff }, /*    42 */
  {   124,   -20,    -8,0, 0x0000,0x0000, 0xf3,0xb0,0x7e,0xff }, /*    43 */
  {   127,    -9,     2,0, 0x0000,0x0000, 0xa3,0x76,0x55,0xff }, /*    44 */
  {   113,   -19,   -24,0, 0x0000,0x0000, 0x9e,0x72,0x52,0xff }, /*    45 */
  {   124,   -20,    -8,0, 0x0000,0x0000, 0xf3,0xb0,0x7e,0xff }, /*    46 */
  {   113,   -19,     2,0, 0x0000,0x0000, 0xd3,0x99,0x6e,0xff }, /*    47 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_51_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 16, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_51_ = {
  gfx_Shape_of___POLYHEDRON_51_,
  vtx_Shape_of___POLYHEDRON_51_, 48, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner9[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_45_[] = {
  {    57,    97,     5,0, 0x0000,0x0000, 0x87,0xe2,0xe2,0xff }, /*     0 */
  {    34,    80,     5,0, 0x0000,0x0000, 0x6e,0xb8,0xb8,0xff }, /*     1 */
  {    81,    69,     9,0, 0x0000,0x0000, 0x93,0xf6,0xf6,0xff }, /*     2 */
  {    34,    80,     5,0, 0x0000,0x0000, 0x6e,0xb8,0xb8,0xff }, /*     3 */
  {    51,    48,     9,0, 0x0000,0x0000, 0x51,0x87,0x87,0xff }, /*     4 */
  {    81,    69,     9,0, 0x0000,0x0000, 0x93,0xf6,0xf6,0xff }, /*     5 */
  {    34,    80,   -22,0, 0x0000,0x0000, 0x73,0xc0,0xc0,0xff }, /*     6 */
  {    81,    69,   -26,0, 0x0000,0x0000, 0x73,0xc1,0xc1,0xff }, /*     7 */
  {    51,    48,   -26,0, 0x0000,0x0000, 0x5d,0x9c,0x9c,0xff }, /*     8 */
  {    34,    80,     5,0, 0x0000,0x0000, 0x6e,0xb8,0xb8,0xff }, /*     9 */
  {    51,    48,   -26,0, 0x0000,0x0000, 0x5d,0x9c,0x9c,0xff }, /*    10 */
  {    51,    48,     9,0, 0x0000,0x0000, 0x51,0x87,0x87,0xff }, /*    11 */
  {    57,    97,   -22,0, 0x0000,0x0000, 0x61,0xa3,0xa3,0xff }, /*    12 */
  {    57,    97,     5,0, 0x0000,0x0000, 0x87,0xe2,0xe2,0xff }, /*    13 */
  {    81,    69,   -26,0, 0x0000,0x0000, 0x73,0xc1,0xc1,0xff }, /*    14 */
  {    57,    97,     5,0, 0x0000,0x0000, 0x87,0xe2,0xe2,0xff }, /*    15 */
  {    38,    99,    -8,0, 0x0000,0x0000, 0x8c,0xe9,0xe9,0xff }, /*    16 */
  {    34,    80,     5,0, 0x0000,0x0000, 0x6e,0xb8,0xb8,0xff }, /*    17 */
  {    57,    97,   -22,0, 0x0000,0x0000, 0x61,0xa3,0xa3,0xff }, /*    18 */
  {    38,    99,    -8,0, 0x0000,0x0000, 0x8c,0xe9,0xe9,0xff }, /*    19 */
  {    57,    97,     5,0, 0x0000,0x0000, 0x87,0xe2,0xe2,0xff }, /*    20 */
  {    34,    80,   -22,0, 0x0000,0x0000, 0x73,0xc0,0xc0,0xff }, /*    21 */
  {    38,    99,    -8,0, 0x0000,0x0000, 0x8c,0xe9,0xe9,0xff }, /*    22 */
  {    57,    97,   -22,0, 0x0000,0x0000, 0x61,0xa3,0xa3,0xff }, /*    23 */
  {    34,    80,     5,0, 0x0000,0x0000, 0x6e,0xb8,0xb8,0xff }, /*    24 */
  {    38,    99,    -8,0, 0x0000,0x0000, 0x8c,0xe9,0xe9,0xff }, /*    25 */
  {    34,    80,   -22,0, 0x0000,0x0000, 0x73,0xc0,0xc0,0xff }, /*    26 */
  {    57,    97,     5,0, 0x0000,0x0000, 0x87,0xe2,0xe2,0xff }, /*    27 */
  {    81,    69,     9,0, 0x0000,0x0000, 0x93,0xf6,0xf6,0xff }, /*    28 */
  {    81,    69,   -26,0, 0x0000,0x0000, 0x73,0xc1,0xc1,0xff }, /*    29 */
  {    34,    80,     5,0, 0x0000,0x0000, 0x6e,0xb8,0xb8,0xff }, /*    30 */
  {    34,    80,   -22,0, 0x0000,0x0000, 0x73,0xc0,0xc0,0xff }, /*    31 */
  {    51,    48,   -26,0, 0x0000,0x0000, 0x5d,0x9c,0x9c,0xff }, /*    32 */
  {    34,    80,   -22,0, 0x0000,0x0000, 0x73,0xc0,0xc0,0xff }, /*    33 */
  {    57,    97,   -22,0, 0x0000,0x0000, 0x61,0xa3,0xa3,0xff }, /*    34 */
  {    81,    69,   -26,0, 0x0000,0x0000, 0x73,0xc1,0xc1,0xff }, /*    35 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_45_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 4, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_45_ = {
  gfx_Shape_of___POLYHEDRON_45_,
  vtx_Shape_of___POLYHEDRON_45_, 36, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner10[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_8_[] = {
  {    57,   188,     0,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*     0 */
  {    36,   132,   -52,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*     1 */
  {    51,   178,   -65,0, 0x0000,0x0000, 0x00,0x43,0x86,0xff }, /*     2 */
  {    51,   178,   -65,0, 0x0000,0x0000, 0x00,0x43,0x86,0xff }, /*     3 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*     4 */
  {     0,   168,   -91,0, 0x0000,0x0000, 0x00,0x3b,0x77,0xff }, /*     5 */
  {    51,   178,   -65,0, 0x0000,0x0000, 0x00,0x43,0x86,0xff }, /*     6 */
  {    47,   148,   -68,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*     7 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x20,0x40,0xff }, /*     8 */
  {     0,   168,   -91,0, 0x0000,0x0000, 0x00,0x3b,0x77,0xff }, /*     9 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*    10 */
  {   -51,   178,   -65,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*    11 */
  {   -51,   178,   -65,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*    12 */
  {   -36,   132,   -52,0, 0x0000,0x0000, 0x00,0x41,0x82,0xff }, /*    13 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0x00,0x44,0x88,0xff }, /*    14 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0xdb,0x9e,0x72,0xff }, /*    15 */
  {   -52,   132,     0,0, 0x0000,0x0000, 0xcb,0x93,0x69,0xff }, /*    16 */
  {   -36,   132,    52,0, 0x0000,0x0000, 0xb3,0x81,0x5d,0xff }, /*    17 */
  {    57,   188,     0,0, 0x0000,0x0000, 0xa7,0x79,0x57,0xff }, /*    18 */
  {    42,   170,    65,0, 0x0000,0x0000, 0xb2,0x54,0x37,0xff }, /*    19 */
  {    36,   132,    52,0, 0x0000,0x0000, 0x9e,0x72,0x52,0xff }, /*    20 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*    21 */
  {    15,    90,     0,0, 0x0000,0x0000, 0xa4,0x77,0x55,0xff }, /*    22 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*    23 */
  {    52,   132,     0,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*    24 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*    25 */
  {    36,   132,   -52,0, 0x0000,0x0000, 0xa7,0x78,0x56,0xff }, /*    26 */
  {    36,   132,   -52,0, 0x0000,0x0000, 0xa7,0x78,0x56,0xff }, /*    27 */
  {    11,   114,   -20,0, 0x0000,0x0000, 0xa8,0x79,0x57,0xff }, /*    28 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0xb3,0x82,0x5d,0xff }, /*    29 */
  {    11,   114,   -20,0, 0x0000,0x0000, 0xa8,0x79,0x57,0xff }, /*    30 */
  {     0,   114,   -28,0, 0x0000,0x0000, 0xb6,0x84,0x5f,0xff }, /*    31 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0xb3,0x82,0x5d,0xff }, /*    32 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0xb3,0x82,0x5d,0xff }, /*    33 */
  {   -11,   114,   -20,0, 0x0000,0x0000, 0xda,0x9e,0x71,0xff }, /*    34 */
  {   -36,   132,   -52,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*    35 */
  {     0,   114,   -28,0, 0x0000,0x0000, 0xb6,0x84,0x5f,0xff }, /*    36 */
  {   -11,    90,   -20,0, 0x0000,0x0000, 0xce,0x95,0x6b,0xff }, /*    37 */
  {   -11,   114,   -20,0, 0x0000,0x0000, 0xda,0x9e,0x71,0xff }, /*    38 */
  {     0,   114,   -28,0, 0x0000,0x0000, 0xb6,0x84,0x5f,0xff }, /*    39 */
  {     0,    90,   -28,0, 0x0000,0x0000, 0xad,0x7d,0x5a,0xff }, /*    40 */
  {   -11,    90,   -20,0, 0x0000,0x0000, 0xce,0x95,0x6b,0xff }, /*    41 */
  {   -36,   132,   -52,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*    42 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*    43 */
  {   -52,   132,     0,0, 0x0000,0x0000, 0xcb,0x93,0x69,0xff }, /*    44 */
  {   -52,   132,     0,0, 0x0000,0x0000, 0xcb,0x93,0x69,0xff }, /*    45 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*    46 */
  {   -36,   132,    52,0, 0x0000,0x0000, 0xb3,0x81,0x5d,0xff }, /*    47 */
  {   -36,   132,    52,0, 0x0000,0x0000, 0xb3,0x81,0x5d,0xff }, /*    48 */
  {   -18,   114,    20,0, 0x0000,0x0000, 0xc9,0x91,0x68,0xff }, /*    49 */
  {     0,   114,    28,0, 0x0000,0x0000, 0xb7,0x84,0x5f,0xff }, /*    50 */
  {     0,   114,    28,0, 0x0000,0x0000, 0xb7,0x84,0x5f,0xff }, /*    51 */
  {    18,   114,    20,0, 0x0000,0x0000, 0xa4,0x77,0x55,0xff }, /*    52 */
  {    36,   132,    52,0, 0x0000,0x0000, 0x9e,0x72,0x52,0xff }, /*    53 */
  {    36,   132,    52,0, 0x0000,0x0000, 0x9e,0x72,0x52,0xff }, /*    54 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*    55 */
  {    52,   132,     0,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*    56 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x20,0x00,0x40,0xff }, /*    57 */
  {    57,   201,   -84,0, 0x0000,0x0000, 0x1e,0x00,0x3c,0xff }, /*    58 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x2d,0x00,0x5b,0xff }, /*    59 */
  {    73,   214,     0,0, 0x0000,0x0000, 0x35,0x00,0x6b,0xff }, /*    60 */
  {    51,   229,    48,0, 0x0000,0x0000, 0x39,0x00,0x72,0xff }, /*    61 */
  {    64,   204,    47,0, 0x0000,0x0000, 0x3a,0x00,0x74,0xff }, /*    62 */
  {    64,   204,    47,0, 0x0000,0x0000, 0x3a,0x00,0x74,0xff }, /*    63 */
  {    51,   229,    48,0, 0x0000,0x0000, 0x39,0x00,0x72,0xff }, /*    64 */
  {     0,   210,    68,0, 0x0000,0x0000, 0x33,0x00,0x67,0xff }, /*    65 */
  {     0,   210,    68,0, 0x0000,0x0000, 0x33,0x00,0x67,0xff }, /*    66 */
  {   -51,   229,    48,0, 0x0000,0x0000, 0x2c,0x00,0x58,0xff }, /*    67 */
  {   -64,   204,    47,0, 0x0000,0x0000, 0x2e,0x00,0x5c,0xff }, /*    68 */
  {   -64,   204,    47,0, 0x0000,0x0000, 0x2e,0x00,0x5c,0xff }, /*    69 */
  {   -51,   229,    48,0, 0x0000,0x0000, 0x2c,0x00,0x58,0xff }, /*    70 */
  {   -73,   214,     0,0, 0x0000,0x0000, 0x34,0x00,0x69,0xff }, /*    71 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x3e,0x00,0x7d,0xff }, /*    72 */
  {   -57,   201,   -84,0, 0x0000,0x0000, 0x41,0x00,0x83,0xff }, /*    73 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x45,0x00,0x8b,0xff }, /*    74 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x45,0x00,0x8b,0xff }, /*    75 */
  {     0,   193,  -119,0, 0x0000,0x0000, 0x31,0x00,0x63,0xff }, /*    76 */
  {     0,   168,  -112,0, 0x0000,0x0000, 0x32,0x00,0x64,0xff }, /*    77 */
  {     0,   168,  -112,0, 0x0000,0x0000, 0x32,0x00,0x64,0xff }, /*    78 */
  {     0,   193,  -119,0, 0x0000,0x0000, 0x31,0x00,0x63,0xff }, /*    79 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x20,0x00,0x40,0xff }, /*    80 */
  {    36,   132,    52,0, 0x0000,0x0000, 0x9e,0x72,0x52,0xff }, /*    81 */
  {    52,   132,     0,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*    82 */
  {    57,   188,     0,0, 0x0000,0x0000, 0xa7,0x79,0x57,0xff }, /*    83 */
  {   -36,   132,    52,0, 0x0000,0x0000, 0xb3,0x81,0x5d,0xff }, /*    84 */
  {   -42,   170,    65,0, 0x0000,0x0000, 0x8a,0x41,0x2b,0xff }, /*    85 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0xdb,0x9e,0x72,0xff }, /*    86 */
  {     0,   114,    28,0, 0x0000,0x0000, 0xb7,0x84,0x5f,0xff }, /*    87 */
  {    36,   132,    52,0, 0x0000,0x0000, 0x9e,0x72,0x52,0xff }, /*    88 */
  {   -36,   132,    52,0, 0x0000,0x0000, 0xb3,0x81,0x5d,0xff }, /*    89 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0x00,0x44,0x88,0xff }, /*    90 */
  {   -71,   160,    11,0, 0x0000,0x0000, 0x00,0x47,0x8f,0xff }, /*    91 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x00,0x3e,0x7d,0xff }, /*    92 */
  {    56,   148,   -81,0, 0x0000,0x0000, 0x00,0x20,0x41,0xff }, /*    93 */
  {    28,   113,   -63,0, 0x0000,0x0000, 0x00,0x00,0x4b,0xff }, /*    94 */
  {     4,   140,  -108,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*    95 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x00,0x3e,0x7d,0xff }, /*    96 */
  {   -64,   174,    43,0, 0x0000,0x0000, 0x00,0x2e,0x5d,0xff }, /*    97 */
  {   -64,   204,    47,0, 0x0000,0x0000, 0x00,0x2e,0x5c,0xff }, /*    98 */
  {   -51,   178,   -65,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*    99 */
  {   -54,   149,   -60,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   100 */
  {   -54,   149,   -72,0, 0x0000,0x0000, 0x00,0x37,0x6e,0xff }, /*   101 */
  {   -64,   204,    47,0, 0x0000,0x0000, 0x00,0x2e,0x5c,0xff }, /*   102 */
  {   -56,   174,    44,0, 0x0000,0x0000, 0x00,0x2c,0x59,0xff }, /*   103 */
  {   -42,   204,    49,0, 0x0000,0x0000, 0x00,0x38,0x71,0xff }, /*   104 */
  {   -65,   160,    12,0, 0x0000,0x0000, 0x00,0x3b,0x76,0xff }, /*   105 */
  {   -66,   138,    24,0, 0x0000,0x0000, 0x00,0x00,0x48,0xff }, /*   106 */
  {   -71,   160,    11,0, 0x0000,0x0000, 0x00,0x47,0x8f,0xff }, /*   107 */
  {   -71,   160,    11,0, 0x0000,0x0000, 0x00,0x47,0x8f,0xff }, /*   108 */
  {   -66,   138,    24,0, 0x0000,0x0000, 0x00,0x00,0x48,0xff }, /*   109 */
  {   -64,   174,    43,0, 0x0000,0x0000, 0x00,0x2e,0x5d,0xff }, /*   110 */
  {   -64,   174,    43,0, 0x0000,0x0000, 0x00,0x2e,0x5d,0xff }, /*   111 */
  {   -66,   138,    24,0, 0x0000,0x0000, 0x00,0x00,0x48,0xff }, /*   112 */
  {   -56,   174,    44,0, 0x0000,0x0000, 0x00,0x2c,0x59,0xff }, /*   113 */
  {    42,   204,    49,0, 0x0000,0x0000, 0x00,0x31,0x62,0xff }, /*   114 */
  {    56,   174,    44,0, 0x0000,0x0000, 0x00,0x37,0x6e,0xff }, /*   115 */
  {    64,   204,    47,0, 0x0000,0x0000, 0x00,0x3a,0x74,0xff }, /*   116 */
  {    64,   204,    47,0, 0x0000,0x0000, 0x00,0x3a,0x74,0xff }, /*   117 */
  {    64,   174,    43,0, 0x0000,0x0000, 0x00,0x30,0x60,0xff }, /*   118 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x00,0x2d,0x5b,0xff }, /*   119 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x00,0x2d,0x5b,0xff }, /*   120 */
  {    71,   160,    11,0, 0x0000,0x0000, 0x00,0x2f,0x5e,0xff }, /*   121 */
  {    57,   188,     0,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*   122 */
  {    56,   174,    44,0, 0x0000,0x0000, 0x00,0x37,0x6e,0xff }, /*   123 */
  {    66,   138,    24,0, 0x0000,0x0000, 0x00,0x00,0x46,0xff }, /*   124 */
  {    64,   174,    43,0, 0x0000,0x0000, 0x00,0x30,0x60,0xff }, /*   125 */
  {    64,   174,    43,0, 0x0000,0x0000, 0x00,0x30,0x60,0xff }, /*   126 */
  {    66,   138,    24,0, 0x0000,0x0000, 0x00,0x00,0x46,0xff }, /*   127 */
  {    71,   160,    11,0, 0x0000,0x0000, 0x00,0x2f,0x5e,0xff }, /*   128 */
  {    64,   174,    43,0, 0x0000,0x0000, 0x00,0x30,0x60,0xff }, /*   129 */
  {    71,   160,    11,0, 0x0000,0x0000, 0x00,0x2f,0x5e,0xff }, /*   130 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x00,0x2d,0x5b,0xff }, /*   131 */
  {    71,   160,    11,0, 0x0000,0x0000, 0x00,0x2f,0x5e,0xff }, /*   132 */
  {    66,   138,    24,0, 0x0000,0x0000, 0x00,0x00,0x46,0xff }, /*   133 */
  {    65,   160,    12,0, 0x0000,0x0000, 0x00,0x48,0x90,0xff }, /*   134 */
  {    57,   188,     0,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*   135 */
  {    65,   158,    -6,0, 0x0000,0x0000, 0x00,0x37,0x6e,0xff }, /*   136 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x00,0x2d,0x5b,0xff }, /*   137 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x00,0x2d,0x5b,0xff }, /*   138 */
  {    73,   158,    -6,0, 0x0000,0x0000, 0x00,0x2d,0x5b,0xff }, /*   139 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x20,0x40,0xff }, /*   140 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x20,0x40,0xff }, /*   141 */
  {    54,   149,   -72,0, 0x0000,0x0000, 0x00,0x33,0x66,0xff }, /*   142 */
  {    51,   178,   -65,0, 0x0000,0x0000, 0x00,0x43,0x86,0xff }, /*   143 */
  {    65,   158,    -6,0, 0x0000,0x0000, 0x00,0x37,0x6e,0xff }, /*   144 */
  {    57,   115,   -36,0, 0x0000,0x0000, 0x00,0x00,0x49,0xff }, /*   145 */
  {    73,   158,    -6,0, 0x0000,0x0000, 0x00,0x2d,0x5b,0xff }, /*   146 */
  {    73,   158,    -6,0, 0x0000,0x0000, 0x00,0x2d,0x5b,0xff }, /*   147 */
  {    57,   115,   -36,0, 0x0000,0x0000, 0x00,0x00,0x49,0xff }, /*   148 */
  {    54,   149,   -72,0, 0x0000,0x0000, 0x00,0x33,0x66,0xff }, /*   149 */
  {    54,   149,   -72,0, 0x0000,0x0000, 0x00,0x33,0x66,0xff }, /*   150 */
  {    57,   115,   -36,0, 0x0000,0x0000, 0x00,0x00,0x49,0xff }, /*   151 */
  {    54,   149,   -60,0, 0x0000,0x0000, 0x00,0x43,0x86,0xff }, /*   152 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x45,0x8b,0xff }, /*   153 */
  {   -73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   154 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x00,0x3e,0x7d,0xff }, /*   155 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x45,0x8b,0xff }, /*   156 */
  {   -54,   149,   -72,0, 0x0000,0x0000, 0x00,0x37,0x6e,0xff }, /*   157 */
  {   -73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   158 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x00,0x3e,0x7d,0xff }, /*   159 */
  {   -65,   158,    -6,0, 0x0000,0x0000, 0x00,0x2a,0x55,0xff }, /*   160 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0x00,0x44,0x88,0xff }, /*   161 */
  {   -51,   178,   -65,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   162 */
  {   -54,   149,   -72,0, 0x0000,0x0000, 0x00,0x37,0x6e,0xff }, /*   163 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x45,0x8b,0xff }, /*   164 */
  {    47,   148,   -68,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*   165 */
  {    28,   113,   -63,0, 0x0000,0x0000, 0x00,0x00,0x4b,0xff }, /*   166 */
  {    56,   148,   -81,0, 0x0000,0x0000, 0x00,0x20,0x41,0xff }, /*   167 */
  {   -54,   149,   -72,0, 0x0000,0x0000, 0x00,0x37,0x6e,0xff }, /*   168 */
  {   -57,   115,   -36,0, 0x0000,0x0000, 0x00,0x00,0x3e,0xff }, /*   169 */
  {   -73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   170 */
  {   -73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   171 */
  {   -57,   115,   -36,0, 0x0000,0x0000, 0x00,0x00,0x3e,0xff }, /*   172 */
  {   -65,   158,    -6,0, 0x0000,0x0000, 0x00,0x2a,0x55,0xff }, /*   173 */
  {    -4,   140,  -108,0, 0x0000,0x0000, 0x00,0x35,0x6a,0xff }, /*   174 */
  {   -28,   113,   -63,0, 0x0000,0x0000, 0x00,0x00,0x49,0xff }, /*   175 */
  {   -56,   148,   -81,0, 0x0000,0x0000, 0x00,0x45,0x8b,0xff }, /*   176 */
  {   -54,   149,   -60,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   177 */
  {   -57,   115,   -36,0, 0x0000,0x0000, 0x00,0x00,0x3e,0xff }, /*   178 */
  {   -54,   149,   -72,0, 0x0000,0x0000, 0x00,0x37,0x6e,0xff }, /*   179 */
  {    -4,   140,  -108,0, 0x0000,0x0000, 0x00,0x35,0x6a,0xff }, /*   180 */
  {   -56,   148,   -81,0, 0x0000,0x0000, 0x00,0x45,0x8b,0xff }, /*   181 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x45,0x8b,0xff }, /*   182 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x20,0x40,0xff }, /*   183 */
  {     4,   140,  -108,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   184 */
  {     0,   168,  -112,0, 0x0000,0x0000, 0x00,0x32,0x64,0xff }, /*   185 */
  {     0,   168,  -112,0, 0x0000,0x0000, 0x00,0x32,0x64,0xff }, /*   186 */
  {     4,   140,  -108,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   187 */
  {     0,   168,   -91,0, 0x0000,0x0000, 0x00,0x3b,0x77,0xff }, /*   188 */
  {     4,   140,  -108,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   189 */
  {     4,   140,   -91,0, 0x0000,0x0000, 0x00,0x35,0x6a,0xff }, /*   190 */
  {     0,   168,   -91,0, 0x0000,0x0000, 0x00,0x3b,0x77,0xff }, /*   191 */
  {     4,   140,  -108,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   192 */
  {    28,   113,   -63,0, 0x0000,0x0000, 0x00,0x00,0x4b,0xff }, /*   193 */
  {     4,   140,   -91,0, 0x0000,0x0000, 0x00,0x35,0x6a,0xff }, /*   194 */
  {     0,   168,   -91,0, 0x0000,0x0000, 0x00,0x3b,0x77,0xff }, /*   195 */
  {    -4,   140,  -108,0, 0x0000,0x0000, 0x00,0x35,0x6a,0xff }, /*   196 */
  {     0,   168,  -112,0, 0x0000,0x0000, 0x00,0x32,0x64,0xff }, /*   197 */
  {     0,   168,   -91,0, 0x0000,0x0000, 0x00,0x3b,0x77,0xff }, /*   198 */
  {    -4,   140,   -91,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*   199 */
  {    -4,   140,  -108,0, 0x0000,0x0000, 0x00,0x35,0x6a,0xff }, /*   200 */
  {     0,   168,  -112,0, 0x0000,0x0000, 0x00,0x32,0x64,0xff }, /*   201 */
  {    -4,   140,  -108,0, 0x0000,0x0000, 0x00,0x35,0x6a,0xff }, /*   202 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x45,0x8b,0xff }, /*   203 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x45,0x8b,0xff }, /*   204 */
  {   -47,   148,   -68,0, 0x0000,0x0000, 0x00,0x31,0x63,0xff }, /*   205 */
  {   -51,   178,   -65,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   206 */
  {    -4,   140,   -91,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*   207 */
  {   -28,   113,   -63,0, 0x0000,0x0000, 0x00,0x00,0x49,0xff }, /*   208 */
  {    -4,   140,  -108,0, 0x0000,0x0000, 0x00,0x35,0x6a,0xff }, /*   209 */
  {   -56,   148,   -81,0, 0x0000,0x0000, 0x00,0x45,0x8b,0xff }, /*   210 */
  {   -28,   113,   -63,0, 0x0000,0x0000, 0x00,0x00,0x49,0xff }, /*   211 */
  {   -47,   148,   -68,0, 0x0000,0x0000, 0x00,0x31,0x63,0xff }, /*   212 */
  {    71,   160,    11,0, 0x0000,0x0000, 0x00,0x2f,0x5e,0xff }, /*   213 */
  {    65,   160,    12,0, 0x0000,0x0000, 0x00,0x48,0x90,0xff }, /*   214 */
  {    57,   188,     0,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*   215 */
  {    42,   170,    65,0, 0x0000,0x0000, 0xb2,0x54,0x37,0xff }, /*   216 */
  {    57,   188,     0,0, 0x0000,0x0000, 0xa7,0x79,0x57,0xff }, /*   217 */
  {    42,   204,    49,0, 0x0000,0x0000, 0x9e,0x73,0x52,0xff }, /*   218 */
  {   -42,   170,    65,0, 0x0000,0x0000, 0x8a,0x41,0x2b,0xff }, /*   219 */
  {   -42,   204,    49,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*   220 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0xdb,0x9e,0x72,0xff }, /*   221 */
  {    36,   132,    52,0, 0x0000,0x0000, 0x9e,0x72,0x52,0xff }, /*   222 */
  {    18,   114,    20,0, 0x0000,0x0000, 0xa4,0x77,0x55,0xff }, /*   223 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   224 */
  {    64,   204,    47,0, 0x0000,0x0000, 0x3a,0x00,0x74,0xff }, /*   225 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x2d,0x00,0x5b,0xff }, /*   226 */
  {    73,   214,     0,0, 0x0000,0x0000, 0x35,0x00,0x6b,0xff }, /*   227 */
  {   -73,   214,     0,0, 0x0000,0x0000, 0x34,0x00,0x69,0xff }, /*   228 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x3e,0x00,0x7d,0xff }, /*   229 */
  {   -64,   204,    47,0, 0x0000,0x0000, 0x2e,0x00,0x5c,0xff }, /*   230 */
  {     0,   275,    27,0, 0x0000,0x0000, 0xf3,0xb5,0x00,0xff }, /*   231 */
  {    -6,   247,    66,0, 0x0000,0x0000, 0x8f,0x6a,0x00,0xff }, /*   232 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x8e,0x6a,0x00,0xff }, /*   233 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x8e,0x6a,0x00,0xff }, /*   234 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0xab,0x7f,0x00,0xff }, /*   235 */
  {   -51,   229,    48,0, 0x0000,0x0000, 0x93,0x6e,0x00,0xff }, /*   236 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x8e,0x6a,0x00,0xff }, /*   237 */
  {    -6,   247,    66,0, 0x0000,0x0000, 0x8f,0x6a,0x00,0xff }, /*   238 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0xab,0x7f,0x00,0xff }, /*   239 */
  {   -51,   229,    48,0, 0x0000,0x0000, 0x93,0x6e,0x00,0xff }, /*   240 */
  {   -59,   273,    28,0, 0x0000,0x0000, 0xcf,0x9a,0x00,0xff }, /*   241 */
  {   -59,   267,    16,0, 0x0000,0x0000, 0xa8,0x7d,0x00,0xff }, /*   242 */
  {   -59,   267,    16,0, 0x0000,0x0000, 0xa8,0x7d,0x00,0xff }, /*   243 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0xf7,0xb8,0x00,0xff }, /*   244 */
  {     0,   275,    27,0, 0x0000,0x0000, 0xf3,0xb5,0x00,0xff }, /*   245 */
  {   -59,   267,    16,0, 0x0000,0x0000, 0xa8,0x7d,0x00,0xff }, /*   246 */
  {   -59,   273,    28,0, 0x0000,0x0000, 0xcf,0x9a,0x00,0xff }, /*   247 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0xf7,0xb8,0x00,0xff }, /*   248 */
  {    51,   229,    48,0, 0x0000,0x0000, 0xbf,0x8e,0x00,0xff }, /*   249 */
  {    50,   241,    55,0, 0x0000,0x0000, 0xb8,0x89,0x00,0xff }, /*   250 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x8e,0x6a,0x00,0xff }, /*   251 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x8e,0x6a,0x00,0xff }, /*   252 */
  {     6,   247,    66,0, 0x0000,0x0000, 0xa7,0x7c,0x00,0xff }, /*   253 */
  {     0,   275,    27,0, 0x0000,0x0000, 0xf3,0xb5,0x00,0xff }, /*   254 */
  {     0,   275,    27,0, 0x0000,0x0000, 0xf3,0xb5,0x00,0xff }, /*   255 */
  {     6,   279,    37,0, 0x0000,0x0000, 0xdf,0xa6,0x00,0xff }, /*   256 */
  {    59,   267,    16,0, 0x0000,0x0000, 0xc2,0x91,0x00,0xff }, /*   257 */
  {     6,   279,    37,0, 0x0000,0x0000, 0xdf,0xa6,0x00,0xff }, /*   258 */
  {    59,   273,    28,0, 0x0000,0x0000, 0xeb,0xaf,0x00,0xff }, /*   259 */
  {    59,   267,    16,0, 0x0000,0x0000, 0xc2,0x91,0x00,0xff }, /*   260 */
  {    59,   267,    16,0, 0x0000,0x0000, 0xc2,0x91,0x00,0xff }, /*   261 */
  {    59,   273,    28,0, 0x0000,0x0000, 0xeb,0xaf,0x00,0xff }, /*   262 */
  {    51,   229,    48,0, 0x0000,0x0000, 0xbf,0x8e,0x00,0xff }, /*   263 */
  {    59,   273,    28,0, 0x0000,0x0000, 0xeb,0xaf,0x00,0xff }, /*   264 */
  {    50,   241,    55,0, 0x0000,0x0000, 0xb8,0x89,0x00,0xff }, /*   265 */
  {    51,   229,    48,0, 0x0000,0x0000, 0xbf,0x8e,0x00,0xff }, /*   266 */
  {     6,   247,    66,0, 0x0000,0x0000, 0xa7,0x7c,0x00,0xff }, /*   267 */
  {     6,   279,    37,0, 0x0000,0x0000, 0xdf,0xa6,0x00,0xff }, /*   268 */
  {     0,   275,    27,0, 0x0000,0x0000, 0xf3,0xb5,0x00,0xff }, /*   269 */
  {    50,   241,    55,0, 0x0000,0x0000, 0xb8,0x89,0x00,0xff }, /*   270 */
  {     6,   247,    66,0, 0x0000,0x0000, 0xa7,0x7c,0x00,0xff }, /*   271 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x8e,0x6a,0x00,0xff }, /*   272 */
  {   -51,   229,    48,0, 0x0000,0x0000, 0x93,0x6e,0x00,0xff }, /*   273 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0xab,0x7f,0x00,0xff }, /*   274 */
  {   -59,   273,    28,0, 0x0000,0x0000, 0xcf,0x9a,0x00,0xff }, /*   275 */
  {     0,   275,    27,0, 0x0000,0x0000, 0xf3,0xb5,0x00,0xff }, /*   276 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0xf7,0xb8,0x00,0xff }, /*   277 */
  {    -6,   247,    66,0, 0x0000,0x0000, 0x8f,0x6a,0x00,0xff }, /*   278 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x45,0x8b,0xff }, /*   279 */
  {   -56,   148,   -81,0, 0x0000,0x0000, 0x00,0x45,0x8b,0xff }, /*   280 */
  {   -47,   148,   -68,0, 0x0000,0x0000, 0x00,0x31,0x63,0xff }, /*   281 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x20,0x40,0xff }, /*   282 */
  {    56,   148,   -81,0, 0x0000,0x0000, 0x00,0x20,0x41,0xff }, /*   283 */
  {     4,   140,  -108,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   284 */
  {    47,   148,   -68,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*   285 */
  {    56,   148,   -81,0, 0x0000,0x0000, 0x00,0x20,0x41,0xff }, /*   286 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x20,0x40,0xff }, /*   287 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x00,0x3e,0x7d,0xff }, /*   288 */
  {   -73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   289 */
  {   -65,   158,    -6,0, 0x0000,0x0000, 0x00,0x2a,0x55,0xff }, /*   290 */
  {    65,   158,    -6,0, 0x0000,0x0000, 0x00,0x37,0x6e,0xff }, /*   291 */
  {    73,   158,    -6,0, 0x0000,0x0000, 0x00,0x2d,0x5b,0xff }, /*   292 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x00,0x2d,0x5b,0xff }, /*   293 */
  {    54,   149,   -72,0, 0x0000,0x0000, 0x00,0x33,0x66,0xff }, /*   294 */
  {    54,   149,   -60,0, 0x0000,0x0000, 0x00,0x43,0x86,0xff }, /*   295 */
  {    51,   178,   -65,0, 0x0000,0x0000, 0x00,0x43,0x86,0xff }, /*   296 */
  {   -36,   132,   -52,0, 0x0000,0x0000, 0x00,0x41,0x82,0xff }, /*   297 */
  {   -52,   132,     0,0, 0x0000,0x0000, 0x00,0x3f,0x7e,0xff }, /*   298 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0x00,0x44,0x88,0xff }, /*   299 */
  {    73,   158,    -6,0, 0x0000,0x0000, 0x00,0x2d,0x5b,0xff }, /*   300 */
  {    54,   149,   -72,0, 0x0000,0x0000, 0x00,0x33,0x66,0xff }, /*   301 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x20,0x40,0xff }, /*   302 */
  {    56,   174,    44,0, 0x0000,0x0000, 0x00,0x37,0x6e,0xff }, /*   303 */
  {    64,   174,    43,0, 0x0000,0x0000, 0x00,0x30,0x60,0xff }, /*   304 */
  {    64,   204,    47,0, 0x0000,0x0000, 0x00,0x3a,0x74,0xff }, /*   305 */
  {   -64,   204,    47,0, 0x0000,0x0000, 0x00,0x2e,0x5c,0xff }, /*   306 */
  {   -64,   174,    43,0, 0x0000,0x0000, 0x00,0x2e,0x5d,0xff }, /*   307 */
  {   -56,   174,    44,0, 0x0000,0x0000, 0x00,0x2c,0x59,0xff }, /*   308 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x00,0x3e,0x7d,0xff }, /*   309 */
  {   -71,   160,    11,0, 0x0000,0x0000, 0x00,0x47,0x8f,0xff }, /*   310 */
  {   -64,   174,    43,0, 0x0000,0x0000, 0x00,0x2e,0x5d,0xff }, /*   311 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0x00,0x44,0x88,0xff }, /*   312 */
  {   -65,   160,    12,0, 0x0000,0x0000, 0x00,0x3b,0x76,0xff }, /*   313 */
  {   -71,   160,    11,0, 0x0000,0x0000, 0x00,0x47,0x8f,0xff }, /*   314 */
  {     0,   193,  -119,0, 0x0000,0x0000, 0x31,0x00,0x63,0xff }, /*   315 */
  {    57,   201,   -84,0, 0x0000,0x0000, 0x1e,0x00,0x3c,0xff }, /*   316 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x20,0x00,0x40,0xff }, /*   317 */
  {    57,   201,   -84,0, 0x0000,0x0000, 0x1e,0x00,0x3c,0xff }, /*   318 */
  {    73,   214,     0,0, 0x0000,0x0000, 0x35,0x00,0x6b,0xff }, /*   319 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x2d,0x00,0x5b,0xff }, /*   320 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x45,0x00,0x8b,0xff }, /*   321 */
  {   -57,   201,   -84,0, 0x0000,0x0000, 0x41,0x00,0x83,0xff }, /*   322 */
  {     0,   193,  -119,0, 0x0000,0x0000, 0x31,0x00,0x63,0xff }, /*   323 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x3e,0x00,0x7d,0xff }, /*   324 */
  {   -73,   214,     0,0, 0x0000,0x0000, 0x34,0x00,0x69,0xff }, /*   325 */
  {   -57,   201,   -84,0, 0x0000,0x0000, 0x41,0x00,0x83,0xff }, /*   326 */
  {     0,   210,    68,0, 0x0000,0x0000, 0x33,0x00,0x67,0xff }, /*   327 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x2a,0x00,0x55,0xff }, /*   328 */
  {   -51,   229,    48,0, 0x0000,0x0000, 0x2c,0x00,0x58,0xff }, /*   329 */
  {    51,   229,    48,0, 0x0000,0x0000, 0x39,0x00,0x72,0xff }, /*   330 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x2a,0x00,0x55,0xff }, /*   331 */
  {     0,   210,    68,0, 0x0000,0x0000, 0x33,0x00,0x67,0xff }, /*   332 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   333 */
  {    18,   114,    20,0, 0x0000,0x0000, 0xa4,0x77,0x55,0xff }, /*   334 */
  {   -18,   114,    20,0, 0x0000,0x0000, 0xc9,0x91,0x68,0xff }, /*   335 */
  {    18,   114,    20,0, 0x0000,0x0000, 0xa4,0x77,0x55,0xff }, /*   336 */
  {     0,   114,    28,0, 0x0000,0x0000, 0xb7,0x84,0x5f,0xff }, /*   337 */
  {   -18,   114,    20,0, 0x0000,0x0000, 0xc9,0x91,0x68,0xff }, /*   338 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   339 */
  {   -18,   114,    20,0, 0x0000,0x0000, 0xc9,0x91,0x68,0xff }, /*   340 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*   341 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*   342 */
  {   -18,   114,    20,0, 0x0000,0x0000, 0xc9,0x91,0x68,0xff }, /*   343 */
  {   -36,   132,    52,0, 0x0000,0x0000, 0xb3,0x81,0x5d,0xff }, /*   344 */
  {   -36,   132,   -52,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*   345 */
  {   -11,   114,   -20,0, 0x0000,0x0000, 0xda,0x9e,0x71,0xff }, /*   346 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*   347 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0xb3,0x82,0x5d,0xff }, /*   348 */
  {     0,   114,   -28,0, 0x0000,0x0000, 0xb6,0x84,0x5f,0xff }, /*   349 */
  {   -11,   114,   -20,0, 0x0000,0x0000, 0xda,0x9e,0x71,0xff }, /*   350 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   351 */
  {    11,   114,   -20,0, 0x0000,0x0000, 0xa8,0x79,0x57,0xff }, /*   352 */
  {    36,   132,   -52,0, 0x0000,0x0000, 0xa7,0x78,0x56,0xff }, /*   353 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   354 */
  {   -36,   132,   -52,0, 0x0000,0x0000, 0x00,0x41,0x82,0xff }, /*   355 */
  {   -51,   178,   -65,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   356 */
  {    51,   178,   -65,0, 0x0000,0x0000, 0x00,0x43,0x86,0xff }, /*   357 */
  {    36,   132,   -52,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*   358 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   359 */
  {    57,   188,     0,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*   360 */
  {    52,   132,     0,0, 0x0000,0x0000, 0x00,0x2f,0x5e,0xff }, /*   361 */
  {    36,   132,   -52,0, 0x0000,0x0000, 0x00,0x34,0x68,0xff }, /*   362 */
  {     6,   247,    66,0, 0x0000,0x0000, 0xa7,0x7c,0x00,0xff }, /*   363 */
  {     9,   247,    60,0, 0x0000,0x0000, 0xef,0xb2,0x00,0xff }, /*   364 */
  {     6,   279,    37,0, 0x0000,0x0000, 0xdf,0xa6,0x00,0xff }, /*   365 */
  {     9,   247,    60,0, 0x0000,0x0000, 0xef,0xb2,0x00,0xff }, /*   366 */
  {     9,   273,    37,0, 0x0000,0x0000, 0xc0,0x8f,0x00,0xff }, /*   367 */
  {     6,   279,    37,0, 0x0000,0x0000, 0xdf,0xa6,0x00,0xff }, /*   368 */
  {     6,   279,    37,0, 0x0000,0x0000, 0xdf,0xa6,0x00,0xff }, /*   369 */
  {    53,   268,    29,0, 0x0000,0x0000, 0xb4,0x86,0x00,0xff }, /*   370 */
  {    59,   273,    28,0, 0x0000,0x0000, 0xeb,0xaf,0x00,0xff }, /*   371 */
  {    59,   273,    28,0, 0x0000,0x0000, 0xeb,0xaf,0x00,0xff }, /*   372 */
  {    53,   268,    29,0, 0x0000,0x0000, 0xb4,0x86,0x00,0xff }, /*   373 */
  {    50,   241,    55,0, 0x0000,0x0000, 0xb8,0x89,0x00,0xff }, /*   374 */
  {    53,   268,    29,0, 0x0000,0x0000, 0xb4,0x86,0x00,0xff }, /*   375 */
  {    46,   242,    50,0, 0x0000,0x0000, 0xe4,0xaa,0x00,0xff }, /*   376 */
  {    50,   241,    55,0, 0x0000,0x0000, 0xb8,0x89,0x00,0xff }, /*   377 */
  {    50,   241,    55,0, 0x0000,0x0000, 0xb8,0x89,0x00,0xff }, /*   378 */
  {     9,   247,    60,0, 0x0000,0x0000, 0xef,0xb2,0x00,0xff }, /*   379 */
  {     6,   247,    66,0, 0x0000,0x0000, 0xa7,0x7c,0x00,0xff }, /*   380 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0xf7,0xb8,0x00,0xff }, /*   381 */
  {    -9,   247,    60,0, 0x0000,0x0000, 0xdf,0xa6,0x00,0xff }, /*   382 */
  {    -6,   247,    66,0, 0x0000,0x0000, 0x8f,0x6a,0x00,0xff }, /*   383 */
  {    -6,   247,    66,0, 0x0000,0x0000, 0x8f,0x6a,0x00,0xff }, /*   384 */
  {    -9,   247,    60,0, 0x0000,0x0000, 0xdf,0xa6,0x00,0xff }, /*   385 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0xab,0x7f,0x00,0xff }, /*   386 */
  {    -9,   247,    60,0, 0x0000,0x0000, 0xdf,0xa6,0x00,0xff }, /*   387 */
  {   -46,   242,    50,0, 0x0000,0x0000, 0xe6,0xab,0x00,0xff }, /*   388 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0xab,0x7f,0x00,0xff }, /*   389 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0xab,0x7f,0x00,0xff }, /*   390 */
  {   -53,   268,    29,0, 0x0000,0x0000, 0xad,0x81,0x00,0xff }, /*   391 */
  {   -59,   273,    28,0, 0x0000,0x0000, 0xcf,0x9a,0x00,0xff }, /*   392 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0xab,0x7f,0x00,0xff }, /*   393 */
  {   -46,   242,    50,0, 0x0000,0x0000, 0xe6,0xab,0x00,0xff }, /*   394 */
  {   -53,   268,    29,0, 0x0000,0x0000, 0xad,0x81,0x00,0xff }, /*   395 */
  {   -59,   273,    28,0, 0x0000,0x0000, 0xcf,0x9a,0x00,0xff }, /*   396 */
  {   -53,   268,    29,0, 0x0000,0x0000, 0xad,0x81,0x00,0xff }, /*   397 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0xf7,0xb8,0x00,0xff }, /*   398 */
  {   -53,   268,    29,0, 0x0000,0x0000, 0xad,0x81,0x00,0xff }, /*   399 */
  {    -9,   273,    37,0, 0x0000,0x0000, 0xbb,0x8b,0x00,0xff }, /*   400 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0xf7,0xb8,0x00,0xff }, /*   401 */
  {     6,   279,    37,0, 0x0000,0x0000, 0xdf,0xa6,0x00,0xff }, /*   402 */
  {     9,   273,    37,0, 0x0000,0x0000, 0xc0,0x8f,0x00,0xff }, /*   403 */
  {    53,   268,    29,0, 0x0000,0x0000, 0xb4,0x86,0x00,0xff }, /*   404 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0xf7,0xb8,0x00,0xff }, /*   405 */
  {    -9,   273,    37,0, 0x0000,0x0000, 0xbb,0x8b,0x00,0xff }, /*   406 */
  {    -9,   247,    60,0, 0x0000,0x0000, 0xdf,0xa6,0x00,0xff }, /*   407 */
  {    50,   241,    55,0, 0x0000,0x0000, 0xb8,0x89,0x00,0xff }, /*   408 */
  {    46,   242,    50,0, 0x0000,0x0000, 0xe4,0xaa,0x00,0xff }, /*   409 */
  {     9,   247,    60,0, 0x0000,0x0000, 0xef,0xb2,0x00,0xff }, /*   410 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   411 */
  {    11,    90,   -20,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*   412 */
  {    11,   114,   -20,0, 0x0000,0x0000, 0xa8,0x79,0x57,0xff }, /*   413 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   414 */
  {    15,    90,     0,0, 0x0000,0x0000, 0xa4,0x77,0x55,0xff }, /*   415 */
  {    11,    90,   -20,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*   416 */
  {    11,   114,   -20,0, 0x0000,0x0000, 0xa8,0x79,0x57,0xff }, /*   417 */
  {     0,    90,   -28,0, 0x0000,0x0000, 0xad,0x7d,0x5a,0xff }, /*   418 */
  {     0,   114,   -28,0, 0x0000,0x0000, 0xb6,0x84,0x5f,0xff }, /*   419 */
  {    11,   114,   -20,0, 0x0000,0x0000, 0xa8,0x79,0x57,0xff }, /*   420 */
  {    11,    90,   -20,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*   421 */
  {     0,    90,   -28,0, 0x0000,0x0000, 0xad,0x7d,0x5a,0xff }, /*   422 */
  {   -11,   114,   -20,0, 0x0000,0x0000, 0xda,0x9e,0x71,0xff }, /*   423 */
  {   -15,    90,     0,0, 0x0000,0x0000, 0xae,0x7e,0x5a,0xff }, /*   424 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*   425 */
  {   -11,   114,   -20,0, 0x0000,0x0000, 0xda,0x9e,0x71,0xff }, /*   426 */
  {   -11,    90,   -20,0, 0x0000,0x0000, 0xce,0x95,0x6b,0xff }, /*   427 */
  {   -15,    90,     0,0, 0x0000,0x0000, 0xae,0x7e,0x5a,0xff }, /*   428 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*   429 */
  {   -15,    90,     0,0, 0x0000,0x0000, 0xae,0x7e,0x5a,0xff }, /*   430 */
  {    15,    90,     0,0, 0x0000,0x0000, 0xa4,0x77,0x55,0xff }, /*   431 */
  {   -42,   204,    49,0, 0x01f0,0xfdf1, 0xa9,0xa9,0xa9,0xff }, /*   432 */
  {     0,   210,    49,0, 0x02cf,0xfe41, 0xa2,0xa2,0xa2,0xff }, /*   433 */
  {     0,   210,    68,0, 0x02cf,0xfdb4, 0x9b,0x9b,0x9b,0xff }, /*   434 */
  {     0,   210,    68,0, 0x02cf,0xfdb4, 0x9b,0x9b,0x9b,0xff }, /*   435 */
  {    42,   204,    49,0, 0x0400,0xfdf1, 0x94,0x94,0x94,0xff }, /*   436 */
  {    64,   204,    47,0, 0x045a,0xfdfd, 0xaf,0xaf,0xaf,0xff }, /*   437 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xb3,0xb3,0xb3,0xff }, /*   438 */
  {   -23,   300,   -84,0, 0x079f,0x073c, 0x78,0x5a,0x97,0xff }, /*   439 */
  {   -20,   319,   -81,0, 0x07d7,0x082f, 0xa2,0xa2,0xa2,0xff }, /*   440 */
  {     0,   292,  -111,0, 0x06cf,0x0784, 0x71,0x55,0x8d,0xff }, /*   441 */
  {    23,   302,  -104,0, 0x0654,0x07ec, 0x7f,0x5f,0x9e,0xff }, /*   442 */
  {    57,   201,   -84,0, 0x0622,0x01a6, 0x5a,0x5a,0x5a,0xff }, /*   443 */
  {    57,   201,   -84,0, 0x0622,0x01a6, 0x5a,0x5a,0x5a,0xff }, /*   444 */
  {    23,   300,   -84,0, 0x062d,0x073c, 0x79,0x5a,0x97,0xff }, /*   445 */
  {    73,   214,     0,0, 0x051f,0xfff2, 0xa1,0xa1,0xa1,0xff }, /*   446 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xb3,0xb3,0xb3,0xff }, /*   447 */
  {    20,   319,   -81,0, 0x0609,0x082f, 0x8a,0x67,0xad,0xff }, /*   448 */
  {    23,   300,   -84,0, 0x062d,0x073c, 0x79,0x5a,0x97,0xff }, /*   449 */
  {    73,   214,     0,0, 0x051f,0xfff2, 0xa1,0xa1,0xa1,0xff }, /*   450 */
  {    23,   300,   -84,0, 0x062d,0x073c, 0x79,0x5a,0x97,0xff }, /*   451 */
  {    59,   267,    16,0, 0x0445,0x0269, 0xaf,0xaf,0xaf,0xff }, /*   452 */
  {   -23,   300,   -84,0, 0x079f,0x073c, 0x78,0x5a,0x97,0xff }, /*   453 */
  {   -23,   302,  -104,0, 0x0764,0x07ec, 0x73,0x56,0x8f,0xff }, /*   454 */
  {   -57,   201,   -84,0, 0x07b0,0x01a6, 0xc4,0xc4,0xc4,0xff }, /*   455 */
  {   -59,   267,    16,0, 0x01c4,0x0269, 0x98,0x98,0x98,0xff }, /*   456 */
  {   -23,   300,   -84,0, 0xffa0,0x073c, 0x78,0x5a,0x97,0xff }, /*   457 */
  {   -73,   214,     0,0, 0x011b,0xfff2, 0x9d,0x9d,0x9d,0xff }, /*   458 */
  {   -73,   214,     0,0, 0x011b,0xfff2, 0x9d,0x9d,0x9d,0xff }, /*   459 */
  {   -23,   300,   -84,0, 0xffa0,0x073c, 0x78,0x5a,0x97,0xff }, /*   460 */
  {   -57,   201,   -84,0, 0xffb1,0x01a6, 0xc4,0xc4,0xc4,0xff }, /*   461 */
  {   -57,   201,   -84,0, 0x07b0,0x01a6, 0xc4,0xc4,0xc4,0xff }, /*   462 */
  {   -23,   302,  -104,0, 0x0764,0x07ec, 0x73,0x56,0x8f,0xff }, /*   463 */
  {     0,   292,  -111,0, 0x06cf,0x0784, 0x71,0x55,0x8d,0xff }, /*   464 */
  {   -57,   201,   -84,0, 0x07b0,0x01a6, 0xc4,0xc4,0xc4,0xff }, /*   465 */
  {     0,   292,  -111,0, 0x06cf,0x0784, 0x71,0x55,0x8d,0xff }, /*   466 */
  {     0,   193,  -119,0, 0x06cf,0x0230, 0x95,0x95,0x95,0xff }, /*   467 */
  {     0,   193,  -119,0, 0x06cf,0x0230, 0x95,0x95,0x95,0xff }, /*   468 */
  {     0,   292,  -111,0, 0x06cf,0x0784, 0x71,0x55,0x8d,0xff }, /*   469 */
  {    57,   201,   -84,0, 0x0622,0x01a6, 0x5a,0x5a,0x5a,0xff }, /*   470 */
  {    23,   302,  -104,0, 0x0654,0x07ec, 0x7f,0x5f,0x9e,0xff }, /*   471 */
  {     0,   292,  -111,0, 0x06cf,0x0784, 0x71,0x55,0x8d,0xff }, /*   472 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xb3,0xb3,0xb3,0xff }, /*   473 */
  {     0,   275,    27,0, 0x02cf,0x0289, 0xdb,0xdb,0xdb,0xff }, /*   474 */
  {     0,   313,   -51,0, 0xfed0,0x06f6, 0xaf,0x83,0xdb,0xff }, /*   475 */
  {   -59,   267,    16,0, 0x01c4,0x0269, 0x98,0x98,0x98,0xff }, /*   476 */
  {     0,   313,   -51,0, 0x06cf,0x06f6, 0xaf,0x83,0xdb,0xff }, /*   477 */
  {   -20,   319,   -81,0, 0x07d7,0x082f, 0xa2,0xa2,0xa2,0xff }, /*   478 */
  {   -59,   267,    16,0, 0x09c4,0x0269, 0x98,0x98,0x98,0xff }, /*   479 */
  {    59,   267,    16,0, 0x0c45,0x0269, 0xaf,0xaf,0xaf,0xff }, /*   480 */
  {     0,   313,   -51,0, 0x06cf,0x06f6, 0xaf,0x83,0xdb,0xff }, /*   481 */
  {     0,   275,    27,0, 0x0acf,0x0289, 0xdb,0xdb,0xdb,0xff }, /*   482 */
  {    59,   267,    16,0, 0x0445,0x0269, 0xaf,0xaf,0xaf,0xff }, /*   483 */
  {    51,   229,    48,0, 0x03f8,0xff5b, 0xac,0xac,0xac,0xff }, /*   484 */
  {    73,   214,     0,0, 0x051f,0xfff2, 0xa1,0xa1,0xa1,0xff }, /*   485 */
  {   -73,   214,     0,0, 0x011b,0xfff2, 0x9d,0x9d,0x9d,0xff }, /*   486 */
  {   -51,   229,    48,0, 0x01f4,0xff5b, 0x85,0x85,0x85,0xff }, /*   487 */
  {   -59,   267,    16,0, 0x01c4,0x0269, 0x98,0x98,0x98,0xff }, /*   488 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xb3,0xb3,0xb3,0xff }, /*   489 */
  {     0,   313,   -51,0, 0x06cf,0x06f6, 0xaf,0x83,0xdb,0xff }, /*   490 */
  {    20,   319,   -81,0, 0x0609,0x082f, 0x8a,0x67,0xad,0xff }, /*   491 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xb3,0xb3,0xb3,0xff }, /*   492 */
  {    23,   300,   -84,0, 0x062d,0x073c, 0x79,0x5a,0x97,0xff }, /*   493 */
  {    23,   302,  -104,0, 0x0654,0x07ec, 0x7f,0x5f,0x9e,0xff }, /*   494 */
  {   -20,   319,   -81,0, 0x07d7,0x082f, 0xa2,0xa2,0xa2,0xff }, /*   495 */
  {     0,   313,   -51,0, 0x06cf,0x06f6, 0xaf,0x83,0xdb,0xff }, /*   496 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xb3,0xb3,0xb3,0xff }, /*   497 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xb3,0xb3,0xb3,0xff }, /*   498 */
  {   -23,   302,  -104,0, 0x0764,0x07ec, 0x73,0x56,0x8f,0xff }, /*   499 */
  {   -23,   300,   -84,0, 0x079f,0x073c, 0x78,0x5a,0x97,0xff }, /*   500 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xb3,0xb3,0xb3,0xff }, /*   501 */
  {     0,   292,  -111,0, 0x06cf,0x0784, 0x71,0x55,0x8d,0xff }, /*   502 */
  {   -23,   302,  -104,0, 0x0764,0x07ec, 0x73,0x56,0x8f,0xff }, /*   503 */
  {    59,   267,    16,0, 0x0445,0x0269, 0xaf,0xaf,0xaf,0xff }, /*   504 */
  {    20,   319,   -81,0, 0x0609,0x082f, 0x8a,0x67,0xad,0xff }, /*   505 */
  {     0,   313,   -51,0, 0x06cf,0x06f6, 0xaf,0x83,0xdb,0xff }, /*   506 */
  {   -59,   267,    16,0, 0x01c4,0x0269, 0x98,0x98,0x98,0xff }, /*   507 */
  {   -20,   319,   -81,0, 0xffd8,0x082f, 0xa2,0xa2,0xa2,0xff }, /*   508 */
  {   -23,   300,   -84,0, 0xffa0,0x073c, 0x78,0x5a,0x97,0xff }, /*   509 */
  {    23,   300,   -84,0, 0x062d,0x073c, 0x79,0x5a,0x97,0xff }, /*   510 */
  {    20,   319,   -81,0, 0x0609,0x082f, 0x8a,0x67,0xad,0xff }, /*   511 */
  {    59,   267,    16,0, 0x0445,0x0269, 0xaf,0xaf,0xaf,0xff }, /*   512 */
  {    57,   201,   -84,0, 0x0622,0x01a6, 0x5a,0x5a,0x5a,0xff }, /*   513 */
  {    23,   302,  -104,0, 0x0654,0x07ec, 0x7f,0x5f,0x9e,0xff }, /*   514 */
  {    23,   300,   -84,0, 0x062d,0x073c, 0x79,0x5a,0x97,0xff }, /*   515 */
  {     0,   210,    68,0, 0x02cf,0xfdb4, 0x9b,0x9b,0x9b,0xff }, /*   516 */
  {     0,   210,    49,0, 0x02cf,0xfe41, 0xa2,0xa2,0xa2,0xff }, /*   517 */
  {    42,   204,    49,0, 0x0400,0xfdf1, 0x94,0x94,0x94,0xff }, /*   518 */
  {     0,   210,    68,0, 0x02cf,0xfdb4, 0x9b,0x9b,0x9b,0xff }, /*   519 */
  {   -64,   204,    47,0, 0x01b7,0xfdfd, 0x8a,0x8a,0x8a,0xff }, /*   520 */
  {   -42,   204,    49,0, 0x01f0,0xfdf1, 0xa9,0xa9,0xa9,0xff }, /*   521 */
  {    36,   132,    52,0, 0x077e,0x0000, 0xa4,0xa4,0xa4,0xff }, /*   522 */
  {     0,   170,    73,0, 0x0400,0x07ff, 0x8d,0x71,0x54,0xff }, /*   523 */
  {   -36,   132,    52,0, 0x0081,0x0000, 0xb9,0xb9,0xb9,0xff }, /*   524 */
  {     0,   170,    73,0, 0x0400,0x07ff, 0x8d,0x71,0x54,0xff }, /*   525 */
  {   -42,   170,    65,0, 0x0000,0x0755, 0x8f,0x5d,0x55,0xff }, /*   526 */
  {   -36,   132,    52,0, 0x0081,0x0000, 0xb9,0xb9,0xb9,0xff }, /*   527 */
  {    36,   132,    52,0, 0x077e,0x0000, 0xa4,0xa4,0xa4,0xff }, /*   528 */
  {    42,   170,    65,0, 0x0800,0x0755, 0xb9,0x79,0x6f,0xff }, /*   529 */
  {     0,   170,    73,0, 0x0400,0x07ff, 0x8d,0x71,0x54,0xff }, /*   530 */
  {   -42,   170,    65,0, 0xf800,0x0000, 0x8f,0x5d,0x55,0xff }, /*   531 */
  {     0,   210,    49,0, 0x0000,0x0800, 0xb4,0xb4,0xb4,0xff }, /*   532 */
  {   -42,   204,    49,0, 0xf800,0x06e5, 0xbc,0xbc,0xbc,0xff }, /*   533 */
  {    42,   170,    65,0, 0x0800,0x0000, 0xb9,0x79,0x6f,0xff }, /*   534 */
  {    42,   204,    49,0, 0x0800,0x06e5, 0xa4,0xa4,0xa4,0xff }, /*   535 */
  {     0,   210,    49,0, 0x0000,0x0800, 0xb4,0xb4,0xb4,0xff }, /*   536 */
  {   -42,   170,    65,0, 0xf800,0x0000, 0x8f,0x5d,0x55,0xff }, /*   537 */
  {     0,   170,    73,0, 0x0000,0x0000, 0x8d,0x71,0x54,0xff }, /*   538 */
  {     0,   210,    49,0, 0x0000,0x0800, 0xb4,0xb4,0xb4,0xff }, /*   539 */
  {     0,   170,    73,0, 0x0000,0x0000, 0x8d,0x71,0x54,0xff }, /*   540 */
  {    42,   170,    65,0, 0x0800,0x0000, 0xb9,0x79,0x6f,0xff }, /*   541 */
  {     0,   210,    49,0, 0x0000,0x0800, 0xb4,0xb4,0xb4,0xff }, /*   542 */
  {    -9,   247,    60,0, 0x0000,0x0000, 0xc2,0x5e,0x39,0xff }, /*   543 */
  {    -9,   273,    37,0, 0x0000,0x0000, 0xc2,0x29,0x66,0xff }, /*   544 */
  {   -46,   242,    50,0, 0x0000,0x0000, 0x02,0x69,0x46,0xff }, /*   545 */
  {     9,   273,    37,0, 0x0000,0x0000, 0x3e,0x29,0x66,0xff }, /*   546 */
  {    46,   242,    50,0, 0x0000,0x0000, 0xfe,0x69,0x46,0xff }, /*   547 */
  {    53,   268,    29,0, 0x0000,0x0000, 0xfa,0x36,0x72,0xff }, /*   548 */
  {     9,   273,    37,0, 0x0000,0x0000, 0x3e,0x29,0x66,0xff }, /*   549 */
  {     9,   247,    60,0, 0x0000,0x0000, 0x3e,0x5e,0x39,0xff }, /*   550 */
  {    46,   242,    50,0, 0x0000,0x0000, 0xfe,0x69,0x46,0xff }, /*   551 */
  {    -9,   273,    37,0, 0x0000,0x0000, 0xc2,0x29,0x66,0xff }, /*   552 */
  {   -53,   268,    29,0, 0x0000,0x0000, 0x06,0x36,0x72,0xff }, /*   553 */
  {   -46,   242,    50,0, 0x0000,0x0000, 0x02,0x69,0x46,0xff }, /*   554 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_8_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP2Triangles(  3,  4,  5,  0,  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP2Triangles( 18, 19, 20,  0, 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP2Triangles(  1,  2,  3,  0,  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 992), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 1024), 31, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsSP1Triangle(  2,  3,  4,  0),
  gsSP1Triangle(  5,  6,  7,  0),
  gsSP1Triangle(  8,  9, 10,  0),
  gsSP1Triangle( 11, 12, 13,  0),
  gsSP1Triangle( 14, 15, 16,  0),
  gsSP1Triangle( 17, 18, 19,  0),
  gsSP1Triangle( 20, 21, 22,  0),
  gsSP1Triangle( 23, 24, 25,  0),
  gsSP1Triangle( 26, 27, 28,  0),
  gsSPVertex( SEGADRS(5, 1520), 1, 31 ),
  gsSP1Triangle( 29, 30, 31,  0),
  gsSPVertex( SEGADRS(5, 1536), 29, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSPVertex( SEGADRS(5, 2000), 3, 29 ),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 2048), 30, 0 ),
  gsSP2Triangles( 30, 31,  0,  0,  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 2528), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 2560), 31, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsSP2Triangles(  2,  3,  4,  0,  5,  6,  7,  0),
  gsSP1Triangle(  8,  9, 10,  0),
  gsSP2Triangles( 11, 12, 13,  0, 14, 15, 16,  0),
  gsSP2Triangles( 17, 18, 19,  0, 20, 21, 22,  0),
  gsSP1Triangle( 23, 24, 25,  0),
  gsSP1Triangle( 26, 27, 28,  0),
  gsSPVertex( SEGADRS(5, 3056), 1, 31 ),
  gsSP1Triangle( 29, 30, 31,  0),
  gsSPVertex( SEGADRS(5, 3072), 29, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP2Triangles(  3,  4,  5,  0,  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP2Triangles( 18, 19, 20,  0, 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSPVertex( SEGADRS(5, 3536), 3, 29 ),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 3584), 7, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSPClearGeometryMode( G_SHADING_SMOOTH ),
  gsSPVertex( SEGADRS(5, 3696), 25, 7 ),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP2Triangles( 10, 11, 12,  0, 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP2Triangles( 19, 20, 21,  0, 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 4096), 23, 0 ),
  gsSP2Triangles( 31,  0,  1,  0,  2,  3,  4,  0),
  gsSP1Triangle(  5,  6,  7,  0),
  gsSP1Triangle(  8,  9, 10,  0),
  gsSP1Triangle( 11, 12, 13,  0),
  gsSP1Triangle( 14, 15, 16,  0),
  gsSP1Triangle( 17, 18, 19,  0),
  gsSP1Triangle( 20, 21, 22,  0),
  gsSPSetGeometryMode( G_SHADING_SMOOTH ),
  gsSPVertex( SEGADRS(5, 4464), 9, 23 ),
  gsSP1Triangle( 23, 24, 25,  0),
  gsSP1Triangle( 26, 27, 28,  0),
  gsSP1Triangle( 29, 30, 31,  0),
  gsSPVertex( SEGADRS(5, 4608), 29, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSPVertex( SEGADRS(5, 5072), 3, 29 ),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 5120), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP2Triangles( 13, 14, 15,  0, 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 5600), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 5632), 11, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsSP1Triangle(  2,  3,  4,  0),
  gsSP1Triangle(  5,  6,  7,  0),
  gsSP1Triangle(  8,  9, 10,  0),
  gsSPClearGeometryMode( G_SHADING_SMOOTH ),
  gsSPVertex( SEGADRS(5, 5808), 21, 11 ),
  gsSP2Triangles( 11, 12, 13,  0, 14, 15, 16,  0),
  gsSP1Triangle( 17, 18, 19,  0),
  gsSP2Triangles( 20, 21, 22,  0, 23, 24, 25,  0),
  gsSP1Triangle( 26, 27, 28,  0),
  gsSP1Triangle( 29, 30, 31,  0),
  gsSPVertex( SEGADRS(5, 6144), 27, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSPSetGeometryMode( G_SHADING_SMOOTH ),
  gsSPVertex( SEGADRS(5, 6576), 5, 27 ),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 6656), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP2Triangles(  1,  2,  3,  0,  4,  5,  6,  0),
  gsSP2Triangles(  7,  8,  9,  0, 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsDPPipeSync(),
  gsDPSetCombineLERP(TEXEL0, 0, SHADE, 0, SHADE, 0, TEXEL0, 0, TEXEL0, 0, SHADE, 0, SHADE, 0, TEXEL0, 0),
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_ON),
  gsDPSetTextureLUT( G_TT_NONE ),
  gsDPLoadMultiBlock( SEGADRS(6,2048), 0, 0,
    G_IM_FMT_RGBA, G_IM_SIZ_16b, 32, 32, 0,
    0, 0, 5, 5, 0, 0 ),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP2Triangles( 22, 23, 24,  0, 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 7136), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 7168), 31, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsSP2Triangles(  2,  3,  4,  0,  5,  6,  7,  0),
  gsSP1Triangle(  8,  9, 10,  0),
  gsSP2Triangles( 11, 12, 13,  0, 14, 15, 16,  0),
  gsSP1Triangle( 17, 18, 19,  0),
  gsSP1Triangle( 20, 21, 22,  0),
  gsSP1Triangle( 23, 24, 25,  0),
  gsSP1Triangle( 26, 27, 28,  0),
  gsSPVertex( SEGADRS(5, 7664), 1, 31 ),
  gsSP1Triangle( 29, 30, 31,  0),
  gsSPVertex( SEGADRS(5, 7680), 29, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP1Triangle(  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSPVertex( SEGADRS(5, 8144), 3, 29 ),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 8192), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsDPPipeSync(),
  gsDPSetTextureLUT( G_TT_NONE ),
  gsDPLoadMultiBlock( SEGADRS(6,4096), 0, 0,
    G_IM_FMT_RGBA, G_IM_SIZ_16b, 32, 32, 0,
    1, 1, 5, 5, 0, 0 ),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsDPPipeSync(),
  gsDPSetTextureLUT( G_TT_NONE ),
  gsDPLoadMultiBlock( SEGADRS(6,6144), 0, 0,
    G_IM_FMT_RGBA, G_IM_SIZ_16b, 32, 32, 0,
    1, 1, 5, 5, 0, 0 ),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 8672), 1, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPSetGeometryMode( G_LIGHTING|G_TEXTURE_GEN ),
  gsDPPipeSync(),
  gsDPSetCombineLERP(TEXEL0, 0, SHADE, 0, 0, 0, 0, TEXEL0, TEXEL0, 0, SHADE, 0, 0, 0, 0, TEXEL0),
  gsSPTexture(0x0800, 0x0800, 0, G_TX_RENDERTILE, G_ON),
  gsDPSetTextureLUT( G_TT_NONE ),
  gsDPLoadMultiBlock( SEGADRS(6,8192), 0, 0,
    G_IM_FMT_RGBA, G_IM_SIZ_16b, 32, 32, 0,
    0, 0, 5, 5, 0, 0 ),
  gsSPVertex( SEGADRS(5, 8688), 12, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_8_ = {
  gfx_Shape_of___POLYHEDRON_8_,
  vtx_Shape_of___POLYHEDRON_8_, 555, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner11[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_69_[] = {
  {    38,   -53,     3,0, 0x0000,0x0000, 0xef,0xad,0x7c,0xff }, /*     0 */
  {    38,   -75,     3,0, 0x0000,0x0000, 0xcc,0x93,0x6a,0xff }, /*     1 */
  {    38,   -53,   -22,0, 0x0000,0x0000, 0x8e,0x67,0x4a,0xff }, /*     2 */
  {    38,   -75,     3,0, 0x0000,0x0000, 0xcc,0x93,0x6a,0xff }, /*     3 */
  {    38,   -75,   -22,0, 0x0000,0x0000, 0x8c,0x65,0x49,0xff }, /*     4 */
  {    38,   -53,   -22,0, 0x0000,0x0000, 0x8e,0x67,0x4a,0xff }, /*     5 */
  {    38,   -53,   -22,0, 0x0000,0x0000, 0x8e,0x67,0x4a,0xff }, /*     6 */
  {    38,   -75,   -22,0, 0x0000,0x0000, 0x8c,0x65,0x49,0xff }, /*     7 */
  {    13,   -53,   -22,0, 0x0000,0x0000, 0xd9,0x9d,0x70,0xff }, /*     8 */
  {    38,   -75,   -22,0, 0x0000,0x0000, 0x8c,0x65,0x49,0xff }, /*     9 */
  {    13,   -75,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4f,0xff }, /*    10 */
  {    13,   -53,   -22,0, 0x0000,0x0000, 0xd9,0x9d,0x70,0xff }, /*    11 */
  {    13,   -53,   -22,0, 0x0000,0x0000, 0xd9,0x9d,0x70,0xff }, /*    12 */
  {    13,   -75,     3,0, 0x0000,0x0000, 0x8e,0x66,0x49,0xff }, /*    13 */
  {    13,   -53,     3,0, 0x0000,0x0000, 0xff,0xbb,0x86,0xff }, /*    14 */
  {    13,   -53,     3,0, 0x0000,0x0000, 0xff,0xbb,0x86,0xff }, /*    15 */
  {    13,   -75,     3,0, 0x0000,0x0000, 0x8e,0x66,0x49,0xff }, /*    16 */
  {    38,   -53,     3,0, 0x0000,0x0000, 0xef,0xad,0x7c,0xff }, /*    17 */
  {    13,   -75,     3,0, 0x0000,0x0000, 0x8e,0x66,0x49,0xff }, /*    18 */
  {    38,   -75,     3,0, 0x0000,0x0000, 0xcc,0x93,0x6a,0xff }, /*    19 */
  {    38,   -53,     3,0, 0x0000,0x0000, 0xef,0xad,0x7c,0xff }, /*    20 */
  {    13,   -53,   -22,0, 0x0000,0x0000, 0xd9,0x9d,0x70,0xff }, /*    21 */
  {    13,   -75,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4f,0xff }, /*    22 */
  {    13,   -75,     3,0, 0x0000,0x0000, 0x8e,0x66,0x49,0xff }, /*    23 */
  {    38,   -75,   -22,0, 0x0000,0x0000, 0x8c,0x65,0x49,0xff }, /*    24 */
  {    26,   -86,    -9,0, 0x0000,0x0000, 0x8f,0x3e,0x2c,0xff }, /*    25 */
  {    13,   -75,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4f,0xff }, /*    26 */
  {    13,   -75,     3,0, 0x0000,0x0000, 0x8e,0x66,0x49,0xff }, /*    27 */
  {    26,   -86,    -9,0, 0x0000,0x0000, 0x8f,0x3e,0x2c,0xff }, /*    28 */
  {    38,   -75,     3,0, 0x0000,0x0000, 0xcc,0x93,0x6a,0xff }, /*    29 */
  {    38,   -75,     3,0, 0x0000,0x0000, 0xcc,0x93,0x6a,0xff }, /*    30 */
  {    26,   -86,    -9,0, 0x0000,0x0000, 0x8f,0x3e,0x2c,0xff }, /*    31 */
  {    38,   -75,   -22,0, 0x0000,0x0000, 0x8c,0x65,0x49,0xff }, /*    32 */
  {    13,   -75,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4f,0xff }, /*    33 */
  {    26,   -86,    -9,0, 0x0000,0x0000, 0x8f,0x3e,0x2c,0xff }, /*    34 */
  {    13,   -75,     3,0, 0x0000,0x0000, 0x8e,0x66,0x49,0xff }, /*    35 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_69_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP2Triangles( 24, 25, 26,  0, 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 4, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_69_ = {
  gfx_Shape_of___POLYHEDRON_69_,
  vtx_Shape_of___POLYHEDRON_69_, 36, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner12[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_73_[] = {
  {    38,  -104,     3,0, 0x0000,0x0000, 0xc0,0x8b,0x64,0xff }, /*     0 */
  {    38,   -80,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*     1 */
  {    13,  -104,     3,0, 0x0000,0x0000, 0x8f,0x67,0x4a,0xff }, /*     2 */
  {    13,  -104,   -22,0, 0x0000,0x0000, 0xea,0xa9,0x7a,0xff }, /*     3 */
  {    13,   -80,   -22,0, 0x0000,0x0000, 0xfa,0xb5,0x82,0xff }, /*     4 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*     5 */
  {    13,   -80,     3,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*     6 */
  {    13,   -80,   -22,0, 0x0000,0x0000, 0xfa,0xb5,0x82,0xff }, /*     7 */
  {    13,  -104,     3,0, 0x0000,0x0000, 0x8f,0x67,0x4a,0xff }, /*     8 */
  {    38,   -80,   -22,0, 0x0000,0x0000, 0xd3,0x98,0x6d,0xff }, /*     9 */
  {    38,   -80,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    10 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*    11 */
  {    38,   -80,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    12 */
  {    26,   -67,    -9,0, 0x0000,0x0000, 0xff,0x70,0x50,0xff }, /*    13 */
  {    13,   -80,     3,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    14 */
  {    38,   -80,   -22,0, 0x0000,0x0000, 0xd3,0x98,0x6d,0xff }, /*    15 */
  {    26,   -67,    -9,0, 0x0000,0x0000, 0xff,0x70,0x50,0xff }, /*    16 */
  {    38,   -80,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    17 */
  {    13,   -80,   -22,0, 0x0000,0x0000, 0xfa,0xb5,0x82,0xff }, /*    18 */
  {    26,   -67,    -9,0, 0x0000,0x0000, 0xff,0x70,0x50,0xff }, /*    19 */
  {    38,   -80,   -22,0, 0x0000,0x0000, 0xd3,0x98,0x6d,0xff }, /*    20 */
  {    13,   -80,     3,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    21 */
  {    26,   -67,    -9,0, 0x0000,0x0000, 0xff,0x70,0x50,0xff }, /*    22 */
  {    13,   -80,   -22,0, 0x0000,0x0000, 0xfa,0xb5,0x82,0xff }, /*    23 */
  {    38,  -104,     3,0, 0x0000,0x0000, 0x77,0x3b,0x00,0xff }, /*    24 */
  {    38,  -138,     3,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*    25 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    26 */
  {    38,  -138,     3,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*    27 */
  {    38,  -138,   -22,0, 0x0000,0x0000, 0x56,0x2b,0x00,0xff }, /*    28 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    29 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    30 */
  {    38,  -138,   -22,0, 0x0000,0x0000, 0x56,0x2b,0x00,0xff }, /*    31 */
  {    13,  -104,   -22,0, 0x0000,0x0000, 0x91,0x48,0x00,0xff }, /*    32 */
  {    38,  -138,   -22,0, 0x0000,0x0000, 0x56,0x2b,0x00,0xff }, /*    33 */
  {    13,  -138,   -22,0, 0x0000,0x0000, 0x6f,0x37,0x00,0xff }, /*    34 */
  {    13,  -104,   -22,0, 0x0000,0x0000, 0x91,0x48,0x00,0xff }, /*    35 */
  {    13,  -104,   -22,0, 0x0000,0x0000, 0x91,0x48,0x00,0xff }, /*    36 */
  {    13,  -138,     3,0, 0x0000,0x0000, 0x56,0x2b,0x00,0xff }, /*    37 */
  {    13,  -104,     3,0, 0x0000,0x0000, 0x59,0x2c,0x00,0xff }, /*    38 */
  {    13,  -104,     3,0, 0x0000,0x0000, 0x59,0x2c,0x00,0xff }, /*    39 */
  {    13,  -138,     3,0, 0x0000,0x0000, 0x56,0x2b,0x00,0xff }, /*    40 */
  {    38,  -104,     3,0, 0x0000,0x0000, 0x77,0x3b,0x00,0xff }, /*    41 */
  {    13,  -138,     3,0, 0x0000,0x0000, 0x56,0x2b,0x00,0xff }, /*    42 */
  {    38,  -138,     3,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*    43 */
  {    38,  -104,     3,0, 0x0000,0x0000, 0x77,0x3b,0x00,0xff }, /*    44 */
  {    13,  -104,   -22,0, 0x0000,0x0000, 0x91,0x48,0x00,0xff }, /*    45 */
  {    13,  -138,   -22,0, 0x0000,0x0000, 0x6f,0x37,0x00,0xff }, /*    46 */
  {    13,  -138,     3,0, 0x0000,0x0000, 0x56,0x2b,0x00,0xff }, /*    47 */
  {    38,   -80,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    48 */
  {    38,  -104,     3,0, 0x0000,0x0000, 0xc0,0x8b,0x64,0xff }, /*    49 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*    50 */
  {    13,   -80,   -22,0, 0x0000,0x0000, 0xfa,0xb5,0x82,0xff }, /*    51 */
  {    13,  -104,   -22,0, 0x0000,0x0000, 0xea,0xa9,0x7a,0xff }, /*    52 */
  {    13,  -104,     3,0, 0x0000,0x0000, 0x8f,0x67,0x4a,0xff }, /*    53 */
  {    13,   -80,   -22,0, 0x0000,0x0000, 0xfa,0xb5,0x82,0xff }, /*    54 */
  {    38,   -80,   -22,0, 0x0000,0x0000, 0xd3,0x98,0x6d,0xff }, /*    55 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*    56 */
  {    38,   -80,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    57 */
  {    13,   -80,     3,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    58 */
  {    13,  -104,     3,0, 0x0000,0x0000, 0x8f,0x67,0x4a,0xff }, /*    59 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_73_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP1Triangle(  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP2Triangles( 24, 25, 26,  0, 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 28, 0 ),
  gsSP2Triangles( 30, 31,  0,  0,  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP2Triangles(  7,  8,  9,  0, 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_73_ = {
  gfx_Shape_of___POLYHEDRON_73_,
  vtx_Shape_of___POLYHEDRON_73_, 60, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner13[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_77_[] = {
  {    10,  -129,   -24,0, 0x0000,0x0000, 0x8a,0x45,0x00,0xff }, /*     0 */
  {    44,  -151,   -30,0, 0x0000,0x0000, 0x57,0x2b,0x00,0xff }, /*     1 */
  {     5,  -151,   -23,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*     2 */
  {    55,  -119,     8,0, 0x0000,0x0000, 0x8f,0x47,0x00,0xff }, /*     3 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*     4 */
  {    52,  -109,    39,0, 0x0000,0x0000, 0x97,0x4b,0x00,0xff }, /*     5 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*     6 */
  {    18,  -109,    45,0, 0x0000,0x0000, 0xa0,0x50,0x00,0xff }, /*     7 */
  {    52,  -109,    39,0, 0x0000,0x0000, 0x97,0x4b,0x00,0xff }, /*     8 */
  {    59,  -151,     7,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*     9 */
  {    13,  -151,    46,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    10 */
  {     5,  -151,    17,0, 0x0000,0x0000, 0x48,0x24,0x00,0xff }, /*    11 */
  {     5,  -151,    17,0, 0x0000,0x0000, 0x48,0x24,0x00,0xff }, /*    12 */
  {    44,  -151,   -30,0, 0x0000,0x0000, 0x57,0x2b,0x00,0xff }, /*    13 */
  {    59,  -151,     7,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    14 */
  {    18,  -109,    45,0, 0x0000,0x0000, 0xa0,0x50,0x00,0xff }, /*    15 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*    16 */
  {    13,  -151,    46,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    17 */
  {    55,  -119,     8,0, 0x0000,0x0000, 0x8f,0x47,0x00,0xff }, /*    18 */
  {    65,  -151,    37,0, 0x0000,0x0000, 0x62,0x31,0x00,0xff }, /*    19 */
  {    59,  -151,     7,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    20 */
  {    13,  -151,    46,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    21 */
  {    39,  -138,    62,0, 0x0000,0x0000, 0x57,0x2b,0x00,0xff }, /*    22 */
  {    18,  -109,    45,0, 0x0000,0x0000, 0xa0,0x50,0x00,0xff }, /*    23 */
  {    65,  -151,    37,0, 0x0000,0x0000, 0x62,0x31,0x00,0xff }, /*    24 */
  {    39,  -138,    62,0, 0x0000,0x0000, 0x57,0x2b,0x00,0xff }, /*    25 */
  {    13,  -151,    46,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    26 */
  {    52,  -109,    39,0, 0x0000,0x0000, 0x97,0x4b,0x00,0xff }, /*    27 */
  {    39,  -138,    62,0, 0x0000,0x0000, 0x57,0x2b,0x00,0xff }, /*    28 */
  {    65,  -151,    37,0, 0x0000,0x0000, 0x62,0x31,0x00,0xff }, /*    29 */
  {    18,  -109,    45,0, 0x0000,0x0000, 0xa0,0x50,0x00,0xff }, /*    30 */
  {    39,  -138,    62,0, 0x0000,0x0000, 0x57,0x2b,0x00,0xff }, /*    31 */
  {    52,  -109,    39,0, 0x0000,0x0000, 0x97,0x4b,0x00,0xff }, /*    32 */
  {    55,  -119,     8,0, 0x0000,0x0000, 0x8f,0x47,0x00,0xff }, /*    33 */
  {    10,  -129,   -24,0, 0x0000,0x0000, 0x8a,0x45,0x00,0xff }, /*    34 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*    35 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*    36 */
  {     5,  -151,   -23,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    37 */
  {     5,  -151,    17,0, 0x0000,0x0000, 0x48,0x24,0x00,0xff }, /*    38 */
  {    59,  -151,     7,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    39 */
  {    41,  -129,   -29,0, 0x0000,0x0000, 0x7e,0x3f,0x00,0xff }, /*    40 */
  {    55,  -119,     8,0, 0x0000,0x0000, 0x8f,0x47,0x00,0xff }, /*    41 */
  {    10,  -129,   -24,0, 0x0000,0x0000, 0x8a,0x45,0x00,0xff }, /*    42 */
  {    41,  -129,   -29,0, 0x0000,0x0000, 0x7e,0x3f,0x00,0xff }, /*    43 */
  {    44,  -151,   -30,0, 0x0000,0x0000, 0x57,0x2b,0x00,0xff }, /*    44 */
  {    59,  -151,     7,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    45 */
  {    44,  -151,   -30,0, 0x0000,0x0000, 0x57,0x2b,0x00,0xff }, /*    46 */
  {    41,  -129,   -29,0, 0x0000,0x0000, 0x7e,0x3f,0x00,0xff }, /*    47 */
  {     5,  -151,    17,0, 0x0000,0x0000, 0x48,0x24,0x00,0xff }, /*    48 */
  {     5,  -151,   -23,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    49 */
  {    44,  -151,   -30,0, 0x0000,0x0000, 0x57,0x2b,0x00,0xff }, /*    50 */
  {    59,  -151,     7,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    51 */
  {    65,  -151,    37,0, 0x0000,0x0000, 0x62,0x31,0x00,0xff }, /*    52 */
  {    13,  -151,    46,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    53 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*    54 */
  {    10,  -129,   -24,0, 0x0000,0x0000, 0x8a,0x45,0x00,0xff }, /*    55 */
  {     5,  -151,   -23,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    56 */
  {    55,  -119,     8,0, 0x0000,0x0000, 0x8f,0x47,0x00,0xff }, /*    57 */
  {    41,  -129,   -29,0, 0x0000,0x0000, 0x7e,0x3f,0x00,0xff }, /*    58 */
  {    10,  -129,   -24,0, 0x0000,0x0000, 0x8a,0x45,0x00,0xff }, /*    59 */
  {    55,  -119,     8,0, 0x0000,0x0000, 0x8f,0x47,0x00,0xff }, /*    60 */
  {    52,  -109,    39,0, 0x0000,0x0000, 0x97,0x4b,0x00,0xff }, /*    61 */
  {    65,  -151,    37,0, 0x0000,0x0000, 0x62,0x31,0x00,0xff }, /*    62 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*    63 */
  {     5,  -151,    17,0, 0x0000,0x0000, 0x48,0x24,0x00,0xff }, /*    64 */
  {    13,  -151,    46,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    65 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_77_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP2Triangles(  3,  4,  5,  0,  6,  7,  8,  0),
  gsSP2Triangles(  9, 10, 11,  0, 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP2Triangles(  7,  8,  9,  0, 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP2Triangles( 16, 17, 18,  0, 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 992), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 1024), 2, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_77_ = {
  gfx_Shape_of___POLYHEDRON_77_,
  vtx_Shape_of___POLYHEDRON_77_, 66, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner14[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_65_[] = {
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*     0 */
  {     0,   -70,    15,0, 0x0000,0x0000, 0x00,0x2c,0x00,0xff }, /*     1 */
  {    53,   -70,    15,0, 0x0000,0x0000, 0x00,0x3f,0x00,0xff }, /*     2 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*     3 */
  {     8,     7,   -24,0, 0x0000,0x0000, 0x00,0x41,0x00,0xff }, /*     4 */
  {     0,    -9,   -35,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*     5 */
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*     6 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*     7 */
  {     0,   -70,    15,0, 0x0000,0x0000, 0x00,0x2c,0x00,0xff }, /*     8 */
  {     0,    -9,   -35,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*     9 */
  {    45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x25,0x00,0xff }, /*    10 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x2f,0x00,0xff }, /*    11 */
  {    45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x25,0x00,0xff }, /*    12 */
  {    53,   -70,   -37,0, 0x0000,0x0000, 0x00,0x32,0x00,0xff }, /*    13 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x2f,0x00,0xff }, /*    14 */
  {    24,     7,   -24,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    15 */
  {     8,     7,   -24,0, 0x0000,0x0000, 0x00,0x41,0x00,0xff }, /*    16 */
  {    24,     7,    -3,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    17 */
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    18 */
  {    24,     7,    -3,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    19 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    20 */
  {     8,     7,   -24,0, 0x0000,0x0000, 0x00,0x41,0x00,0xff }, /*    21 */
  {     8,     7,    -3,0, 0x0000,0x0000, 0x00,0x59,0x00,0xff }, /*    22 */
  {    24,     7,    -3,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    23 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    24 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x2f,0x00,0xff }, /*    25 */
  {     0,   -70,    15,0, 0x0000,0x0000, 0x00,0x2c,0x00,0xff }, /*    26 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    27 */
  {     0,    -9,   -35,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*    28 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x2f,0x00,0xff }, /*    29 */
  {    45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x25,0x00,0xff }, /*    30 */
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    31 */
  {    53,   -70,   -37,0, 0x0000,0x0000, 0x00,0x32,0x00,0xff }, /*    32 */
  {    45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x25,0x00,0xff }, /*    33 */
  {    24,     7,   -24,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    34 */
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    35 */
  {     0,    -9,   -35,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*    36 */
  {    24,     7,   -24,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    37 */
  {    45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x25,0x00,0xff }, /*    38 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    39 */
  {     8,     7,    -3,0, 0x0000,0x0000, 0x00,0x59,0x00,0xff }, /*    40 */
  {     8,     7,   -24,0, 0x0000,0x0000, 0x00,0x41,0x00,0xff }, /*    41 */
  {     0,    -9,   -35,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*    42 */
  {     8,     7,   -24,0, 0x0000,0x0000, 0x00,0x41,0x00,0xff }, /*    43 */
  {    24,     7,   -24,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    44 */
  {    24,     7,   -24,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    45 */
  {    24,     7,    -3,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    46 */
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    47 */
  {    24,     7,    -3,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    48 */
  {     8,     7,    -3,0, 0x0000,0x0000, 0x00,0x59,0x00,0xff }, /*    49 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    50 */
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    51 */
  {    53,   -70,    15,0, 0x0000,0x0000, 0x00,0x3f,0x00,0xff }, /*    52 */
  {    53,   -70,   -37,0, 0x0000,0x0000, 0x00,0x32,0x00,0xff }, /*    53 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_65_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP2Triangles(  9, 10, 11,  0, 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP2Triangles( 24, 25, 26,  0, 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 22, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_65_ = {
  gfx_Shape_of___POLYHEDRON_65_,
  vtx_Shape_of___POLYHEDRON_65_, 54, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner15[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_63_[] = {
  {     5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x33,0x00,0xff }, /*     0 */
  {   -30,    11,    22,0, 0x0000,0x0000, 0x00,0x47,0x00,0xff }, /*     1 */
  {    -5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x39,0x00,0xff }, /*     2 */
  {     6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x2e,0x00,0xff }, /*     3 */
  {    30,    11,    22,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*     4 */
  {     5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x33,0x00,0xff }, /*     5 */
  {     6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x2e,0x00,0xff }, /*     6 */
  {    30,    11,   -37,0, 0x0000,0x0000, 0x00,0x26,0x00,0xff }, /*     7 */
  {    30,    11,    22,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*     8 */
  {     5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x33,0x00,0xff }, /*     9 */
  {    30,    11,    22,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    10 */
  {   -30,    11,    22,0, 0x0000,0x0000, 0x00,0x47,0x00,0xff }, /*    11 */
  {    -6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x33,0x00,0xff }, /*    12 */
  {    30,    11,   -37,0, 0x0000,0x0000, 0x00,0x26,0x00,0xff }, /*    13 */
  {     6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x2e,0x00,0xff }, /*    14 */
  {   -30,    11,    22,0, 0x0000,0x0000, 0x00,0x47,0x00,0xff }, /*    15 */
  {   -30,    11,   -37,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*    16 */
  {    -6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x33,0x00,0xff }, /*    17 */
  {    -6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x33,0x00,0xff }, /*    18 */
  {   -30,    11,   -37,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*    19 */
  {    30,    11,   -37,0, 0x0000,0x0000, 0x00,0x26,0x00,0xff }, /*    20 */
  {   -30,    11,    22,0, 0x0000,0x0000, 0x00,0x47,0x00,0xff }, /*    21 */
  {    30,    11,   -37,0, 0x0000,0x0000, 0x00,0x26,0x00,0xff }, /*    22 */
  {   -30,    11,   -37,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*    23 */
  {   -30,    11,    22,0, 0x0000,0x0000, 0x00,0x47,0x00,0xff }, /*    24 */
  {    30,    11,    22,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    25 */
  {    30,    11,   -37,0, 0x0000,0x0000, 0x00,0x26,0x00,0xff }, /*    26 */
  {    -5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x39,0x00,0xff }, /*    27 */
  {   -30,    11,    22,0, 0x0000,0x0000, 0x00,0x47,0x00,0xff }, /*    28 */
  {    -6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x33,0x00,0xff }, /*    29 */
  {     6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x2e,0x00,0xff }, /*    30 */
  {    -5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x39,0x00,0xff }, /*    31 */
  {    -6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x33,0x00,0xff }, /*    32 */
  {     6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x2e,0x00,0xff }, /*    33 */
  {     5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x33,0x00,0xff }, /*    34 */
  {    -5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x39,0x00,0xff }, /*    35 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_63_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP2Triangles( 12, 13, 14,  0, 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP2Triangles( 21, 22, 23,  0, 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 4, 0 ),
  gsSP2Triangles( 30, 31,  0,  0,  1,  2,  3,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_63_ = {
  gfx_Shape_of___POLYHEDRON_63_,
  vtx_Shape_of___POLYHEDRON_63_, 36, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner16[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_61_[] = {
  {   -53,   -70,    15,0, 0x0000,0x0000, 0x00,0x30,0x00,0xff }, /*     0 */
  {     0,   -70,    15,0, 0x0000,0x0000, 0x00,0x36,0x00,0xff }, /*     1 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x54,0x00,0xff }, /*     2 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x30,0x00,0xff }, /*     3 */
  {   -45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x47,0x00,0xff }, /*     4 */
  {     1,    -9,   -35,0, 0x0000,0x0000, 0x00,0x2b,0x00,0xff }, /*     5 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x30,0x00,0xff }, /*     6 */
  {   -53,   -70,   -37,0, 0x0000,0x0000, 0x00,0x31,0x00,0xff }, /*     7 */
  {   -45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x47,0x00,0xff }, /*     8 */
  {   -24,     7,    -3,0, 0x0000,0x0000, 0x00,0x54,0x00,0xff }, /*     9 */
  {    -8,     7,   -24,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    10 */
  {   -24,     7,   -24,0, 0x0000,0x0000, 0x00,0x43,0x00,0xff }, /*    11 */
  {   -24,     7,    -3,0, 0x0000,0x0000, 0x00,0x54,0x00,0xff }, /*    12 */
  {    -8,     7,    -3,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    13 */
  {    -8,     7,   -24,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    14 */
  {     0,   -70,    15,0, 0x0000,0x0000, 0x00,0x36,0x00,0xff }, /*    15 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x30,0x00,0xff }, /*    16 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    17 */
  {   -53,   -70,   -37,0, 0x0000,0x0000, 0x00,0x31,0x00,0xff }, /*    18 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x54,0x00,0xff }, /*    19 */
  {   -45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x47,0x00,0xff }, /*    20 */
  {   -53,   -70,   -37,0, 0x0000,0x0000, 0x00,0x31,0x00,0xff }, /*    21 */
  {   -53,   -70,    15,0, 0x0000,0x0000, 0x00,0x30,0x00,0xff }, /*    22 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x54,0x00,0xff }, /*    23 */
  {     0,   -70,    15,0, 0x0000,0x0000, 0x00,0x36,0x00,0xff }, /*    24 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    25 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x54,0x00,0xff }, /*    26 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    27 */
  {   -24,     7,    -3,0, 0x0000,0x0000, 0x00,0x54,0x00,0xff }, /*    28 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x54,0x00,0xff }, /*    29 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x54,0x00,0xff }, /*    30 */
  {   -24,     7,    -3,0, 0x0000,0x0000, 0x00,0x54,0x00,0xff }, /*    31 */
  {   -24,     7,   -24,0, 0x0000,0x0000, 0x00,0x43,0x00,0xff }, /*    32 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x54,0x00,0xff }, /*    33 */
  {   -24,     7,   -24,0, 0x0000,0x0000, 0x00,0x43,0x00,0xff }, /*    34 */
  {   -45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x47,0x00,0xff }, /*    35 */
  {   -45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x47,0x00,0xff }, /*    36 */
  {   -24,     7,   -24,0, 0x0000,0x0000, 0x00,0x43,0x00,0xff }, /*    37 */
  {     1,    -9,   -35,0, 0x0000,0x0000, 0x00,0x2b,0x00,0xff }, /*    38 */
  {   -24,     7,   -24,0, 0x0000,0x0000, 0x00,0x43,0x00,0xff }, /*    39 */
  {    -8,     7,   -24,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    40 */
  {     1,    -9,   -35,0, 0x0000,0x0000, 0x00,0x2b,0x00,0xff }, /*    41 */
  {     1,    -9,   -35,0, 0x0000,0x0000, 0x00,0x2b,0x00,0xff }, /*    42 */
  {    -8,     7,   -24,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    43 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    44 */
  {    -8,     7,   -24,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    45 */
  {    -8,     7,    -3,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    46 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    47 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    48 */
  {    -8,     7,    -3,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    49 */
  {   -24,     7,    -3,0, 0x0000,0x0000, 0x00,0x54,0x00,0xff }, /*    50 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x30,0x00,0xff }, /*    51 */
  {     1,    -9,   -35,0, 0x0000,0x0000, 0x00,0x2b,0x00,0xff }, /*    52 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    53 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_61_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP2Triangles(  3,  4,  5,  0,  6,  7,  8,  0),
  gsSP2Triangles(  9, 10, 11,  0, 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP2Triangles( 18, 19, 20,  0, 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 22, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP2Triangles(  4,  5,  6,  0,  7,  8,  9,  0),
  gsSP2Triangles( 10, 11, 12,  0, 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_61_ = {
  gfx_Shape_of___POLYHEDRON_61_,
  vtx_Shape_of___POLYHEDRON_61_, 54, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner17[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_71_[] = {
  {   -13,  -104,     3,0, 0x0000,0x0000, 0xb1,0x80,0x5c,0xff }, /*     0 */
  {   -38,   -80,     3,0, 0x0000,0x0000, 0xc0,0x8b,0x64,0xff }, /*     1 */
  {   -38,  -104,     3,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*     2 */
  {   -13,  -104,     3,0, 0x0000,0x0000, 0xb1,0x80,0x5c,0xff }, /*     3 */
  {   -13,   -80,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*     4 */
  {   -38,   -80,     3,0, 0x0000,0x0000, 0xc0,0x8b,0x64,0xff }, /*     5 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*     6 */
  {   -13,   -80,   -22,0, 0x0000,0x0000, 0xc4,0x8e,0x66,0xff }, /*     7 */
  {   -13,  -104,   -22,0, 0x0000,0x0000, 0xa1,0x74,0x54,0xff }, /*     8 */
  {   -13,  -104,     3,0, 0x0000,0x0000, 0xb1,0x80,0x5c,0xff }, /*     9 */
  {   -13,   -80,   -22,0, 0x0000,0x0000, 0xc4,0x8e,0x66,0xff }, /*    10 */
  {   -13,   -80,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    11 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    12 */
  {   -38,   -80,     3,0, 0x0000,0x0000, 0xc0,0x8b,0x64,0xff }, /*    13 */
  {   -38,   -80,   -22,0, 0x0000,0x0000, 0xfd,0xb7,0x84,0xff }, /*    14 */
  {   -13,   -80,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    15 */
  {   -26,   -67,    -9,0, 0x0000,0x0000, 0xf6,0x6a,0x4c,0xff }, /*    16 */
  {   -38,   -80,     3,0, 0x0000,0x0000, 0xc0,0x8b,0x64,0xff }, /*    17 */
  {   -38,   -80,     3,0, 0x0000,0x0000, 0xc0,0x8b,0x64,0xff }, /*    18 */
  {   -26,   -67,    -9,0, 0x0000,0x0000, 0xf6,0x6a,0x4c,0xff }, /*    19 */
  {   -38,   -80,   -22,0, 0x0000,0x0000, 0xfd,0xb7,0x84,0xff }, /*    20 */
  {   -38,   -80,   -22,0, 0x0000,0x0000, 0xfd,0xb7,0x84,0xff }, /*    21 */
  {   -26,   -67,    -9,0, 0x0000,0x0000, 0xf6,0x6a,0x4c,0xff }, /*    22 */
  {   -13,   -80,   -22,0, 0x0000,0x0000, 0xc4,0x8e,0x66,0xff }, /*    23 */
  {   -13,   -80,   -22,0, 0x0000,0x0000, 0xc4,0x8e,0x66,0xff }, /*    24 */
  {   -26,   -67,    -9,0, 0x0000,0x0000, 0xf6,0x6a,0x4c,0xff }, /*    25 */
  {   -13,   -80,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    26 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0x89,0x44,0x00,0xff }, /*    27 */
  {   -38,  -138,     3,0, 0x0000,0x0000, 0x4d,0x26,0x00,0xff }, /*    28 */
  {   -38,  -104,     3,0, 0x0000,0x0000, 0x5f,0x2f,0x00,0xff }, /*    29 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0x89,0x44,0x00,0xff }, /*    30 */
  {   -38,  -138,   -22,0, 0x0000,0x0000, 0x72,0x39,0x00,0xff }, /*    31 */
  {   -38,  -138,     3,0, 0x0000,0x0000, 0x4d,0x26,0x00,0xff }, /*    32 */
  {   -13,  -104,   -22,0, 0x0000,0x0000, 0x64,0x32,0x00,0xff }, /*    33 */
  {   -38,  -138,   -22,0, 0x0000,0x0000, 0x72,0x39,0x00,0xff }, /*    34 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0x89,0x44,0x00,0xff }, /*    35 */
  {   -13,  -104,   -22,0, 0x0000,0x0000, 0x64,0x32,0x00,0xff }, /*    36 */
  {   -13,  -138,   -22,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    37 */
  {   -38,  -138,   -22,0, 0x0000,0x0000, 0x72,0x39,0x00,0xff }, /*    38 */
  {   -13,  -104,     3,0, 0x0000,0x0000, 0x6e,0x37,0x00,0xff }, /*    39 */
  {   -13,  -138,     3,0, 0x0000,0x0000, 0x67,0x33,0x00,0xff }, /*    40 */
  {   -13,  -104,   -22,0, 0x0000,0x0000, 0x64,0x32,0x00,0xff }, /*    41 */
  {   -38,  -104,     3,0, 0x0000,0x0000, 0x5f,0x2f,0x00,0xff }, /*    42 */
  {   -13,  -138,     3,0, 0x0000,0x0000, 0x67,0x33,0x00,0xff }, /*    43 */
  {   -13,  -104,     3,0, 0x0000,0x0000, 0x6e,0x37,0x00,0xff }, /*    44 */
  {   -38,  -104,     3,0, 0x0000,0x0000, 0x5f,0x2f,0x00,0xff }, /*    45 */
  {   -38,  -138,     3,0, 0x0000,0x0000, 0x4d,0x26,0x00,0xff }, /*    46 */
  {   -13,  -138,     3,0, 0x0000,0x0000, 0x67,0x33,0x00,0xff }, /*    47 */
  {   -13,  -138,     3,0, 0x0000,0x0000, 0x67,0x33,0x00,0xff }, /*    48 */
  {   -13,  -138,   -22,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    49 */
  {   -13,  -104,   -22,0, 0x0000,0x0000, 0x64,0x32,0x00,0xff }, /*    50 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    51 */
  {   -38,  -104,     3,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*    52 */
  {   -38,   -80,     3,0, 0x0000,0x0000, 0xc0,0x8b,0x64,0xff }, /*    53 */
  {   -13,  -104,     3,0, 0x0000,0x0000, 0xb1,0x80,0x5c,0xff }, /*    54 */
  {   -13,  -104,   -22,0, 0x0000,0x0000, 0xa1,0x74,0x54,0xff }, /*    55 */
  {   -13,   -80,   -22,0, 0x0000,0x0000, 0xc4,0x8e,0x66,0xff }, /*    56 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    57 */
  {   -38,   -80,   -22,0, 0x0000,0x0000, 0xfd,0xb7,0x84,0xff }, /*    58 */
  {   -13,   -80,   -22,0, 0x0000,0x0000, 0xc4,0x8e,0x66,0xff }, /*    59 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_71_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 28, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP2Triangles(  1,  2,  3,  0,  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP2Triangles( 10, 11, 12,  0, 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_71_ = {
  gfx_Shape_of___POLYHEDRON_71_,
  vtx_Shape_of___POLYHEDRON_71_, 60, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner18[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_75_[] = {
  {    -5,  -151,   -23,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*     0 */
  {   -44,  -151,   -30,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*     1 */
  {   -10,  -129,   -24,0, 0x0000,0x0000, 0x71,0x38,0x00,0xff }, /*     2 */
  {   -52,  -109,    39,0, 0x0000,0x0000, 0xaa,0x55,0x00,0xff }, /*     3 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x83,0x41,0x00,0xff }, /*     4 */
  {   -55,  -119,     8,0, 0x0000,0x0000, 0xad,0x56,0x00,0xff }, /*     5 */
  {   -52,  -109,    39,0, 0x0000,0x0000, 0xaa,0x55,0x00,0xff }, /*     6 */
  {   -18,  -109,    45,0, 0x0000,0x0000, 0x91,0x48,0x00,0xff }, /*     7 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x83,0x41,0x00,0xff }, /*     8 */
  {    -5,  -151,    17,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*     9 */
  {   -13,  -151,    46,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    10 */
  {   -59,  -151,     7,0, 0x0000,0x0000, 0x64,0x32,0x00,0xff }, /*    11 */
  {   -59,  -151,     7,0, 0x0000,0x0000, 0x64,0x32,0x00,0xff }, /*    12 */
  {   -44,  -151,   -30,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    13 */
  {    -5,  -151,    17,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*    14 */
  {   -44,  -151,   -30,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    15 */
  {    -5,  -151,   -23,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    16 */
  {    -5,  -151,    17,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*    17 */
  {   -13,  -151,    46,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    18 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x83,0x41,0x00,0xff }, /*    19 */
  {   -18,  -109,    45,0, 0x0000,0x0000, 0x91,0x48,0x00,0xff }, /*    20 */
  {   -59,  -151,     7,0, 0x0000,0x0000, 0x64,0x32,0x00,0xff }, /*    21 */
  {   -65,  -151,    37,0, 0x0000,0x0000, 0x5f,0x2f,0x00,0xff }, /*    22 */
  {   -55,  -119,     8,0, 0x0000,0x0000, 0xad,0x56,0x00,0xff }, /*    23 */
  {   -13,  -151,    46,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    24 */
  {    -5,  -151,    17,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*    25 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x83,0x41,0x00,0xff }, /*    26 */
  {   -18,  -109,    45,0, 0x0000,0x0000, 0x91,0x48,0x00,0xff }, /*    27 */
  {   -39,  -138,    62,0, 0x0000,0x0000, 0x50,0x28,0x00,0xff }, /*    28 */
  {   -13,  -151,    46,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    29 */
  {   -13,  -151,    46,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    30 */
  {   -39,  -138,    62,0, 0x0000,0x0000, 0x50,0x28,0x00,0xff }, /*    31 */
  {   -65,  -151,    37,0, 0x0000,0x0000, 0x5f,0x2f,0x00,0xff }, /*    32 */
  {   -65,  -151,    37,0, 0x0000,0x0000, 0x5f,0x2f,0x00,0xff }, /*    33 */
  {   -39,  -138,    62,0, 0x0000,0x0000, 0x50,0x28,0x00,0xff }, /*    34 */
  {   -52,  -109,    39,0, 0x0000,0x0000, 0xaa,0x55,0x00,0xff }, /*    35 */
  {   -52,  -109,    39,0, 0x0000,0x0000, 0xaa,0x55,0x00,0xff }, /*    36 */
  {   -39,  -138,    62,0, 0x0000,0x0000, 0x50,0x28,0x00,0xff }, /*    37 */
  {   -18,  -109,    45,0, 0x0000,0x0000, 0x91,0x48,0x00,0xff }, /*    38 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x83,0x41,0x00,0xff }, /*    39 */
  {   -10,  -129,   -24,0, 0x0000,0x0000, 0x71,0x38,0x00,0xff }, /*    40 */
  {   -55,  -119,     8,0, 0x0000,0x0000, 0xad,0x56,0x00,0xff }, /*    41 */
  {    -5,  -151,    17,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*    42 */
  {    -5,  -151,   -23,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    43 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x83,0x41,0x00,0xff }, /*    44 */
  {   -55,  -119,     8,0, 0x0000,0x0000, 0xad,0x56,0x00,0xff }, /*    45 */
  {   -41,  -129,   -29,0, 0x0000,0x0000, 0x7d,0x3e,0x00,0xff }, /*    46 */
  {   -59,  -151,     7,0, 0x0000,0x0000, 0x64,0x32,0x00,0xff }, /*    47 */
  {   -41,  -129,   -29,0, 0x0000,0x0000, 0x7d,0x3e,0x00,0xff }, /*    48 */
  {   -44,  -151,   -30,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    49 */
  {   -59,  -151,     7,0, 0x0000,0x0000, 0x64,0x32,0x00,0xff }, /*    50 */
  {    -5,  -151,   -23,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    51 */
  {   -10,  -129,   -24,0, 0x0000,0x0000, 0x71,0x38,0x00,0xff }, /*    52 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x83,0x41,0x00,0xff }, /*    53 */
  {   -10,  -129,   -24,0, 0x0000,0x0000, 0x71,0x38,0x00,0xff }, /*    54 */
  {   -41,  -129,   -29,0, 0x0000,0x0000, 0x7d,0x3e,0x00,0xff }, /*    55 */
  {   -55,  -119,     8,0, 0x0000,0x0000, 0xad,0x56,0x00,0xff }, /*    56 */
  {   -65,  -151,    37,0, 0x0000,0x0000, 0x5f,0x2f,0x00,0xff }, /*    57 */
  {   -52,  -109,    39,0, 0x0000,0x0000, 0xaa,0x55,0x00,0xff }, /*    58 */
  {   -55,  -119,     8,0, 0x0000,0x0000, 0xad,0x56,0x00,0xff }, /*    59 */
  {   -13,  -151,    46,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    60 */
  {   -65,  -151,    37,0, 0x0000,0x0000, 0x5f,0x2f,0x00,0xff }, /*    61 */
  {   -59,  -151,     7,0, 0x0000,0x0000, 0x64,0x32,0x00,0xff }, /*    62 */
  {   -44,  -151,   -30,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    63 */
  {   -41,  -129,   -29,0, 0x0000,0x0000, 0x7d,0x3e,0x00,0xff }, /*    64 */
  {   -10,  -129,   -24,0, 0x0000,0x0000, 0x71,0x38,0x00,0xff }, /*    65 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_75_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP2Triangles(  3,  4,  5,  0,  6,  7,  8,  0),
  gsSP2Triangles(  9, 10, 11,  0, 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP2Triangles( 21, 22, 23,  0, 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP2Triangles( 13, 14, 15,  0, 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 992), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 1024), 2, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_75_ = {
  gfx_Shape_of___POLYHEDRON_75_,
  vtx_Shape_of___POLYHEDRON_75_, 66, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_eat_aligner19[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_67_[] = {
  {   -38,   -53,   -22,0, 0x0000,0x0000, 0xcb,0x92,0x69,0xff }, /*     0 */
  {   -38,   -75,     3,0, 0x0000,0x0000, 0x92,0x69,0x4c,0xff }, /*     1 */
  {   -38,   -53,     3,0, 0x0000,0x0000, 0xff,0xba,0x85,0xff }, /*     2 */
  {   -38,   -53,     3,0, 0x0000,0x0000, 0xff,0xba,0x85,0xff }, /*     3 */
  {   -13,   -75,     3,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*     4 */
  {   -13,   -53,     3,0, 0x0000,0x0000, 0xe4,0xa5,0x77,0xff }, /*     5 */
  {   -38,   -53,     3,0, 0x0000,0x0000, 0xff,0xba,0x85,0xff }, /*     6 */
  {   -38,   -75,     3,0, 0x0000,0x0000, 0x92,0x69,0x4c,0xff }, /*     7 */
  {   -13,   -75,     3,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*     8 */
  {   -38,   -53,   -22,0, 0x0000,0x0000, 0xcb,0x92,0x69,0xff }, /*     9 */
  {   -38,   -75,   -22,0, 0x0000,0x0000, 0xa6,0x78,0x56,0xff }, /*    10 */
  {   -38,   -75,     3,0, 0x0000,0x0000, 0x92,0x69,0x4c,0xff }, /*    11 */
  {   -13,   -53,   -22,0, 0x0000,0x0000, 0x95,0x6b,0x4d,0xff }, /*    12 */
  {   -38,   -75,   -22,0, 0x0000,0x0000, 0xa6,0x78,0x56,0xff }, /*    13 */
  {   -38,   -53,   -22,0, 0x0000,0x0000, 0xcb,0x92,0x69,0xff }, /*    14 */
  {   -13,   -53,   -22,0, 0x0000,0x0000, 0x95,0x6b,0x4d,0xff }, /*    15 */
  {   -13,   -75,   -22,0, 0x0000,0x0000, 0x9a,0x6f,0x50,0xff }, /*    16 */
  {   -38,   -75,   -22,0, 0x0000,0x0000, 0xa6,0x78,0x56,0xff }, /*    17 */
  {   -13,   -53,     3,0, 0x0000,0x0000, 0xe4,0xa5,0x77,0xff }, /*    18 */
  {   -13,   -75,     3,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*    19 */
  {   -13,   -53,   -22,0, 0x0000,0x0000, 0x95,0x6b,0x4d,0xff }, /*    20 */
  {   -13,   -75,     3,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*    21 */
  {   -13,   -75,   -22,0, 0x0000,0x0000, 0x9a,0x6f,0x50,0xff }, /*    22 */
  {   -13,   -53,   -22,0, 0x0000,0x0000, 0x95,0x6b,0x4d,0xff }, /*    23 */
  {   -13,   -75,   -22,0, 0x0000,0x0000, 0x9a,0x6f,0x50,0xff }, /*    24 */
  {   -26,   -86,    -9,0, 0x0000,0x0000, 0x90,0x3e,0x2d,0xff }, /*    25 */
  {   -38,   -75,   -22,0, 0x0000,0x0000, 0xa6,0x78,0x56,0xff }, /*    26 */
  {   -38,   -75,   -22,0, 0x0000,0x0000, 0xa6,0x78,0x56,0xff }, /*    27 */
  {   -26,   -86,    -9,0, 0x0000,0x0000, 0x90,0x3e,0x2d,0xff }, /*    28 */
  {   -38,   -75,     3,0, 0x0000,0x0000, 0x92,0x69,0x4c,0xff }, /*    29 */
  {   -38,   -75,     3,0, 0x0000,0x0000, 0x92,0x69,0x4c,0xff }, /*    30 */
  {   -26,   -86,    -9,0, 0x0000,0x0000, 0x90,0x3e,0x2d,0xff }, /*    31 */
  {   -13,   -75,     3,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*    32 */
  {   -13,   -75,     3,0, 0x0000,0x0000, 0xbf,0x8a,0x63,0xff }, /*    33 */
  {   -26,   -86,    -9,0, 0x0000,0x0000, 0x90,0x3e,0x2d,0xff }, /*    34 */
  {   -13,   -75,   -22,0, 0x0000,0x0000, 0x9a,0x6f,0x50,0xff }, /*    35 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_67_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP2Triangles( 12, 13, 14,  0, 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 4, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_67_ = {
  gfx_Shape_of___POLYHEDRON_67_,
  vtx_Shape_of___POLYHEDRON_67_, 36, 5,
  bmp_frake_eat,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

/* ============================================================
          Geometric Animation Data Area
 ============================================================ */
static s16 Animation_for_body_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch0_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_body_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch1_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -3.3506, -6.7002,
  -10.0475, -13.3916, -16.7313, -16.7313, -16.7313, -16.7313, -16.7313,
  -16.7313, -16.7313, -16.7313, -16.7313, -16.7313, -16.7313, -16.7313,
  -16.7313, -16.7313, -16.7313, -16.7313, -16.7313, -16.7313, -16.7313,
  -16.7313, -16.7313,
};

static s16 Animation_for_body_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch2_val[]={
  -12.3406, -12.3406, -12.3406, -12.3406, -12.3406, -12.3406, -12.3529,
  -12.3897, -12.4511, -12.5367, -12.6466, -12.6466, -12.6466, -12.6466,
  -12.6466, -12.6466, -12.6466, -12.6466, -12.6466, -12.6466, -12.6466,
  -12.6466, -12.6466, -12.6466, -12.6466, -12.6466, -12.6466, -12.6466,
  -12.6466, -12.6466, -12.6466,
};

static s16 Animation_for_body_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch3_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0446, 0.0893, 0.1339,
  0.1785, 0.2232, 0.2232, 0.2232, 0.2232, 0.2232, 0.2232, 0.2232, 0.2232,
  0.2232, 0.2232, 0.2232, 0.2232, 0.2232, 0.2232, 0.2232, 0.2232, 0.2232,
  0.2232, 0.2232, 0.2232, 0.2232,
};

static s16 Animation_for_body_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch4_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_body_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch5_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_body_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_body_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_body_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_body_motion_ch0_key, Animation_for_body_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_body_motion_ch1_key, Animation_for_body_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_body_motion_ch2_key, Animation_for_body_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_body_motion_ch3_key, Animation_for_body_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_body_motion_ch4_key, Animation_for_body_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_body_motion_ch5_key, Animation_for_body_motion_ch5_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_body_motion_ch6_key, Animation_for_body_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_body_motion_ch7_key, Animation_for_body_motion_ch7_val},
};

static NDR_ANIM Animation_for_body_motion={
  Animation_for_body_motion_chlist, 8, 30
};

static s16 Animation_for_R_uparm_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch0_val[]={
  0.0000, 2.2361, 4.9015, 8.0163, 11.5964, 15.6450, -2.6754, -23.1622,
  -44.3524, -64.7791, -83.0718, -83.0718, -83.0718, -83.0718, -83.0718,
  -83.0718, -83.0718, -83.0718, -83.0718, -83.0718, -83.0718, -83.0718,
  -83.0718, -83.0718, -83.0718, -83.0718, -83.0718, -83.0718, -83.0718,
  -83.0718, -83.0718,
};

static s16 Animation_for_R_uparm_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch1_val[]={
  0.0000, 3.3591, 9.0939, 16.8876, 26.3255, 36.9144, 25.7461, 19.9013,
  19.6959, 25.0555, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366,
  35.5366, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366,
  35.5366, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366,
};

static s16 Animation_for_R_uparm_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch2_val[]={
  0.0000, 15.0681, 28.9813, 41.2701, 51.5388, 59.4860, 60.7246, 62.2115,
  63.8357, 65.4606, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375,
  66.9375, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375,
  66.9375, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375,
};

static s16 Animation_for_R_uparm_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch3_val[]={
  0.0000, -0.1852, -0.3719, -0.5629, -0.7603, -0.9654, -0.9332, -0.9034,
  -0.8750, -0.8470, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184,
  -0.8184, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184,
  -0.8184, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184,
};

static s16 Animation_for_R_uparm_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch4_val[]={
  0.0000, 0.0781, 0.1570, 0.2320, 0.2988, 0.3527, 0.3101, 0.2614, 0.2065,
  0.1459, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799,
  0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799,
  0.0799, 0.0799, 0.0799, 0.0799,
};

static s16 Animation_for_R_uparm_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch5_val[]={
  0.0000, 0.0030, -0.0165, -0.0587, -0.1235, -0.2100, -0.4154, -0.6203,
  -0.8240, -1.0262, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267,
  -1.2267, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267,
  -1.2267, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267,
};

static s16 Animation_for_R_uparm_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_uparm_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_uparm_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_uparm_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_uparm_motion_ch0_key, Animation_for_R_uparm_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_uparm_motion_ch1_key, Animation_for_R_uparm_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_uparm_motion_ch2_key, Animation_for_R_uparm_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_uparm_motion_ch3_key, Animation_for_R_uparm_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_uparm_motion_ch4_key, Animation_for_R_uparm_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_uparm_motion_ch5_key, Animation_for_R_uparm_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_uparm_motion_ch6_key, Animation_for_R_uparm_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_uparm_motion_ch7_key, Animation_for_R_uparm_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_uparm_motion_ch8_key, Animation_for_R_uparm_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_uparm_motion={
  Animation_for_R_uparm_motion_chlist, 9, 30
};

static s16 Animation_for_R_lowarm_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch0_val[]={
  -0.0000, 0.4373, 0.7861, 1.0433, 1.2062, 1.2734, 1.0945, 0.4857, -0.5264,
  -1.8974, -3.5669, -3.5669, -3.5669, -3.5669, -3.5669, -3.5669, -3.5669,
  -3.5669, -3.5669, -3.5669, -3.5669, -3.5669, -3.5669, -3.5669, -3.5669,
  -3.5669, -3.5669, -3.5669, -3.5669, -3.5669, -3.5669,
};

static s16 Animation_for_R_lowarm_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch1_val[]={
  0.0000, 0.6005, 1.0799, 1.4333, 1.6571, 1.7494, 1.5038, 0.6672, -0.7234,
  -2.6067, -4.8999, -4.8999, -4.8999, -4.8999, -4.8999, -4.8999, -4.8999,
  -4.8999, -4.8999, -4.8999, -4.8999, -4.8999, -4.8999, -4.8999, -4.8999,
  -4.8999, -4.8999, -4.8999, -4.8999, -4.8999, -4.8999,
};

static s16 Animation_for_R_lowarm_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch2_val[]={
  0.0000, -1.4898, -3.0453, -4.6517, -6.2933, -7.9541, -11.4886, -14.8820,
  -17.9848, -20.6606, -22.7909, -22.7909, -22.7909, -22.7909, -22.7909,
  -22.7909, -22.7909, -22.7909, -22.7909, -22.7909, -22.7909, -22.7909,
  -22.7909, -22.7909, -22.7909, -22.7909, -22.7909, -22.7909, -22.7909,
  -22.7909, -22.7909,
};

static s16 Animation_for_R_lowarm_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch3_val[]={
  0.0000, -0.0797, -0.1600, -0.2413, -0.3243, -0.4093, -0.6004, -0.8080,
  -1.0349, -1.2799, -1.5371, -1.5371, -1.5371, -1.5371, -1.5371, -1.5371,
  -1.5371, -1.5371, -1.5371, -1.5371, -1.5371, -1.5371, -1.5371, -1.5371,
  -1.5371, -1.5371, -1.5371, -1.5371, -1.5371, -1.5371, -1.5371,
};

static s16 Animation_for_R_lowarm_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch4_val[]={
  0.0000, 0.0579, 0.1154, 0.1722, 0.2279, 0.2820, 0.3902, 0.4844, 0.5592,
  0.6089, 0.6289, 0.6289, 0.6289, 0.6289, 0.6289, 0.6289, 0.6289, 0.6289,
  0.6289, 0.6289, 0.6289, 0.6289, 0.6289, 0.6289, 0.6289, 0.6289, 0.6289,
  0.6289, 0.6289, 0.6289, 0.6289,
};

static s16 Animation_for_R_lowarm_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch5_val[]={
  -0.0000, -0.0023, -0.0093, -0.0209, -0.0374, -0.0589, -0.1222, -0.2105,
  -0.3240, -0.4596, -0.6094, -0.6094, -0.6094, -0.6094, -0.6094, -0.6094,
  -0.6094, -0.6094, -0.6094, -0.6094, -0.6094, -0.6094, -0.6094, -0.6094,
  -0.6094, -0.6094, -0.6094, -0.6094, -0.6094, -0.6094, -0.6094,
};

static s16 Animation_for_R_lowarm_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_lowarm_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_lowarm_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_lowarm_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch0_key, Animation_for_R_lowarm_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch1_key, Animation_for_R_lowarm_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch2_key, Animation_for_R_lowarm_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch3_key, Animation_for_R_lowarm_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch4_key, Animation_for_R_lowarm_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch5_key, Animation_for_R_lowarm_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch6_key, Animation_for_R_lowarm_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch7_key, Animation_for_R_lowarm_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch8_key, Animation_for_R_lowarm_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_lowarm_motion={
  Animation_for_R_lowarm_motion_chlist, 9, 30
};

static s16 Animation_for_R_hand_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch0_val[]={
  -0.0000, 0.1666, -1.6982, -5.4873, -10.9820, -17.8665, -16.5111, -15.1922,
  -13.9122, -12.6737, -11.4789, -11.4789, -11.4789, -11.4789, -11.4789,
  -11.4789, -11.4789, -11.4789, -11.4789, -11.4789, -11.4789, -11.4789,
  -11.4789, -11.4789, -11.4789, -11.4789, -11.4789, -11.4789, -11.4789,
  -11.4789, -11.4789,
};

static s16 Animation_for_R_hand_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch1_val[]={
  0.0000, 0.2133, -2.2936, -7.3762, -14.7423, -23.9692, -22.1460, -20.3715,
  -18.6495, -16.9826, -15.3745, -15.3745, -15.3745, -15.3745, -15.3745,
  -15.3745, -15.3745, -15.3745, -15.3745, -15.3745, -15.3745, -15.3745,
  -15.3745, -15.3745, -15.3745, -15.3745, -15.3745, -15.3745, -15.3745,
  -15.3745, -15.3745,
};

static s16 Animation_for_R_hand_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch2_val[]={
  0.0000, -14.2457, -28.1451, -40.9007, -51.7812, -60.1627, -58.8343,
  -57.4065, -55.8821, -54.2640, -52.5556, -52.5556, -52.5556, -52.5556,
  -52.5556, -52.5556, -52.5556, -52.5556, -52.5556, -52.5556, -52.5556,
  -52.5556, -52.5556, -52.5556, -52.5556, -52.5556, -52.5556, -52.5556,
  -52.5556, -52.5556, -52.5556,
};

static s16 Animation_for_R_hand_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch3_val[]={
  0.0000, -0.1924, -0.3928, -0.6090, -0.8479, -1.1131, -1.0632, -1.0141,
  -0.9659, -0.9186, -0.8722, -0.8722, -0.8722, -0.8722, -0.8722, -0.8722,
  -0.8722, -0.8722, -0.8722, -0.8722, -0.8722, -0.8722, -0.8722, -0.8722,
  -0.8722, -0.8722, -0.8722, -0.8722, -0.8722, -0.8722, -0.8722,
};

static s16 Animation_for_R_hand_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch4_val[]={
  0.0000, 0.1428, 0.2800, 0.4057, 0.5124, 0.5917, 0.5796, 0.5663, 0.5518,
  0.5363, 0.5198, 0.5198, 0.5198, 0.5198, 0.5198, 0.5198, 0.5198, 0.5198,
  0.5198, 0.5198, 0.5198, 0.5198, 0.5198, 0.5198, 0.5198, 0.5198, 0.5198,
  0.5198, 0.5198, 0.5198, 0.5198,
};

static s16 Animation_for_R_hand_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch5_val[]={
  0.0000, -0.0139, -0.0564, -0.1299, -0.2369, -0.3773, -0.3491, -0.3219,
  -0.2958, -0.2708, -0.2468, -0.2468, -0.2468, -0.2468, -0.2468, -0.2468,
  -0.2468, -0.2468, -0.2468, -0.2468, -0.2468, -0.2468, -0.2468, -0.2468,
  -0.2468, -0.2468, -0.2468, -0.2468, -0.2468, -0.2468, -0.2468,
};

static s16 Animation_for_R_hand_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_hand_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_hand_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_hand_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_hand_motion_ch0_key, Animation_for_R_hand_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_hand_motion_ch1_key, Animation_for_R_hand_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_hand_motion_ch2_key, Animation_for_R_hand_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_hand_motion_ch3_key, Animation_for_R_hand_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_hand_motion_ch4_key, Animation_for_R_hand_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_hand_motion_ch5_key, Animation_for_R_hand_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_hand_motion_ch6_key, Animation_for_R_hand_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_hand_motion_ch7_key, Animation_for_R_hand_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_hand_motion_ch8_key, Animation_for_R_hand_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_hand_motion={
  Animation_for_R_hand_motion_chlist, 9, 30
};

static s16 Animation_for_R_sode_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch0_val[]={
  0.0000, 2.2361, 4.9015, 8.0163, 11.5964, 15.6450, -2.6754, -23.1622,
  -44.3524, -64.7791, -83.0718, -83.0718, -83.0718, -83.0718, -83.0718,
  -83.0718, -83.0718, -83.0718, -83.0718, -83.0718, -83.0718, -83.0718,
  -83.0718, -83.0718, -83.0718, -83.0718, -83.0718, -83.0718, -83.0718,
  -83.0718, -83.0718,
};

static s16 Animation_for_R_sode_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch1_val[]={
  0.0000, 3.3591, 9.0939, 16.8876, 26.3255, 36.9144, 25.7461, 19.9013,
  19.6959, 25.0555, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366,
  35.5366, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366,
  35.5366, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366, 35.5366,
};

static s16 Animation_for_R_sode_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch2_val[]={
  0.0000, 15.0681, 28.9813, 41.2701, 51.5388, 59.4860, 60.7246, 62.2115,
  63.8357, 65.4606, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375,
  66.9375, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375,
  66.9375, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375, 66.9375,
};

static s16 Animation_for_R_sode_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch3_val[]={
  0.0000, -0.1852, -0.3719, -0.5629, -0.7603, -0.9654, -0.9332, -0.9034,
  -0.8750, -0.8470, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184,
  -0.8184, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184,
  -0.8184, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184, -0.8184,
};

static s16 Animation_for_R_sode_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch4_val[]={
  0.0000, 0.0781, 0.1570, 0.2320, 0.2988, 0.3527, 0.3101, 0.2614, 0.2065,
  0.1459, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799,
  0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799, 0.0799,
  0.0799, 0.0799, 0.0799, 0.0799,
};

static s16 Animation_for_R_sode_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch5_val[]={
  0.0000, 0.0030, -0.0165, -0.0587, -0.1235, -0.2100, -0.4154, -0.6203,
  -0.8240, -1.0262, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267,
  -1.2267, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267,
  -1.2267, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267, -1.2267,
};

static s16 Animation_for_R_sode_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_sode_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_sode_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_sode_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_sode_motion_ch0_key, Animation_for_R_sode_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_sode_motion_ch1_key, Animation_for_R_sode_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_sode_motion_ch2_key, Animation_for_R_sode_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_sode_motion_ch3_key, Animation_for_R_sode_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_sode_motion_ch4_key, Animation_for_R_sode_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_sode_motion_ch5_key, Animation_for_R_sode_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_sode_motion_ch6_key, Animation_for_R_sode_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_sode_motion_ch7_key, Animation_for_R_sode_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_sode_motion_ch8_key, Animation_for_R_sode_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_sode_motion={
  Animation_for_R_sode_motion_chlist, 9, 30
};

static s16 Animation_for_L_uparm_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch0_val[]={
  0.0000, -0.1076, -1.1041, -3.1604, -6.3771, -10.7634, 8.1993, 28.6794,
  49.2263, 68.4395, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626,
  85.0626, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626,
  85.0626, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626,
};

static s16 Animation_for_L_uparm_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch1_val[]={
  0.0000, 2.7315, 8.1355, 15.7294, 24.8962, 34.9203, 25.7357, 21.9791,
  23.7712, 30.8569, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406,
  42.6406, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406,
  42.6406, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406,
};

static s16 Animation_for_L_uparm_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch2_val[]={
  0.0000, 16.1897, 30.9607, 43.7209, 53.9856, 61.4058, 63.4271, 65.5641,
  67.6965, 69.6866, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920,
  71.3920, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920,
  71.3920, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920,
};

static s16 Animation_for_L_uparm_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch3_val[]={
  0.0000, -0.2010, -0.4042, -0.6133, -0.8310, -1.0584, -1.0209, -0.9857,
  -0.9517, -0.9182, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846,
  -0.8846, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846,
  -0.8846, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846,
};

static s16 Animation_for_L_uparm_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch4_val[]={
  0.0000, -0.0876, -0.1754, -0.2578, -0.3288, -0.3826, -0.3312, -0.2738,
  -0.2106, -0.1420, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687,
  -0.0687, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687,
  -0.0687, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687,
};

static s16 Animation_for_L_uparm_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch5_val[]={
  0.0000, 0.0227, 0.0717, 0.1475, 0.2499, 0.3772, 0.5725, 0.7669, 0.9598,
  1.1511, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407,
  1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407,
  1.3407, 1.3407, 1.3407, 1.3407,
};

static s16 Animation_for_L_uparm_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_uparm_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_uparm_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_uparm_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_uparm_motion_ch0_key, Animation_for_L_uparm_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_uparm_motion_ch1_key, Animation_for_L_uparm_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_uparm_motion_ch2_key, Animation_for_L_uparm_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_uparm_motion_ch3_key, Animation_for_L_uparm_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_uparm_motion_ch4_key, Animation_for_L_uparm_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_uparm_motion_ch5_key, Animation_for_L_uparm_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_uparm_motion_ch6_key, Animation_for_L_uparm_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_uparm_motion_ch7_key, Animation_for_L_uparm_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_uparm_motion_ch8_key, Animation_for_L_uparm_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_uparm_motion={
  Animation_for_L_uparm_motion_chlist, 9, 30
};

static s16 Animation_for_L_lowarm_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch0_val[]={
  0.0000, -0.1941, -0.3728, -0.5363, -0.6840, -0.8157, -1.2700, -0.9888,
  0.0070, 1.6443, 3.8014, 3.8014, 3.8014, 3.8014, 3.8014, 3.8014, 3.8014,
  3.8014, 3.8014, 3.8014, 3.8014, 3.8014, 3.8014, 3.8014, 3.8014, 3.8014,
  3.8014, 3.8014, 3.8014, 3.8014, 3.8014,
};

static s16 Animation_for_L_lowarm_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch1_val[]={
  0.0000, 0.2667, 0.5123, 0.7369, 0.9398, 1.1207, 1.7449, 1.3584, -0.0097,
  -2.2588, -5.2224, -5.2224, -5.2224, -5.2224, -5.2224, -5.2224, -5.2224,
  -5.2224, -5.2224, -5.2224, -5.2224, -5.2224, -5.2224, -5.2224, -5.2224,
  -5.2224, -5.2224, -5.2224, -5.2224, -5.2224, -5.2224,
};

static s16 Animation_for_L_lowarm_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch2_val[]={
  0.0000, -0.6164, -1.2459, -1.8874, -2.5400, -3.2022, -7.7354, -12.3088,
  -16.5842, -20.2457, -23.0221, -23.0221, -23.0221, -23.0221, -23.0221,
  -23.0221, -23.0221, -23.0221, -23.0221, -23.0221, -23.0221, -23.0221,
  -23.0221, -23.0221, -23.0221, -23.0221, -23.0221, -23.0221, -23.0221,
  -23.0221, -23.0221,
};

static s16 Animation_for_L_lowarm_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch3_val[]={
  0.0000, -0.0334, -0.0669, -0.1005, -0.1341, -0.1680, -0.3980, -0.6478,
  -0.9267, -1.2376, -1.5708, -1.5708, -1.5708, -1.5708, -1.5708, -1.5708,
  -1.5708, -1.5708, -1.5708, -1.5708, -1.5708, -1.5708, -1.5708, -1.5708,
  -1.5708, -1.5708, -1.5708, -1.5708, -1.5708, -1.5708, -1.5708,
};

static s16 Animation_for_L_lowarm_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch4_val[]={
  0.0000, -0.0243, -0.0486, -0.0729, -0.0970, -0.1211, -0.2750, -0.4139,
  -0.5271, -0.6025, -0.6292, -0.6292, -0.6292, -0.6292, -0.6292, -0.6292,
  -0.6292, -0.6292, -0.6292, -0.6292, -0.6292, -0.6292, -0.6292, -0.6292,
  -0.6292, -0.6292, -0.6292, -0.6292, -0.6292, -0.6292, -0.6292,
};

static s16 Animation_for_L_lowarm_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch5_val[]={
  -0.0000, 0.0004, 0.0016, 0.0037, 0.0065, 0.0102, 0.0558, 0.1407, 0.2680,
  0.4355, 0.6292, 0.6292, 0.6292, 0.6292, 0.6292, 0.6292, 0.6292, 0.6292,
  0.6292, 0.6292, 0.6292, 0.6292, 0.6292, 0.6292, 0.6292, 0.6292, 0.6292,
  0.6292, 0.6292, 0.6292, 0.6292,
};

static s16 Animation_for_L_lowarm_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_lowarm_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_lowarm_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_lowarm_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch0_key, Animation_for_L_lowarm_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch1_key, Animation_for_L_lowarm_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch2_key, Animation_for_L_lowarm_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch3_key, Animation_for_L_lowarm_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch4_key, Animation_for_L_lowarm_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch5_key, Animation_for_L_lowarm_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch6_key, Animation_for_L_lowarm_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch7_key, Animation_for_L_lowarm_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch8_key, Animation_for_L_lowarm_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_lowarm_motion={
  Animation_for_L_lowarm_motion_chlist, 9, 30
};

static s16 Animation_for_L_hand_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch0_val[]={
  0.0000, -0.3391, -0.1687, 0.5088, 1.6840, 3.3394, 4.6957, 6.2463,
  7.9815, 9.8902, 11.9606, 11.9606, 11.9606, 11.9606, 11.9606, 11.9606,
  11.9606, 11.9606, 11.9606, 11.9606, 11.9606, 11.9606, 11.9606, 11.9606,
  11.9606, 11.9606, 11.9606, 11.9606, 11.9606, 11.9606, 11.9606,
};

static s16 Animation_for_L_hand_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch1_val[]={
  -0.0000, 0.4469, 0.2113, -0.7032, -2.2837, -4.5071, -6.3130, -8.3788,
  -10.6915, -13.2368, -15.9981, -15.9981, -15.9981, -15.9981, -15.9981,
  -15.9981, -15.9981, -15.9981, -15.9981, -15.9981, -15.9981, -15.9981,
  -15.9981, -15.9981, -15.9981, -15.9981, -15.9981, -15.9981, -15.9981,
  -15.9981, -15.9981,
};

static s16 Animation_for_L_hand_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch2_val[]={
  0.0000, -7.0970, -14.2094, -21.2355, -28.0745, -34.6285, -38.7913,
  -42.7620, -46.5150, -50.0264, -53.2743, -53.2743, -53.2743, -53.2743,
  -53.2743, -53.2743, -53.2743, -53.2743, -53.2743, -53.2743, -53.2743,
  -53.2743, -53.2743, -53.2743, -53.2743, -53.2743, -53.2743, -53.2743,
  -53.2743, -53.2743, -53.2743,
};

static s16 Animation_for_L_hand_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch3_val[]={
  -0.0000, -0.0952, -0.1914, -0.2895, -0.3906, -0.4956, -0.5701, -0.6470,
  -0.7265, -0.8089, -0.8941, -0.8941, -0.8941, -0.8941, -0.8941, -0.8941,
  -0.8941, -0.8941, -0.8941, -0.8941, -0.8941, -0.8941, -0.8941, -0.8941,
  -0.8941, -0.8941, -0.8941, -0.8941, -0.8941, -0.8941, -0.8941,
};

static s16 Animation_for_L_hand_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch4_val[]={
  -0.0000, -0.0715, -0.1424, -0.2119, -0.2793, -0.3439, -0.3849, -0.4240,
  -0.4609, -0.4952, -0.5267, -0.5267, -0.5267, -0.5267, -0.5267, -0.5267,
  -0.5267, -0.5267, -0.5267, -0.5267, -0.5267, -0.5267, -0.5267, -0.5267,
  -0.5267, -0.5267, -0.5267, -0.5267, -0.5267, -0.5267, -0.5267,
};

static s16 Animation_for_L_hand_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch5_val[]={
  0.0000, 0.0034, 0.0138, 0.0313, 0.0561, 0.0886, 0.1147, 0.1445, 0.1781,
  0.2154, 0.2565, 0.2565, 0.2565, 0.2565, 0.2565, 0.2565, 0.2565, 0.2565,
  0.2565, 0.2565, 0.2565, 0.2565, 0.2565, 0.2565, 0.2565, 0.2565, 0.2565,
  0.2565, 0.2565, 0.2565, 0.2565,
};

static s16 Animation_for_L_hand_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_hand_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_hand_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_hand_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_hand_motion_ch0_key, Animation_for_L_hand_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_hand_motion_ch1_key, Animation_for_L_hand_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_hand_motion_ch2_key, Animation_for_L_hand_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_hand_motion_ch3_key, Animation_for_L_hand_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_hand_motion_ch4_key, Animation_for_L_hand_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_hand_motion_ch5_key, Animation_for_L_hand_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_hand_motion_ch6_key, Animation_for_L_hand_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_hand_motion_ch7_key, Animation_for_L_hand_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_hand_motion_ch8_key, Animation_for_L_hand_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_hand_motion={
  Animation_for_L_hand_motion_chlist, 9, 30
};

static s16 Animation_for_L_sode_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch0_val[]={
  0.0000, -0.1076, -1.1041, -3.1604, -6.3771, -10.7634, 8.1993, 28.6794,
  49.2263, 68.4395, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626,
  85.0626, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626,
  85.0626, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626, 85.0626,
};

static s16 Animation_for_L_sode_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch1_val[]={
  0.0000, 2.7315, 8.1355, 15.7294, 24.8962, 34.9203, 25.7357, 21.9791,
  23.7712, 30.8569, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406,
  42.6406, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406,
  42.6406, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406, 42.6406,
};

static s16 Animation_for_L_sode_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch2_val[]={
  0.0000, 16.1897, 30.9607, 43.7209, 53.9856, 61.4058, 63.4271, 65.5641,
  67.6965, 69.6866, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920,
  71.3920, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920,
  71.3920, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920, 71.3920,
};

static s16 Animation_for_L_sode_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch3_val[]={
  0.0000, -0.2010, -0.4042, -0.6133, -0.8310, -1.0584, -1.0209, -0.9857,
  -0.9517, -0.9182, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846,
  -0.8846, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846,
  -0.8846, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846, -0.8846,
};

static s16 Animation_for_L_sode_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch4_val[]={
  0.0000, -0.0876, -0.1754, -0.2578, -0.3288, -0.3826, -0.3312, -0.2738,
  -0.2106, -0.1420, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687,
  -0.0687, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687,
  -0.0687, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687, -0.0687,
};

static s16 Animation_for_L_sode_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch5_val[]={
  0.0000, 0.0227, 0.0717, 0.1475, 0.2499, 0.3772, 0.5725, 0.7669, 0.9598,
  1.1511, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407,
  1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407, 1.3407,
  1.3407, 1.3407, 1.3407, 1.3407,
};

static s16 Animation_for_L_sode_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_sode_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_sode_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_sode_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_sode_motion_ch0_key, Animation_for_L_sode_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_sode_motion_ch1_key, Animation_for_L_sode_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_sode_motion_ch2_key, Animation_for_L_sode_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_sode_motion_ch3_key, Animation_for_L_sode_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_sode_motion_ch4_key, Animation_for_L_sode_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_sode_motion_ch5_key, Animation_for_L_sode_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_sode_motion_ch6_key, Animation_for_L_sode_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_sode_motion_ch7_key, Animation_for_L_sode_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_sode_motion_ch8_key, Animation_for_L_sode_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_sode_motion={
  Animation_for_L_sode_motion_chlist, 9, 30
};

static s16 Animation_for_head_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch0_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  -0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_head_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch1_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_head_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch2_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0000, -0.0000,
  0.0000, 0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_head_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch3_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  -0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_head_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch4_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_head_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch5_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_head_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_head_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_head_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_head_motion_ch0_key, Animation_for_head_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_head_motion_ch1_key, Animation_for_head_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_head_motion_ch2_key, Animation_for_head_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_head_motion_ch3_key, Animation_for_head_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_head_motion_ch4_key, Animation_for_head_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_head_motion_ch5_key, Animation_for_head_motion_ch5_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_head_motion_ch6_key, Animation_for_head_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_head_motion_ch7_key, Animation_for_head_motion_ch7_val},
};

static NDR_ANIM Animation_for_head_motion={
  Animation_for_head_motion_chlist, 8, 30
};

static s16 Animation_for_L_upleg_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch0_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0089, -0.0324,
  -0.0695, -0.1187, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779,
  -0.1779, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779,
  -0.1779, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779,
};

static s16 Animation_for_L_upleg_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch1_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -2.5701, -5.1470,
  -7.7360, -10.3426, -12.9717, -12.9717, -12.9717, -12.9717, -12.9717,
  -12.9717, -12.9717, -12.9717, -12.9717, -12.9717, -12.9717, -12.9717,
  -12.9717, -12.9717, -12.9717, -12.9717, -12.9717, -12.9717, -12.9717,
  -12.9717, -12.9717,
};

static s16 Animation_for_L_upleg_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch2_val[]={
  -12.3406, -12.3406, -12.3406, -12.3406, -12.3406, -12.3406, -12.3764,
  -12.4676, -12.6133, -12.8126, -13.0639, -13.0639, -13.0639, -13.0639,
  -13.0639, -13.0639, -13.0639, -13.0639, -13.0639, -13.0639, -13.0639,
  -13.0639, -13.0639, -13.0639, -13.0639, -13.0639, -13.0639, -13.0639,
  -13.0639, -13.0639, -13.0639,
};

static s16 Animation_for_L_upleg_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch3_val[]={
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0736, -0.1471,
  -0.2206, -0.2940, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673,
  -0.3673, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673,
  -0.3673, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673,
};

static s16 Animation_for_L_upleg_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch4_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0008, -0.0028,
  -0.0057, -0.0097, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147,
  -0.0147, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147,
  -0.0147, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147,
};

static s16 Animation_for_L_upleg_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch5_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0219, 0.0439, 0.0663,
  0.0889, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121,
  0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121,
  0.1121, 0.1121, 0.1121, 0.1121,
};

static s16 Animation_for_L_upleg_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_upleg_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_upleg_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_upleg_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_upleg_motion_ch0_key, Animation_for_L_upleg_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_upleg_motion_ch1_key, Animation_for_L_upleg_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_upleg_motion_ch2_key, Animation_for_L_upleg_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_upleg_motion_ch3_key, Animation_for_L_upleg_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_upleg_motion_ch4_key, Animation_for_L_upleg_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_upleg_motion_ch5_key, Animation_for_L_upleg_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_upleg_motion_ch6_key, Animation_for_L_upleg_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_upleg_motion_ch7_key, Animation_for_L_upleg_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_upleg_motion_ch8_key, Animation_for_L_upleg_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_upleg_motion={
  Animation_for_L_upleg_motion_chlist, 9, 30
};

static s16 Animation_for_L_lowleg_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch0_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_L_lowleg_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch1_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -1.9637, -6.1591,
  -12.4585, -20.6703, -30.5445, -30.5445, -30.5445, -30.5445, -30.5445,
  -30.5445, -30.5445, -30.5445, -30.5445, -30.5445, -30.5445, -30.5445,
  -30.5445, -30.5445, -30.5445, -30.5445, -30.5445, -30.5445, -30.5445,
  -30.5445, -30.5445,
};

static s16 Animation_for_L_lowleg_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch2_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 13.0139, 25.4885,
  37.0443, 47.3297, 56.0314, 56.0314, 56.0314, 56.0314, 56.0314, 56.0314,
  56.0314, 56.0314, 56.0314, 56.0314, 56.0314, 56.0314, 56.0314, 56.0314,
  56.0314, 56.0314, 56.0314, 56.0314, 56.0314, 56.0314, 56.0314,
};

static s16 Animation_for_L_lowleg_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch3_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.1747, 0.3493, 0.5240,
  0.6987, 0.8733, 0.8733, 0.8733, 0.8733, 0.8733, 0.8733, 0.8733, 0.8733,
  0.8733, 0.8733, 0.8733, 0.8733, 0.8733, 0.8733, 0.8733, 0.8733, 0.8733,
  0.8733, 0.8733, 0.8733, 0.8733,
};

static s16 Animation_for_L_lowleg_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch4_val[]={
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_lowleg_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch5_val[]={
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_lowleg_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_lowleg_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_lowleg_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_lowleg_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch0_key, Animation_for_L_lowleg_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch1_key, Animation_for_L_lowleg_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch2_key, Animation_for_L_lowleg_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch3_key, Animation_for_L_lowleg_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch4_key, Animation_for_L_lowleg_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch5_key, Animation_for_L_lowleg_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch6_key, Animation_for_L_lowleg_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch7_key, Animation_for_L_lowleg_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch8_key, Animation_for_L_lowleg_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_lowleg_motion={
  Animation_for_L_lowleg_motion_chlist, 9, 30
};

static s16 Animation_for_L_shoe_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch0_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -2.1427, -4.2678,
  -6.3573, -8.3941, -10.3611, -10.3611, -10.3611, -10.3611, -10.3611,
  -10.3611, -10.3611, -10.3611, -10.3611, -10.3611, -10.3611, -10.3611,
  -10.3611, -10.3611, -10.3611, -10.3611, -10.3611, -10.3611, -10.3611,
  -10.3611, -10.3611,
};

static s16 Animation_for_L_shoe_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch1_val[]={
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.5537, -2.2302,
  -5.0160, -8.8873, -13.8125, -13.8125, -13.8125, -13.8125, -13.8125,
  -13.8125, -13.8125, -13.8125, -13.8125, -13.8125, -13.8125, -13.8125,
  -13.8125, -13.8125, -13.8125, -13.8125, -13.8125, -13.8125, -13.8125,
  -13.8125, -13.8125,
};

static s16 Animation_for_L_shoe_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch2_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -12.1522, -24.2039,
  -36.0550, -47.6067, -58.7619, -58.7619, -58.7619, -58.7619, -58.7619,
  -58.7619, -58.7619, -58.7619, -58.7619, -58.7619, -58.7619, -58.7619,
  -58.7619, -58.7619, -58.7619, -58.7619, -58.7619, -58.7619, -58.7619,
  -58.7619, -58.7619,
};

static s16 Animation_for_L_shoe_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch3_val[]={
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0899, -0.1798,
  -0.2697, -0.3595, -0.4493, -0.4493, -0.4493, -0.4493, -0.4493, -0.4493,
  -0.4493, -0.4493, -0.4493, -0.4493, -0.4493, -0.4493, -0.4493, -0.4493,
  -0.4493, -0.4493, -0.4493, -0.4493, -0.4493, -0.4493, -0.4493,
};

static s16 Animation_for_L_shoe_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch4_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0007, 0.0028, 0.0064,
  0.0113, 0.0175, 0.0175, 0.0175, 0.0175, 0.0175, 0.0175, 0.0175, 0.0175,
  0.0175, 0.0175, 0.0175, 0.0175, 0.0175, 0.0175, 0.0175, 0.0175, 0.0175,
  0.0175, 0.0175, 0.0175, 0.0175,
};

static s16 Animation_for_L_shoe_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch5_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0158, 0.0315, 0.0470,
  0.0621, 0.0766, 0.0766, 0.0766, 0.0766, 0.0766, 0.0766, 0.0766, 0.0766,
  0.0766, 0.0766, 0.0766, 0.0766, 0.0766, 0.0766, 0.0766, 0.0766, 0.0766,
  0.0766, 0.0766, 0.0766, 0.0766,
};

static s16 Animation_for_L_shoe_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_shoe_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_shoe_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_shoe_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_shoe_motion_ch0_key, Animation_for_L_shoe_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_shoe_motion_ch1_key, Animation_for_L_shoe_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_shoe_motion_ch2_key, Animation_for_L_shoe_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_shoe_motion_ch3_key, Animation_for_L_shoe_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_shoe_motion_ch4_key, Animation_for_L_shoe_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_shoe_motion_ch5_key, Animation_for_L_shoe_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_shoe_motion_ch6_key, Animation_for_L_shoe_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_shoe_motion_ch7_key, Animation_for_L_shoe_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_shoe_motion_ch8_key, Animation_for_L_shoe_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_shoe_motion={
  Animation_for_L_shoe_motion_chlist, 9, 30
};

static s16 Animation_for_L_leg_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch0_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0089, -0.0324,
  -0.0695, -0.1187, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779,
  -0.1779, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779,
  -0.1779, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779, -0.1779,
};

static s16 Animation_for_L_leg_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch1_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -2.5701, -5.1470,
  -7.7360, -10.3426, -12.9717, -12.9717, -12.9717, -12.9717, -12.9717,
  -12.9717, -12.9717, -12.9717, -12.9717, -12.9717, -12.9717, -12.9717,
  -12.9717, -12.9717, -12.9717, -12.9717, -12.9717, -12.9717, -12.9717,
  -12.9717, -12.9717,
};

static s16 Animation_for_L_leg_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch2_val[]={
  -12.3406, -12.3406, -12.3406, -12.3406, -12.3406, -12.3406, -12.3764,
  -12.4676, -12.6133, -12.8126, -13.0639, -13.0639, -13.0639, -13.0639,
  -13.0639, -13.0639, -13.0639, -13.0639, -13.0639, -13.0639, -13.0639,
  -13.0639, -13.0639, -13.0639, -13.0639, -13.0639, -13.0639, -13.0639,
  -13.0639, -13.0639, -13.0639,
};

static s16 Animation_for_L_leg_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch3_val[]={
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0736, -0.1471,
  -0.2206, -0.2940, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673,
  -0.3673, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673,
  -0.3673, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673, -0.3673,
};

static s16 Animation_for_L_leg_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch4_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0008, -0.0028,
  -0.0057, -0.0097, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147,
  -0.0147, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147,
  -0.0147, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147, -0.0147,
};

static s16 Animation_for_L_leg_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch5_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0219, 0.0439, 0.0663,
  0.0889, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121,
  0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121, 0.1121,
  0.1121, 0.1121, 0.1121, 0.1121,
};

static s16 Animation_for_L_leg_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_leg_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_leg_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_leg_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_leg_motion_ch0_key, Animation_for_L_leg_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_leg_motion_ch1_key, Animation_for_L_leg_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_leg_motion_ch2_key, Animation_for_L_leg_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_leg_motion_ch3_key, Animation_for_L_leg_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_leg_motion_ch4_key, Animation_for_L_leg_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_leg_motion_ch5_key, Animation_for_L_leg_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_leg_motion_ch6_key, Animation_for_L_leg_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_leg_motion_ch7_key, Animation_for_L_leg_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_leg_motion_ch8_key, Animation_for_L_leg_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_leg_motion={
  Animation_for_L_leg_motion_chlist, 9, 30
};

static s16 Animation_for_hip_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch0_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0002, 0.0006, 0.0014,
  0.0025, 0.0039, 0.0039, 0.0039, 0.0039, 0.0039, 0.0039, 0.0039, 0.0039,
  0.0039, 0.0039, 0.0039, 0.0039, 0.0039, 0.0039, 0.0039, 0.0039, 0.0039,
  0.0039, 0.0039, 0.0039, 0.0039,
};

static s16 Animation_for_hip_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch1_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -2.7662, -5.5325,
  -8.2987, -11.0650, -13.8312, -13.8312, -13.8312, -13.8312, -13.8312,
  -13.8312, -13.8312, -13.8312, -13.8312, -13.8312, -13.8312, -13.8312,
  -13.8312, -13.8312, -13.8312, -13.8312, -13.8312, -13.8312, -13.8312,
  -13.8312, -13.8312,
};

static s16 Animation_for_hip_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch2_val[]={
  -12.3406, -12.3406, -12.3406, -12.3406, -12.3406, -12.3406, -12.3406,
  -12.3408, -12.3410, -12.3413, -12.3417, -12.3417, -12.3417, -12.3417,
  -12.3417, -12.3417, -12.3417, -12.3417, -12.3417, -12.3417, -12.3417,
  -12.3417, -12.3417, -12.3417, -12.3417, -12.3417, -12.3417, -12.3417,
  -12.3417, -12.3417, -12.3417,
};

static s16 Animation_for_hip_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch3_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0027, -0.0055,
  -0.0082, -0.0109, -0.0137, -0.0137, -0.0137, -0.0137, -0.0137, -0.0137,
  -0.0137, -0.0137, -0.0137, -0.0137, -0.0137, -0.0137, -0.0137, -0.0137,
  -0.0137, -0.0137, -0.0137, -0.0137, -0.0137, -0.0137, -0.0137,
};

static s16 Animation_for_hip_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch4_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0000, -0.0001,
  -0.0001, -0.0002, -0.0003, -0.0003, -0.0003, -0.0003, -0.0003, -0.0003,
  -0.0003, -0.0003, -0.0003, -0.0003, -0.0003, -0.0003, -0.0003, -0.0003,
  -0.0003, -0.0003, -0.0003, -0.0003, -0.0003, -0.0003, -0.0003,
};

static s16 Animation_for_hip_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch5_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0093, -0.0187,
  -0.0280, -0.0373, -0.0466, -0.0466, -0.0466, -0.0466, -0.0466, -0.0466,
  -0.0466, -0.0466, -0.0466, -0.0466, -0.0466, -0.0466, -0.0466, -0.0466,
  -0.0466, -0.0466, -0.0466, -0.0466, -0.0466, -0.0466, -0.0466,
};

static s16 Animation_for_hip_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_hip_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_hip_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_hip_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_hip_motion_ch0_key, Animation_for_hip_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_hip_motion_ch1_key, Animation_for_hip_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_hip_motion_ch2_key, Animation_for_hip_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_hip_motion_ch3_key, Animation_for_hip_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_hip_motion_ch4_key, Animation_for_hip_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_hip_motion_ch5_key, Animation_for_hip_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_hip_motion_ch6_key, Animation_for_hip_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_hip_motion_ch7_key, Animation_for_hip_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_hip_motion_ch8_key, Animation_for_hip_motion_ch8_val},
};

static NDR_ANIM Animation_for_hip_motion={
  Animation_for_hip_motion_chlist, 9, 30
};

static s16 Animation_for_R_leg_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch0_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0020, 0.0083, 0.0187,
  0.0328, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504,
  0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504,
  0.0504, 0.0504, 0.0504, 0.0504,
};

static s16 Animation_for_R_leg_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch1_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.2762, 0.5499, 0.8187,
  1.0800, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314,
  1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314,
  1.3314, 1.3314, 1.3314, 1.3314,
};

static s16 Animation_for_R_leg_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch2_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0196, -0.0791,
  -0.1784, -0.3169, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943,
  -0.4943, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943,
  -0.4943, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943,
};

static s16 Animation_for_R_leg_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch3_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0584, -0.1168,
  -0.1751, -0.2334, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916,
  -0.2916, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916,
  -0.2916, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916,
};

static s16 Animation_for_R_leg_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch4_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0004, 0.0018, 0.0041,
  0.0073, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114,
  0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114,
  0.0114, 0.0114, 0.0114, 0.0114,
};

static s16 Animation_for_R_leg_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch5_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0001, 0.0001, -0.0000,
  -0.0003, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008,
  -0.0008, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008,
  -0.0008, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008,
};

static s16 Animation_for_R_leg_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_leg_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_leg_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_leg_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_leg_motion_ch0_key, Animation_for_R_leg_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_leg_motion_ch1_key, Animation_for_R_leg_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_leg_motion_ch2_key, Animation_for_R_leg_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_leg_motion_ch3_key, Animation_for_R_leg_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_leg_motion_ch4_key, Animation_for_R_leg_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_leg_motion_ch5_key, Animation_for_R_leg_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_leg_motion_ch6_key, Animation_for_R_leg_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_leg_motion_ch7_key, Animation_for_R_leg_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_leg_motion_ch8_key, Animation_for_R_leg_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_leg_motion={
  Animation_for_R_leg_motion_chlist, 9, 30
};

static s16 Animation_for_R_lowleg_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch0_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0001, 0.0001, 0.0001, 0.0001, 0.0001, 0.0001, 0.0001, 0.0001, 0.0001,
  0.0001, 0.0001, 0.0001, 0.0001, 0.0001, 0.0001, 0.0001, 0.0001, 0.0001,
  0.0001, 0.0001, 0.0001, 0.0001,
};

static s16 Animation_for_R_lowleg_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch1_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -1.7381, -5.3925,
  -10.8681, -18.0223, -26.6687, -26.6687, -26.6687, -26.6687, -26.6687,
  -26.6687, -26.6687, -26.6687, -26.6687, -26.6687, -26.6687, -26.6687,
  -26.6687, -26.6687, -26.6687, -26.6687, -26.6687, -26.6687, -26.6687,
  -26.6687, -26.6687,
};

static s16 Animation_for_R_lowleg_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch2_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 12.0527, 23.6689,
  34.5458, 44.4001, 52.9752, 52.9752, 52.9752, 52.9752, 52.9752, 52.9752,
  52.9752, 52.9752, 52.9752, 52.9752, 52.9752, 52.9752, 52.9752, 52.9752,
  52.9752, 52.9752, 52.9752, 52.9752, 52.9752, 52.9752, 52.9752,
};

static s16 Animation_for_R_lowleg_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch3_val[]={
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, 0.1616, 0.3232,
  0.4847, 0.6463, 0.8079, 0.8079, 0.8079, 0.8079, 0.8079, 0.8079, 0.8079,
  0.8079, 0.8079, 0.8079, 0.8079, 0.8079, 0.8079, 0.8079, 0.8079, 0.8079,
  0.8079, 0.8079, 0.8079, 0.8079, 0.8079,
};

static s16 Animation_for_R_lowleg_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch4_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_lowleg_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch5_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_R_lowleg_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_lowleg_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_lowleg_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_lowleg_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch0_key, Animation_for_R_lowleg_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch1_key, Animation_for_R_lowleg_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch2_key, Animation_for_R_lowleg_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch3_key, Animation_for_R_lowleg_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch4_key, Animation_for_R_lowleg_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch5_key, Animation_for_R_lowleg_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch6_key, Animation_for_R_lowleg_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch7_key, Animation_for_R_lowleg_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch8_key, Animation_for_R_lowleg_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_lowleg_motion={
  Animation_for_R_lowleg_motion_chlist, 9, 30
};

static s16 Animation_for_R_shoe_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch0_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 2.1255, 4.5087, 7.1268,
  9.9536, 12.9583, 12.9583, 12.9583, 12.9583, 12.9583, 12.9583, 12.9583,
  12.9583, 12.9583, 12.9583, 12.9583, 12.9583, 12.9583, 12.9583, 12.9583,
  12.9583, 12.9583, 12.9583, 12.9583, 12.9583, 12.9583,
};

static s16 Animation_for_R_shoe_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch1_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.5032, -2.0290,
  -4.5656, -8.0937, -12.5864, -12.5864, -12.5864, -12.5864, -12.5864,
  -12.5864, -12.5864, -12.5864, -12.5864, -12.5864, -12.5864, -12.5864,
  -12.5864, -12.5864, -12.5864, -12.5864, -12.5864, -12.5864, -12.5864,
  -12.5864, -12.5864,
};

static s16 Animation_for_R_shoe_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch2_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -11.2546, -22.3688,
  -33.2515, -43.8142, -53.9716, -53.9716, -53.9716, -53.9716, -53.9716,
  -53.9716, -53.9716, -53.9716, -53.9716, -53.9716, -53.9716, -53.9716,
  -53.9716, -53.9716, -53.9716, -53.9716, -53.9716, -53.9716, -53.9716,
  -53.9716, -53.9716,
};

static s16 Animation_for_R_shoe_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch3_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0856, -0.1708,
  -0.2557, -0.3401, -0.4242, -0.4242, -0.4242, -0.4242, -0.4242, -0.4242,
  -0.4242, -0.4242, -0.4242, -0.4242, -0.4242, -0.4242, -0.4242, -0.4242,
  -0.4242, -0.4242, -0.4242, -0.4242, -0.4242, -0.4242, -0.4242,
};

static s16 Animation_for_R_shoe_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch4_val[]={
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0127, -0.0267,
  -0.0420, -0.0585, -0.0762, -0.0762, -0.0762, -0.0762, -0.0762, -0.0762,
  -0.0762, -0.0762, -0.0762, -0.0762, -0.0762, -0.0762, -0.0762, -0.0762,
  -0.0762, -0.0762, -0.0762, -0.0762, -0.0762, -0.0762, -0.0762,
};

static s16 Animation_for_R_shoe_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch5_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0151, -0.0301,
  -0.0449, -0.0594, -0.0736, -0.0736, -0.0736, -0.0736, -0.0736, -0.0736,
  -0.0736, -0.0736, -0.0736, -0.0736, -0.0736, -0.0736, -0.0736, -0.0736,
  -0.0736, -0.0736, -0.0736, -0.0736, -0.0736, -0.0736, -0.0736,
};

static s16 Animation_for_R_shoe_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_shoe_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_shoe_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_shoe_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_shoe_motion_ch0_key, Animation_for_R_shoe_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_shoe_motion_ch1_key, Animation_for_R_shoe_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_shoe_motion_ch2_key, Animation_for_R_shoe_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_shoe_motion_ch3_key, Animation_for_R_shoe_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_shoe_motion_ch4_key, Animation_for_R_shoe_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_shoe_motion_ch5_key, Animation_for_R_shoe_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_shoe_motion_ch6_key, Animation_for_R_shoe_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_shoe_motion_ch7_key, Animation_for_R_shoe_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_shoe_motion_ch8_key, Animation_for_R_shoe_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_shoe_motion={
  Animation_for_R_shoe_motion_chlist, 9, 30
};

static s16 Animation_for_R_upleg_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch0_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0020, 0.0083, 0.0187,
  0.0328, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504,
  0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504, 0.0504,
  0.0504, 0.0504, 0.0504, 0.0504,
};

static s16 Animation_for_R_upleg_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch1_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.2762, 0.5499, 0.8187,
  1.0800, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314,
  1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314, 1.3314,
  1.3314, 1.3314, 1.3314, 1.3314,
};

static s16 Animation_for_R_upleg_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch2_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0196, -0.0791,
  -0.1784, -0.3169, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943,
  -0.4943, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943,
  -0.4943, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943, -0.4943,
};

static s16 Animation_for_R_upleg_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch3_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0584, -0.1168,
  -0.1751, -0.2334, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916,
  -0.2916, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916,
  -0.2916, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916, -0.2916,
};

static s16 Animation_for_R_upleg_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch4_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0004, 0.0018, 0.0041,
  0.0073, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114,
  0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114, 0.0114,
  0.0114, 0.0114, 0.0114, 0.0114,
};

static s16 Animation_for_R_upleg_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch5_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0001, 0.0001, -0.0000,
  -0.0003, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008,
  -0.0008, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008,
  -0.0008, -0.0008, -0.0008, -0.0008, -0.0008, -0.0008,
};

static s16 Animation_for_R_upleg_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_upleg_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_upleg_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_upleg_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_upleg_motion_ch0_key, Animation_for_R_upleg_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_upleg_motion_ch1_key, Animation_for_R_upleg_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_upleg_motion_ch2_key, Animation_for_R_upleg_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_upleg_motion_ch3_key, Animation_for_R_upleg_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_upleg_motion_ch4_key, Animation_for_R_upleg_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_upleg_motion_ch5_key, Animation_for_R_upleg_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_upleg_motion_ch6_key, Animation_for_R_upleg_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_upleg_motion_ch7_key, Animation_for_R_upleg_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_upleg_motion_ch8_key, Animation_for_R_upleg_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_upleg_motion={
  Animation_for_R_upleg_motion_chlist, 9, 30
};

/* ============================================================
          Object Structure Area
 ============================================================ */
/* Object #0 */
static NDR_OBJ _dummy_obj={
  NDC_TYPE_NULL, 0, 0x00000000,
  {0,0,0, 0,0,0, 1,1,1},
  -1,
  0x0000, 0x00000000, 0x00000000, 0x00000000,
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  -1, 0, 0, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #1 */
static u16 _body_link[]={ 1, 4, 5, 8, 9, 10, 13, 14 };
static NDR_OBJ _body={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,-12.340581,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  0,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  0, 0, 2, 0x010203,
  {-1, -1}, 8, _body_link
};

/* Object #2 */
static u16 _R_uparm_link[]={ 1 };
static NDR_OBJ _R_uparm={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000000,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  1,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  1, 0, 2, 0x010203,
  {-1, -1}, 1, _R_uparm_link
};

/* Object #3 */
static u16 _R_lowarm_link[]={ 1 };
static NDR_OBJ _R_lowarm={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {-0.000005,0.000000,0.000000,
   0.000000,0.000000,-0.000000,
   1.000000,1.000000,1.000000},
  2,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  2, 0, 2, 0x010203,
  {-1, -1}, 1, _R_lowarm_link
};

/* Object #4 */
static NDR_OBJ _R_hand={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {-0.000005,0.000004,0.000000,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  3,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  3, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #5 */
static NDR_OBJ _R_sode={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000000,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  4,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  4, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #6 */
static u16 _L_uparm_link[]={ 1 };
static NDR_OBJ _L_uparm={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000000,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  5,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  5, 0, 2, 0x010203,
  {-1, -1}, 1, _L_uparm_link
};

/* Object #7 */
static u16 _L_lowarm_link[]={ 1 };
static NDR_OBJ _L_lowarm={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000005,0.000000,
   0.000000,0.000000,-0.000000,
   1.000000,1.000000,1.000000},
  6,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  6, 0, 2, 0x010203,
  {-1, -1}, 1, _L_lowarm_link
};

/* Object #8 */
static NDR_OBJ _L_hand={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,-0.000010,0.000000,
   -0.000000,-0.000000,0.000000,
   1.000000,1.000000,1.000000},
  7,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  7, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #9 */
static NDR_OBJ _L_sode={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000000,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  8,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  8, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #10 */
static NDR_OBJ _head={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000000,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  9,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  9, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #11 */
static u16 _L_upleg_link[]={ 1 };
static NDR_OBJ _L_upleg={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,-12.340581,
   -0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  10,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  10, 0, 2, 0x010203,
  {-1, -1}, 1, _L_upleg_link
};

/* Object #12 */
static u16 _L_lowleg_link[]={ 1 };
static NDR_OBJ _L_lowleg={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000005,0.000000,
   0.000000,-0.000000,-0.000000,
   1.000000,1.000000,1.000000},
  11,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  11, 0, 2, 0x010203,
  {-1, -1}, 1, _L_lowleg_link
};

/* Object #13 */
static NDR_OBJ _L_shoe={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,-0.000005,0.000000,
   -0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  12,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  12, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #14 */
static NDR_OBJ _L_leg={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,-12.340581,
   -0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  13,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  13, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #15 */
static u16 _hip_link[]={ 1, 4 };
static NDR_OBJ _hip={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,-12.340586,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  14,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  14, 0, 2, 0x010203,
  {-1, -1}, 2, _hip_link
};

/* Object #16 */
static u16 _R_leg_link[]={ 1 };
static NDR_OBJ _R_leg={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000005,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  15,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  15, 0, 2, 0x010203,
  {-1, -1}, 1, _R_leg_link
};

/* Object #17 */
static u16 _R_lowleg_link[]={ 1 };
static NDR_OBJ _R_lowleg={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000000,
   -0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  16,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  16, 0, 2, 0x010203,
  {-1, -1}, 1, _R_lowleg_link
};

/* Object #18 */
static NDR_OBJ _R_shoe={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000000,
   0.000000,-0.000000,0.000000,
   1.000000,1.000000,1.000000},
  17,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  17, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #19 */
static NDR_OBJ _R_upleg={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000005,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  18,
  0x0000, 0x00000001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  18, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #20 */
static NDR_OBJ _NULL_OBJECT_1={
  NDC_TYPE_NULL, 0, 0x00000000,
  {0,0,0, 0,0,0, 1,1,1},
  -1,
  0x0000, 0x00000000, 0x00000000, 0x00000000,
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  -1, 0, 0, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #21 */
static NDR_OBJ _NULL_OBJECT_2={
  NDC_TYPE_NULL, 0, 0x00000000,
  {0,0,0, 0,0,0, 1,1,1},
  -1,
  0x0000, 0x00000000, 0x00000000, 0x00000000,
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  -1, 0, 0, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #22 */
static NDR_OBJ _NULL_OBJECT_3={
  NDC_TYPE_NULL, 0, 0x00000000,
  {0,0,0, 0,0,0, 1,1,1},
  -1,
  0x0000, 0x00000000, 0x00000000, 0x00000000,
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  -1, 0, 0, 0x010203,
  {-1, -1}, 0, NULL
};

/* ============================================================
          Scene Structure Area
 ============================================================ */
static NDR_SCN_INST_OBJ eat_scene_instlist[]={
 {
  {0,0,0, 0,0,0, 1,1,1},
  20 /* NULL_OBJECT_1 */
 },
 {
  {0,0,0, 0,0,0, 1,1,1},
  22 /* NULL_OBJECT_3 */
 },
 {
  {0,0,0, 0,0,0, 1,1,1},
  21 /* NULL_OBJECT_2 */
 },
 {
  {0.000000,0.000000,-12.340581,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  1 /* body */
 },
};

static NDR_SCN_LIGHT _eat_scene_light[]={
  { 0, 0xffffff00, 0.57735, 0.57735, 0.57735, 0, 0 }
};

static NDR_SCENE eat_scene={
  {950.000000, 1000.000000, 0x4c, 0x4c, 0x4c, 0xff},
  {1, 45.841393, 1.333330, 1.000000, 0, 0, 0, 0, 1, 10000, 0, 1, 2},
  { 0x000000ff, 1, _eat_scene_light },
  0x0000, 0x00fc,
  eat_scene_instlist, 4,
};

/* ============================================================
          Shape Structure List
 ============================================================ */
static NDR_SHAPE *shapelist_frake_eat[] = {
  &shape_Shape_of___POLYHEDRON_41_,
  &shape_Shape_of___POLYHEDRON_55_,
  &shape_Shape_of___POLYHEDRON_57_,
  &shape_Shape_of___POLYHEDRON_59_,
  &shape_Shape_of___POLYHEDRON_53_,
  &shape_Shape_of___POLYHEDRON_47_,
  &shape_Shape_of___POLYHEDRON_49_,
  &shape_Shape_of___POLYHEDRON_51_,
  &shape_Shape_of___POLYHEDRON_45_,
  &shape_Shape_of___POLYHEDRON_8_,
  &shape_Shape_of___POLYHEDRON_69_,
  &shape_Shape_of___POLYHEDRON_73_,
  &shape_Shape_of___POLYHEDRON_77_,
  &shape_Shape_of___POLYHEDRON_65_,
  &shape_Shape_of___POLYHEDRON_63_,
  &shape_Shape_of___POLYHEDRON_61_,
  &shape_Shape_of___POLYHEDRON_71_,
  &shape_Shape_of___POLYHEDRON_75_,
  &shape_Shape_of___POLYHEDRON_67_,
};

/* ============================================================
          Geometric Animation List
 ============================================================ */
static NDR_ANIM *animlist_frake_eat[]={
  &Animation_for_body_motion,
  &Animation_for_R_uparm_motion,
  &Animation_for_R_lowarm_motion,
  &Animation_for_R_hand_motion,
  &Animation_for_R_sode_motion,
  &Animation_for_L_uparm_motion,
  &Animation_for_L_lowarm_motion,
  &Animation_for_L_hand_motion,
  &Animation_for_L_sode_motion,
  &Animation_for_head_motion,
  &Animation_for_L_upleg_motion,
  &Animation_for_L_lowleg_motion,
  &Animation_for_L_shoe_motion,
  &Animation_for_L_leg_motion,
  &Animation_for_hip_motion,
  &Animation_for_R_leg_motion,
  &Animation_for_R_lowleg_motion,
  &Animation_for_R_shoe_motion,
  &Animation_for_R_upleg_motion,
};

/* ============================================================
          Object Struct List
 ============================================================ */
static NDR_OBJ *objlist_frake_eat[]={
  &_dummy_obj, /* 0 */
  &_body, /* 1 */
  &_R_uparm, /* 2 */
  &_R_lowarm, /* 3 */
  &_R_hand, /* 4 */
  &_R_sode, /* 5 */
  &_L_uparm, /* 6 */
  &_L_lowarm, /* 7 */
  &_L_hand, /* 8 */
  &_L_sode, /* 9 */
  &_head, /* 10 */
  &_L_upleg, /* 11 */
  &_L_lowleg, /* 12 */
  &_L_shoe, /* 13 */
  &_L_leg, /* 14 */
  &_hip, /* 15 */
  &_R_leg, /* 16 */
  &_R_lowleg, /* 17 */
  &_R_shoe, /* 18 */
  &_R_upleg, /* 19 */
  &_NULL_OBJECT_1, /* 20 */
  &_NULL_OBJECT_2, /* 21 */
  &_NULL_OBJECT_3, /* 22 */
};

/* ############################################################
          Resource Information
 ############################################################ */
NDR_RESINFO resinfo_frake_eat={
  &eat_scene,
  20.000000,
  objlist_frake_eat,
  shapelist_frake_eat,
  animlist_frake_eat,
  NULL,
  NULL,
  NULL,
  23, 19, 19, 0, 0, 0
};

