
/* niff to gfx converter v1.2 */

/* ============================================================
          Header Area
 ============================================================ */

#ifndef __ND_COMMONS__
#define __ND_COMMONS__

/* ------------------ macros */

#define SEGTOP(sno) ((u32)(sno)<<24)
#define SEGADRS(sno,adrs) (SEGTOP(sno)|((u32)(adrs)&0x00ffffff))

/* ------------------ geometric animation */

enum{
  /* NDR_ANIM_CHANNEL.kind */
  NDC_ANIMKIND_TRS_X  = 0x00,
  NDC_ANIMKIND_TRS_Y  = 0x01,
  NDC_ANIMKIND_TRS_Z  = 0x02,
  NDC_ANIMKIND_ROT_X  = 0x03,
  NDC_ANIMKIND_ROT_Y  = 0x04,
  NDC_ANIMKIND_ROT_Z  = 0x05,
  NDC_ANIMKIND_SCL_X  = 0x06,
  NDC_ANIMKIND_SCL_Y  = 0x07,
  NDC_ANIMKIND_SCL_Z  = 0x08
};

typedef struct {
  u16 kind;
  s16 start;
  s16 end;
  s16 nkey;
  s16 *key;
  float *value;
} NDR_ANIM_CHANNEL;

typedef struct {
  NDR_ANIM_CHANNEL *chlist;
  u16 nch;
  u16 total;
} NDR_ANIM;

/* ------------------ shape */

typedef struct{
  s16 *key;
  s16 *tblno;
  u16 nkey;
  u16 nelem;
} NDR_DEFORM;

typedef struct{
  Gfx *gfx;
  Vtx *vtx;
  u16 nvtx;
  u16 vtxsegno;

  /* using texture segment */
  u8 *img;
  u16 *tlut;
  u16 imgsegno, tlutsegno;

  /* shape deform data link */
  s16 *shaperef;
  NDR_DEFORM *vtx_deform;
  NDR_DEFORM *vtxcol_deform;
  NDR_DEFORM *vtxnv_deform;

} NDR_SHAPE;

/* ------------------ object */

enum{
  /* object types */
  NDC_TYPE_NULL = 0x01,
  NDC_TYPE_3D = 0x02,
  NDC_TYPE_BILLBOARD = 0x03,

  /* ColorCombine Mode */
  NDC_CC_COMBINED = 0x00,
  NDC_CC_TEX0 = 0x01,
  NDC_CC_TEX1 = 0x02,
  NDC_CC_PRIMITIVE = 0x03,
  NDC_CC_SHADE = 0x04,
  NDC_CC_ENV = 0x05,
  NDC_CC_KEY_CENTER = 0x10,
  NDC_CC_KEY_SCALE = 0x11,
  NDC_CC_COMBINED_ALPHA = 0x06,
  NDC_CC_TEX0_ALPHA = 0x07,
  NDC_CC_TEX1_ALPHA = 0x08,
  NDC_CC_PRIMITIVE_ALPHA = 0x09,
  NDC_CC_SHADE_ALPHA = 0x0a,
  NDC_CC_ENV_ALPHA = 0x0b,
  NDC_CC_LOD = 0x0c,
  NDC_CC_PRIM_LOD = 0x0d,
  NDC_CC_NOISE = 0x12,
  NDC_CC_K4 = 0x14,
  NDC_CC_K5 = 0x15,
  NDC_CC_1 = 0x0e,
  NDC_CC_0 = 0x0f,

  /* Render Mode(complexed) */
  NDC_RM_AA_EN                     = 0x80000000,
  NDC_RM_Z_CMP                     = 0x40000000,
  NDC_RM_Z_UPD                     = 0x20000000,
  NDC_RM_IM_RD                     = 0x10000000,
  NDC_RM_CVG_DST                   = 0x0c000000,
    NDC_RM_CVG_DST_CLAMP           = 0x00000000,
    NDC_RM_CVG_DST_WRAP            = 0x04000000,
    NDC_RM_CVG_DST_FULL            = 0x08000000,
    NDC_RM_CVG_DST_SAVE            = 0x0c000000,
  NDC_RM_CLR_ON_CVG                = 0x02000000,
  NDC_RM_CVG_X_ALPHA               = 0x01000000,
  NDC_RM_ALPHA_CVG_SEL             = 0x00800000,
  NDC_RM_FORCE_BL                  = 0x00400000,
  NDC_RM_ZMODE                     = 0x00300000,
    NDC_RM_ZMODE_OPA               = 0x00000000,
    NDC_RM_ZMODE_INTER             = 0x00100000,
    NDC_RM_ZMODE_XLU               = 0x00200000,
    NDC_RM_ZMODE_DEC               = 0x00300000,
  NDC_RM_ALPHA_COMPARE             = 0x000c0000,
    NDC_RM_ALPHA_COMPARE_NONE      = 0x00000000,
    NDC_RM_ALPHA_COMPARE_THRESHOLD = 0x00040000,
    NDC_RM_ALPHA_COMPARE_DITHER    = 0x00080000,
  NDC_RM_DITHER_ALPHA              = 0x00030000,
    NDC_RM_DITHER_ALPHA_PATTERN    = 0x00000000,
    NDC_RM_DITHER_ALPHA_NOPATTERN  = 0x00010000,
    NDC_RM_DITHER_ALPHA_NOISE      = 0x00020000,
    NDC_RM_DITHER_ALPHA_DISABLE    = 0x00030000,
  NDC_RM_BLENDER_MUX0              = 0x0000ff00,
  NDC_RM_BLENDER_MUX1              = 0x000000ff,

  /* NDR_OBJ.regflag(bit assign) */
  NDC_REGF_2CYC        = 0x4000
};

typedef struct{
  float tx, ty, tz;
  float rx, ry, rz;
  float sx, sy, sz;
} TRX;

typedef struct ndr_objTag{

  /* object propaties */
  u16 type, gid;
  u32 prio;
  TRX trx;

  /* resourse index */
  s16 shape;

  /* RSP, RDP status register setting */
  u16 regflag;
  u32 geommode;
  u32 rdmode;
  u32 primcol;
  u8 cc1[8], cc2[8];

  /* geometric animation setting */
  s16 geom_anim;
  u16 loop;
  u16 framerate;
  u32 order;

  /* children links */
  s16 bb_obj[2];
  u16 nchild;
  u16 *child;

} NDR_OBJ;

/* ------------------ scene */

typedef struct{
  TRX trx;
  u16 objno;
} NDR_SCN_INST_OBJ;

typedef struct{
  int type;
  u32 color;
  float x, y, z;
  float a1,a2;
} NDR_SCN_LIGHT;

typedef struct{
  struct{
    u16 near, far;
    u8 r,g,b,a;
  }fog;
  struct{
    u32 mode;
    float fovy, aspect, scale;
    u16 top, bottom, left, right, near, far;
    u16 eye_inst, upper_inst, lookat_inst;
  }cam;
  struct{
    u32 amb_color;
    int nlight;
    NDR_SCN_LIGHT *light;
  }light;
  u16 fill_col, fill_depth;
  NDR_SCN_INST_OBJ *entryobj;
  u16 nentobj;
} NDR_SCENE;

typedef struct{
  NDR_SCENE *scene;
  float convert_scale;

  NDR_OBJ **obj;
  NDR_SHAPE **shape;
  NDR_ANIM **anim;
  s16 **rawvtx;
  u32 **rawvtxcol;
  s8 **rawvtxnv;

  u16 nobj;
  u16 nanim;
  u16 nshape;
  u16 nrawvtx;
  u16 nrawvtxcol;
  u16 nrawvtxnv;
} NDR_RESINFO;

#endif    /* #ifndef __ND_COMMONS__ */

#ifdef PREVIEW_TYPE
#undef PREVIEW_TYPE
#endif  /* #ifdef PREVIEW_TYPE */

#ifdef RESINFO_NAME
#undef RESINFO_NAME
#endif  /* #ifdef RESINFO_NAME */

#define PREVIEW_TYPE 28
extern NDR_RESINFO resinfo_frake_run;
#define RESINFO_NAME resinfo_frake_run

static Gfx _frake_run_aligner0[] = { gsSPEndDisplayList() };

/* ============================================================
          Bitmap
 ============================================================ */
static u8 bmp_frake_run[] = {
/* name:t_shirt3  type:RGBA  pixel/bit: 16 bits
  offset:0x00000000  length:2048 bytes. */
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xa7,0xb4,
0xaf,0x6e,0xaf,0x6e,0xaf,0x6e,0xaf,0x6e,
0xa7,0xb4,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xd0,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xd0,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xd0,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xd0,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xd0,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xd0,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x42,0xcd,0xd0,0xcd,0xd0,0x9f,0xfa,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xa7,0xb6,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xa7,0xb6,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xa7,0xb6,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xa7,0xb6,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xd5,0x88,0xbe,0x9e,0xbe,0x9e,0x9f,0xfc,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xcd,0xce,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xa7,0x72,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xaf,0x6e,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xaf,0x6e,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xaf,0x6e,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0xcd,0xd0,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xdd,0x40,0xdd,0x40,0xdd,0x40,
0xdd,0x40,0xaf,0x6e,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfa,
0xa7,0xb6,0xa7,0xb6,0xa7,0xb6,0xa7,0xb6,
0xa7,0xb6,0xa7,0xb6,0xa7,0xb6,0xa7,0xb6,
0xa7,0xb6,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x5c,0x1e,0x39,0x88,0x52,0x0a,0x6a,0x8e,
0x6a,0x8e,0x5a,0x4c,0x39,0x88,0x31,0xcc,
0x65,0x6a,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x21,0x8c,0x8b,0x92,
0xf6,0x20,0xfe,0x62,0xfe,0x62,0xfe,0x62,
0xfe,0x62,0xfe,0x62,0xfe,0x62,0xf6,0x20,
0x8b,0x92,0x18,0x82,0x7e,0xb4,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x6d,0xac,0xa4,0x16,0xfe,0x62,
0xfe,0x62,0xfe,0x62,0xfe,0x62,0xfe,0x62,
0xfe,0x62,0xfe,0x62,0xfe,0x62,0xfe,0x62,
0xfe,0x62,0xd5,0x5c,0x10,0xc6,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,0x9f,0xfe,
/* name:belt2  type:RGBA  pixel/bit: 16 bits
  offset:0x00000800  length:2048 bytes. */
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x24,0x48,0x24,0x48,0x24,0x48,0x26,
0x50,0x66,0x50,0x66,0x48,0x26,0x48,0x24,
0x48,0x24,0x48,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x48,0x24,0x48,0x26,0x48,0x24,0x40,0x24,
0x08,0x02,0x18,0x88,0x49,0xd6,0x73,0x24,
0x8b,0xaa,0x8b,0xaa,0x73,0x24,0x41,0xd6,
0x18,0x88,0x08,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x28,0xcc,0x6a,0xe2,0x10,0x86,0x00,0x02,
0x00,0x00,0x10,0x86,0x4a,0x16,0x7b,0xa4,
0x8c,0x2c,0x8c,0x2c,0x7b,0xa4,0x42,0x14,
0x10,0x84,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x18,0xc8,0x73,0x22,0x18,0xc6,0x00,0x00,
0x08,0x02,0x18,0x88,0x49,0xd8,0x7b,0x66,
0x8b,0xaa,0x8b,0xaa,0x73,0x24,0x41,0xd6,
0x18,0x88,0x08,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,
0x18,0x8a,0x6a,0xe2,0x20,0xca,0x00,0x02,
0x48,0x24,0x48,0x24,0x48,0x24,0x48,0x26,
0x50,0x66,0x50,0x66,0x48,0x26,0x48,0x24,
0x48,0x24,0x48,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x40,0x24,0x40,0x24,0x40,0x24,0x40,0x24,
0x48,0x24,0x48,0x26,0x48,0x24,0x40,0x24,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
0x48,0x26,0x48,0x26,0x48,0x26,0x48,0x26,
/* name:mouth2_32  type:RGBA  pixel/bit: 16 bits
  offset:0x00001000  length:2048 bytes. */
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x5e,0xed,0x5e,0xed,0x5e,0xed,0x5e,
0xe5,0x1c,0xdd,0x1c,0xed,0x5e,0xed,0x5e,
0xf5,0x5e,0xf5,0x5e,0xf5,0x9e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x9e,0xed,0x5e,0xed,0x5e,0xd4,0xdc,
0xb4,0x16,0x9b,0x54,0x8b,0x12,0x72,0x8e,
0x62,0x0c,0x62,0x0c,0x6a,0x8e,0x7a,0xd0,
0x93,0x52,0xa3,0xd4,0xbc,0x18,0xd4,0xda,
0xed,0x5e,0xf5,0x9e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0x9e,0xed,0x5e,
0xcc,0x9a,0x8b,0x52,0x6a,0x4e,0x5a,0x0c,
0x41,0x48,0x31,0x06,0x29,0x06,0x20,0xc4,
0x20,0x84,0x28,0xc4,0x20,0xc4,0x20,0xc4,
0x31,0x06,0x39,0x48,0x41,0x88,0x5a,0x0c,
0x83,0x10,0xc4,0x58,0xed,0x5e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x5e,0xe5,0x1e,0xc4,0x58,0x8b,0x52,
0x51,0xca,0x29,0x06,0x20,0x84,0x20,0xc4,
0x31,0x06,0x39,0x46,0x51,0xca,0x62,0x0c,
0x6a,0x4e,0x7a,0x8e,0x6a,0x4e,0x49,0xca,
0x41,0x88,0x39,0x46,0x28,0xc4,0x28,0xc4,
0x28,0xc4,0x51,0xca,0x9b,0x54,0xdc,0xdc,
0xf5,0x5e,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xed,0x5e,
0xc4,0x58,0x8b,0x10,0x49,0x8a,0x31,0x06,
0x20,0x84,0x39,0x46,0x5a,0x0a,0x7a,0xd0,
0x93,0x52,0xa3,0xd4,0xc4,0x58,0xdc,0xdc,
0xed,0x5e,0xed,0x5e,0xe5,0x1c,0xcc,0x9a,
0xb4,0x16,0xa3,0x94,0x83,0x10,0x6a,0x4c,
0x31,0x06,0x28,0xc4,0x31,0x06,0x6a,0x4c,
0xbc,0x58,0xed,0x5e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xed,0x5e,0xa3,0xd4,
0x49,0xca,0x29,0x06,0x20,0x84,0x39,0x48,
0x6a,0x4c,0xa3,0x94,0xd4,0xdc,0xed,0x5e,
0xf5,0x5e,0xf5,0x5e,0xf5,0x9e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x9e,0xf5,0x5e,0xed,0x5e,0xd4,0xdc,
0x9b,0x94,0x51,0xca,0x31,0x06,0x20,0xc4,
0x51,0xca,0xbc,0x18,0xed,0x5e,0xf5,0xa0,
0xf5,0x9e,0xe5,0x1c,0xa3,0x94,0x41,0x88,
0x20,0x84,0x39,0x48,0x62,0x4c,0xab,0xd6,
0xdc,0xdc,0xed,0x5e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xed,0x5e,0xcc,0x9a,0x82,0xd0,0x39,0x46,
0x28,0xc4,0x5a,0x0a,0xc4,0x58,0xed,0x5e,
0xdc,0xdc,0x8b,0x52,0x39,0x48,0x20,0x84,
0x41,0x88,0x9b,0x94,0xdc,0xdc,0xed,0x5e,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0x5e,0xe5,0x1c,0x8b,0x12,
0x31,0x46,0x28,0xc4,0x62,0x4c,0xcc,0x9a,
0x9b,0x94,0x39,0x46,0x20,0xc4,0x51,0xca,
0xab,0xd6,0xed,0x5e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0x9e,0xdd,0x1c,
0x82,0xd0,0x31,0x06,0x31,0x06,0x8b,0x12,
0x6a,0x8e,0x28,0xc4,0x5a,0x0a,0xbc,0x18,
0xed,0x5e,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0x9e,
0xd4,0xda,0x72,0x8e,0x39,0x48,0x82,0xd0,
0xc4,0x58,0x9b,0x94,0xc4,0x9a,0xed,0x5e,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x5e,0xd4,0xda,0xb4,0x16,0xd4,0x9a,
0xf5,0x9e,0xf5,0x5e,0xf5,0x9e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0x5e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
/* name:eye_32b  type:RGBA  pixel/bit: 16 bits
  offset:0x00001800  length:2048 bytes. */
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xe2,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x9e,0xdc,0xdc,0x8b,0x12,0x72,0x8e,
0x83,0x10,0x6a,0x4c,0x31,0x06,0x39,0x46,
0x7a,0xd0,0x7a,0xd0,0xbc,0x18,0xed,0x5e,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xed,0x5e,0xa3,0x94,
0x5a,0x0c,0x49,0xca,0x6a,0xd4,0x73,0x16,
0xac,0xe4,0xde,0xb2,0xe6,0xf4,0xe6,0xf4,
0xad,0x26,0x6b,0x14,0x6a,0x90,0x39,0x46,
0xab,0xd6,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0x9b,0x54,0x31,0x46,0x31,0x8a,
0x21,0xd0,0x11,0x90,0x00,0x86,0x09,0x0c,
0x22,0x56,0x5b,0xe2,0xef,0xbe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xcd,0xec,
0x31,0x06,0x9b,0x54,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0x8b,0x52,0x31,0x4a,0x21,0x8e,0x01,0x0c,
0x02,0x16,0x03,0x24,0x03,0x64,0x03,0x64,
0x02,0x5a,0x01,0x0c,0x11,0x4c,0xa5,0xf0,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0xd6,0x72,0x31,0x48,0x93,0x52,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0x93,0x54,
0x31,0x4a,0x10,0xc8,0x02,0x16,0x03,0xa8,
0x02,0xe0,0x01,0x4e,0x00,0xc8,0x01,0x90,
0x03,0x22,0x03,0xa8,0x02,0x5a,0x00,0x86,
0xa5,0xf0,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xde,0xf4,0x39,0x48,0x83,0x10,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xc4,0x58,0x39,0x8a,
0x21,0x8c,0x02,0x5a,0x03,0xa8,0x03,0x22,
0x00,0x86,0x00,0x00,0x00,0x00,0x00,0x00,
0x01,0x4e,0x03,0xa8,0x03,0xa8,0x02,0x5a,
0x09,0x4c,0xef,0x7c,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xde,0xb2,0x41,0x48,
0xc4,0x58,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x7a,0xd0,0x31,0x8e,
0x01,0x90,0x03,0xa8,0x03,0xa8,0x01,0x92,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x86,0x03,0xa8,0x03,0xa8,0x03,0xa8,
0x01,0x4e,0x53,0x9e,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0x73,0x14,
0x82,0xd0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x7b,0x14,0x21,0x4c,
0x03,0x22,0x03,0xa8,0x03,0x64,0x21,0x8e,
0x21,0x08,0x00,0x42,0x00,0x00,0x00,0x00,
0x00,0x86,0x03,0xa8,0x03,0xa8,0x03,0xa8,
0x02,0x9c,0x22,0x14,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xce,0x30,
0x51,0xca,0xf5,0x9e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x39,0x8a,0x11,0x08,
0x03,0x66,0x03,0x22,0x84,0xa4,0xef,0x7a,
0xef,0x7a,0x84,0x20,0x00,0x00,0x00,0x00,
0x00,0xca,0x03,0xa8,0x03,0xa8,0x03,0xa8,
0x03,0x64,0x01,0x0a,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xf7,0xbc,
0x52,0x0e,0xd4,0x9a,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x29,0x48,0x11,0x0a,
0x03,0xa6,0x33,0x60,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0x18,0xc6,0x00,0x02,
0x02,0xde,0x03,0xa8,0x03,0xa8,0x03,0xa8,
0x03,0x66,0x00,0x88,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0x6a,0xd4,0x82,0xd0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x5a,0x50,0x19,0x4a,
0x03,0x64,0x74,0x64,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0x5b,0x18,0x02,0x5a,
0x03,0xa8,0x03,0xa8,0x03,0xa8,0x03,0xa8,
0x03,0x22,0x09,0x4c,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0xbd,0x68,0x72,0x8e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x83,0x54,0x21,0x4a,
0x02,0x16,0x5c,0x24,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0x4b,0x5c,0x03,0xa8,
0x03,0xa8,0x03,0xa8,0x03,0xa8,0x03,0xa8,
0x01,0xd6,0x22,0x56,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0x9c,0xa0,0x7a,0x8e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xdc,0xdc,0x29,0x48,
0x01,0x0a,0x13,0x20,0xe7,0x38,0xff,0xfe,
0xff,0xfe,0xef,0x7a,0x0a,0x9a,0x03,0xa8,
0x03,0xa8,0x03,0xa8,0x03,0xa8,0x03,0x66,
0x00,0xca,0x85,0x2a,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0x62,0x92,0x7a,0xd0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0x8b,0x96,
0x10,0xc6,0x01,0x4e,0x23,0x1e,0xa5,0x6c,
0xa5,0x6a,0x2b,0x1e,0x03,0x66,0x03,0xa8,
0x03,0xa8,0x03,0xa8,0x03,0x20,0x01,0x0a,
0x3b,0x1c,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xef,0x7a,
0x41,0x8a,0xc4,0x98,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xed,0x5e,
0x6a,0xd2,0x08,0x86,0x00,0x84,0x01,0xd4,
0x03,0x22,0x03,0x64,0x03,0x64,0x03,0x20,
0x01,0xd6,0x01,0x4e,0x00,0xc8,0x53,0xe2,
0xf7,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0x62,0xd4,
0x72,0x8e,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xd5,0x1e,0x52,0x94,0x08,0x86,
0x00,0x86,0x00,0x84,0x00,0xca,0x1a,0x56,
0x2a,0x56,0x64,0x66,0xce,0xf8,0xff,0xfe,
0xff,0xfe,0xff,0xfe,0xff,0xfe,0xff,0xfe,
0xff,0xfe,0xe7,0x38,0x62,0xd6,0x5a,0x0c,
0xed,0x5e,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xac,0x5a,0x62,0x92,0x4a,0x10,0x7b,0xde,
0x6b,0x5a,0x73,0x9c,0xad,0x6a,0xe7,0x38,
0xe7,0x38,0xe7,0x38,0xa5,0x28,0x6b,0x5a,
0x73,0x5a,0x31,0x48,0x7a,0xd0,0xf5,0x9e,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0x9e,0xe5,0x1e,0xb4,0x18,
0x83,0x14,0x7b,0x14,0x83,0x98,0x5a,0x92,
0x29,0x48,0x4a,0x0e,0x7b,0x16,0x7b,0x14,
0xcc,0x9a,0xed,0x5e,0xf5,0x9e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x6a,0x4e,0x5a,0x0c,
0x62,0x0c,0x9b,0x94,0xed,0x5e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x51,0xca,0x00,0x00,
0x08,0x40,0x08,0x00,0x49,0xca,0xb4,0x16,
0xd4,0xda,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0x8b,0x12,0x18,0x82,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x18,0x82,0x51,0xca,0xb4,0x16,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0x9e,0xbc,0x58,
0x51,0xca,0x08,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x51,0xca,
0xb4,0x16,0xed,0x5e,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0x5e,0xb4,0x16,0x5a,0x0a,0x08,0x42,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x08,0x00,0x31,0x46,0x7a,0x8e,0xbc,0x58,
0xe5,0x1e,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xc4,0x58,
0x5a,0x0c,0x10,0x42,0x00,0x00,0x08,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x41,0x88,0xab,0xd6,0xf5,0x5e,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xcc,0x9a,0x7a,0xd0,0x9b,0x94,
0x8b,0x52,0x7a,0xd0,0x39,0x48,0x20,0xc4,
0x7a,0xd0,0x7a,0xd0,0x83,0x12,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,
0xf5,0xa0,0xf5,0xa0,0xf5,0xa0,0xf5,0xe2,
/* name:refmap  type:RGBA  pixel/bit: 16 bits
  offset:0x00002000  length:2048 bytes. */
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x18,0xc9,0x10,0x87,0x08,0x43,0x08,0x01,
0x08,0x43,0x08,0x43,0x08,0x43,0x21,0x09,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x10,0x45,0x20,0xc7,
0x10,0x85,0x10,0x85,0x10,0x85,0x10,0x85,
0x10,0x85,0x10,0x85,0x10,0x85,0x10,0x45,
0x10,0x85,0x10,0x43,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x08,0x43,0x18,0xc7,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x10,0x85,0x21,0x09,
0x10,0x85,0x10,0x85,0x10,0x45,0x10,0x43,
0x10,0x45,0x10,0x43,0x10,0x85,0x10,0x43,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x21,0x09,0x18,0xc7,
0x21,0x09,0x21,0x07,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x20,0xc7,0x10,0x85,0x10,0x45,0x10,0x43,
0x18,0x85,0x08,0x01,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x08,0x43,0x18,0xc7,0x21,0x09,
0x10,0x85,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x20,0xc7,0x20,0xc7,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x18,0xc7,0x08,0x03,
0x10,0x85,0x18,0x85,0x08,0x43,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x08,0x43,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x18,0xc7,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x18,0xc7,
0x18,0x87,0x10,0x43,0x10,0x43,0x10,0x85,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x20,0xc7,
0x08,0x43,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x18,0x87,0x20,0xc7,
0x18,0xc7,0x18,0x85,0x10,0x45,0x20,0xc9,
0x10,0x85,0x10,0x85,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x10,0x85,0x10,0x43,0x10,0x43,
0x20,0xc7,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x43,
0x18,0xc5,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x10,0x85,0x08,0x43,0x08,0x43,
0x10,0x43,0x10,0x43,0x10,0x45,0x10,0x45,
0x10,0x85,0x18,0x87,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x08,0x43,0x10,0x43,
0x10,0x43,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x10,0x87,0x10,0x43,
0x20,0xc5,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x10,0x85,0x08,0x43,0x08,0x43,
0x08,0x43,0x08,0x43,0x10,0x45,0x10,0x85,
0x10,0x85,0x21,0x09,0x10,0x85,0x21,0x09,
0x21,0x09,0x21,0x07,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x10,0x85,
0x10,0x43,0x18,0xc7,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x10,0x85,0x10,0x43,
0x20,0xc5,0x20,0xc5,0x21,0x09,0x21,0x09,
0x08,0x01,0x10,0x43,0x10,0x43,0x18,0x85,
0x10,0x43,0x08,0x43,0x10,0x45,0x21,0x09,
0x10,0x85,0x10,0x85,0x10,0x85,0x21,0x09,
0x21,0x0b,0x21,0x0b,0x08,0x43,0x21,0x0b,
0x21,0x0b,0x21,0x09,0x21,0x07,0x21,0x09,
0x08,0x43,0x18,0x85,0x00,0x00,0x00,0x00,
0x00,0x00,0x18,0xc7,0x08,0x43,0x10,0x43,
0x20,0xc5,0x20,0xc5,0x21,0x09,0x10,0x43,
0x10,0x43,0x10,0x83,0x18,0x83,0x18,0x83,
0x18,0xc7,0x18,0xc7,0x10,0x85,0x10,0x85,
0x10,0x85,0x10,0x85,0x10,0x85,0x21,0x0b,
0x21,0x0b,0x21,0x0b,0x21,0x0b,0x21,0x09,
0x21,0x09,0x20,0xc5,0x20,0xc5,0x21,0x09,
0x18,0xc7,0x10,0x43,0x08,0x01,0x00,0x00,
0x00,0x00,0x10,0x87,0x08,0x43,0x10,0x85,
0x18,0x85,0x21,0x09,0x21,0x09,0x08,0x41,
0x18,0x85,0x20,0xc7,0x20,0xc7,0x10,0x83,
0x10,0x43,0x10,0x43,0x10,0x85,0x10,0x85,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x21,0x09,
0x21,0x0b,0x21,0x09,0x20,0xc5,0x20,0xc5,
0x20,0xc5,0x20,0xc5,0x18,0x85,0x21,0x07,
0x10,0x85,0x10,0x43,0x08,0x43,0x00,0x00,
0x00,0x00,0x10,0x85,0x08,0x01,0x21,0x09,
0x10,0x83,0x18,0x85,0x10,0x43,0x08,0x03,
0x18,0x85,0x18,0x85,0x18,0xc5,0x18,0xc7,
0x10,0x85,0x10,0x85,0x08,0x43,0x21,0x09,
0x18,0x85,0x10,0x85,0x10,0x85,0x10,0x85,
0x08,0x43,0x08,0x43,0x10,0x83,0x10,0x83,
0x08,0x43,0x08,0x43,0x21,0x09,0x10,0x83,
0x10,0x85,0x08,0x43,0x10,0x43,0x00,0x00,
0x18,0x87,0x08,0x43,0x08,0x01,0x21,0x09,
0x18,0x85,0x08,0x01,0x08,0x43,0x08,0x01,
0x08,0x01,0x08,0x01,0x08,0x01,0x08,0x03,
0x08,0x03,0x10,0x43,0x08,0x01,0x08,0x43,
0x08,0x43,0x08,0x43,0x08,0x43,0x08,0x43,
0x08,0x43,0x08,0x43,0x10,0x43,0x10,0x43,
0x08,0x43,0x08,0x43,0x21,0x09,0x10,0x85,
0x10,0x85,0x08,0x43,0x10,0x85,0x18,0x85,
0x10,0x87,0x08,0x43,0x08,0x43,0x10,0x85,
0x18,0x85,0x08,0x43,0x08,0x43,0x08,0x43,
0x08,0x43,0x10,0x83,0x21,0x09,0x21,0x07,
0x08,0x43,0x08,0x43,0x08,0x03,0x08,0x43,
0x08,0x43,0x10,0x43,0x10,0x85,0x08,0x45,
0x10,0x85,0x10,0x85,0x08,0x43,0x08,0x43,
0x08,0x43,0x08,0x03,0x21,0x09,0x18,0xc7,
0x18,0xc7,0x08,0x43,0x10,0x85,0x21,0x07,
0x10,0x85,0x08,0x43,0x10,0x43,0x08,0x43,
0x10,0x43,0x10,0x43,0x20,0xc7,0x21,0x07,
0x10,0x85,0x10,0x87,0x21,0x09,0x10,0x85,
0x10,0x85,0x08,0x43,0x08,0x03,0x10,0x45,
0x10,0x87,0x10,0x85,0x10,0x87,0x10,0x45,
0x08,0x43,0x10,0x87,0x08,0x43,0x10,0x45,
0x08,0x43,0x08,0x03,0x18,0xc7,0x18,0x85,
0x10,0x85,0x10,0x43,0x10,0x85,0x10,0x85,
0x10,0x85,0x08,0x03,0x08,0x43,0x08,0x43,
0x08,0x01,0x10,0x43,0x10,0x85,0x10,0x85,
0x10,0x85,0x10,0x87,0x10,0x85,0x10,0x43,
0x08,0x43,0x08,0x43,0x08,0x43,0x10,0x45,
0x10,0x87,0x10,0xc7,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x19,0x09,0x10,0x85,0x08,0x45,
0x08,0x45,0x08,0x03,0x08,0x43,0x10,0x85,
0x10,0x85,0x10,0x45,0x10,0x85,0x10,0x85,
0x10,0x85,0x08,0x43,0x10,0x85,0x18,0x87,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x18,0xc7,
0x18,0xc7,0x18,0x87,0x10,0x85,0x10,0x43,
0x10,0x85,0x08,0x43,0x08,0x43,0x18,0xc7,
0x18,0xc9,0x10,0xc7,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x10,0x85,0x10,0x85,
0x08,0x43,0x18,0x87,0x18,0xc7,0x18,0xc7,
0x18,0xc7,0x18,0xc7,0x18,0x85,0x10,0x43,
0x08,0x43,0x10,0x43,0x18,0xc7,0x18,0xc7,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x18,0xc7,
0x18,0xc7,0x20,0xc9,0x20,0xc9,0x21,0x09,
0x21,0x09,0x10,0x87,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x20,0xc9,0x10,0x85,
0x10,0x85,0x10,0x85,0x10,0x43,0x10,0x85,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x18,0xc7,
0x18,0xc7,0x20,0xc7,0x18,0xc7,0x18,0xc7,
0x00,0x00,0x10,0x85,0x10,0xc7,0x18,0xc7,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x18,0xc7,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x19,0x09,
0x21,0x09,0x21,0x09,0x21,0x09,0x21,0x09,
0x21,0x09,0x21,0x09,0x29,0x09,0x29,0x09,
0x21,0x09,0x21,0x09,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x00,0x00,
0x00,0x00,0x10,0x85,0x18,0xc7,0x18,0xc7,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x18,0xc7,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x21,0x09,
0x21,0x09,0x10,0x45,0x21,0x0b,0x21,0x0b,
0x29,0x4b,0x29,0x4b,0x29,0x4d,0x29,0x4b,
0x29,0x4b,0x29,0x0b,0x21,0x09,0x21,0x0b,
0x21,0x09,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x18,0xc7,0x18,0xc7,0x18,0xc9,0x00,0x00,
0x00,0x00,0x10,0x87,0x18,0xc7,0x18,0xc9,
0x18,0xc7,0x18,0xc7,0x18,0xc7,0x18,0xc7,
0x18,0xc9,0x18,0xc9,0x21,0x09,0x21,0x09,
0x21,0x09,0x29,0x0b,0x21,0x0b,0x29,0x4b,
0x29,0x4d,0x31,0x8d,0x31,0x8f,0x39,0xcf,
0x31,0x8f,0x31,0x8d,0x29,0x4b,0x21,0x0b,
0x21,0x0b,0x21,0x09,0x18,0xc9,0x18,0xc7,
0x18,0xc7,0x18,0xc9,0x18,0xc9,0x00,0x00,
0x00,0x00,0x00,0x00,0x18,0xc7,0x18,0xc9,
0x18,0xc9,0x18,0xc7,0x18,0xc7,0x18,0xc9,
0x20,0xc9,0x21,0x09,0x19,0x09,0x21,0x09,
0x21,0x0b,0x21,0x0b,0x29,0x4d,0x29,0x4d,
0x29,0x8f,0x39,0xd1,0x42,0x13,0x4a,0x53,
0x42,0x13,0x39,0xd1,0x31,0x8f,0x21,0x4b,
0x21,0x0b,0x21,0x09,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x20,0xc7,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x18,0xc7,0x18,0xc7,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x19,0x09,
0x19,0x0b,0x29,0x4b,0x21,0x4d,0x29,0x8f,
0x39,0xd1,0x52,0x97,0x6b,0x9f,0x73,0x9f,
0x6b,0x5d,0x52,0x95,0x31,0xd1,0x29,0x4d,
0x21,0x0b,0x21,0x09,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x18,0xc7,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x19,0x0b,0x21,0x0b,0x21,0x4d,0x31,0x8f,
0x42,0x55,0x7c,0x23,0xb5,0xf1,0xde,0xf9,
0xb5,0xaf,0x73,0x9f,0x42,0x13,0x31,0x8d,
0x21,0x0b,0x18,0xc9,0x18,0xc9,0x21,0x09,
0x18,0xc9,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x21,0x09,
0x18,0xc9,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x18,0xc7,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x19,0x0b,0x21,0x0b,0x29,0x4d,0x31,0xd1,
0x52,0x99,0x9c,0xe9,0xff,0xff,0xff,0xff,
0xe7,0x3b,0x84,0x23,0x4a,0x55,0x31,0x8f,
0x29,0x4b,0x18,0xc9,0x18,0xc9,0x21,0x09,
0x21,0x09,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x21,0x09,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xc7,0x10,0xc9,0x18,0xc9,
0x18,0xc9,0x19,0x0b,0x21,0x4d,0x31,0xd1,
0x52,0xd9,0x94,0xe9,0xf7,0xbf,0xff,0xff,
0xc6,0x75,0x6b,0x5d,0x42,0x13,0x29,0x4d,
0x21,0x0b,0x19,0x09,0x18,0xc9,0x21,0x09,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x21,0x09,0x21,0x09,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x10,0xc9,0x10,0xc9,
0x18,0xc9,0x18,0xcb,0x21,0x0d,0x29,0x8f,
0x42,0x13,0x5b,0x1b,0x84,0x23,0x84,0x65,
0x6b,0x5d,0x42,0x13,0x29,0x4d,0x21,0x0b,
0x21,0x0b,0x19,0x0b,0x19,0x0b,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x21,0x09,0x21,0x0b,
0x19,0x09,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xc9,0x18,0xcb,0x21,0x0b,
0x29,0x8f,0x39,0xd1,0x42,0x13,0x42,0x13,
0x31,0xd1,0x29,0x8f,0x21,0x0d,0x21,0x0b,
0x19,0x0b,0x19,0x0b,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x21,0x0b,0x18,0xc9,0x18,0xc9,0x18,0xc9,
0x18,0xc9,0x18,0xcb,0x19,0x0b,0x19,0x0b,
0x21,0x0d,0x21,0x4d,0x29,0x4d,0x29,0x4d,
0x21,0x4d,0x21,0x0d,0x21,0x0b,0x19,0x0b,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x21,0x0b,0x19,0x0b,0x18,0xcb,
0x18,0xcb,0x19,0x0b,0x19,0x0b,0x21,0x0b,
0x21,0x0b,0x21,0x0b,0x21,0x0b,0x21,0x0b,
0x21,0x0b,0x19,0x0b,0x19,0x0b,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x19,0x0b,0x19,0x0b,0x19,0x0b,0x19,0x0b,
0x19,0x0b,0x19,0x0b,0x19,0x0b,0x21,0x0b,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
};

static Gfx _frake_run_aligner1[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_39_[] = {
  {    44,     5,   -40,0, 0x0000,0x0000, 0x2c,0x7a,0x7a,0xff }, /*     0 */
  {   -44,     5,   -40,0, 0x0000,0x0000, 0x33,0x8f,0x8f,0xff }, /*     1 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x48,0x78,0x78,0xff }, /*     2 */
  {    41,    31,    28,0, 0x0000,0x0000, 0x87,0xe1,0xe1,0xff }, /*     3 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x48,0x78,0x78,0xff }, /*     4 */
  {    47,    98,     6,0, 0x0000,0x0000, 0x85,0xdf,0xdf,0xff }, /*     5 */
  {   -11,   107,    -2,0, 0x0000,0x0000, 0x8d,0xec,0xec,0xff }, /*     6 */
  {    11,   107,   -25,0, 0x0000,0x0000, 0x8d,0xeb,0xeb,0xff }, /*     7 */
  {   -11,   107,   -25,0, 0x0000,0x0000, 0x8b,0xe9,0xe9,0xff }, /*     8 */
  {   -11,   107,    -2,0, 0x0000,0x0000, 0x8d,0xec,0xec,0xff }, /*     9 */
  {    11,   107,    -2,0, 0x0000,0x0000, 0x94,0xf8,0xf8,0xff }, /*    10 */
  {    11,   107,   -25,0, 0x0000,0x0000, 0x8d,0xeb,0xeb,0xff }, /*    11 */
  {   -40,    31,   -38,0, 0x0000,0x0000, 0x84,0xdd,0xdd,0xff }, /*    12 */
  {   -47,    98,     6,0, 0x0000,0x0000, 0x8c,0xe9,0xe9,0xff }, /*    13 */
  {   -46,    98,   -33,0, 0x0000,0x0000, 0x6d,0xb7,0xb7,0xff }, /*    14 */
  {   -47,    98,     6,0, 0x0000,0x0000, 0x8c,0xe9,0xe9,0xff }, /*    15 */
  {   -11,   107,   -25,0, 0x0000,0x0000, 0x8b,0xe9,0xe9,0xff }, /*    16 */
  {   -46,    98,   -33,0, 0x0000,0x0000, 0x6d,0xb7,0xb7,0xff }, /*    17 */
  {   -47,    98,     6,0, 0x0000,0x0000, 0x8c,0xe9,0xe9,0xff }, /*    18 */
  {   -11,   107,    -2,0, 0x0000,0x0000, 0x8d,0xec,0xec,0xff }, /*    19 */
  {   -11,   107,   -25,0, 0x0000,0x0000, 0x8b,0xe9,0xe9,0xff }, /*    20 */
  {   -46,    98,   -33,0, 0x0000,0x0000, 0x6d,0xb7,0xb7,0xff }, /*    21 */
  {    11,   107,   -25,0, 0x0000,0x0000, 0x8d,0xeb,0xeb,0xff }, /*    22 */
  {    46,    98,   -33,0, 0x0000,0x0000, 0x5b,0x98,0x98,0xff }, /*    23 */
  {    46,    98,   -33,0, 0x0000,0x0000, 0x5b,0x98,0x98,0xff }, /*    24 */
  {    11,   107,   -25,0, 0x0000,0x0000, 0x8d,0xeb,0xeb,0xff }, /*    25 */
  {    47,    98,     6,0, 0x0000,0x0000, 0x85,0xdf,0xdf,0xff }, /*    26 */
  {    41,    31,    28,0, 0x0000,0x0000, 0x87,0xe1,0xe1,0xff }, /*    27 */
  {    44,     5,   -40,0, 0x0000,0x0000, 0x2c,0x7a,0x7a,0xff }, /*    28 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x48,0x78,0x78,0xff }, /*    29 */
  {    41,    31,    28,0, 0x0000,0x0000, 0x87,0xe1,0xe1,0xff }, /*    30 */
  {    44,     5,    31,0, 0x0000,0x0000, 0x28,0x71,0x71,0xff }, /*    31 */
  {    44,     5,   -40,0, 0x0000,0x0000, 0x2c,0x7a,0x7a,0xff }, /*    32 */
  {   -41,    31,    28,0, 0x0000,0x0000, 0x7e,0xd3,0xd3,0xff }, /*    33 */
  {    44,     5,    31,0, 0x0000,0x0000, 0x28,0x71,0x71,0xff }, /*    34 */
  {    41,    31,    28,0, 0x0000,0x0000, 0x87,0xe1,0xe1,0xff }, /*    35 */
  {   -41,    31,    28,0, 0x0000,0x0000, 0x7e,0xd3,0xd3,0xff }, /*    36 */
  {   -44,     5,    31,0, 0x0000,0x0000, 0x29,0x73,0x73,0xff }, /*    37 */
  {    44,     5,    31,0, 0x0000,0x0000, 0x28,0x71,0x71,0xff }, /*    38 */
  {   -40,    31,   -38,0, 0x0000,0x0000, 0x84,0xdd,0xdd,0xff }, /*    39 */
  {   -44,     5,   -40,0, 0x0000,0x0000, 0x33,0x8f,0x8f,0xff }, /*    40 */
  {   -41,    31,    28,0, 0x0000,0x0000, 0x7e,0xd3,0xd3,0xff }, /*    41 */
  {   -40,    31,   -38,0, 0x0000,0x0000, 0x84,0xdd,0xdd,0xff }, /*    42 */
  {   -46,    98,   -33,0, 0x0000,0x0000, 0x6d,0xb7,0xb7,0xff }, /*    43 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x48,0x78,0x78,0xff }, /*    44 */
  {   -46,    98,   -33,0, 0x0000,0x0000, 0x6d,0xb7,0xb7,0xff }, /*    45 */
  {    46,    98,   -33,0, 0x0000,0x0000, 0x5b,0x98,0x98,0xff }, /*    46 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x48,0x78,0x78,0xff }, /*    47 */
  {   -44,     5,   -40,0, 0x0000,0x0000, 0x33,0x8f,0x8f,0xff }, /*    48 */
  {   -44,     5,    31,0, 0x0000,0x0000, 0x29,0x73,0x73,0xff }, /*    49 */
  {   -41,    31,    28,0, 0x0000,0x0000, 0x7e,0xd3,0xd3,0xff }, /*    50 */
  {    11,   107,   -25,0, 0x0000,0x0000, 0x8d,0xeb,0xeb,0xff }, /*    51 */
  {    11,   107,    -2,0, 0x0000,0x0000, 0x94,0xf8,0xf8,0xff }, /*    52 */
  {    47,    98,     6,0, 0x0000,0x0000, 0x85,0xdf,0xdf,0xff }, /*    53 */
  {   -46,    98,   -33,0, 0x0000,0x0000, 0x6d,0xb7,0xb7,0xff }, /*    54 */
  {   -11,   107,   -25,0, 0x0000,0x0000, 0x8b,0xe9,0xe9,0xff }, /*    55 */
  {    11,   107,   -25,0, 0x0000,0x0000, 0x8d,0xeb,0xeb,0xff }, /*    56 */
  {   -40,    31,   -38,0, 0x0000,0x0000, 0x84,0xdd,0xdd,0xff }, /*    57 */
  {   -41,    31,    28,0, 0x0000,0x0000, 0x7e,0xd3,0xd3,0xff }, /*    58 */
  {   -47,    98,     6,0, 0x0000,0x0000, 0x8c,0xe9,0xe9,0xff }, /*    59 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x48,0x78,0x78,0xff }, /*    60 */
  {    46,    98,   -33,0, 0x0000,0x0000, 0x5b,0x98,0x98,0xff }, /*    61 */
  {    47,    98,     6,0, 0x0000,0x0000, 0x85,0xdf,0xdf,0xff }, /*    62 */
  {   -44,     5,   -40,0, 0x0000,0x0000, 0x33,0x8f,0x8f,0xff }, /*    63 */
  {   -40,    31,   -38,0, 0x0000,0x0000, 0x84,0xdd,0xdd,0xff }, /*    64 */
  {    40,    31,   -38,0, 0x0000,0x0000, 0x48,0x78,0x78,0xff }, /*    65 */
  {    41,    31,    28,0, 0x0779,0x0000, 0xe1,0xe1,0xe1,0xff }, /*    66 */
  {    47,    98,     6,0, 0x07ff,0x070b, 0xdf,0xdf,0xdf,0xff }, /*    67 */
  {   -41,    31,    28,0, 0x0086,0x0000, 0xd3,0xd3,0xd3,0xff }, /*    68 */
  {    47,    98,     6,0, 0x07ff,0x070b, 0xdf,0xdf,0xdf,0xff }, /*    69 */
  {   -11,   107,    -2,0, 0x0302,0x0800, 0xec,0xec,0xec,0xff }, /*    70 */
  {   -47,    98,     6,0, 0x0000,0x070b, 0xe9,0xe9,0xe9,0xff }, /*    71 */
  {    47,    98,     6,0, 0x07ff,0x070b, 0xdf,0xdf,0xdf,0xff }, /*    72 */
  {    11,   107,    -2,0, 0x04fd,0x0800, 0xf8,0xf8,0xf8,0xff }, /*    73 */
  {   -11,   107,    -2,0, 0x0302,0x0800, 0xec,0xec,0xec,0xff }, /*    74 */
  {    47,    98,     6,0, 0x07ff,0x070b, 0xdf,0xdf,0xdf,0xff }, /*    75 */
  {   -47,    98,     6,0, 0x0000,0x070b, 0xe9,0xe9,0xe9,0xff }, /*    76 */
  {   -41,    31,    28,0, 0x0086,0x0000, 0xd3,0xd3,0xd3,0xff }, /*    77 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_39_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP2Triangles(  1,  2,  3,  0,  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 992), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 1024), 14, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsDPPipeSync(),
  gsDPSetCombineLERP(TEXEL0, 0, SHADE, 0, SHADE, 0, TEXEL0, 0, TEXEL0, 0, SHADE, 0, SHADE, 0, TEXEL0, 0),
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_ON),
  gsDPSetTextureLUT( G_TT_NONE ),
  gsDPLoadMultiBlock( SEGADRS(6,0), 0, 0,
    G_IM_FMT_RGBA, G_IM_SIZ_16b, 32, 32, 0,
    0, 0, 5, 5, 0, 0 ),
  gsSP1Triangle(  2,  3,  4,  0),
  gsSP1Triangle(  5,  6,  7,  0),
  gsSP1Triangle(  8,  9, 10,  0),
  gsSP1Triangle( 11, 12, 13,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_39_ = {
  gfx_Shape_of___POLYHEDRON_39_,
  vtx_Shape_of___POLYHEDRON_39_, 78, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner2[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_53_[] = {
  {   -92,    45,     4,0, 0x0000,0x0000, 0xa7,0x79,0x57,0xff }, /*     0 */
  {   -52,    54,     5,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*     1 */
  {   -75,    71,     5,0, 0x0000,0x0000, 0xf7,0xb2,0x80,0xff }, /*     2 */
  {   -92,    45,     4,0, 0x0000,0x0000, 0xa7,0x79,0x57,0xff }, /*     3 */
  {   -71,    30,     4,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*     4 */
  {   -52,    54,     5,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*     5 */
  {   -71,    30,   -20,0, 0x0000,0x0000, 0xb3,0x82,0x5d,0xff }, /*     6 */
  {   -92,    45,   -20,0, 0x0000,0x0000, 0xd9,0x9d,0x71,0xff }, /*     7 */
  {   -52,    54,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4e,0xff }, /*     8 */
  {   -71,    30,     4,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*     9 */
  {   -71,    30,   -20,0, 0x0000,0x0000, 0xb3,0x82,0x5d,0xff }, /*    10 */
  {   -52,    54,     5,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*    11 */
  {   -92,    45,   -20,0, 0x0000,0x0000, 0xd9,0x9d,0x71,0xff }, /*    12 */
  {   -75,    71,     5,0, 0x0000,0x0000, 0xf7,0xb2,0x80,0xff }, /*    13 */
  {   -75,    71,   -22,0, 0x0000,0x0000, 0xb8,0x85,0x60,0xff }, /*    14 */
  {   -52,    54,     5,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*    15 */
  {   -56,    73,    -8,0, 0x0000,0x0000, 0xd4,0x9a,0x6e,0xff }, /*    16 */
  {   -75,    71,     5,0, 0x0000,0x0000, 0xf7,0xb2,0x80,0xff }, /*    17 */
  {   -75,    71,     5,0, 0x0000,0x0000, 0xf7,0xb2,0x80,0xff }, /*    18 */
  {   -56,    73,    -8,0, 0x0000,0x0000, 0xd4,0x9a,0x6e,0xff }, /*    19 */
  {   -75,    71,   -22,0, 0x0000,0x0000, 0xb8,0x85,0x60,0xff }, /*    20 */
  {   -75,    71,   -22,0, 0x0000,0x0000, 0xb8,0x85,0x60,0xff }, /*    21 */
  {   -56,    73,    -8,0, 0x0000,0x0000, 0xd4,0x9a,0x6e,0xff }, /*    22 */
  {   -52,    54,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4e,0xff }, /*    23 */
  {   -52,    54,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4e,0xff }, /*    24 */
  {   -56,    73,    -8,0, 0x0000,0x0000, 0xd4,0x9a,0x6e,0xff }, /*    25 */
  {   -52,    54,     5,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*    26 */
  {   -92,    45,   -20,0, 0x0000,0x0000, 0xd9,0x9d,0x71,0xff }, /*    27 */
  {   -92,    45,     4,0, 0x0000,0x0000, 0xa7,0x79,0x57,0xff }, /*    28 */
  {   -75,    71,     5,0, 0x0000,0x0000, 0xf7,0xb2,0x80,0xff }, /*    29 */
  {   -71,    30,   -20,0, 0x0000,0x0000, 0xb3,0x82,0x5d,0xff }, /*    30 */
  {   -52,    54,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4e,0xff }, /*    31 */
  {   -52,    54,     5,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*    32 */
  {   -92,    45,   -20,0, 0x0000,0x0000, 0xd9,0x9d,0x71,0xff }, /*    33 */
  {   -75,    71,   -22,0, 0x0000,0x0000, 0xb8,0x85,0x60,0xff }, /*    34 */
  {   -52,    54,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4e,0xff }, /*    35 */
  {   -92,    45,     4,0, 0x0000,0x0000, 0xa7,0x79,0x57,0xff }, /*    36 */
  {   -88,    29,    -8,0, 0x0000,0x0000, 0x98,0x42,0x2f,0xff }, /*    37 */
  {   -71,    30,     4,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*    38 */
  {   -71,    30,   -20,0, 0x0000,0x0000, 0xb3,0x82,0x5d,0xff }, /*    39 */
  {   -88,    29,    -8,0, 0x0000,0x0000, 0x98,0x42,0x2f,0xff }, /*    40 */
  {   -92,    45,   -20,0, 0x0000,0x0000, 0xd9,0x9d,0x71,0xff }, /*    41 */
  {   -92,    45,   -20,0, 0x0000,0x0000, 0xd9,0x9d,0x71,0xff }, /*    42 */
  {   -88,    29,    -8,0, 0x0000,0x0000, 0x98,0x42,0x2f,0xff }, /*    43 */
  {   -92,    45,     4,0, 0x0000,0x0000, 0xa7,0x79,0x57,0xff }, /*    44 */
  {   -71,    30,     4,0, 0x0000,0x0000, 0x98,0x6e,0x4f,0xff }, /*    45 */
  {   -88,    29,    -8,0, 0x0000,0x0000, 0x98,0x42,0x2f,0xff }, /*    46 */
  {   -71,    30,   -20,0, 0x0000,0x0000, 0xb3,0x82,0x5d,0xff }, /*    47 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_53_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 16, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP2Triangles(  1,  2,  3,  0,  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_53_ = {
  gfx_Shape_of___POLYHEDRON_53_,
  vtx_Shape_of___POLYHEDRON_53_, 48, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner3[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_55_[] = {
  {  -110,    15,    -1,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*     0 */
  {   -74,    29,     3,0, 0x0000,0x0000, 0xd6,0x9b,0x6f,0xff }, /*     1 */
  {   -92,    42,     3,0, 0x0000,0x0000, 0xfb,0xb5,0x82,0xff }, /*     2 */
  {  -110,    15,    -1,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*     3 */
  {   -94,     3,    -1,0, 0x0000,0x0000, 0x94,0x6b,0x4d,0xff }, /*     4 */
  {   -74,    29,     3,0, 0x0000,0x0000, 0xd6,0x9b,0x6f,0xff }, /*     5 */
  {   -94,     3,   -15,0, 0x0000,0x0000, 0xaf,0x7f,0x5b,0xff }, /*     6 */
  {  -110,    15,   -15,0, 0x0000,0x0000, 0xd7,0x9c,0x70,0xff }, /*     7 */
  {   -74,    29,   -19,0, 0x0000,0x0000, 0x9f,0x73,0x52,0xff }, /*     8 */
  {   -94,     3,    -1,0, 0x0000,0x0000, 0x94,0x6b,0x4d,0xff }, /*     9 */
  {   -94,     3,   -15,0, 0x0000,0x0000, 0xaf,0x7f,0x5b,0xff }, /*    10 */
  {   -74,    29,     3,0, 0x0000,0x0000, 0xd6,0x9b,0x6f,0xff }, /*    11 */
  {  -110,    15,   -15,0, 0x0000,0x0000, 0xd7,0x9c,0x70,0xff }, /*    12 */
  {   -92,    42,     3,0, 0x0000,0x0000, 0xfb,0xb5,0x82,0xff }, /*    13 */
  {   -92,    42,   -19,0, 0x0000,0x0000, 0xb7,0x85,0x5f,0xff }, /*    14 */
  {   -74,    29,     3,0, 0x0000,0x0000, 0xd6,0x9b,0x6f,0xff }, /*    15 */
  {   -77,    44,    -8,0, 0x0000,0x0000, 0xd5,0x9a,0x6e,0xff }, /*    16 */
  {   -92,    42,     3,0, 0x0000,0x0000, 0xfb,0xb5,0x82,0xff }, /*    17 */
  {   -92,    42,     3,0, 0x0000,0x0000, 0xfb,0xb5,0x82,0xff }, /*    18 */
  {   -77,    44,    -8,0, 0x0000,0x0000, 0xd5,0x9a,0x6e,0xff }, /*    19 */
  {   -92,    42,   -19,0, 0x0000,0x0000, 0xb7,0x85,0x5f,0xff }, /*    20 */
  {   -92,    42,   -19,0, 0x0000,0x0000, 0xb7,0x85,0x5f,0xff }, /*    21 */
  {   -77,    44,    -8,0, 0x0000,0x0000, 0xd5,0x9a,0x6e,0xff }, /*    22 */
  {   -74,    29,   -19,0, 0x0000,0x0000, 0x9f,0x73,0x52,0xff }, /*    23 */
  {   -74,    29,   -19,0, 0x0000,0x0000, 0x9f,0x73,0x52,0xff }, /*    24 */
  {   -77,    44,    -8,0, 0x0000,0x0000, 0xd5,0x9a,0x6e,0xff }, /*    25 */
  {   -74,    29,     3,0, 0x0000,0x0000, 0xd6,0x9b,0x6f,0xff }, /*    26 */
  {  -110,    15,   -15,0, 0x0000,0x0000, 0xd7,0x9c,0x70,0xff }, /*    27 */
  {  -110,    15,    -1,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    28 */
  {   -92,    42,     3,0, 0x0000,0x0000, 0xfb,0xb5,0x82,0xff }, /*    29 */
  {   -94,     3,   -15,0, 0x0000,0x0000, 0xaf,0x7f,0x5b,0xff }, /*    30 */
  {   -74,    29,   -19,0, 0x0000,0x0000, 0x9f,0x73,0x52,0xff }, /*    31 */
  {   -74,    29,     3,0, 0x0000,0x0000, 0xd6,0x9b,0x6f,0xff }, /*    32 */
  {  -110,    15,   -15,0, 0x0000,0x0000, 0xd7,0x9c,0x70,0xff }, /*    33 */
  {   -92,    42,   -19,0, 0x0000,0x0000, 0xb7,0x85,0x5f,0xff }, /*    34 */
  {   -74,    29,   -19,0, 0x0000,0x0000, 0x9f,0x73,0x52,0xff }, /*    35 */
  {   -94,     3,   -15,0, 0x0000,0x0000, 0xaf,0x7f,0x5b,0xff }, /*    36 */
  {  -107,     2,    -8,0, 0x0000,0x0000, 0x8f,0x67,0x4a,0xff }, /*    37 */
  {  -110,    15,   -15,0, 0x0000,0x0000, 0xd7,0x9c,0x70,0xff }, /*    38 */
  {  -110,    15,   -15,0, 0x0000,0x0000, 0xd7,0x9c,0x70,0xff }, /*    39 */
  {  -107,     2,    -8,0, 0x0000,0x0000, 0x8f,0x67,0x4a,0xff }, /*    40 */
  {  -110,    15,    -1,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    41 */
  {  -110,    15,    -1,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    42 */
  {  -107,     2,    -8,0, 0x0000,0x0000, 0x8f,0x67,0x4a,0xff }, /*    43 */
  {   -94,     3,    -1,0, 0x0000,0x0000, 0x94,0x6b,0x4d,0xff }, /*    44 */
  {   -94,     3,    -1,0, 0x0000,0x0000, 0x94,0x6b,0x4d,0xff }, /*    45 */
  {  -107,     2,    -8,0, 0x0000,0x0000, 0x8f,0x67,0x4a,0xff }, /*    46 */
  {   -94,     3,   -15,0, 0x0000,0x0000, 0xaf,0x7f,0x5b,0xff }, /*    47 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_55_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 16, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_55_ = {
  gfx_Shape_of___POLYHEDRON_55_,
  vtx_Shape_of___POLYHEDRON_55_, 48, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner4[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_57_[] = {
  {  -127,    -9,     2,0, 0x0000,0x0000, 0xac,0x7d,0x59,0xff }, /*     0 */
  {  -103,    -5,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*     1 */
  {  -116,     3,     3,0, 0x0000,0x0000, 0xff,0xb9,0x85,0xff }, /*     2 */
  {  -127,    -9,     2,0, 0x0000,0x0000, 0xac,0x7d,0x59,0xff }, /*     3 */
  {  -113,   -19,     2,0, 0x0000,0x0000, 0x91,0x69,0x4b,0xff }, /*     4 */
  {  -103,    -5,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*     5 */
  {  -113,   -19,   -24,0, 0x0000,0x0000, 0xaf,0x7e,0x5b,0xff }, /*     6 */
  {  -127,    -9,   -24,0, 0x0000,0x0000, 0xce,0x95,0x6b,0xff }, /*     7 */
  {  -103,    -5,   -19,0, 0x0000,0x0000, 0x9a,0x6f,0x50,0xff }, /*     8 */
  {  -113,   -19,     2,0, 0x0000,0x0000, 0x91,0x69,0x4b,0xff }, /*     9 */
  {  -113,   -19,   -24,0, 0x0000,0x0000, 0xaf,0x7e,0x5b,0xff }, /*    10 */
  {  -103,    -5,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    11 */
  {  -127,    -9,   -24,0, 0x0000,0x0000, 0xce,0x95,0x6b,0xff }, /*    12 */
  {  -116,     3,     3,0, 0x0000,0x0000, 0xff,0xb9,0x85,0xff }, /*    13 */
  {  -116,     3,   -19,0, 0x0000,0x0000, 0xd3,0x98,0x6d,0xff }, /*    14 */
  {  -103,    -5,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    15 */
  {  -103,     7,    -8,0, 0x0000,0x0000, 0xda,0x9e,0x71,0xff }, /*    16 */
  {  -116,     3,     3,0, 0x0000,0x0000, 0xff,0xb9,0x85,0xff }, /*    17 */
  {  -127,    -9,   -24,0, 0x0000,0x0000, 0xce,0x95,0x6b,0xff }, /*    18 */
  {  -127,    -9,     2,0, 0x0000,0x0000, 0xac,0x7d,0x59,0xff }, /*    19 */
  {  -116,     3,     3,0, 0x0000,0x0000, 0xff,0xb9,0x85,0xff }, /*    20 */
  {  -116,     3,     3,0, 0x0000,0x0000, 0xff,0xb9,0x85,0xff }, /*    21 */
  {  -103,     7,    -8,0, 0x0000,0x0000, 0xda,0x9e,0x71,0xff }, /*    22 */
  {  -116,     3,   -19,0, 0x0000,0x0000, 0xd3,0x98,0x6d,0xff }, /*    23 */
  {  -116,     3,   -19,0, 0x0000,0x0000, 0xd3,0x98,0x6d,0xff }, /*    24 */
  {  -103,     7,    -8,0, 0x0000,0x0000, 0xda,0x9e,0x71,0xff }, /*    25 */
  {  -103,    -5,   -19,0, 0x0000,0x0000, 0x9a,0x6f,0x50,0xff }, /*    26 */
  {  -103,    -5,   -19,0, 0x0000,0x0000, 0x9a,0x6f,0x50,0xff }, /*    27 */
  {  -103,     7,    -8,0, 0x0000,0x0000, 0xda,0x9e,0x71,0xff }, /*    28 */
  {  -103,    -5,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    29 */
  {  -113,   -19,   -24,0, 0x0000,0x0000, 0xaf,0x7e,0x5b,0xff }, /*    30 */
  {  -103,    -5,   -19,0, 0x0000,0x0000, 0x9a,0x6f,0x50,0xff }, /*    31 */
  {  -103,    -5,     3,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    32 */
  {  -127,    -9,   -24,0, 0x0000,0x0000, 0xce,0x95,0x6b,0xff }, /*    33 */
  {  -116,     3,   -19,0, 0x0000,0x0000, 0xd3,0x98,0x6d,0xff }, /*    34 */
  {  -103,    -5,   -19,0, 0x0000,0x0000, 0x9a,0x6f,0x50,0xff }, /*    35 */
  {  -113,   -19,   -24,0, 0x0000,0x0000, 0xaf,0x7e,0x5b,0xff }, /*    36 */
  {  -124,   -20,    -8,0, 0x0000,0x0000, 0x8b,0x65,0x48,0xff }, /*    37 */
  {  -127,    -9,   -24,0, 0x0000,0x0000, 0xce,0x95,0x6b,0xff }, /*    38 */
  {  -127,    -9,   -24,0, 0x0000,0x0000, 0xce,0x95,0x6b,0xff }, /*    39 */
  {  -124,   -20,    -8,0, 0x0000,0x0000, 0x8b,0x65,0x48,0xff }, /*    40 */
  {  -127,    -9,     2,0, 0x0000,0x0000, 0xac,0x7d,0x59,0xff }, /*    41 */
  {  -127,    -9,     2,0, 0x0000,0x0000, 0xac,0x7d,0x59,0xff }, /*    42 */
  {  -124,   -20,    -8,0, 0x0000,0x0000, 0x8b,0x65,0x48,0xff }, /*    43 */
  {  -113,   -19,     2,0, 0x0000,0x0000, 0x91,0x69,0x4b,0xff }, /*    44 */
  {  -113,   -19,     2,0, 0x0000,0x0000, 0x91,0x69,0x4b,0xff }, /*    45 */
  {  -124,   -20,    -8,0, 0x0000,0x0000, 0x8b,0x65,0x48,0xff }, /*    46 */
  {  -113,   -19,   -24,0, 0x0000,0x0000, 0xaf,0x7e,0x5b,0xff }, /*    47 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_57_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 16, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_57_ = {
  gfx_Shape_of___POLYHEDRON_57_,
  vtx_Shape_of___POLYHEDRON_57_, 48, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner5[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_51_[] = {
  {   -81,    69,     9,0, 0x0000,0x0000, 0x64,0xa7,0xa7,0xff }, /*     0 */
  {   -34,    80,     5,0, 0x0000,0x0000, 0x89,0xe4,0xe4,0xff }, /*     1 */
  {   -57,    97,     5,0, 0x0000,0x0000, 0x96,0xfa,0xfa,0xff }, /*     2 */
  {   -81,    69,     9,0, 0x0000,0x0000, 0x64,0xa7,0xa7,0xff }, /*     3 */
  {   -51,    48,     9,0, 0x0000,0x0000, 0x64,0xa7,0xa7,0xff }, /*     4 */
  {   -34,    80,     5,0, 0x0000,0x0000, 0x89,0xe4,0xe4,0xff }, /*     5 */
  {   -51,    48,   -26,0, 0x0000,0x0000, 0x6e,0xb8,0xb8,0xff }, /*     6 */
  {   -81,    69,   -26,0, 0x0000,0x0000, 0x89,0xe4,0xe4,0xff }, /*     7 */
  {   -34,    80,   -22,0, 0x0000,0x0000, 0x51,0x88,0x88,0xff }, /*     8 */
  {   -51,    48,     9,0, 0x0000,0x0000, 0x64,0xa7,0xa7,0xff }, /*     9 */
  {   -51,    48,   -26,0, 0x0000,0x0000, 0x6e,0xb8,0xb8,0xff }, /*    10 */
  {   -34,    80,     5,0, 0x0000,0x0000, 0x89,0xe4,0xe4,0xff }, /*    11 */
  {   -81,    69,   -26,0, 0x0000,0x0000, 0x89,0xe4,0xe4,0xff }, /*    12 */
  {   -57,    97,     5,0, 0x0000,0x0000, 0x96,0xfa,0xfa,0xff }, /*    13 */
  {   -57,    97,   -22,0, 0x0000,0x0000, 0x7a,0xcc,0xcc,0xff }, /*    14 */
  {   -34,    80,     5,0, 0x0000,0x0000, 0x89,0xe4,0xe4,0xff }, /*    15 */
  {   -38,    99,    -8,0, 0x0000,0x0000, 0x84,0xdc,0xdc,0xff }, /*    16 */
  {   -57,    97,     5,0, 0x0000,0x0000, 0x96,0xfa,0xfa,0xff }, /*    17 */
  {   -57,    97,     5,0, 0x0000,0x0000, 0x96,0xfa,0xfa,0xff }, /*    18 */
  {   -38,    99,    -8,0, 0x0000,0x0000, 0x84,0xdc,0xdc,0xff }, /*    19 */
  {   -57,    97,   -22,0, 0x0000,0x0000, 0x7a,0xcc,0xcc,0xff }, /*    20 */
  {   -57,    97,   -22,0, 0x0000,0x0000, 0x7a,0xcc,0xcc,0xff }, /*    21 */
  {   -38,    99,    -8,0, 0x0000,0x0000, 0x84,0xdc,0xdc,0xff }, /*    22 */
  {   -34,    80,   -22,0, 0x0000,0x0000, 0x51,0x88,0x88,0xff }, /*    23 */
  {   -34,    80,   -22,0, 0x0000,0x0000, 0x51,0x88,0x88,0xff }, /*    24 */
  {   -38,    99,    -8,0, 0x0000,0x0000, 0x84,0xdc,0xdc,0xff }, /*    25 */
  {   -34,    80,     5,0, 0x0000,0x0000, 0x89,0xe4,0xe4,0xff }, /*    26 */
  {   -81,    69,   -26,0, 0x0000,0x0000, 0x89,0xe4,0xe4,0xff }, /*    27 */
  {   -81,    69,     9,0, 0x0000,0x0000, 0x64,0xa7,0xa7,0xff }, /*    28 */
  {   -57,    97,     5,0, 0x0000,0x0000, 0x96,0xfa,0xfa,0xff }, /*    29 */
  {   -51,    48,   -26,0, 0x0000,0x0000, 0x6e,0xb8,0xb8,0xff }, /*    30 */
  {   -34,    80,   -22,0, 0x0000,0x0000, 0x51,0x88,0x88,0xff }, /*    31 */
  {   -34,    80,     5,0, 0x0000,0x0000, 0x89,0xe4,0xe4,0xff }, /*    32 */
  {   -81,    69,   -26,0, 0x0000,0x0000, 0x89,0xe4,0xe4,0xff }, /*    33 */
  {   -57,    97,   -22,0, 0x0000,0x0000, 0x7a,0xcc,0xcc,0xff }, /*    34 */
  {   -34,    80,   -22,0, 0x0000,0x0000, 0x51,0x88,0x88,0xff }, /*    35 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_51_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 4, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_51_ = {
  gfx_Shape_of___POLYHEDRON_51_,
  vtx_Shape_of___POLYHEDRON_51_, 36, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner6[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_45_[] = {
  {    75,    71,     5,0, 0x0000,0x0000, 0xeb,0xaa,0x7a,0xff }, /*     0 */
  {    52,    54,     5,0, 0x0000,0x0000, 0x9c,0x71,0x51,0xff }, /*     1 */
  {    92,    45,     4,0, 0x0000,0x0000, 0xd0,0x96,0x6c,0xff }, /*     2 */
  {    52,    54,     5,0, 0x0000,0x0000, 0x9c,0x71,0x51,0xff }, /*     3 */
  {    71,    30,     4,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*     4 */
  {    92,    45,     4,0, 0x0000,0x0000, 0xd0,0x96,0x6c,0xff }, /*     5 */
  {    71,    30,   -20,0, 0x0000,0x0000, 0xb9,0x85,0x60,0xff }, /*     6 */
  {    88,    29,    -8,0, 0x0000,0x0000, 0xaa,0x4a,0x35,0xff }, /*     7 */
  {    71,    30,     4,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*     8 */
  {    52,    54,   -22,0, 0x0000,0x0000, 0xed,0xac,0x7b,0xff }, /*     9 */
  {    92,    45,   -20,0, 0x0000,0x0000, 0x68,0x4b,0x36,0xff }, /*    10 */
  {    71,    30,   -20,0, 0x0000,0x0000, 0xb9,0x85,0x60,0xff }, /*    11 */
  {    52,    54,     5,0, 0x0000,0x0000, 0x9c,0x71,0x51,0xff }, /*    12 */
  {    71,    30,   -20,0, 0x0000,0x0000, 0xb9,0x85,0x60,0xff }, /*    13 */
  {    71,    30,     4,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*    14 */
  {    75,    71,   -22,0, 0x0000,0x0000, 0xbe,0x89,0x62,0xff }, /*    15 */
  {    75,    71,     5,0, 0x0000,0x0000, 0xeb,0xaa,0x7a,0xff }, /*    16 */
  {    92,    45,   -20,0, 0x0000,0x0000, 0x68,0x4b,0x36,0xff }, /*    17 */
  {    75,    71,     5,0, 0x0000,0x0000, 0xeb,0xaa,0x7a,0xff }, /*    18 */
  {    92,    45,     4,0, 0x0000,0x0000, 0xd0,0x96,0x6c,0xff }, /*    19 */
  {    92,    45,   -20,0, 0x0000,0x0000, 0x68,0x4b,0x36,0xff }, /*    20 */
  {    75,    71,     5,0, 0x0000,0x0000, 0xeb,0xaa,0x7a,0xff }, /*    21 */
  {    56,    73,    -8,0, 0x0000,0x0000, 0xf6,0xb2,0x80,0xff }, /*    22 */
  {    52,    54,     5,0, 0x0000,0x0000, 0x9c,0x71,0x51,0xff }, /*    23 */
  {    75,    71,   -22,0, 0x0000,0x0000, 0xbe,0x89,0x62,0xff }, /*    24 */
  {    56,    73,    -8,0, 0x0000,0x0000, 0xf6,0xb2,0x80,0xff }, /*    25 */
  {    75,    71,     5,0, 0x0000,0x0000, 0xeb,0xaa,0x7a,0xff }, /*    26 */
  {    52,    54,   -22,0, 0x0000,0x0000, 0xed,0xac,0x7b,0xff }, /*    27 */
  {    56,    73,    -8,0, 0x0000,0x0000, 0xf6,0xb2,0x80,0xff }, /*    28 */
  {    75,    71,   -22,0, 0x0000,0x0000, 0xbe,0x89,0x62,0xff }, /*    29 */
  {    52,    54,     5,0, 0x0000,0x0000, 0x9c,0x71,0x51,0xff }, /*    30 */
  {    56,    73,    -8,0, 0x0000,0x0000, 0xf6,0xb2,0x80,0xff }, /*    31 */
  {    52,    54,   -22,0, 0x0000,0x0000, 0xed,0xac,0x7b,0xff }, /*    32 */
  {    52,    54,     5,0, 0x0000,0x0000, 0x9c,0x71,0x51,0xff }, /*    33 */
  {    52,    54,   -22,0, 0x0000,0x0000, 0xed,0xac,0x7b,0xff }, /*    34 */
  {    71,    30,   -20,0, 0x0000,0x0000, 0xb9,0x85,0x60,0xff }, /*    35 */
  {    52,    54,   -22,0, 0x0000,0x0000, 0xed,0xac,0x7b,0xff }, /*    36 */
  {    75,    71,   -22,0, 0x0000,0x0000, 0xbe,0x89,0x62,0xff }, /*    37 */
  {    92,    45,   -20,0, 0x0000,0x0000, 0x68,0x4b,0x36,0xff }, /*    38 */
  {    92,    45,   -20,0, 0x0000,0x0000, 0x68,0x4b,0x36,0xff }, /*    39 */
  {    88,    29,    -8,0, 0x0000,0x0000, 0xaa,0x4a,0x35,0xff }, /*    40 */
  {    71,    30,   -20,0, 0x0000,0x0000, 0xb9,0x85,0x60,0xff }, /*    41 */
  {    92,    45,     4,0, 0x0000,0x0000, 0xd0,0x96,0x6c,0xff }, /*    42 */
  {    88,    29,    -8,0, 0x0000,0x0000, 0xaa,0x4a,0x35,0xff }, /*    43 */
  {    92,    45,   -20,0, 0x0000,0x0000, 0x68,0x4b,0x36,0xff }, /*    44 */
  {    71,    30,     4,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*    45 */
  {    88,    29,    -8,0, 0x0000,0x0000, 0xaa,0x4a,0x35,0xff }, /*    46 */
  {    92,    45,     4,0, 0x0000,0x0000, 0xd0,0x96,0x6c,0xff }, /*    47 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_45_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 16, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_45_ = {
  gfx_Shape_of___POLYHEDRON_45_,
  vtx_Shape_of___POLYHEDRON_45_, 48, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner7[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_47_[] = {
  {    92,    42,     3,0, 0x0000,0x0000, 0xe9,0xa8,0x79,0xff }, /*     0 */
  {    74,    29,     3,0, 0x0000,0x0000, 0x9c,0x70,0x51,0xff }, /*     1 */
  {   110,    15,    -1,0, 0x0000,0x0000, 0xd2,0x97,0x6d,0xff }, /*     2 */
  {    74,    29,     3,0, 0x0000,0x0000, 0x9c,0x70,0x51,0xff }, /*     3 */
  {    94,     3,    -1,0, 0x0000,0x0000, 0xb4,0x82,0x5e,0xff }, /*     4 */
  {   110,    15,    -1,0, 0x0000,0x0000, 0xd2,0x97,0x6d,0xff }, /*     5 */
  {    74,    29,   -19,0, 0x0000,0x0000, 0xed,0xac,0x7b,0xff }, /*     6 */
  {   110,    15,   -15,0, 0x0000,0x0000, 0x62,0x46,0x33,0xff }, /*     7 */
  {    94,     3,   -15,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*     8 */
  {    74,    29,     3,0, 0x0000,0x0000, 0x9c,0x70,0x51,0xff }, /*     9 */
  {    94,     3,   -15,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*    10 */
  {    94,     3,    -1,0, 0x0000,0x0000, 0xb4,0x82,0x5e,0xff }, /*    11 */
  {    92,    42,   -19,0, 0x0000,0x0000, 0xbc,0x88,0x61,0xff }, /*    12 */
  {    92,    42,     3,0, 0x0000,0x0000, 0xe9,0xa8,0x79,0xff }, /*    13 */
  {   110,    15,   -15,0, 0x0000,0x0000, 0x62,0x46,0x33,0xff }, /*    14 */
  {    92,    42,     3,0, 0x0000,0x0000, 0xe9,0xa8,0x79,0xff }, /*    15 */
  {   110,    15,    -1,0, 0x0000,0x0000, 0xd2,0x97,0x6d,0xff }, /*    16 */
  {   110,    15,   -15,0, 0x0000,0x0000, 0x62,0x46,0x33,0xff }, /*    17 */
  {    92,    42,     3,0, 0x0000,0x0000, 0xe9,0xa8,0x79,0xff }, /*    18 */
  {    77,    44,    -8,0, 0x0000,0x0000, 0xff,0xba,0x86,0xff }, /*    19 */
  {    74,    29,     3,0, 0x0000,0x0000, 0x9c,0x70,0x51,0xff }, /*    20 */
  {    92,    42,   -19,0, 0x0000,0x0000, 0xbc,0x88,0x61,0xff }, /*    21 */
  {    77,    44,    -8,0, 0x0000,0x0000, 0xff,0xba,0x86,0xff }, /*    22 */
  {    92,    42,     3,0, 0x0000,0x0000, 0xe9,0xa8,0x79,0xff }, /*    23 */
  {    74,    29,   -19,0, 0x0000,0x0000, 0xed,0xac,0x7b,0xff }, /*    24 */
  {    77,    44,    -8,0, 0x0000,0x0000, 0xff,0xba,0x86,0xff }, /*    25 */
  {    92,    42,   -19,0, 0x0000,0x0000, 0xbc,0x88,0x61,0xff }, /*    26 */
  {    74,    29,     3,0, 0x0000,0x0000, 0x9c,0x70,0x51,0xff }, /*    27 */
  {    77,    44,    -8,0, 0x0000,0x0000, 0xff,0xba,0x86,0xff }, /*    28 */
  {    74,    29,   -19,0, 0x0000,0x0000, 0xed,0xac,0x7b,0xff }, /*    29 */
  {    74,    29,     3,0, 0x0000,0x0000, 0x9c,0x70,0x51,0xff }, /*    30 */
  {    74,    29,   -19,0, 0x0000,0x0000, 0xed,0xac,0x7b,0xff }, /*    31 */
  {    94,     3,   -15,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*    32 */
  {    74,    29,   -19,0, 0x0000,0x0000, 0xed,0xac,0x7b,0xff }, /*    33 */
  {    92,    42,   -19,0, 0x0000,0x0000, 0xbc,0x88,0x61,0xff }, /*    34 */
  {   110,    15,   -15,0, 0x0000,0x0000, 0x62,0x46,0x33,0xff }, /*    35 */
  {   110,    15,   -15,0, 0x0000,0x0000, 0x62,0x46,0x33,0xff }, /*    36 */
  {   107,     2,    -8,0, 0x0000,0x0000, 0xaa,0x7b,0x58,0xff }, /*    37 */
  {    94,     3,   -15,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*    38 */
  {    94,     3,   -15,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*    39 */
  {   107,     2,    -8,0, 0x0000,0x0000, 0xaa,0x7b,0x58,0xff }, /*    40 */
  {    94,     3,    -1,0, 0x0000,0x0000, 0xb4,0x82,0x5e,0xff }, /*    41 */
  {   110,    15,    -1,0, 0x0000,0x0000, 0xd2,0x97,0x6d,0xff }, /*    42 */
  {   107,     2,    -8,0, 0x0000,0x0000, 0xaa,0x7b,0x58,0xff }, /*    43 */
  {   110,    15,   -15,0, 0x0000,0x0000, 0x62,0x46,0x33,0xff }, /*    44 */
  {    94,     3,    -1,0, 0x0000,0x0000, 0xb4,0x82,0x5e,0xff }, /*    45 */
  {   107,     2,    -8,0, 0x0000,0x0000, 0xaa,0x7b,0x58,0xff }, /*    46 */
  {   110,    15,    -1,0, 0x0000,0x0000, 0xd2,0x97,0x6d,0xff }, /*    47 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_47_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP2Triangles( 12, 13, 14,  0, 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 16, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP2Triangles(  4,  5,  6,  0,  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_47_ = {
  gfx_Shape_of___POLYHEDRON_47_,
  vtx_Shape_of___POLYHEDRON_47_, 48, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner8[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_49_[] = {
  {   116,     3,     3,0, 0x0000,0x0000, 0xea,0xa9,0x79,0xff }, /*     0 */
  {   103,    -5,     3,0, 0x0000,0x0000, 0xa0,0x74,0x53,0xff }, /*     1 */
  {   127,    -9,     2,0, 0x0000,0x0000, 0xd1,0x97,0x6d,0xff }, /*     2 */
  {   103,    -5,     3,0, 0x0000,0x0000, 0xa0,0x74,0x53,0xff }, /*     3 */
  {   113,   -19,     2,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*     4 */
  {   127,    -9,     2,0, 0x0000,0x0000, 0xd1,0x97,0x6d,0xff }, /*     5 */
  {   103,    -5,   -19,0, 0x0000,0x0000, 0xf8,0xb3,0x81,0xff }, /*     6 */
  {   127,    -9,   -24,0, 0x0000,0x0000, 0x60,0x45,0x32,0xff }, /*     7 */
  {   113,   -19,   -24,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*     8 */
  {   103,    -5,     3,0, 0x0000,0x0000, 0xa0,0x74,0x53,0xff }, /*     9 */
  {   113,   -19,   -24,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    10 */
  {   113,   -19,     2,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*    11 */
  {   116,     3,   -19,0, 0x0000,0x0000, 0xd3,0x99,0x6e,0xff }, /*    12 */
  {   116,     3,     3,0, 0x0000,0x0000, 0xea,0xa9,0x79,0xff }, /*    13 */
  {   127,    -9,   -24,0, 0x0000,0x0000, 0x60,0x45,0x32,0xff }, /*    14 */
  {   116,     3,     3,0, 0x0000,0x0000, 0xea,0xa9,0x79,0xff }, /*    15 */
  {   103,     7,    -8,0, 0x0000,0x0000, 0xff,0xc5,0x8d,0xff }, /*    16 */
  {   103,    -5,     3,0, 0x0000,0x0000, 0xa0,0x74,0x53,0xff }, /*    17 */
  {   116,     3,   -19,0, 0x0000,0x0000, 0xd3,0x99,0x6e,0xff }, /*    18 */
  {   103,     7,    -8,0, 0x0000,0x0000, 0xff,0xc5,0x8d,0xff }, /*    19 */
  {   116,     3,     3,0, 0x0000,0x0000, 0xea,0xa9,0x79,0xff }, /*    20 */
  {   103,    -5,   -19,0, 0x0000,0x0000, 0xf8,0xb3,0x81,0xff }, /*    21 */
  {   103,     7,    -8,0, 0x0000,0x0000, 0xff,0xc5,0x8d,0xff }, /*    22 */
  {   116,     3,   -19,0, 0x0000,0x0000, 0xd3,0x99,0x6e,0xff }, /*    23 */
  {   103,    -5,     3,0, 0x0000,0x0000, 0xa0,0x74,0x53,0xff }, /*    24 */
  {   103,     7,    -8,0, 0x0000,0x0000, 0xff,0xc5,0x8d,0xff }, /*    25 */
  {   103,    -5,   -19,0, 0x0000,0x0000, 0xf8,0xb3,0x81,0xff }, /*    26 */
  {   116,     3,     3,0, 0x0000,0x0000, 0xea,0xa9,0x79,0xff }, /*    27 */
  {   127,    -9,     2,0, 0x0000,0x0000, 0xd1,0x97,0x6d,0xff }, /*    28 */
  {   127,    -9,   -24,0, 0x0000,0x0000, 0x60,0x45,0x32,0xff }, /*    29 */
  {   103,    -5,     3,0, 0x0000,0x0000, 0xa0,0x74,0x53,0xff }, /*    30 */
  {   103,    -5,   -19,0, 0x0000,0x0000, 0xf8,0xb3,0x81,0xff }, /*    31 */
  {   113,   -19,   -24,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    32 */
  {   103,    -5,   -19,0, 0x0000,0x0000, 0xf8,0xb3,0x81,0xff }, /*    33 */
  {   116,     3,   -19,0, 0x0000,0x0000, 0xd3,0x99,0x6e,0xff }, /*    34 */
  {   127,    -9,   -24,0, 0x0000,0x0000, 0x60,0x45,0x32,0xff }, /*    35 */
  {   127,    -9,   -24,0, 0x0000,0x0000, 0x60,0x45,0x32,0xff }, /*    36 */
  {   124,   -20,    -8,0, 0x0000,0x0000, 0xab,0x7b,0x59,0xff }, /*    37 */
  {   113,   -19,   -24,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    38 */
  {   127,    -9,     2,0, 0x0000,0x0000, 0xd1,0x97,0x6d,0xff }, /*    39 */
  {   124,   -20,    -8,0, 0x0000,0x0000, 0xab,0x7b,0x59,0xff }, /*    40 */
  {   127,    -9,   -24,0, 0x0000,0x0000, 0x60,0x45,0x32,0xff }, /*    41 */
  {   113,   -19,     2,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*    42 */
  {   124,   -20,    -8,0, 0x0000,0x0000, 0xab,0x7b,0x59,0xff }, /*    43 */
  {   127,    -9,     2,0, 0x0000,0x0000, 0xd1,0x97,0x6d,0xff }, /*    44 */
  {   113,   -19,   -24,0, 0x0000,0x0000, 0xa9,0x7a,0x58,0xff }, /*    45 */
  {   124,   -20,    -8,0, 0x0000,0x0000, 0xab,0x7b,0x59,0xff }, /*    46 */
  {   113,   -19,     2,0, 0x0000,0x0000, 0xb0,0x7f,0x5b,0xff }, /*    47 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_49_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 16, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_49_ = {
  gfx_Shape_of___POLYHEDRON_49_,
  vtx_Shape_of___POLYHEDRON_49_, 48, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner9[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_43_[] = {
  {    57,    97,     5,0, 0x0000,0x0000, 0x95,0xf8,0xf8,0xff }, /*     0 */
  {    34,    80,     5,0, 0x0000,0x0000, 0x6a,0xb1,0xb1,0xff }, /*     1 */
  {    81,    69,     9,0, 0x0000,0x0000, 0x7d,0xd1,0xd1,0xff }, /*     2 */
  {    34,    80,     5,0, 0x0000,0x0000, 0x6a,0xb1,0xb1,0xff }, /*     3 */
  {    51,    48,     9,0, 0x0000,0x0000, 0x72,0xbe,0xbe,0xff }, /*     4 */
  {    81,    69,     9,0, 0x0000,0x0000, 0x7d,0xd1,0xd1,0xff }, /*     5 */
  {    34,    80,   -22,0, 0x0000,0x0000, 0x93,0xf6,0xf6,0xff }, /*     6 */
  {    81,    69,   -26,0, 0x0000,0x0000, 0x44,0x72,0x72,0xff }, /*     7 */
  {    51,    48,   -26,0, 0x0000,0x0000, 0x72,0xbe,0xbe,0xff }, /*     8 */
  {    34,    80,     5,0, 0x0000,0x0000, 0x6a,0xb1,0xb1,0xff }, /*     9 */
  {    51,    48,   -26,0, 0x0000,0x0000, 0x72,0xbe,0xbe,0xff }, /*    10 */
  {    51,    48,     9,0, 0x0000,0x0000, 0x72,0xbe,0xbe,0xff }, /*    11 */
  {    57,    97,   -22,0, 0x0000,0x0000, 0x7d,0xd0,0xd0,0xff }, /*    12 */
  {    57,    97,     5,0, 0x0000,0x0000, 0x95,0xf8,0xf8,0xff }, /*    13 */
  {    81,    69,   -26,0, 0x0000,0x0000, 0x44,0x72,0x72,0xff }, /*    14 */
  {    57,    97,     5,0, 0x0000,0x0000, 0x95,0xf8,0xf8,0xff }, /*    15 */
  {    81,    69,     9,0, 0x0000,0x0000, 0x7d,0xd1,0xd1,0xff }, /*    16 */
  {    81,    69,   -26,0, 0x0000,0x0000, 0x44,0x72,0x72,0xff }, /*    17 */
  {    57,    97,     5,0, 0x0000,0x0000, 0x95,0xf8,0xf8,0xff }, /*    18 */
  {    38,    99,    -8,0, 0x0000,0x0000, 0x92,0xf3,0xf3,0xff }, /*    19 */
  {    34,    80,     5,0, 0x0000,0x0000, 0x6a,0xb1,0xb1,0xff }, /*    20 */
  {    57,    97,   -22,0, 0x0000,0x0000, 0x7d,0xd0,0xd0,0xff }, /*    21 */
  {    38,    99,    -8,0, 0x0000,0x0000, 0x92,0xf3,0xf3,0xff }, /*    22 */
  {    57,    97,     5,0, 0x0000,0x0000, 0x95,0xf8,0xf8,0xff }, /*    23 */
  {    34,    80,   -22,0, 0x0000,0x0000, 0x93,0xf6,0xf6,0xff }, /*    24 */
  {    38,    99,    -8,0, 0x0000,0x0000, 0x92,0xf3,0xf3,0xff }, /*    25 */
  {    57,    97,   -22,0, 0x0000,0x0000, 0x7d,0xd0,0xd0,0xff }, /*    26 */
  {    34,    80,     5,0, 0x0000,0x0000, 0x6a,0xb1,0xb1,0xff }, /*    27 */
  {    38,    99,    -8,0, 0x0000,0x0000, 0x92,0xf3,0xf3,0xff }, /*    28 */
  {    34,    80,   -22,0, 0x0000,0x0000, 0x93,0xf6,0xf6,0xff }, /*    29 */
  {    34,    80,     5,0, 0x0000,0x0000, 0x6a,0xb1,0xb1,0xff }, /*    30 */
  {    34,    80,   -22,0, 0x0000,0x0000, 0x93,0xf6,0xf6,0xff }, /*    31 */
  {    51,    48,   -26,0, 0x0000,0x0000, 0x72,0xbe,0xbe,0xff }, /*    32 */
  {    34,    80,   -22,0, 0x0000,0x0000, 0x93,0xf6,0xf6,0xff }, /*    33 */
  {    57,    97,   -22,0, 0x0000,0x0000, 0x7d,0xd0,0xd0,0xff }, /*    34 */
  {    81,    69,   -26,0, 0x0000,0x0000, 0x44,0x72,0x72,0xff }, /*    35 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_43_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP2Triangles( 12, 13, 14,  0, 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 4, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_43_ = {
  gfx_Shape_of___POLYHEDRON_43_,
  vtx_Shape_of___POLYHEDRON_43_, 36, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner10[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_8_[] = {
  {    57,   188,     0,0, 0x0000,0x0000, 0x00,0x39,0x72,0xff }, /*     0 */
  {    36,   132,   -52,0, 0x0000,0x0000, 0x00,0x32,0x65,0xff }, /*     1 */
  {    51,   178,   -65,0, 0x0000,0x0000, 0x00,0x44,0x88,0xff }, /*     2 */
  {    51,   178,   -65,0, 0x0000,0x0000, 0x00,0x44,0x88,0xff }, /*     3 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0x00,0x38,0x71,0xff }, /*     4 */
  {     0,   168,   -91,0, 0x0000,0x0000, 0x00,0x40,0x81,0xff }, /*     5 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x00,0x3d,0x7a,0xff }, /*     6 */
  {   -65,   158,    -6,0, 0x0000,0x0000, 0x00,0x32,0x65,0xff }, /*     7 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0x00,0x41,0x83,0xff }, /*     8 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x00,0x3d,0x7a,0xff }, /*     9 */
  {   -73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*    10 */
  {   -65,   158,    -6,0, 0x0000,0x0000, 0x00,0x32,0x65,0xff }, /*    11 */
  {     0,   168,   -91,0, 0x0000,0x0000, 0x00,0x40,0x81,0xff }, /*    12 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0x00,0x38,0x71,0xff }, /*    13 */
  {   -51,   178,   -65,0, 0x0000,0x0000, 0x00,0x3c,0x79,0xff }, /*    14 */
  {   -65,   160,    12,0, 0x0000,0x0000, 0x00,0x3d,0x7a,0xff }, /*    15 */
  {   -66,   138,    24,0, 0x0000,0x0000, 0x00,0x00,0x46,0xff }, /*    16 */
  {   -71,   160,    11,0, 0x0000,0x0000, 0x00,0x46,0x8d,0xff }, /*    17 */
  {   -51,   178,   -65,0, 0x0000,0x0000, 0x00,0x3c,0x79,0xff }, /*    18 */
  {   -36,   132,   -52,0, 0x0000,0x0000, 0x00,0x40,0x81,0xff }, /*    19 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0x00,0x41,0x83,0xff }, /*    20 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0xd3,0x98,0x6d,0xff }, /*    21 */
  {   -52,   132,     0,0, 0x0000,0x0000, 0xc0,0x8b,0x64,0xff }, /*    22 */
  {   -36,   132,    52,0, 0x0000,0x0000, 0xac,0x7d,0x59,0xff }, /*    23 */
  {    57,   188,     0,0, 0x0000,0x0000, 0xb8,0x85,0x5f,0xff }, /*    24 */
  {    42,   170,    65,0, 0x0000,0x0000, 0xc2,0x5c,0x3c,0xff }, /*    25 */
  {    36,   132,    52,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*    26 */
  {    52,   132,     0,0, 0x0000,0x0000, 0x9b,0x70,0x50,0xff }, /*    27 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*    28 */
  {    36,   132,   -52,0, 0x0000,0x0000, 0xa2,0x75,0x54,0xff }, /*    29 */
  {    36,   132,   -52,0, 0x0000,0x0000, 0xa2,0x75,0x54,0xff }, /*    30 */
  {    11,   114,   -20,0, 0x0000,0x0000, 0xa4,0x76,0x55,0xff }, /*    31 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0xb6,0x84,0x5f,0xff }, /*    32 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0xb6,0x84,0x5f,0xff }, /*    33 */
  {   -11,   114,   -20,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*    34 */
  {   -36,   132,   -52,0, 0x0000,0x0000, 0xcf,0x96,0x6c,0xff }, /*    35 */
  {   -36,   132,   -52,0, 0x0000,0x0000, 0xcf,0x96,0x6c,0xff }, /*    36 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbe,0x89,0x63,0xff }, /*    37 */
  {   -52,   132,     0,0, 0x0000,0x0000, 0xc0,0x8b,0x64,0xff }, /*    38 */
  {   -52,   132,     0,0, 0x0000,0x0000, 0xc0,0x8b,0x64,0xff }, /*    39 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbe,0x89,0x63,0xff }, /*    40 */
  {   -36,   132,    52,0, 0x0000,0x0000, 0xac,0x7d,0x59,0xff }, /*    41 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbe,0x89,0x63,0xff }, /*    42 */
  {   -18,   114,    20,0, 0x0000,0x0000, 0xc1,0x8c,0x64,0xff }, /*    43 */
  {   -36,   132,    52,0, 0x0000,0x0000, 0xac,0x7d,0x59,0xff }, /*    44 */
  {   -36,   132,    52,0, 0x0000,0x0000, 0xac,0x7d,0x59,0xff }, /*    45 */
  {   -18,   114,    20,0, 0x0000,0x0000, 0xc1,0x8c,0x64,0xff }, /*    46 */
  {     0,   114,    28,0, 0x0000,0x0000, 0xc1,0x8c,0x64,0xff }, /*    47 */
  {     0,   114,    28,0, 0x0000,0x0000, 0xc1,0x8c,0x64,0xff }, /*    48 */
  {    18,   114,    20,0, 0x0000,0x0000, 0xa8,0x7a,0x57,0xff }, /*    49 */
  {    36,   132,    52,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*    50 */
  {    36,   132,    52,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*    51 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*    52 */
  {    52,   132,     0,0, 0x0000,0x0000, 0x9b,0x70,0x50,0xff }, /*    53 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x23,0x00,0x46,0xff }, /*    54 */
  {    57,   201,   -84,0, 0x0000,0x0000, 0x1b,0x00,0x36,0xff }, /*    55 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x2e,0x00,0x5c,0xff }, /*    56 */
  {    73,   214,     0,0, 0x0000,0x0000, 0x33,0x00,0x67,0xff }, /*    57 */
  {    51,   229,    48,0, 0x0000,0x0000, 0x3d,0x00,0x7b,0xff }, /*    58 */
  {    64,   204,    47,0, 0x0000,0x0000, 0x3e,0x00,0x7c,0xff }, /*    59 */
  {    64,   204,    47,0, 0x0000,0x0000, 0x3e,0x00,0x7c,0xff }, /*    60 */
  {    51,   229,    48,0, 0x0000,0x0000, 0x3d,0x00,0x7b,0xff }, /*    61 */
  {     0,   210,    68,0, 0x0000,0x0000, 0x2e,0x00,0x5c,0xff }, /*    62 */
  {    51,   229,    48,0, 0x0000,0x0000, 0x3d,0x00,0x7b,0xff }, /*    63 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x2f,0x00,0x5f,0xff }, /*    64 */
  {     0,   210,    68,0, 0x0000,0x0000, 0x2e,0x00,0x5c,0xff }, /*    65 */
  {     0,   210,    68,0, 0x0000,0x0000, 0x2e,0x00,0x5c,0xff }, /*    66 */
  {   -51,   229,    48,0, 0x0000,0x0000, 0x34,0x00,0x68,0xff }, /*    67 */
  {   -64,   204,    47,0, 0x0000,0x0000, 0x35,0x00,0x6b,0xff }, /*    68 */
  {   -64,   204,    47,0, 0x0000,0x0000, 0x35,0x00,0x6b,0xff }, /*    69 */
  {   -51,   229,    48,0, 0x0000,0x0000, 0x34,0x00,0x68,0xff }, /*    70 */
  {   -73,   214,     0,0, 0x0000,0x0000, 0x35,0x00,0x6a,0xff }, /*    71 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x3d,0x00,0x7a,0xff }, /*    72 */
  {   -57,   201,   -84,0, 0x0000,0x0000, 0x43,0x00,0x86,0xff }, /*    73 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x46,0x00,0x8c,0xff }, /*    74 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x46,0x00,0x8c,0xff }, /*    75 */
  {     0,   193,  -119,0, 0x0000,0x0000, 0x31,0x00,0x63,0xff }, /*    76 */
  {     0,   168,  -112,0, 0x0000,0x0000, 0x30,0x00,0x60,0xff }, /*    77 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x46,0x00,0x8c,0xff }, /*    78 */
  {   -57,   201,   -84,0, 0x0000,0x0000, 0x43,0x00,0x86,0xff }, /*    79 */
  {     0,   193,  -119,0, 0x0000,0x0000, 0x31,0x00,0x63,0xff }, /*    80 */
  {     0,   168,  -112,0, 0x0000,0x0000, 0x30,0x00,0x60,0xff }, /*    81 */
  {     0,   193,  -119,0, 0x0000,0x0000, 0x31,0x00,0x63,0xff }, /*    82 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x23,0x00,0x46,0xff }, /*    83 */
  {    36,   132,    52,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*    84 */
  {    52,   132,     0,0, 0x0000,0x0000, 0x9b,0x70,0x50,0xff }, /*    85 */
  {    57,   188,     0,0, 0x0000,0x0000, 0xb8,0x85,0x5f,0xff }, /*    86 */
  {   -36,   132,    52,0, 0x0000,0x0000, 0xac,0x7d,0x59,0xff }, /*    87 */
  {   -42,   170,    65,0, 0x0000,0x0000, 0xa7,0x4f,0x34,0xff }, /*    88 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0xd3,0x98,0x6d,0xff }, /*    89 */
  {     0,   114,    28,0, 0x0000,0x0000, 0xc1,0x8c,0x64,0xff }, /*    90 */
  {    36,   132,    52,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*    91 */
  {   -36,   132,    52,0, 0x0000,0x0000, 0xac,0x7d,0x59,0xff }, /*    92 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0x00,0x41,0x83,0xff }, /*    93 */
  {   -71,   160,    11,0, 0x0000,0x0000, 0x00,0x46,0x8d,0xff }, /*    94 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x00,0x3d,0x7a,0xff }, /*    95 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0x00,0x38,0x71,0xff }, /*    96 */
  {   -36,   132,   -52,0, 0x0000,0x0000, 0x00,0x40,0x81,0xff }, /*    97 */
  {   -51,   178,   -65,0, 0x0000,0x0000, 0x00,0x3c,0x79,0xff }, /*    98 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x00,0x3d,0x7a,0xff }, /*    99 */
  {   -64,   174,    43,0, 0x0000,0x0000, 0x00,0x2d,0x5a,0xff }, /*   100 */
  {   -64,   204,    47,0, 0x0000,0x0000, 0x00,0x35,0x6b,0xff }, /*   101 */
  {   -64,   204,    47,0, 0x0000,0x0000, 0x00,0x35,0x6b,0xff }, /*   102 */
  {   -56,   174,    44,0, 0x0000,0x0000, 0x00,0x2e,0x5c,0xff }, /*   103 */
  {   -42,   204,    49,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   104 */
  {   -71,   160,    11,0, 0x0000,0x0000, 0x00,0x46,0x8d,0xff }, /*   105 */
  {   -66,   138,    24,0, 0x0000,0x0000, 0x00,0x00,0x46,0xff }, /*   106 */
  {   -64,   174,    43,0, 0x0000,0x0000, 0x00,0x2d,0x5a,0xff }, /*   107 */
  {   -54,   149,   -60,0, 0x0000,0x0000, 0x00,0x38,0x71,0xff }, /*   108 */
  {   -57,   115,   -36,0, 0x0000,0x0000, 0x00,0x00,0x40,0xff }, /*   109 */
  {   -54,   149,   -72,0, 0x0000,0x0000, 0x00,0x3a,0x74,0xff }, /*   110 */
  {    51,   178,   -65,0, 0x0000,0x0000, 0x00,0x44,0x88,0xff }, /*   111 */
  {    47,   148,   -68,0, 0x0000,0x0000, 0x00,0x3a,0x74,0xff }, /*   112 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x23,0x46,0xff }, /*   113 */
  {   -51,   178,   -65,0, 0x0000,0x0000, 0x00,0x3c,0x79,0xff }, /*   114 */
  {   -54,   149,   -60,0, 0x0000,0x0000, 0x00,0x38,0x71,0xff }, /*   115 */
  {   -54,   149,   -72,0, 0x0000,0x0000, 0x00,0x3a,0x74,0xff }, /*   116 */
  {   -64,   174,    43,0, 0x0000,0x0000, 0x00,0x2d,0x5a,0xff }, /*   117 */
  {   -66,   138,    24,0, 0x0000,0x0000, 0x00,0x00,0x46,0xff }, /*   118 */
  {   -56,   174,    44,0, 0x0000,0x0000, 0x00,0x2e,0x5c,0xff }, /*   119 */
  {    42,   204,    49,0, 0x0000,0x0000, 0x00,0x2c,0x59,0xff }, /*   120 */
  {    56,   174,    44,0, 0x0000,0x0000, 0x00,0x32,0x65,0xff }, /*   121 */
  {    64,   204,    47,0, 0x0000,0x0000, 0x00,0x3e,0x7c,0xff }, /*   122 */
  {    64,   204,    47,0, 0x0000,0x0000, 0x00,0x3e,0x7c,0xff }, /*   123 */
  {    64,   174,    43,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   124 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x00,0x2e,0x5c,0xff }, /*   125 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x00,0x2e,0x5c,0xff }, /*   126 */
  {    71,   160,    11,0, 0x0000,0x0000, 0x00,0x30,0x61,0xff }, /*   127 */
  {    57,   188,     0,0, 0x0000,0x0000, 0x00,0x39,0x72,0xff }, /*   128 */
  {    56,   174,    44,0, 0x0000,0x0000, 0x00,0x32,0x65,0xff }, /*   129 */
  {    66,   138,    24,0, 0x0000,0x0000, 0x00,0x00,0x46,0xff }, /*   130 */
  {    64,   174,    43,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   131 */
  {    64,   174,    43,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   132 */
  {    66,   138,    24,0, 0x0000,0x0000, 0x00,0x00,0x46,0xff }, /*   133 */
  {    71,   160,    11,0, 0x0000,0x0000, 0x00,0x30,0x61,0xff }, /*   134 */
  {    56,   148,   -81,0, 0x0000,0x0000, 0x00,0x23,0x46,0xff }, /*   135 */
  {    28,   113,   -63,0, 0x0000,0x0000, 0x00,0x00,0x4b,0xff }, /*   136 */
  {     4,   140,  -108,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   137 */
  {    64,   174,    43,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   138 */
  {    71,   160,    11,0, 0x0000,0x0000, 0x00,0x30,0x61,0xff }, /*   139 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x00,0x2e,0x5c,0xff }, /*   140 */
  {    71,   160,    11,0, 0x0000,0x0000, 0x00,0x30,0x61,0xff }, /*   141 */
  {    66,   138,    24,0, 0x0000,0x0000, 0x00,0x00,0x46,0xff }, /*   142 */
  {    65,   160,    12,0, 0x0000,0x0000, 0x00,0x47,0x8f,0xff }, /*   143 */
  {    57,   188,     0,0, 0x0000,0x0000, 0x00,0x39,0x72,0xff }, /*   144 */
  {    65,   158,    -6,0, 0x0000,0x0000, 0x00,0x31,0x63,0xff }, /*   145 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x00,0x2e,0x5c,0xff }, /*   146 */
  {    65,   158,    -6,0, 0x0000,0x0000, 0x00,0x31,0x63,0xff }, /*   147 */
  {    73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   148 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x00,0x2e,0x5c,0xff }, /*   149 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x00,0x2e,0x5c,0xff }, /*   150 */
  {    73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   151 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x23,0x46,0xff }, /*   152 */
  {    57,   188,     0,0, 0x0000,0x0000, 0x00,0x39,0x72,0xff }, /*   153 */
  {    52,   132,     0,0, 0x0000,0x0000, 0x00,0x30,0x60,0xff }, /*   154 */
  {    36,   132,   -52,0, 0x0000,0x0000, 0x00,0x32,0x65,0xff }, /*   155 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x23,0x46,0xff }, /*   156 */
  {    54,   149,   -72,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   157 */
  {    51,   178,   -65,0, 0x0000,0x0000, 0x00,0x44,0x88,0xff }, /*   158 */
  {    65,   158,    -6,0, 0x0000,0x0000, 0x00,0x31,0x63,0xff }, /*   159 */
  {    57,   115,   -36,0, 0x0000,0x0000, 0x00,0x00,0x46,0xff }, /*   160 */
  {    73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   161 */
  {    73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   162 */
  {    57,   115,   -36,0, 0x0000,0x0000, 0x00,0x00,0x46,0xff }, /*   163 */
  {    54,   149,   -72,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   164 */
  {    54,   149,   -72,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   165 */
  {    57,   115,   -36,0, 0x0000,0x0000, 0x00,0x00,0x46,0xff }, /*   166 */
  {    54,   149,   -60,0, 0x0000,0x0000, 0x00,0x42,0x85,0xff }, /*   167 */
  {    54,   149,   -72,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   168 */
  {    54,   149,   -60,0, 0x0000,0x0000, 0x00,0x42,0x85,0xff }, /*   169 */
  {    51,   178,   -65,0, 0x0000,0x0000, 0x00,0x44,0x88,0xff }, /*   170 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x46,0x8c,0xff }, /*   171 */
  {   -73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   172 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x00,0x3d,0x7a,0xff }, /*   173 */
  {    47,   148,   -68,0, 0x0000,0x0000, 0x00,0x3a,0x74,0xff }, /*   174 */
  {    56,   148,   -81,0, 0x0000,0x0000, 0x00,0x23,0x46,0xff }, /*   175 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x23,0x46,0xff }, /*   176 */
  {   -51,   178,   -65,0, 0x0000,0x0000, 0x00,0x3c,0x79,0xff }, /*   177 */
  {   -54,   149,   -72,0, 0x0000,0x0000, 0x00,0x3a,0x74,0xff }, /*   178 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x46,0x8c,0xff }, /*   179 */
  {   -54,   149,   -72,0, 0x0000,0x0000, 0x00,0x3a,0x74,0xff }, /*   180 */
  {   -57,   115,   -36,0, 0x0000,0x0000, 0x00,0x00,0x40,0xff }, /*   181 */
  {   -73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   182 */
  {   -73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   183 */
  {   -57,   115,   -36,0, 0x0000,0x0000, 0x00,0x00,0x40,0xff }, /*   184 */
  {   -65,   158,    -6,0, 0x0000,0x0000, 0x00,0x32,0x65,0xff }, /*   185 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x23,0x46,0xff }, /*   186 */
  {     4,   140,  -108,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   187 */
  {     0,   168,  -112,0, 0x0000,0x0000, 0x00,0x30,0x60,0xff }, /*   188 */
  {     0,   168,  -112,0, 0x0000,0x0000, 0x00,0x30,0x60,0xff }, /*   189 */
  {     4,   140,  -108,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   190 */
  {     0,   168,   -91,0, 0x0000,0x0000, 0x00,0x40,0x81,0xff }, /*   191 */
  {     4,   140,  -108,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   192 */
  {     4,   140,   -91,0, 0x0000,0x0000, 0x00,0x35,0x6a,0xff }, /*   193 */
  {     0,   168,   -91,0, 0x0000,0x0000, 0x00,0x40,0x81,0xff }, /*   194 */
  {    47,   148,   -68,0, 0x0000,0x0000, 0x00,0x3a,0x74,0xff }, /*   195 */
  {    28,   113,   -63,0, 0x0000,0x0000, 0x00,0x00,0x4b,0xff }, /*   196 */
  {    56,   148,   -81,0, 0x0000,0x0000, 0x00,0x23,0x46,0xff }, /*   197 */
  {     4,   140,  -108,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   198 */
  {    28,   113,   -63,0, 0x0000,0x0000, 0x00,0x00,0x4b,0xff }, /*   199 */
  {     4,   140,   -91,0, 0x0000,0x0000, 0x00,0x35,0x6a,0xff }, /*   200 */
  {     0,   168,   -91,0, 0x0000,0x0000, 0x00,0x40,0x81,0xff }, /*   201 */
  {    -4,   140,  -108,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   202 */
  {     0,   168,  -112,0, 0x0000,0x0000, 0x00,0x30,0x60,0xff }, /*   203 */
  {     0,   168,   -91,0, 0x0000,0x0000, 0x00,0x40,0x81,0xff }, /*   204 */
  {    -4,   140,   -91,0, 0x0000,0x0000, 0x00,0x39,0x72,0xff }, /*   205 */
  {    -4,   140,  -108,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   206 */
  {    56,   174,    44,0, 0x0000,0x0000, 0x00,0x32,0x65,0xff }, /*   207 */
  {    64,   174,    43,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   208 */
  {    64,   204,    47,0, 0x0000,0x0000, 0x00,0x3e,0x7c,0xff }, /*   209 */
  {   -64,   204,    47,0, 0x0000,0x0000, 0x00,0x35,0x6b,0xff }, /*   210 */
  {   -64,   174,    43,0, 0x0000,0x0000, 0x00,0x2d,0x5a,0xff }, /*   211 */
  {   -56,   174,    44,0, 0x0000,0x0000, 0x00,0x2e,0x5c,0xff }, /*   212 */
  {     0,   168,  -112,0, 0x0000,0x0000, 0x00,0x30,0x60,0xff }, /*   213 */
  {    -4,   140,  -108,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   214 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x46,0x8c,0xff }, /*   215 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x46,0x8c,0xff }, /*   216 */
  {   -47,   148,   -68,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   217 */
  {   -51,   178,   -65,0, 0x0000,0x0000, 0x00,0x3c,0x79,0xff }, /*   218 */
  {    -4,   140,   -91,0, 0x0000,0x0000, 0x00,0x39,0x72,0xff }, /*   219 */
  {   -28,   113,   -63,0, 0x0000,0x0000, 0x00,0x00,0x49,0xff }, /*   220 */
  {    -4,   140,  -108,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   221 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x00,0x3d,0x7a,0xff }, /*   222 */
  {   -71,   160,    11,0, 0x0000,0x0000, 0x00,0x46,0x8d,0xff }, /*   223 */
  {   -64,   174,    43,0, 0x0000,0x0000, 0x00,0x2d,0x5a,0xff }, /*   224 */
  {    -4,   140,  -108,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   225 */
  {   -28,   113,   -63,0, 0x0000,0x0000, 0x00,0x00,0x49,0xff }, /*   226 */
  {   -56,   148,   -81,0, 0x0000,0x0000, 0x00,0x44,0x89,0xff }, /*   227 */
  {   -56,   148,   -81,0, 0x0000,0x0000, 0x00,0x44,0x89,0xff }, /*   228 */
  {   -28,   113,   -63,0, 0x0000,0x0000, 0x00,0x00,0x49,0xff }, /*   229 */
  {   -47,   148,   -68,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   230 */
  {    51,   178,   -65,0, 0x0000,0x0000, 0x00,0x44,0x88,0xff }, /*   231 */
  {    36,   132,   -52,0, 0x0000,0x0000, 0x00,0x32,0x65,0xff }, /*   232 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0x00,0x38,0x71,0xff }, /*   233 */
  {    42,   170,    65,0, 0x0000,0x0000, 0xc2,0x5c,0x3c,0xff }, /*   234 */
  {    57,   188,     0,0, 0x0000,0x0000, 0xb8,0x85,0x5f,0xff }, /*   235 */
  {    42,   204,    49,0, 0x0000,0x0000, 0x8f,0x67,0x4a,0xff }, /*   236 */
  {   -42,   170,    65,0, 0x0000,0x0000, 0xa7,0x4f,0x34,0xff }, /*   237 */
  {   -42,   204,    49,0, 0x0000,0x0000, 0xa6,0x78,0x56,0xff }, /*   238 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0xd3,0x98,0x6d,0xff }, /*   239 */
  {    64,   204,    47,0, 0x0000,0x0000, 0x3e,0x00,0x7c,0xff }, /*   240 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x2e,0x00,0x5c,0xff }, /*   241 */
  {    73,   214,     0,0, 0x0000,0x0000, 0x33,0x00,0x67,0xff }, /*   242 */
  {   -73,   214,     0,0, 0x0000,0x0000, 0x35,0x00,0x6a,0xff }, /*   243 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x3d,0x00,0x7a,0xff }, /*   244 */
  {   -64,   204,    47,0, 0x0000,0x0000, 0x35,0x00,0x6b,0xff }, /*   245 */
  {     0,   193,  -119,0, 0x0000,0x0000, 0x31,0x00,0x63,0xff }, /*   246 */
  {    57,   201,   -84,0, 0x0000,0x0000, 0x1b,0x00,0x36,0xff }, /*   247 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x23,0x00,0x46,0xff }, /*   248 */
  {     0,   275,    27,0, 0x0000,0x0000, 0x68,0x2f,0x35,0xff }, /*   249 */
  {    -6,   247,    66,0, 0x0000,0x0000, 0x11,0x28,0x76,0xff }, /*   250 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x00,0x06,0x7e,0xff }, /*   251 */
  {     0,   275,    27,0, 0x0000,0x0000, 0x68,0x2f,0x35,0xff }, /*   252 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0x14,0x71,0x35,0xff }, /*   253 */
  {    -6,   247,    66,0, 0x0000,0x0000, 0x11,0x28,0x76,0xff }, /*   254 */
  {     0,   236,    62,0, 0x0000,0x0000, 0xeb,0xc5,0x6e,0xff }, /*   255 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0xbf,0x1a,0x69,0xff }, /*   256 */
  {   -51,   229,    48,0, 0x0000,0x0000, 0xa7,0x05,0x59,0xff }, /*   257 */
  {     0,   236,    62,0, 0x0000,0x0000, 0xeb,0xc5,0x6e,0xff }, /*   258 */
  {    -6,   247,    66,0, 0x0000,0x0000, 0x11,0x28,0x76,0xff }, /*   259 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0xbf,0x1a,0x69,0xff }, /*   260 */
  {   -51,   229,    48,0, 0x0000,0x0000, 0x83,0xf2,0x0e,0xff }, /*   261 */
  {   -59,   273,    28,0, 0x0000,0x0000, 0xb1,0x5c,0x23,0xff }, /*   262 */
  {   -59,   267,    16,0, 0x0000,0x0000, 0x97,0x47,0x03,0xff }, /*   263 */
  {   -59,   267,    16,0, 0x0000,0x0000, 0xfb,0x73,0xcc,0xff }, /*   264 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0x14,0x71,0x35,0xff }, /*   265 */
  {     0,   275,    27,0, 0x0000,0x0000, 0x00,0x7b,0x1b,0xff }, /*   266 */
  {   -59,   267,    16,0, 0x0000,0x0000, 0xfb,0x73,0xcb,0xff }, /*   267 */
  {   -59,   273,    28,0, 0x0000,0x0000, 0xb1,0x5c,0x23,0xff }, /*   268 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0x14,0x71,0x35,0xff }, /*   269 */
  {    51,   229,    48,0, 0x0000,0x0000, 0x15,0xc5,0x6e,0xff }, /*   270 */
  {    50,   241,    55,0, 0x0000,0x0000, 0x41,0x1a,0x69,0xff }, /*   271 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x00,0x06,0x7e,0xff }, /*   272 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x98,0x2f,0x35,0xff }, /*   273 */
  {     6,   247,    66,0, 0x0000,0x0000, 0xef,0x28,0x76,0xff }, /*   274 */
  {     0,   275,    27,0, 0x0000,0x0000, 0x00,0x7b,0x1b,0xff }, /*   275 */
  {     0,   275,    27,0, 0x0000,0x0000, 0x05,0x73,0xcc,0xff }, /*   276 */
  {     6,   279,    37,0, 0x0000,0x0000, 0xec,0x71,0x35,0xff }, /*   277 */
  {    59,   267,    16,0, 0x0000,0x0000, 0x69,0x47,0x03,0xff }, /*   278 */
  {    59,   267,    16,0, 0x0000,0x0000, 0x7d,0xf2,0x0e,0xff }, /*   279 */
  {    59,   273,    28,0, 0x0000,0x0000, 0x4f,0x5c,0x23,0xff }, /*   280 */
  {    51,   229,    48,0, 0x0000,0x0000, 0x59,0x05,0x59,0xff }, /*   281 */
  {    59,   273,    28,0, 0x0000,0x0000, 0x79,0xfd,0x23,0xff }, /*   282 */
  {    50,   241,    55,0, 0x0000,0x0000, 0x41,0x1a,0x69,0xff }, /*   283 */
  {    51,   229,    48,0, 0x0000,0x0000, 0x59,0x05,0x59,0xff }, /*   284 */
  {     6,   247,    66,0, 0x0000,0x0000, 0x98,0x2f,0x35,0xff }, /*   285 */
  {     6,   279,    37,0, 0x0000,0x0000, 0xec,0x71,0x35,0xff }, /*   286 */
  {     0,   275,    27,0, 0x0000,0x0000, 0x00,0x7b,0x1b,0xff }, /*   287 */
  {     6,   279,    37,0, 0x0000,0x0000, 0x05,0x73,0xcb,0xff }, /*   288 */
  {    59,   273,    28,0, 0x0000,0x0000, 0x4f,0x5c,0x23,0xff }, /*   289 */
  {    59,   267,    16,0, 0x0000,0x0000, 0x69,0x47,0x03,0xff }, /*   290 */
  {    50,   241,    55,0, 0x0000,0x0000, 0x15,0xc5,0x6e,0xff }, /*   291 */
  {     6,   247,    66,0, 0x0000,0x0000, 0xef,0x28,0x76,0xff }, /*   292 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x00,0x06,0x7e,0xff }, /*   293 */
  {   -51,   229,    48,0, 0x0000,0x0000, 0x87,0xfd,0x23,0xff }, /*   294 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0xbf,0x1a,0x69,0xff }, /*   295 */
  {   -59,   273,    28,0, 0x0000,0x0000, 0xb1,0x5c,0x23,0xff }, /*   296 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x46,0x8c,0xff }, /*   297 */
  {   -56,   148,   -81,0, 0x0000,0x0000, 0x00,0x44,0x89,0xff }, /*   298 */
  {   -47,   148,   -68,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   299 */
  {    -4,   140,  -108,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   300 */
  {   -56,   148,   -81,0, 0x0000,0x0000, 0x00,0x44,0x89,0xff }, /*   301 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x46,0x8c,0xff }, /*   302 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x23,0x46,0xff }, /*   303 */
  {    56,   148,   -81,0, 0x0000,0x0000, 0x00,0x23,0x46,0xff }, /*   304 */
  {     4,   140,  -108,0, 0x0000,0x0000, 0x00,0x37,0x6f,0xff }, /*   305 */
  {   -53,   178,   -80,0, 0x0000,0x0000, 0x00,0x46,0x8c,0xff }, /*   306 */
  {   -54,   149,   -72,0, 0x0000,0x0000, 0x00,0x3a,0x74,0xff }, /*   307 */
  {   -73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   308 */
  {    73,   158,    -6,0, 0x0000,0x0000, 0x00,0x33,0x67,0xff }, /*   309 */
  {    54,   149,   -72,0, 0x0000,0x0000, 0x00,0x36,0x6d,0xff }, /*   310 */
  {    53,   178,   -80,0, 0x0000,0x0000, 0x00,0x23,0x46,0xff }, /*   311 */
  {    71,   160,    11,0, 0x0000,0x0000, 0x00,0x30,0x61,0xff }, /*   312 */
  {    65,   160,    12,0, 0x0000,0x0000, 0x00,0x47,0x8f,0xff }, /*   313 */
  {    57,   188,     0,0, 0x0000,0x0000, 0x00,0x39,0x72,0xff }, /*   314 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0x00,0x41,0x83,0xff }, /*   315 */
  {   -65,   160,    12,0, 0x0000,0x0000, 0x00,0x3d,0x7a,0xff }, /*   316 */
  {   -71,   160,    11,0, 0x0000,0x0000, 0x00,0x46,0x8d,0xff }, /*   317 */
  {   -76,   188,     0,0, 0x0000,0x0000, 0x3d,0x00,0x7a,0xff }, /*   318 */
  {   -73,   214,     0,0, 0x0000,0x0000, 0x35,0x00,0x6a,0xff }, /*   319 */
  {   -57,   201,   -84,0, 0x0000,0x0000, 0x43,0x00,0x86,0xff }, /*   320 */
  {     0,   210,    68,0, 0x0000,0x0000, 0x2e,0x00,0x5c,0xff }, /*   321 */
  {     0,   236,    62,0, 0x0000,0x0000, 0x2f,0x00,0x5f,0xff }, /*   322 */
  {   -51,   229,    48,0, 0x0000,0x0000, 0x34,0x00,0x68,0xff }, /*   323 */
  {    57,   201,   -84,0, 0x0000,0x0000, 0x1b,0x00,0x36,0xff }, /*   324 */
  {    73,   214,     0,0, 0x0000,0x0000, 0x33,0x00,0x67,0xff }, /*   325 */
  {    76,   188,     0,0, 0x0000,0x0000, 0x2e,0x00,0x5c,0xff }, /*   326 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   327 */
  {    18,   114,    20,0, 0x0000,0x0000, 0xa8,0x7a,0x57,0xff }, /*   328 */
  {   -18,   114,    20,0, 0x0000,0x0000, 0xc1,0x8c,0x64,0xff }, /*   329 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   330 */
  {   -18,   114,    20,0, 0x0000,0x0000, 0xc1,0x8c,0x64,0xff }, /*   331 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbe,0x89,0x63,0xff }, /*   332 */
  {    18,   114,    20,0, 0x0000,0x0000, 0xa8,0x7a,0x57,0xff }, /*   333 */
  {     0,   114,    28,0, 0x0000,0x0000, 0xc1,0x8c,0x64,0xff }, /*   334 */
  {   -18,   114,    20,0, 0x0000,0x0000, 0xc1,0x8c,0x64,0xff }, /*   335 */
  {    36,   132,    52,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*   336 */
  {    18,   114,    20,0, 0x0000,0x0000, 0xa8,0x7a,0x57,0xff }, /*   337 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   338 */
  {   -36,   132,   -52,0, 0x0000,0x0000, 0xcf,0x96,0x6c,0xff }, /*   339 */
  {   -11,   114,   -20,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*   340 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbe,0x89,0x63,0xff }, /*   341 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0xb6,0x84,0x5f,0xff }, /*   342 */
  {     0,   114,   -28,0, 0x0000,0x0000, 0xba,0x86,0x60,0xff }, /*   343 */
  {   -11,   114,   -20,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*   344 */
  {    11,   114,   -20,0, 0x0000,0x0000, 0xa4,0x76,0x55,0xff }, /*   345 */
  {     0,   114,   -28,0, 0x0000,0x0000, 0xba,0x86,0x60,0xff }, /*   346 */
  {     0,   132,   -74,0, 0x0000,0x0000, 0xb6,0x84,0x5f,0xff }, /*   347 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   348 */
  {    11,   114,   -20,0, 0x0000,0x0000, 0xa4,0x76,0x55,0xff }, /*   349 */
  {    36,   132,   -52,0, 0x0000,0x0000, 0xa2,0x75,0x54,0xff }, /*   350 */
  {   -36,   132,   -52,0, 0x0000,0x0000, 0x00,0x40,0x81,0xff }, /*   351 */
  {   -52,   132,     0,0, 0x0000,0x0000, 0x00,0x3c,0x78,0xff }, /*   352 */
  {   -57,   188,     0,0, 0x0000,0x0000, 0x00,0x41,0x83,0xff }, /*   353 */
  {     6,   247,    66,0, 0x0000,0x0000, 0x65,0x33,0x38,0xff }, /*   354 */
  {     9,   247,    60,0, 0x0000,0x0000, 0x3e,0x5e,0x39,0xff }, /*   355 */
  {     6,   279,    37,0, 0x0000,0x0000, 0xec,0x71,0x35,0xff }, /*   356 */
  {     6,   279,    37,0, 0x0000,0x0000, 0x16,0x00,0x7d,0xff }, /*   357 */
  {    53,   268,    29,0, 0x0000,0x0000, 0xfa,0x36,0x72,0xff }, /*   358 */
  {    59,   273,    28,0, 0x0000,0x0000, 0x4f,0x5c,0x23,0xff }, /*   359 */
  {    59,   273,    28,0, 0x0000,0x0000, 0xcd,0x52,0x51,0xff }, /*   360 */
  {    53,   268,    29,0, 0x0000,0x0000, 0xfa,0x36,0x72,0xff }, /*   361 */
  {    50,   241,    55,0, 0x0000,0x0000, 0x41,0x1a,0x69,0xff }, /*   362 */
  {    50,   241,    55,0, 0x0000,0x0000, 0x17,0x7a,0x17,0xff }, /*   363 */
  {     9,   247,    60,0, 0x0000,0x0000, 0x3e,0x5e,0x39,0xff }, /*   364 */
  {     6,   247,    66,0, 0x0000,0x0000, 0xef,0x28,0x76,0xff }, /*   365 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0x9b,0x33,0x38,0xff }, /*   366 */
  {    -9,   247,    60,0, 0x0000,0x0000, 0xc2,0x5e,0x39,0xff }, /*   367 */
  {    -6,   247,    66,0, 0x0000,0x0000, 0x11,0x28,0x76,0xff }, /*   368 */
  {    -6,   247,    66,0, 0x0000,0x0000, 0xe9,0x7a,0x17,0xff }, /*   369 */
  {    -9,   247,    60,0, 0x0000,0x0000, 0xc2,0x5e,0x39,0xff }, /*   370 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0xbf,0x1a,0x69,0xff }, /*   371 */
  {     9,   247,    60,0, 0x0000,0x0000, 0x69,0x30,0x34,0xff }, /*   372 */
  {     9,   273,    37,0, 0x0000,0x0000, 0x3e,0x29,0x66,0xff }, /*   373 */
  {     6,   279,    37,0, 0x0000,0x0000, 0xec,0x71,0x35,0xff }, /*   374 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0x33,0x52,0x51,0xff }, /*   375 */
  {   -53,   268,    29,0, 0x0000,0x0000, 0x06,0x36,0x72,0xff }, /*   376 */
  {   -59,   273,    28,0, 0x0000,0x0000, 0xb1,0x5c,0x23,0xff }, /*   377 */
  {   -59,   273,    28,0, 0x0000,0x0000, 0xea,0x00,0x7d,0xff }, /*   378 */
  {   -53,   268,    29,0, 0x0000,0x0000, 0x06,0x36,0x72,0xff }, /*   379 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0x14,0x71,0x35,0xff }, /*   380 */
  {   -53,   268,    29,0, 0x0000,0x0000, 0xec,0xfb,0x7d,0xff }, /*   381 */
  {    -9,   273,    37,0, 0x0000,0x0000, 0xc2,0x29,0x66,0xff }, /*   382 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0x14,0x71,0x35,0xff }, /*   383 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0x39,0x51,0x4f,0xff }, /*   384 */
  {   -46,   242,    50,0, 0x0000,0x0000, 0x02,0x69,0x46,0xff }, /*   385 */
  {   -53,   268,    29,0, 0x0000,0x0000, 0x06,0x36,0x72,0xff }, /*   386 */
  {    -9,   247,    60,0, 0x0000,0x0000, 0xea,0x7b,0x11,0xff }, /*   387 */
  {   -46,   242,    50,0, 0x0000,0x0000, 0x02,0x69,0x46,0xff }, /*   388 */
  {   -50,   241,    55,0, 0x0000,0x0000, 0xbf,0x1a,0x69,0xff }, /*   389 */
  {    -6,   279,    37,0, 0x0000,0x0000, 0x97,0x30,0x34,0xff }, /*   390 */
  {    -9,   273,    37,0, 0x0000,0x0000, 0xc2,0x29,0x66,0xff }, /*   391 */
  {    -9,   247,    60,0, 0x0000,0x0000, 0xc2,0x5e,0x39,0xff }, /*   392 */
  {    50,   241,    55,0, 0x0000,0x0000, 0x16,0x7b,0x11,0xff }, /*   393 */
  {    46,   242,    50,0, 0x0000,0x0000, 0xfe,0x69,0x46,0xff }, /*   394 */
  {     9,   247,    60,0, 0x0000,0x0000, 0x3e,0x5e,0x39,0xff }, /*   395 */
  {    53,   268,    29,0, 0x0000,0x0000, 0xc7,0x51,0x4f,0xff }, /*   396 */
  {    46,   242,    50,0, 0x0000,0x0000, 0xfe,0x69,0x46,0xff }, /*   397 */
  {    50,   241,    55,0, 0x0000,0x0000, 0x41,0x1a,0x69,0xff }, /*   398 */
  {     6,   279,    37,0, 0x0000,0x0000, 0x14,0xfb,0x7d,0xff }, /*   399 */
  {     9,   273,    37,0, 0x0000,0x0000, 0x3e,0x29,0x66,0xff }, /*   400 */
  {    53,   268,    29,0, 0x0000,0x0000, 0xfa,0x36,0x72,0xff }, /*   401 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbe,0x89,0x63,0xff }, /*   402 */
  {    15,    90,     0,0, 0x0000,0x0000, 0x9a,0x6f,0x50,0xff }, /*   403 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   404 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   405 */
  {    11,    90,   -20,0, 0x0000,0x0000, 0xa5,0x77,0x55,0xff }, /*   406 */
  {    11,   114,   -20,0, 0x0000,0x0000, 0xa4,0x76,0x55,0xff }, /*   407 */
  {    11,   114,   -20,0, 0x0000,0x0000, 0xa4,0x76,0x55,0xff }, /*   408 */
  {     0,    90,   -28,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*   409 */
  {     0,   114,   -28,0, 0x0000,0x0000, 0xba,0x86,0x60,0xff }, /*   410 */
  {    11,   114,   -20,0, 0x0000,0x0000, 0xa4,0x76,0x55,0xff }, /*   411 */
  {    11,    90,   -20,0, 0x0000,0x0000, 0xa5,0x77,0x55,0xff }, /*   412 */
  {     0,    90,   -28,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*   413 */
  {    15,   114,     0,0, 0x0000,0x0000, 0xab,0x7c,0x59,0xff }, /*   414 */
  {    15,    90,     0,0, 0x0000,0x0000, 0x9a,0x6f,0x50,0xff }, /*   415 */
  {    11,    90,   -20,0, 0x0000,0x0000, 0xa5,0x77,0x55,0xff }, /*   416 */
  {     0,   114,   -28,0, 0x0000,0x0000, 0xba,0x86,0x60,0xff }, /*   417 */
  {   -11,    90,   -20,0, 0x0000,0x0000, 0xcd,0x94,0x6a,0xff }, /*   418 */
  {   -11,   114,   -20,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*   419 */
  {     0,   114,   -28,0, 0x0000,0x0000, 0xba,0x86,0x60,0xff }, /*   420 */
  {     0,    90,   -28,0, 0x0000,0x0000, 0xb5,0x83,0x5e,0xff }, /*   421 */
  {   -11,    90,   -20,0, 0x0000,0x0000, 0xcd,0x94,0x6a,0xff }, /*   422 */
  {   -11,   114,   -20,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*   423 */
  {   -15,    90,     0,0, 0x0000,0x0000, 0xab,0x7b,0x59,0xff }, /*   424 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbe,0x89,0x63,0xff }, /*   425 */
  {   -11,   114,   -20,0, 0x0000,0x0000, 0xd2,0x98,0x6d,0xff }, /*   426 */
  {   -11,    90,   -20,0, 0x0000,0x0000, 0xcd,0x94,0x6a,0xff }, /*   427 */
  {   -15,    90,     0,0, 0x0000,0x0000, 0xab,0x7b,0x59,0xff }, /*   428 */
  {   -15,   114,     0,0, 0x0000,0x0000, 0xbe,0x89,0x63,0xff }, /*   429 */
  {   -15,    90,     0,0, 0x0000,0x0000, 0xab,0x7b,0x59,0xff }, /*   430 */
  {    15,    90,     0,0, 0x0000,0x0000, 0x9a,0x6f,0x50,0xff }, /*   431 */
  {   -42,   204,    49,0, 0x01f0,0xfdf1, 0x9b,0x9b,0x9b,0xff }, /*   432 */
  {     0,   210,    49,0, 0x02cf,0xfe41, 0x93,0x93,0x93,0xff }, /*   433 */
  {     0,   210,    68,0, 0x02cf,0xfdb4, 0x8b,0x8b,0x8b,0xff }, /*   434 */
  {    59,   267,    16,0, 0x0445,0x0269, 0xa6,0xa6,0xa6,0xff }, /*   435 */
  {    51,   229,    48,0, 0x03f8,0xff5b, 0xb9,0xb9,0xb9,0xff }, /*   436 */
  {    73,   214,     0,0, 0x051f,0xfff2, 0x9a,0x9a,0x9a,0xff }, /*   437 */
  {     0,   210,    68,0, 0x02cf,0xfdb4, 0x8b,0x8b,0x8b,0xff }, /*   438 */
  {    42,   204,    49,0, 0x0400,0xfdf1, 0x85,0x85,0x85,0xff }, /*   439 */
  {    64,   204,    47,0, 0x045a,0xfdfd, 0xbb,0xbb,0xbb,0xff }, /*   440 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xab,0xab,0xab,0xff }, /*   441 */
  {   -23,   300,   -84,0, 0x079f,0x073c, 0x70,0x54,0x8c,0xff }, /*   442 */
  {   -20,   319,   -81,0, 0x07d7,0x082f, 0x99,0x99,0x99,0xff }, /*   443 */
  {    57,   201,   -84,0, 0x0622,0x01a6, 0x51,0x51,0x51,0xff }, /*   444 */
  {    23,   300,   -84,0, 0x062d,0x073c, 0x64,0x4b,0x7d,0xff }, /*   445 */
  {    73,   214,     0,0, 0x051f,0xfff2, 0x9a,0x9a,0x9a,0xff }, /*   446 */
  {    73,   214,     0,0, 0x051f,0xfff2, 0x9a,0x9a,0x9a,0xff }, /*   447 */
  {    23,   300,   -84,0, 0x062d,0x073c, 0x64,0x4b,0x7d,0xff }, /*   448 */
  {    59,   267,    16,0, 0x0445,0x0269, 0xa6,0xa6,0xa6,0xff }, /*   449 */
  {   -59,   267,    16,0, 0x01c4,0x0269, 0x96,0x96,0x96,0xff }, /*   450 */
  {   -23,   300,   -84,0, 0xffa0,0x073c, 0x70,0x54,0x8c,0xff }, /*   451 */
  {   -73,   214,     0,0, 0x011b,0xfff2, 0x9f,0x9f,0x9f,0xff }, /*   452 */
  {   -73,   214,     0,0, 0x011b,0xfff2, 0x9f,0x9f,0x9f,0xff }, /*   453 */
  {   -23,   300,   -84,0, 0xffa0,0x073c, 0x70,0x54,0x8c,0xff }, /*   454 */
  {   -57,   201,   -84,0, 0xffb1,0x01a6, 0xca,0xca,0xca,0xff }, /*   455 */
  {   -57,   201,   -84,0, 0x07b0,0x01a6, 0xca,0xca,0xca,0xff }, /*   456 */
  {     0,   292,  -111,0, 0x06cf,0x0784, 0x65,0x4c,0x7f,0xff }, /*   457 */
  {     0,   193,  -119,0, 0x06cf,0x0230, 0x96,0x96,0x96,0xff }, /*   458 */
  {     0,   193,  -119,0, 0x06cf,0x0230, 0x96,0x96,0x96,0xff }, /*   459 */
  {     0,   292,  -111,0, 0x06cf,0x0784, 0x65,0x4c,0x7f,0xff }, /*   460 */
  {    57,   201,   -84,0, 0x0622,0x01a6, 0x51,0x51,0x51,0xff }, /*   461 */
  {    23,   302,  -104,0, 0x0654,0x07ec, 0x72,0x56,0x8f,0xff }, /*   462 */
  {     0,   292,  -111,0, 0x06cf,0x0784, 0x65,0x4c,0x7f,0xff }, /*   463 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xab,0xab,0xab,0xff }, /*   464 */
  {     0,   275,    27,0, 0x02cf,0x0289, 0xd4,0xd4,0xd4,0xff }, /*   465 */
  {     0,   313,   -51,0, 0xfed0,0x06f6, 0xaa,0x7f,0xd5,0xff }, /*   466 */
  {   -59,   267,    16,0, 0x01c4,0x0269, 0x96,0x96,0x96,0xff }, /*   467 */
  {    59,   267,    16,0, 0x0c45,0x0269, 0xa6,0xa6,0xa6,0xff }, /*   468 */
  {     0,   313,   -51,0, 0x06cf,0x06f6, 0xaa,0x7f,0xd5,0xff }, /*   469 */
  {     0,   275,    27,0, 0x0acf,0x0289, 0xd4,0xd4,0xd4,0xff }, /*   470 */
  {     0,   210,    68,0, 0x02cf,0xfdb4, 0x8b,0x8b,0x8b,0xff }, /*   471 */
  {     0,   210,    49,0, 0x02cf,0xfe41, 0x93,0x93,0x93,0xff }, /*   472 */
  {    42,   204,    49,0, 0x0400,0xfdf1, 0x85,0x85,0x85,0xff }, /*   473 */
  {   -73,   214,     0,0, 0x011b,0xfff2, 0x9f,0x9f,0x9f,0xff }, /*   474 */
  {   -51,   229,    48,0, 0x01f4,0xff5b, 0x9b,0x9b,0x9b,0xff }, /*   475 */
  {   -59,   267,    16,0, 0x01c4,0x0269, 0x96,0x96,0x96,0xff }, /*   476 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xab,0xab,0xab,0xff }, /*   477 */
  {     0,   313,   -51,0, 0x06cf,0x06f6, 0xaa,0x7f,0xd5,0xff }, /*   478 */
  {    20,   319,   -81,0, 0x0609,0x082f, 0x80,0x60,0xa0,0xff }, /*   479 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xab,0xab,0xab,0xff }, /*   480 */
  {    20,   319,   -81,0, 0x0609,0x082f, 0x80,0x60,0xa0,0xff }, /*   481 */
  {    23,   300,   -84,0, 0x062d,0x073c, 0x64,0x4b,0x7d,0xff }, /*   482 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xab,0xab,0xab,0xff }, /*   483 */
  {    23,   300,   -84,0, 0x062d,0x073c, 0x64,0x4b,0x7d,0xff }, /*   484 */
  {    23,   302,  -104,0, 0x0654,0x07ec, 0x72,0x56,0x8f,0xff }, /*   485 */
  {   -20,   319,   -81,0, 0x07d7,0x082f, 0x99,0x99,0x99,0xff }, /*   486 */
  {     0,   313,   -51,0, 0x06cf,0x06f6, 0xaa,0x7f,0xd5,0xff }, /*   487 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xab,0xab,0xab,0xff }, /*   488 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xab,0xab,0xab,0xff }, /*   489 */
  {   -23,   302,  -104,0, 0x0764,0x07ec, 0x69,0x4f,0x84,0xff }, /*   490 */
  {   -23,   300,   -84,0, 0x079f,0x073c, 0x70,0x54,0x8c,0xff }, /*   491 */
  {     0,   302,   -81,0, 0x06cf,0x073d, 0xab,0xab,0xab,0xff }, /*   492 */
  {     0,   292,  -111,0, 0x06cf,0x0784, 0x65,0x4c,0x7f,0xff }, /*   493 */
  {   -23,   302,  -104,0, 0x0764,0x07ec, 0x69,0x4f,0x84,0xff }, /*   494 */
  {    59,   267,    16,0, 0x0445,0x0269, 0xa6,0xa6,0xa6,0xff }, /*   495 */
  {    20,   319,   -81,0, 0x0609,0x082f, 0x80,0x60,0xa0,0xff }, /*   496 */
  {     0,   313,   -51,0, 0x06cf,0x06f6, 0xaa,0x7f,0xd5,0xff }, /*   497 */
  {     0,   313,   -51,0, 0x06cf,0x06f6, 0xaa,0x7f,0xd5,0xff }, /*   498 */
  {   -20,   319,   -81,0, 0x07d7,0x082f, 0x99,0x99,0x99,0xff }, /*   499 */
  {   -59,   267,    16,0, 0x09c4,0x0269, 0x96,0x96,0x96,0xff }, /*   500 */
  {   -23,   300,   -84,0, 0x079f,0x073c, 0x70,0x54,0x8c,0xff }, /*   501 */
  {   -23,   302,  -104,0, 0x0764,0x07ec, 0x69,0x4f,0x84,0xff }, /*   502 */
  {   -57,   201,   -84,0, 0x07b0,0x01a6, 0xca,0xca,0xca,0xff }, /*   503 */
  {     0,   292,  -111,0, 0x06cf,0x0784, 0x65,0x4c,0x7f,0xff }, /*   504 */
  {    23,   302,  -104,0, 0x0654,0x07ec, 0x72,0x56,0x8f,0xff }, /*   505 */
  {    57,   201,   -84,0, 0x0622,0x01a6, 0x51,0x51,0x51,0xff }, /*   506 */
  {   -57,   201,   -84,0, 0x07b0,0x01a6, 0xca,0xca,0xca,0xff }, /*   507 */
  {   -23,   302,  -104,0, 0x0764,0x07ec, 0x69,0x4f,0x84,0xff }, /*   508 */
  {     0,   292,  -111,0, 0x06cf,0x0784, 0x65,0x4c,0x7f,0xff }, /*   509 */
  {   -59,   267,    16,0, 0x01c4,0x0269, 0x96,0x96,0x96,0xff }, /*   510 */
  {   -20,   319,   -81,0, 0xffd8,0x082f, 0x99,0x99,0x99,0xff }, /*   511 */
  {   -23,   300,   -84,0, 0xffa0,0x073c, 0x70,0x54,0x8c,0xff }, /*   512 */
  {    23,   300,   -84,0, 0x062d,0x073c, 0x64,0x4b,0x7d,0xff }, /*   513 */
  {    20,   319,   -81,0, 0x0609,0x082f, 0x80,0x60,0xa0,0xff }, /*   514 */
  {    59,   267,    16,0, 0x0445,0x0269, 0xa6,0xa6,0xa6,0xff }, /*   515 */
  {    57,   201,   -84,0, 0x0622,0x01a6, 0x51,0x51,0x51,0xff }, /*   516 */
  {    23,   302,  -104,0, 0x0654,0x07ec, 0x72,0x56,0x8f,0xff }, /*   517 */
  {    23,   300,   -84,0, 0x062d,0x073c, 0x64,0x4b,0x7d,0xff }, /*   518 */
  {     0,   210,    68,0, 0x02cf,0xfdb4, 0x8b,0x8b,0x8b,0xff }, /*   519 */
  {   -64,   204,    47,0, 0x01b7,0xfdfd, 0xa1,0xa1,0xa1,0xff }, /*   520 */
  {   -42,   204,    49,0, 0x01f0,0xfdf1, 0x9b,0x9b,0x9b,0xff }, /*   521 */
  {    36,   132,    52,0, 0x077e,0x0000, 0x96,0x96,0x96,0xff }, /*   522 */
  {     0,   170,    73,0, 0x0400,0x07ff, 0x9c,0x7d,0x5e,0xff }, /*   523 */
  {   -36,   132,    52,0, 0x0081,0x0000, 0xb3,0xb3,0xb3,0xff }, /*   524 */
  {     0,   170,    73,0, 0x0400,0x07ff, 0x9c,0x7d,0x5e,0xff }, /*   525 */
  {   -42,   170,    65,0, 0x0000,0x0755, 0xad,0x71,0x68,0xff }, /*   526 */
  {   -36,   132,    52,0, 0x0081,0x0000, 0xb3,0xb3,0xb3,0xff }, /*   527 */
  {    36,   132,    52,0, 0x077e,0x0000, 0x96,0x96,0x96,0xff }, /*   528 */
  {    42,   170,    65,0, 0x0800,0x0755, 0xc9,0x83,0x78,0xff }, /*   529 */
  {     0,   170,    73,0, 0x0400,0x07ff, 0x9c,0x7d,0x5e,0xff }, /*   530 */
  {   -42,   170,    65,0, 0xf800,0x0000, 0xad,0x71,0x68,0xff }, /*   531 */
  {     0,   210,    49,0, 0x0000,0x0800, 0xa4,0xa4,0xa4,0xff }, /*   532 */
  {   -42,   204,    49,0, 0xf800,0x06e5, 0xac,0xac,0xac,0xff }, /*   533 */
  {    42,   170,    65,0, 0x0800,0x0000, 0xc9,0x83,0x78,0xff }, /*   534 */
  {    42,   204,    49,0, 0x0800,0x06e5, 0x93,0x93,0x93,0xff }, /*   535 */
  {     0,   210,    49,0, 0x0000,0x0800, 0xa4,0xa4,0xa4,0xff }, /*   536 */
  {   -42,   170,    65,0, 0xf800,0x0000, 0xad,0x71,0x68,0xff }, /*   537 */
  {     0,   170,    73,0, 0x0000,0x0000, 0x9c,0x7d,0x5e,0xff }, /*   538 */
  {     0,   210,    49,0, 0x0000,0x0800, 0xa4,0xa4,0xa4,0xff }, /*   539 */
  {     0,   170,    73,0, 0x0000,0x0000, 0x9c,0x7d,0x5e,0xff }, /*   540 */
  {    42,   170,    65,0, 0x0800,0x0000, 0xc9,0x83,0x78,0xff }, /*   541 */
  {     0,   210,    49,0, 0x0000,0x0800, 0xa4,0xa4,0xa4,0xff }, /*   542 */
  {    -9,   247,    60,0, 0x0000,0x0000, 0xc2,0x5e,0x39,0xff }, /*   543 */
  {    -9,   273,    37,0, 0x0000,0x0000, 0xc2,0x29,0x66,0xff }, /*   544 */
  {   -46,   242,    50,0, 0x0000,0x0000, 0x02,0x69,0x46,0xff }, /*   545 */
  {     9,   273,    37,0, 0x0000,0x0000, 0x3e,0x29,0x66,0xff }, /*   546 */
  {    46,   242,    50,0, 0x0000,0x0000, 0xfe,0x69,0x46,0xff }, /*   547 */
  {    53,   268,    29,0, 0x0000,0x0000, 0xfa,0x36,0x72,0xff }, /*   548 */
  {     9,   273,    37,0, 0x0000,0x0000, 0x3e,0x29,0x66,0xff }, /*   549 */
  {     9,   247,    60,0, 0x0000,0x0000, 0x3e,0x5e,0x39,0xff }, /*   550 */
  {    46,   242,    50,0, 0x0000,0x0000, 0xfe,0x69,0x46,0xff }, /*   551 */
  {    -9,   273,    37,0, 0x0000,0x0000, 0xc2,0x29,0x66,0xff }, /*   552 */
  {   -53,   268,    29,0, 0x0000,0x0000, 0x06,0x36,0x72,0xff }, /*   553 */
  {   -46,   242,    50,0, 0x0000,0x0000, 0x02,0x69,0x46,0xff }, /*   554 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_8_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP2Triangles(  3,  4,  5,  0,  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP2Triangles( 12, 13, 14,  0, 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP2Triangles( 21, 22, 23,  0, 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP2Triangles(  7,  8,  9,  0, 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 992), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 1024), 31, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsSP1Triangle(  2,  3,  4,  0),
  gsSP1Triangle(  5,  6,  7,  0),
  gsSP1Triangle(  8,  9, 10,  0),
  gsSP2Triangles( 11, 12, 13,  0, 14, 15, 16,  0),
  gsSP1Triangle( 17, 18, 19,  0),
  gsSP1Triangle( 20, 21, 22,  0),
  gsSP1Triangle( 23, 24, 25,  0),
  gsSP1Triangle( 26, 27, 28,  0),
  gsSPVertex( SEGADRS(5, 1520), 1, 31 ),
  gsSPVertex( SEGADRS(5, 1536), 29, 0 ),
  gsSP2Triangles( 29, 30, 31,  0,  0,  1,  2,  0),
  gsSP1Triangle(  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP2Triangles(  9, 10, 11,  0, 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSPVertex( SEGADRS(5, 2000), 3, 29 ),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 2048), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP2Triangles(  4,  5,  6,  0,  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP2Triangles( 16, 17, 18,  0, 19, 20, 21,  0),
  gsSP2Triangles( 22, 23, 24,  0, 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 2528), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 2560), 31, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsSP1Triangle(  2,  3,  4,  0),
  gsSP2Triangles(  5,  6,  7,  0,  8,  9, 10,  0),
  gsSP2Triangles( 11, 12, 13,  0, 14, 15, 16,  0),
  gsSP1Triangle( 17, 18, 19,  0),
  gsSP1Triangle( 20, 21, 22,  0),
  gsSP1Triangle( 23, 24, 25,  0),
  gsSP1Triangle( 26, 27, 28,  0),
  gsSPVertex( SEGADRS(5, 3056), 1, 31 ),
  gsSPVertex( SEGADRS(5, 3072), 29, 0 ),
  gsSP2Triangles( 29, 30, 31,  0,  0,  1,  2,  0),
  gsSP1Triangle(  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP2Triangles(  9, 10, 11,  0, 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSPVertex( SEGADRS(5, 3536), 3, 29 ),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 3584), 25, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP2Triangles(  4,  5,  6,  0,  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP2Triangles( 19, 20, 21,  0, 22, 23, 24,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xda, 0xa2, 0x00, 0xff),
  gsSPClearGeometryMode( G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, ENVIRONMENT, 0, 0, 0, ENVIRONMENT, 0, 1, 0, ENVIRONMENT, 0, 0, 0, ENVIRONMENT, 0),
  gsSPVertex( SEGADRS(5, 3984), 7, 25 ),
  gsSP2Triangles( 25, 26, 27,  0, 28, 29, 30,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0x9b, 0x73, 0x00, 0xff),
  gsSPVertex( SEGADRS(5, 4096), 31, 0 ),
  gsSP2Triangles( 31,  0,  1,  0,  2,  3,  4,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xc1, 0x90, 0x00, 0xff),
  gsSP1Triangle(  5,  6,  7,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xcc, 0x98, 0x00, 0xff),
  gsSP2Triangles(  8,  9, 10,  0, 11, 12, 13,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0x8e, 0x69, 0x00, 0xff),
  gsSP1Triangle( 14, 15, 16,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xbc, 0x8c, 0x00, 0xff),
  gsSP1Triangle( 17, 18, 19,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xcc, 0x98, 0x00, 0xff),
  gsSP1Triangle( 20, 21, 22,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xa9, 0x7e, 0x00, 0xff),
  gsSP1Triangle( 23, 24, 25,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xbc, 0x8c, 0x00, 0xff),
  gsSP1Triangle( 26, 27, 28,  0),
  gsSPVertex( SEGADRS(5, 4592), 1, 31 ),
  gsSP1Triangle( 29, 30, 31,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xcc, 0x98, 0x00, 0xff),
  gsSPVertex( SEGADRS(5, 4608), 9, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0x8d, 0x69, 0x00, 0xff),
  gsSP1Triangle(  3,  4,  5,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xa5, 0x7b, 0x00, 0xff),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSPSetGeometryMode( G_SHADING_SMOOTH ),
  gsDPPipeSync(),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 4752), 23, 9 ),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 5120), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP2Triangles(  7,  8,  9,  0, 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 5600), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 5632), 2, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xd8, 0xa1, 0x00, 0xff),
  gsSPClearGeometryMode( G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, ENVIRONMENT, 0, 0, 0, ENVIRONMENT, 0, 1, 0, ENVIRONMENT, 0, 0, 0, ENVIRONMENT, 0),
  gsSPVertex( SEGADRS(5, 5664), 30, 2 ),
  gsSP1Triangle(  2,  3,  4,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xa7, 0x7c, 0x00, 0xff),
  gsSP1Triangle(  5,  6,  7,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xe5, 0xab, 0x00, 0xff),
  gsSP1Triangle(  8,  9, 10,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xf4, 0xb5, 0x00, 0xff),
  gsSP1Triangle( 11, 12, 13,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xbe, 0x8d, 0x00, 0xff),
  gsSP1Triangle( 14, 15, 16,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xd7, 0xa0, 0x00, 0xff),
  gsSP2Triangles( 17, 18, 19,  0, 20, 21, 22,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xf3, 0xb5, 0x00, 0xff),
  gsSP1Triangle( 23, 24, 25,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xaa, 0x7e, 0x00, 0xff),
  gsSP1Triangle( 26, 27, 28,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xa6, 0x7c, 0x00, 0xff),
  gsSP1Triangle( 29, 30, 31,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xf1, 0xb4, 0x00, 0xff),
  gsSPVertex( SEGADRS(5, 6144), 18, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xd5, 0x9e, 0x00, 0xff),
  gsSP1Triangle(  3,  4,  5,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xb7, 0x88, 0x00, 0xff),
  gsSP1Triangle(  6,  7,  8,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xf0, 0xb3, 0x00, 0xff),
  gsSP1Triangle(  9, 10, 11,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xe4, 0xaa, 0x00, 0xff),
  gsSP1Triangle( 12, 13, 14,  0),
  gsDPPipeSync(),
  gsDPSetEnvColor(0xa7, 0x7d, 0x00, 0xff),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSPSetGeometryMode( G_SHADING_SMOOTH ),
  gsDPPipeSync(),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 6432), 14, 18 ),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP2Triangles( 21, 22, 23,  0, 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 6656), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP2Triangles(  1,  2,  3,  0,  4,  5,  6,  0),
  gsSP2Triangles(  7,  8,  9,  0, 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsDPPipeSync(),
  gsDPSetCombineLERP(TEXEL0, 0, SHADE, 0, SHADE, 0, TEXEL0, 0, TEXEL0, 0, SHADE, 0, SHADE, 0, TEXEL0, 0),
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_ON),
  gsDPSetTextureLUT( G_TT_NONE ),
  gsDPLoadMultiBlock( SEGADRS(6,2048), 0, 0,
    G_IM_FMT_RGBA, G_IM_SIZ_16b, 32, 32, 0,
    0, 0, 5, 5, 0, 0 ),
  gsSP2Triangles( 16, 17, 18,  0, 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 7136), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 7168), 31, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsSP1Triangle(  2,  3,  4,  0),
  gsSP1Triangle(  5,  6,  7,  0),
  gsSP1Triangle(  8,  9, 10,  0),
  gsSP1Triangle( 11, 12, 13,  0),
  gsSP1Triangle( 14, 15, 16,  0),
  gsSP1Triangle( 17, 18, 19,  0),
  gsSP1Triangle( 20, 21, 22,  0),
  gsSP1Triangle( 23, 24, 25,  0),
  gsSP1Triangle( 26, 27, 28,  0),
  gsSPVertex( SEGADRS(5, 7664), 1, 31 ),
  gsSP1Triangle( 29, 30, 31,  0),
  gsSPVertex( SEGADRS(5, 7680), 29, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP1Triangle(  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP1Triangle(  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSPVertex( SEGADRS(5, 8144), 3, 29 ),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 8192), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsDPPipeSync(),
  gsDPSetTextureLUT( G_TT_NONE ),
  gsDPLoadMultiBlock( SEGADRS(6,4096), 0, 0,
    G_IM_FMT_RGBA, G_IM_SIZ_16b, 32, 32, 0,
    1, 1, 5, 5, 0, 0 ),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsDPPipeSync(),
  gsDPSetTextureLUT( G_TT_NONE ),
  gsDPLoadMultiBlock( SEGADRS(6,6144), 0, 0,
    G_IM_FMT_RGBA, G_IM_SIZ_16b, 32, 32, 0,
    1, 1, 5, 5, 0, 0 ),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 8672), 1, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPSetGeometryMode( G_LIGHTING|G_TEXTURE_GEN ),
  gsDPPipeSync(),
  gsDPSetCombineLERP(TEXEL0, 0, SHADE, 0, 0, 0, 0, TEXEL0, TEXEL0, 0, SHADE, 0, 0, 0, 0, TEXEL0),
  gsSPTexture(0x0800, 0x0800, 0, G_TX_RENDERTILE, G_ON),
  gsDPSetTextureLUT( G_TT_NONE ),
  gsDPLoadMultiBlock( SEGADRS(6,8192), 0, 0,
    G_IM_FMT_RGBA, G_IM_SIZ_16b, 32, 32, 0,
    0, 0, 5, 5, 0, 0 ),
  gsSPVertex( SEGADRS(5, 8688), 12, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_8_ = {
  gfx_Shape_of___POLYHEDRON_8_,
  vtx_Shape_of___POLYHEDRON_8_, 555, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner11[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_67_[] = {
  {    38,   -53,     3,0, 0x0000,0x0000, 0xe1,0xa3,0x75,0xff }, /*     0 */
  {    38,   -75,     3,0, 0x0000,0x0000, 0xa3,0x76,0x54,0xff }, /*     1 */
  {    38,   -53,   -22,0, 0x0000,0x0000, 0xb4,0x82,0x5d,0xff }, /*     2 */
  {    38,   -75,     3,0, 0x0000,0x0000, 0xa3,0x76,0x54,0xff }, /*     3 */
  {    38,   -75,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4e,0xff }, /*     4 */
  {    38,   -53,   -22,0, 0x0000,0x0000, 0xb4,0x82,0x5d,0xff }, /*     5 */
  {    38,   -53,   -22,0, 0x0000,0x0000, 0xb4,0x82,0x5d,0xff }, /*     6 */
  {    38,   -75,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4e,0xff }, /*     7 */
  {    13,   -53,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*     8 */
  {    38,   -75,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4e,0xff }, /*     9 */
  {    13,   -75,   -22,0, 0x0000,0x0000, 0xbd,0x89,0x62,0xff }, /*    10 */
  {    13,   -53,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    11 */
  {    13,   -53,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    12 */
  {    13,   -75,     3,0, 0x0000,0x0000, 0x8e,0x67,0x4a,0xff }, /*    13 */
  {    13,   -53,     3,0, 0x0000,0x0000, 0xff,0xba,0x86,0xff }, /*    14 */
  {    13,   -53,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    15 */
  {    13,   -75,   -22,0, 0x0000,0x0000, 0xbd,0x89,0x62,0xff }, /*    16 */
  {    13,   -75,     3,0, 0x0000,0x0000, 0x8e,0x67,0x4a,0xff }, /*    17 */
  {    13,   -53,     3,0, 0x0000,0x0000, 0xff,0xba,0x86,0xff }, /*    18 */
  {    13,   -75,     3,0, 0x0000,0x0000, 0x8e,0x67,0x4a,0xff }, /*    19 */
  {    38,   -53,     3,0, 0x0000,0x0000, 0xe1,0xa3,0x75,0xff }, /*    20 */
  {    13,   -75,     3,0, 0x0000,0x0000, 0x8e,0x67,0x4a,0xff }, /*    21 */
  {    38,   -75,     3,0, 0x0000,0x0000, 0xa3,0x76,0x54,0xff }, /*    22 */
  {    38,   -53,     3,0, 0x0000,0x0000, 0xe1,0xa3,0x75,0xff }, /*    23 */
  {    38,   -75,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4e,0xff }, /*    24 */
  {    26,   -86,    -9,0, 0x0000,0x0000, 0x85,0x39,0x29,0xff }, /*    25 */
  {    13,   -75,   -22,0, 0x0000,0x0000, 0xbd,0x89,0x62,0xff }, /*    26 */
  {    38,   -75,     3,0, 0x0000,0x0000, 0xa3,0x76,0x54,0xff }, /*    27 */
  {    26,   -86,    -9,0, 0x0000,0x0000, 0x85,0x39,0x29,0xff }, /*    28 */
  {    38,   -75,   -22,0, 0x0000,0x0000, 0x97,0x6d,0x4e,0xff }, /*    29 */
  {    13,   -75,     3,0, 0x0000,0x0000, 0x8e,0x67,0x4a,0xff }, /*    30 */
  {    26,   -86,    -9,0, 0x0000,0x0000, 0x85,0x39,0x29,0xff }, /*    31 */
  {    38,   -75,     3,0, 0x0000,0x0000, 0xa3,0x76,0x54,0xff }, /*    32 */
  {    13,   -75,   -22,0, 0x0000,0x0000, 0xbd,0x89,0x62,0xff }, /*    33 */
  {    26,   -86,    -9,0, 0x0000,0x0000, 0x85,0x39,0x29,0xff }, /*    34 */
  {    13,   -75,     3,0, 0x0000,0x0000, 0x8e,0x67,0x4a,0xff }, /*    35 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_67_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP2Triangles( 12, 13, 14,  0, 15, 16, 17,  0),
  gsSP2Triangles( 18, 19, 20,  0, 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 4, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_67_ = {
  gfx_Shape_of___POLYHEDRON_67_,
  vtx_Shape_of___POLYHEDRON_67_, 36, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner12[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_71_[] = {
  {    38,  -104,     3,0, 0x0000,0x0000, 0xdd,0xa0,0x73,0xff }, /*     0 */
  {    38,   -80,     3,0, 0x0000,0x0000, 0xeb,0xaa,0x7a,0xff }, /*     1 */
  {    13,  -104,     3,0, 0x0000,0x0000, 0xb1,0x80,0x5c,0xff }, /*     2 */
  {    13,  -104,   -22,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*     3 */
  {    13,   -80,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*     4 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x82,0x5e,0x43,0xff }, /*     5 */
  {    13,   -80,     3,0, 0x0000,0x0000, 0xf2,0xaf,0x7e,0xff }, /*     6 */
  {    13,   -80,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*     7 */
  {    13,  -104,     3,0, 0x0000,0x0000, 0xb1,0x80,0x5c,0xff }, /*     8 */
  {    13,   -80,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*     9 */
  {    13,  -104,   -22,0, 0x0000,0x0000, 0xe0,0xa2,0x74,0xff }, /*    10 */
  {    13,  -104,     3,0, 0x0000,0x0000, 0xb1,0x80,0x5c,0xff }, /*    11 */
  {    38,   -80,   -22,0, 0x0000,0x0000, 0x92,0x6a,0x4c,0xff }, /*    12 */
  {    38,   -80,     3,0, 0x0000,0x0000, 0xeb,0xaa,0x7a,0xff }, /*    13 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x82,0x5e,0x43,0xff }, /*    14 */
  {    38,   -80,     3,0, 0x0000,0x0000, 0xeb,0xaa,0x7a,0xff }, /*    15 */
  {    26,   -67,    -9,0, 0x0000,0x0000, 0xfe,0x6e,0x4f,0xff }, /*    16 */
  {    13,   -80,     3,0, 0x0000,0x0000, 0xf2,0xaf,0x7e,0xff }, /*    17 */
  {    38,   -80,   -22,0, 0x0000,0x0000, 0x92,0x6a,0x4c,0xff }, /*    18 */
  {    26,   -67,    -9,0, 0x0000,0x0000, 0xfe,0x6e,0x4f,0xff }, /*    19 */
  {    38,   -80,     3,0, 0x0000,0x0000, 0xeb,0xaa,0x7a,0xff }, /*    20 */
  {    13,   -80,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    21 */
  {    26,   -67,    -9,0, 0x0000,0x0000, 0xfe,0x6e,0x4f,0xff }, /*    22 */
  {    38,   -80,   -22,0, 0x0000,0x0000, 0x92,0x6a,0x4c,0xff }, /*    23 */
  {    13,   -80,     3,0, 0x0000,0x0000, 0xf2,0xaf,0x7e,0xff }, /*    24 */
  {    26,   -67,    -9,0, 0x0000,0x0000, 0xfe,0x6e,0x4f,0xff }, /*    25 */
  {    13,   -80,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    26 */
  {    38,  -104,     3,0, 0x0000,0x0000, 0x89,0x44,0x00,0xff }, /*    27 */
  {    38,  -138,     3,0, 0x0000,0x0000, 0x5f,0x2f,0x00,0xff }, /*    28 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x50,0x28,0x00,0xff }, /*    29 */
  {    38,  -138,     3,0, 0x0000,0x0000, 0x5f,0x2f,0x00,0xff }, /*    30 */
  {    38,  -138,   -22,0, 0x0000,0x0000, 0x59,0x2c,0x00,0xff }, /*    31 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x50,0x28,0x00,0xff }, /*    32 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x50,0x28,0x00,0xff }, /*    33 */
  {    38,  -138,   -22,0, 0x0000,0x0000, 0x59,0x2c,0x00,0xff }, /*    34 */
  {    13,  -104,   -22,0, 0x0000,0x0000, 0x8b,0x45,0x00,0xff }, /*    35 */
  {    38,  -138,   -22,0, 0x0000,0x0000, 0x59,0x2c,0x00,0xff }, /*    36 */
  {    13,  -138,   -22,0, 0x0000,0x0000, 0x5d,0x2e,0x00,0xff }, /*    37 */
  {    13,  -104,   -22,0, 0x0000,0x0000, 0x8b,0x45,0x00,0xff }, /*    38 */
  {    13,  -104,   -22,0, 0x0000,0x0000, 0x8b,0x45,0x00,0xff }, /*    39 */
  {    13,  -138,     3,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    40 */
  {    13,  -104,     3,0, 0x0000,0x0000, 0x6e,0x37,0x00,0xff }, /*    41 */
  {    13,  -104,     3,0, 0x0000,0x0000, 0x6e,0x37,0x00,0xff }, /*    42 */
  {    13,  -138,     3,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    43 */
  {    38,  -104,     3,0, 0x0000,0x0000, 0x89,0x44,0x00,0xff }, /*    44 */
  {    13,  -138,     3,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    45 */
  {    38,  -138,     3,0, 0x0000,0x0000, 0x5f,0x2f,0x00,0xff }, /*    46 */
  {    38,  -104,     3,0, 0x0000,0x0000, 0x89,0x44,0x00,0xff }, /*    47 */
  {    13,  -104,   -22,0, 0x0000,0x0000, 0x8b,0x45,0x00,0xff }, /*    48 */
  {    13,  -138,   -22,0, 0x0000,0x0000, 0x5d,0x2e,0x00,0xff }, /*    49 */
  {    13,  -138,     3,0, 0x0000,0x0000, 0x54,0x2a,0x00,0xff }, /*    50 */
  {    38,   -80,     3,0, 0x0000,0x0000, 0xeb,0xaa,0x7a,0xff }, /*    51 */
  {    38,  -104,     3,0, 0x0000,0x0000, 0xdd,0xa0,0x73,0xff }, /*    52 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x82,0x5e,0x43,0xff }, /*    53 */
  {    13,   -80,   -22,0, 0x0000,0x0000, 0xdc,0x9f,0x72,0xff }, /*    54 */
  {    38,   -80,   -22,0, 0x0000,0x0000, 0x92,0x6a,0x4c,0xff }, /*    55 */
  {    38,  -104,   -22,0, 0x0000,0x0000, 0x82,0x5e,0x43,0xff }, /*    56 */
  {    38,   -80,     3,0, 0x0000,0x0000, 0xeb,0xaa,0x7a,0xff }, /*    57 */
  {    13,   -80,     3,0, 0x0000,0x0000, 0xf2,0xaf,0x7e,0xff }, /*    58 */
  {    13,  -104,     3,0, 0x0000,0x0000, 0xb1,0x80,0x5c,0xff }, /*    59 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_71_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP1Triangle(  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 28, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP2Triangles(  1,  2,  3,  0,  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP2Triangles( 10, 11, 12,  0, 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_71_ = {
  gfx_Shape_of___POLYHEDRON_71_,
  vtx_Shape_of___POLYHEDRON_71_, 60, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner13[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_75_[] = {
  {    10,  -129,   -24,0, 0x0000,0x0000, 0x8d,0x46,0x00,0xff }, /*     0 */
  {    44,  -151,   -30,0, 0x0000,0x0000, 0x5a,0x2d,0x00,0xff }, /*     1 */
  {     5,  -151,   -23,0, 0x0000,0x0000, 0x65,0x32,0x00,0xff }, /*     2 */
  {    55,  -119,     8,0, 0x0000,0x0000, 0x7f,0x3f,0x00,0xff }, /*     3 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xb2,0x59,0x00,0xff }, /*     4 */
  {    52,  -109,    39,0, 0x0000,0x0000, 0x92,0x49,0x00,0xff }, /*     5 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xb2,0x59,0x00,0xff }, /*     6 */
  {    18,  -109,    45,0, 0x0000,0x0000, 0xa6,0x53,0x00,0xff }, /*     7 */
  {    52,  -109,    39,0, 0x0000,0x0000, 0x92,0x49,0x00,0xff }, /*     8 */
  {    59,  -151,     7,0, 0x0000,0x0000, 0x63,0x31,0x00,0xff }, /*     9 */
  {    13,  -151,    46,0, 0x0000,0x0000, 0x4d,0x26,0x00,0xff }, /*    10 */
  {     5,  -151,    17,0, 0x0000,0x0000, 0x52,0x29,0x00,0xff }, /*    11 */
  {     5,  -151,    17,0, 0x0000,0x0000, 0x52,0x29,0x00,0xff }, /*    12 */
  {    44,  -151,   -30,0, 0x0000,0x0000, 0x5a,0x2d,0x00,0xff }, /*    13 */
  {    59,  -151,     7,0, 0x0000,0x0000, 0x63,0x31,0x00,0xff }, /*    14 */
  {    18,  -109,    45,0, 0x0000,0x0000, 0xa6,0x53,0x00,0xff }, /*    15 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xb2,0x59,0x00,0xff }, /*    16 */
  {    13,  -151,    46,0, 0x0000,0x0000, 0x4d,0x26,0x00,0xff }, /*    17 */
  {    55,  -119,     8,0, 0x0000,0x0000, 0x7f,0x3f,0x00,0xff }, /*    18 */
  {    65,  -151,    37,0, 0x0000,0x0000, 0x67,0x33,0x00,0xff }, /*    19 */
  {    59,  -151,     7,0, 0x0000,0x0000, 0x63,0x31,0x00,0xff }, /*    20 */
  {    13,  -151,    46,0, 0x0000,0x0000, 0x4d,0x26,0x00,0xff }, /*    21 */
  {    39,  -138,    62,0, 0x0000,0x0000, 0x56,0x2b,0x00,0xff }, /*    22 */
  {    18,  -109,    45,0, 0x0000,0x0000, 0xa6,0x53,0x00,0xff }, /*    23 */
  {    65,  -151,    37,0, 0x0000,0x0000, 0x67,0x33,0x00,0xff }, /*    24 */
  {    39,  -138,    62,0, 0x0000,0x0000, 0x56,0x2b,0x00,0xff }, /*    25 */
  {    13,  -151,    46,0, 0x0000,0x0000, 0x4d,0x26,0x00,0xff }, /*    26 */
  {    52,  -109,    39,0, 0x0000,0x0000, 0x92,0x49,0x00,0xff }, /*    27 */
  {    39,  -138,    62,0, 0x0000,0x0000, 0x56,0x2b,0x00,0xff }, /*    28 */
  {    65,  -151,    37,0, 0x0000,0x0000, 0x67,0x33,0x00,0xff }, /*    29 */
  {    18,  -109,    45,0, 0x0000,0x0000, 0xa6,0x53,0x00,0xff }, /*    30 */
  {    39,  -138,    62,0, 0x0000,0x0000, 0x56,0x2b,0x00,0xff }, /*    31 */
  {    52,  -109,    39,0, 0x0000,0x0000, 0x92,0x49,0x00,0xff }, /*    32 */
  {    55,  -119,     8,0, 0x0000,0x0000, 0x7f,0x3f,0x00,0xff }, /*    33 */
  {    10,  -129,   -24,0, 0x0000,0x0000, 0x8d,0x46,0x00,0xff }, /*    34 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xb2,0x59,0x00,0xff }, /*    35 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xb2,0x59,0x00,0xff }, /*    36 */
  {     5,  -151,   -23,0, 0x0000,0x0000, 0x65,0x32,0x00,0xff }, /*    37 */
  {     5,  -151,    17,0, 0x0000,0x0000, 0x52,0x29,0x00,0xff }, /*    38 */
  {    59,  -151,     7,0, 0x0000,0x0000, 0x63,0x31,0x00,0xff }, /*    39 */
  {    41,  -129,   -29,0, 0x0000,0x0000, 0x71,0x38,0x00,0xff }, /*    40 */
  {    55,  -119,     8,0, 0x0000,0x0000, 0x7f,0x3f,0x00,0xff }, /*    41 */
  {    59,  -151,     7,0, 0x0000,0x0000, 0x63,0x31,0x00,0xff }, /*    42 */
  {    44,  -151,   -30,0, 0x0000,0x0000, 0x5a,0x2d,0x00,0xff }, /*    43 */
  {    41,  -129,   -29,0, 0x0000,0x0000, 0x71,0x38,0x00,0xff }, /*    44 */
  {     5,  -151,    17,0, 0x0000,0x0000, 0x52,0x29,0x00,0xff }, /*    45 */
  {     5,  -151,   -23,0, 0x0000,0x0000, 0x65,0x32,0x00,0xff }, /*    46 */
  {    44,  -151,   -30,0, 0x0000,0x0000, 0x5a,0x2d,0x00,0xff }, /*    47 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xb2,0x59,0x00,0xff }, /*    48 */
  {    10,  -129,   -24,0, 0x0000,0x0000, 0x8d,0x46,0x00,0xff }, /*    49 */
  {     5,  -151,   -23,0, 0x0000,0x0000, 0x65,0x32,0x00,0xff }, /*    50 */
  {    55,  -119,     8,0, 0x0000,0x0000, 0x7f,0x3f,0x00,0xff }, /*    51 */
  {    41,  -129,   -29,0, 0x0000,0x0000, 0x71,0x38,0x00,0xff }, /*    52 */
  {    10,  -129,   -24,0, 0x0000,0x0000, 0x8d,0x46,0x00,0xff }, /*    53 */
  {    55,  -119,     8,0, 0x0000,0x0000, 0x7f,0x3f,0x00,0xff }, /*    54 */
  {    52,  -109,    39,0, 0x0000,0x0000, 0x92,0x49,0x00,0xff }, /*    55 */
  {    65,  -151,    37,0, 0x0000,0x0000, 0x67,0x33,0x00,0xff }, /*    56 */
  {    10,  -119,    16,0, 0x0000,0x0000, 0xb2,0x59,0x00,0xff }, /*    57 */
  {     5,  -151,    17,0, 0x0000,0x0000, 0x52,0x29,0x00,0xff }, /*    58 */
  {    13,  -151,    46,0, 0x0000,0x0000, 0x4d,0x26,0x00,0xff }, /*    59 */
  {    59,  -151,     7,0, 0x0000,0x0000, 0x63,0x31,0x00,0xff }, /*    60 */
  {    65,  -151,    37,0, 0x0000,0x0000, 0x67,0x33,0x00,0xff }, /*    61 */
  {    13,  -151,    46,0, 0x0000,0x0000, 0x4d,0x26,0x00,0xff }, /*    62 */
  {    10,  -129,   -24,0, 0x0000,0x0000, 0x8d,0x46,0x00,0xff }, /*    63 */
  {    41,  -129,   -29,0, 0x0000,0x0000, 0x71,0x38,0x00,0xff }, /*    64 */
  {    44,  -151,   -30,0, 0x0000,0x0000, 0x5a,0x2d,0x00,0xff }, /*    65 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_75_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP2Triangles(  3,  4,  5,  0,  6,  7,  8,  0),
  gsSP2Triangles(  9, 10, 11,  0, 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 992), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 1024), 2, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_75_ = {
  gfx_Shape_of___POLYHEDRON_75_,
  vtx_Shape_of___POLYHEDRON_75_, 66, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner14[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_63_[] = {
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*     0 */
  {     0,   -70,    15,0, 0x0000,0x0000, 0x00,0x31,0x00,0xff }, /*     1 */
  {    53,   -70,    15,0, 0x0000,0x0000, 0x00,0x32,0x00,0xff }, /*     2 */
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*     3 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*     4 */
  {     0,   -70,    15,0, 0x0000,0x0000, 0x00,0x31,0x00,0xff }, /*     5 */
  {     0,    -9,   -35,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*     6 */
  {    45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x2c,0x00,0xff }, /*     7 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x37,0x00,0xff }, /*     8 */
  {     0,    -9,   -35,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*     9 */
  {    24,     7,   -24,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    10 */
  {    45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x2c,0x00,0xff }, /*    11 */
  {    45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x2c,0x00,0xff }, /*    12 */
  {    53,   -70,   -37,0, 0x0000,0x0000, 0x00,0x31,0x00,0xff }, /*    13 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x37,0x00,0xff }, /*    14 */
  {    24,     7,   -24,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    15 */
  {     8,     7,   -24,0, 0x0000,0x0000, 0x00,0x50,0x00,0xff }, /*    16 */
  {    24,     7,    -3,0, 0x0000,0x0000, 0x00,0x52,0x00,0xff }, /*    17 */
  {    24,     7,    -3,0, 0x0000,0x0000, 0x00,0x52,0x00,0xff }, /*    18 */
  {     8,     7,    -3,0, 0x0000,0x0000, 0x00,0x52,0x00,0xff }, /*    19 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    20 */
  {     8,     7,   -24,0, 0x0000,0x0000, 0x00,0x50,0x00,0xff }, /*    21 */
  {     8,     7,    -3,0, 0x0000,0x0000, 0x00,0x52,0x00,0xff }, /*    22 */
  {    24,     7,    -3,0, 0x0000,0x0000, 0x00,0x52,0x00,0xff }, /*    23 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    24 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x37,0x00,0xff }, /*    25 */
  {     0,   -70,    15,0, 0x0000,0x0000, 0x00,0x31,0x00,0xff }, /*    26 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    27 */
  {     0,    -9,   -35,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*    28 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x37,0x00,0xff }, /*    29 */
  {    45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x2c,0x00,0xff }, /*    30 */
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    31 */
  {    53,   -70,   -37,0, 0x0000,0x0000, 0x00,0x31,0x00,0xff }, /*    32 */
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    33 */
  {    24,     7,    -3,0, 0x0000,0x0000, 0x00,0x52,0x00,0xff }, /*    34 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    35 */
  {    45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x2c,0x00,0xff }, /*    36 */
  {    24,     7,   -24,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    37 */
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    38 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    39 */
  {     8,     7,   -24,0, 0x0000,0x0000, 0x00,0x50,0x00,0xff }, /*    40 */
  {     0,    -9,   -35,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*    41 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    42 */
  {     8,     7,    -3,0, 0x0000,0x0000, 0x00,0x52,0x00,0xff }, /*    43 */
  {     8,     7,   -24,0, 0x0000,0x0000, 0x00,0x50,0x00,0xff }, /*    44 */
  {     0,    -9,   -35,0, 0x0000,0x0000, 0x00,0x48,0x00,0xff }, /*    45 */
  {     8,     7,   -24,0, 0x0000,0x0000, 0x00,0x50,0x00,0xff }, /*    46 */
  {    24,     7,   -24,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    47 */
  {    24,     7,   -24,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    48 */
  {    24,     7,    -3,0, 0x0000,0x0000, 0x00,0x52,0x00,0xff }, /*    49 */
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    50 */
  {    46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    51 */
  {    53,   -70,    15,0, 0x0000,0x0000, 0x00,0x32,0x00,0xff }, /*    52 */
  {    53,   -70,   -37,0, 0x0000,0x0000, 0x00,0x31,0x00,0xff }, /*    53 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_63_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP2Triangles( 24, 25, 26,  0, 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 22, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP2Triangles(  7,  8,  9,  0, 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_63_ = {
  gfx_Shape_of___POLYHEDRON_63_,
  vtx_Shape_of___POLYHEDRON_63_, 54, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner15[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_61_[] = {
  {     5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x34,0x00,0xff }, /*     0 */
  {   -30,    11,    22,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*     1 */
  {    -5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x3a,0x00,0xff }, /*     2 */
  {     5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x34,0x00,0xff }, /*     3 */
  {    30,    11,    22,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*     4 */
  {   -30,    11,    22,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*     5 */
  {    -6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x35,0x00,0xff }, /*     6 */
  {    30,    11,   -37,0, 0x0000,0x0000, 0x00,0x28,0x00,0xff }, /*     7 */
  {     6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x2f,0x00,0xff }, /*     8 */
  {   -30,    11,    22,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*     9 */
  {   -30,    11,   -37,0, 0x0000,0x0000, 0x00,0x49,0x00,0xff }, /*    10 */
  {    -6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x35,0x00,0xff }, /*    11 */
  {    -6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x35,0x00,0xff }, /*    12 */
  {   -30,    11,   -37,0, 0x0000,0x0000, 0x00,0x49,0x00,0xff }, /*    13 */
  {    30,    11,   -37,0, 0x0000,0x0000, 0x00,0x28,0x00,0xff }, /*    14 */
  {   -30,    11,    22,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    15 */
  {    30,    11,   -37,0, 0x0000,0x0000, 0x00,0x28,0x00,0xff }, /*    16 */
  {   -30,    11,   -37,0, 0x0000,0x0000, 0x00,0x49,0x00,0xff }, /*    17 */
  {   -30,    11,    22,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    18 */
  {    30,    11,    22,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    19 */
  {    30,    11,   -37,0, 0x0000,0x0000, 0x00,0x28,0x00,0xff }, /*    20 */
  {     6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x2f,0x00,0xff }, /*    21 */
  {    30,    11,    22,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    22 */
  {     5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x34,0x00,0xff }, /*    23 */
  {     6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x2f,0x00,0xff }, /*    24 */
  {    30,    11,   -37,0, 0x0000,0x0000, 0x00,0x28,0x00,0xff }, /*    25 */
  {    30,    11,    22,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    26 */
  {    -5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x3a,0x00,0xff }, /*    27 */
  {   -30,    11,    22,0, 0x0000,0x0000, 0x00,0x44,0x00,0xff }, /*    28 */
  {    -6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x35,0x00,0xff }, /*    29 */
  {     6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x2f,0x00,0xff }, /*    30 */
  {    -5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x3a,0x00,0xff }, /*    31 */
  {    -6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x35,0x00,0xff }, /*    32 */
  {     6,    -7,   -15,0, 0x0000,0x0000, 0x00,0x2f,0x00,0xff }, /*    33 */
  {     5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x34,0x00,0xff }, /*    34 */
  {    -5,    -7,    -4,0, 0x0000,0x0000, 0x00,0x3a,0x00,0xff }, /*    35 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_61_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP2Triangles( 21, 22, 23,  0, 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 4, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_61_ = {
  gfx_Shape_of___POLYHEDRON_61_,
  vtx_Shape_of___POLYHEDRON_61_, 36, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner16[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_59_[] = {
  {   -53,   -70,    15,0, 0x0000,0x0000, 0x00,0x2e,0x00,0xff }, /*     0 */
  {     0,   -70,    15,0, 0x0000,0x0000, 0x00,0x32,0x00,0xff }, /*     1 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4f,0x00,0xff }, /*     2 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x34,0x00,0xff }, /*     3 */
  {   -45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x45,0x00,0xff }, /*     4 */
  {     1,    -9,   -35,0, 0x0000,0x0000, 0x00,0x2e,0x00,0xff }, /*     5 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x34,0x00,0xff }, /*     6 */
  {   -53,   -70,   -37,0, 0x0000,0x0000, 0x00,0x38,0x00,0xff }, /*     7 */
  {   -45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x45,0x00,0xff }, /*     8 */
  {   -24,     7,    -3,0, 0x0000,0x0000, 0x00,0x53,0x00,0xff }, /*     9 */
  {    -8,     7,   -24,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    10 */
  {   -24,     7,   -24,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    11 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    12 */
  {    -8,     7,    -3,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    13 */
  {   -24,     7,    -3,0, 0x0000,0x0000, 0x00,0x53,0x00,0xff }, /*    14 */
  {   -24,     7,    -3,0, 0x0000,0x0000, 0x00,0x53,0x00,0xff }, /*    15 */
  {    -8,     7,    -3,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    16 */
  {    -8,     7,   -24,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    17 */
  {     0,   -70,    15,0, 0x0000,0x0000, 0x00,0x32,0x00,0xff }, /*    18 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x34,0x00,0xff }, /*    19 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    20 */
  {   -53,   -70,   -37,0, 0x0000,0x0000, 0x00,0x38,0x00,0xff }, /*    21 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4f,0x00,0xff }, /*    22 */
  {   -45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x45,0x00,0xff }, /*    23 */
  {   -53,   -70,   -37,0, 0x0000,0x0000, 0x00,0x38,0x00,0xff }, /*    24 */
  {   -53,   -70,    15,0, 0x0000,0x0000, 0x00,0x2e,0x00,0xff }, /*    25 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4f,0x00,0xff }, /*    26 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    27 */
  {   -24,     7,    -3,0, 0x0000,0x0000, 0x00,0x53,0x00,0xff }, /*    28 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4f,0x00,0xff }, /*    29 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4f,0x00,0xff }, /*    30 */
  {   -24,     7,   -24,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    31 */
  {   -45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x45,0x00,0xff }, /*    32 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4f,0x00,0xff }, /*    33 */
  {   -24,     7,    -3,0, 0x0000,0x0000, 0x00,0x53,0x00,0xff }, /*    34 */
  {   -24,     7,   -24,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    35 */
  {   -45,   -16,   -35,0, 0x0000,0x0000, 0x00,0x45,0x00,0xff }, /*    36 */
  {   -24,     7,   -24,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    37 */
  {     1,    -9,   -35,0, 0x0000,0x0000, 0x00,0x2e,0x00,0xff }, /*    38 */
  {     1,    -9,   -35,0, 0x0000,0x0000, 0x00,0x2e,0x00,0xff }, /*    39 */
  {    -8,     7,   -24,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    40 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    41 */
  {    -8,     7,   -24,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    42 */
  {    -8,     7,    -3,0, 0x0000,0x0000, 0x00,0x51,0x00,0xff }, /*    43 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    44 */
  {   -24,     7,   -24,0, 0x0000,0x0000, 0x00,0x4b,0x00,0xff }, /*    45 */
  {    -8,     7,   -24,0, 0x0000,0x0000, 0x00,0x4d,0x00,0xff }, /*    46 */
  {     1,    -9,   -35,0, 0x0000,0x0000, 0x00,0x2e,0x00,0xff }, /*    47 */
  {     0,   -70,   -37,0, 0x0000,0x0000, 0x00,0x34,0x00,0xff }, /*    48 */
  {     1,    -9,   -35,0, 0x0000,0x0000, 0x00,0x2e,0x00,0xff }, /*    49 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    50 */
  {     0,   -70,    15,0, 0x0000,0x0000, 0x00,0x32,0x00,0xff }, /*    51 */
  {     0,    -9,    12,0, 0x0000,0x0000, 0x00,0x4e,0x00,0xff }, /*    52 */
  {   -46,   -16,    12,0, 0x0000,0x0000, 0x00,0x4f,0x00,0xff }, /*    53 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_59_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP2Triangles(  3,  4,  5,  0,  6,  7,  8,  0),
  gsSP2Triangles(  9, 10, 11,  0, 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP2Triangles( 21, 22, 23,  0, 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 22, 0 ),
  gsSP2Triangles( 30, 31,  0,  0,  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP1Triangle( 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_59_ = {
  gfx_Shape_of___POLYHEDRON_59_,
  vtx_Shape_of___POLYHEDRON_59_, 54, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner17[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_69_[] = {
  {   -13,  -104,     3,0, 0x0000,0x0000, 0xe1,0xa3,0x75,0xff }, /*     0 */
  {   -38,   -80,     3,0, 0x0000,0x0000, 0xf2,0xaf,0x7e,0xff }, /*     1 */
  {   -38,  -104,     3,0, 0x0000,0x0000, 0xb1,0x80,0x5c,0xff }, /*     2 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0xd8,0x9c,0x70,0xff }, /*     3 */
  {   -13,   -80,   -22,0, 0x0000,0x0000, 0xa1,0x74,0x54,0xff }, /*     4 */
  {   -13,  -104,   -22,0, 0x0000,0x0000, 0x8e,0x67,0x4a,0xff }, /*     5 */
  {   -13,  -104,     3,0, 0x0000,0x0000, 0xe1,0xa3,0x75,0xff }, /*     6 */
  {   -13,   -80,   -22,0, 0x0000,0x0000, 0xa1,0x74,0x54,0xff }, /*     7 */
  {   -13,   -80,     3,0, 0x0000,0x0000, 0xf1,0xae,0x7d,0xff }, /*     8 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0xd8,0x9c,0x70,0xff }, /*     9 */
  {   -38,   -80,     3,0, 0x0000,0x0000, 0xf2,0xaf,0x7e,0xff }, /*    10 */
  {   -38,   -80,   -22,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*    11 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0xd8,0x9c,0x70,0xff }, /*    12 */
  {   -38,  -104,     3,0, 0x0000,0x0000, 0xb1,0x80,0x5c,0xff }, /*    13 */
  {   -38,   -80,     3,0, 0x0000,0x0000, 0xf2,0xaf,0x7e,0xff }, /*    14 */
  {   -13,   -80,     3,0, 0x0000,0x0000, 0xf1,0xae,0x7d,0xff }, /*    15 */
  {   -26,   -67,    -9,0, 0x0000,0x0000, 0xff,0x6e,0x4f,0xff }, /*    16 */
  {   -38,   -80,     3,0, 0x0000,0x0000, 0xf2,0xaf,0x7e,0xff }, /*    17 */
  {   -38,   -80,     3,0, 0x0000,0x0000, 0xf2,0xaf,0x7e,0xff }, /*    18 */
  {   -26,   -67,    -9,0, 0x0000,0x0000, 0xff,0x6e,0x4f,0xff }, /*    19 */
  {   -38,   -80,   -22,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*    20 */
  {   -38,   -80,   -22,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*    21 */
  {   -26,   -67,    -9,0, 0x0000,0x0000, 0xff,0x6e,0x4f,0xff }, /*    22 */
  {   -13,   -80,   -22,0, 0x0000,0x0000, 0xa1,0x74,0x54,0xff }, /*    23 */
  {   -13,   -80,   -22,0, 0x0000,0x0000, 0xa1,0x74,0x54,0xff }, /*    24 */
  {   -26,   -67,    -9,0, 0x0000,0x0000, 0xff,0x6e,0x4f,0xff }, /*    25 */
  {   -13,   -80,     3,0, 0x0000,0x0000, 0xf1,0xae,0x7d,0xff }, /*    26 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0x86,0x43,0x00,0xff }, /*    27 */
  {   -38,  -138,     3,0, 0x0000,0x0000, 0x50,0x28,0x00,0xff }, /*    28 */
  {   -38,  -104,     3,0, 0x0000,0x0000, 0x6e,0x37,0x00,0xff }, /*    29 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0x86,0x43,0x00,0xff }, /*    30 */
  {   -38,  -138,   -22,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    31 */
  {   -38,  -138,     3,0, 0x0000,0x0000, 0x50,0x28,0x00,0xff }, /*    32 */
  {   -13,  -104,   -22,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    33 */
  {   -38,  -138,   -22,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    34 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0x86,0x43,0x00,0xff }, /*    35 */
  {   -13,  -104,   -22,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    36 */
  {   -13,  -138,   -22,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    37 */
  {   -38,  -138,   -22,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    38 */
  {   -13,  -104,     3,0, 0x0000,0x0000, 0x8c,0x46,0x00,0xff }, /*    39 */
  {   -13,  -138,     3,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    40 */
  {   -13,  -104,   -22,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    41 */
  {   -38,  -104,     3,0, 0x0000,0x0000, 0x6e,0x37,0x00,0xff }, /*    42 */
  {   -13,  -138,     3,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    43 */
  {   -13,  -104,     3,0, 0x0000,0x0000, 0x8c,0x46,0x00,0xff }, /*    44 */
  {   -38,  -104,     3,0, 0x0000,0x0000, 0x6e,0x37,0x00,0xff }, /*    45 */
  {   -38,  -138,     3,0, 0x0000,0x0000, 0x50,0x28,0x00,0xff }, /*    46 */
  {   -13,  -138,     3,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    47 */
  {   -13,  -138,     3,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    48 */
  {   -13,  -138,   -22,0, 0x0000,0x0000, 0x5e,0x2f,0x00,0xff }, /*    49 */
  {   -13,  -104,   -22,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    50 */
  {   -13,  -104,     3,0, 0x0000,0x0000, 0xe1,0xa3,0x75,0xff }, /*    51 */
  {   -13,  -104,   -22,0, 0x0000,0x0000, 0x8e,0x67,0x4a,0xff }, /*    52 */
  {   -13,   -80,   -22,0, 0x0000,0x0000, 0xa1,0x74,0x54,0xff }, /*    53 */
  {   -38,  -104,   -22,0, 0x0000,0x0000, 0xd8,0x9c,0x70,0xff }, /*    54 */
  {   -38,   -80,   -22,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*    55 */
  {   -13,   -80,   -22,0, 0x0000,0x0000, 0xa1,0x74,0x54,0xff }, /*    56 */
  {   -13,  -104,     3,0, 0x0000,0x0000, 0xe1,0xa3,0x75,0xff }, /*    57 */
  {   -13,   -80,     3,0, 0x0000,0x0000, 0xf1,0xae,0x7d,0xff }, /*    58 */
  {   -38,   -80,     3,0, 0x0000,0x0000, 0xf2,0xaf,0x7e,0xff }, /*    59 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_69_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP1Triangle(  3,  4,  5,  0),
  gsSP1Triangle(  6,  7,  8,  0),
  gsSP2Triangles(  9, 10, 11,  0, 12, 13, 14,  0),
  gsSP1Triangle( 15, 16, 17,  0),
  gsSP1Triangle( 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 28, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP2Triangles(  1,  2,  3,  0,  4,  5,  6,  0),
  gsSP1Triangle(  7,  8,  9,  0),
  gsSP2Triangles( 10, 11, 12,  0, 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_69_ = {
  gfx_Shape_of___POLYHEDRON_69_,
  vtx_Shape_of___POLYHEDRON_69_, 60, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner18[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_73_[] = {
  {    -5,  -151,   -23,0, 0x0000,0x0000, 0x59,0x2c,0x00,0xff }, /*     0 */
  {   -44,  -151,   -30,0, 0x0000,0x0000, 0x4b,0x25,0x00,0xff }, /*     1 */
  {   -10,  -129,   -24,0, 0x0000,0x0000, 0x77,0x3b,0x00,0xff }, /*     2 */
  {   -52,  -109,    39,0, 0x0000,0x0000, 0xab,0x55,0x00,0xff }, /*     3 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x80,0x40,0x00,0xff }, /*     4 */
  {   -55,  -119,     8,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*     5 */
  {   -52,  -109,    39,0, 0x0000,0x0000, 0xab,0x55,0x00,0xff }, /*     6 */
  {   -18,  -109,    45,0, 0x0000,0x0000, 0x94,0x4a,0x00,0xff }, /*     7 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x80,0x40,0x00,0xff }, /*     8 */
  {    -5,  -151,    17,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*     9 */
  {   -13,  -151,    46,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    10 */
  {   -59,  -151,     7,0, 0x0000,0x0000, 0x53,0x29,0x00,0xff }, /*    11 */
  {   -59,  -151,     7,0, 0x0000,0x0000, 0x53,0x29,0x00,0xff }, /*    12 */
  {   -44,  -151,   -30,0, 0x0000,0x0000, 0x4b,0x25,0x00,0xff }, /*    13 */
  {    -5,  -151,    17,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*    14 */
  {   -13,  -151,    46,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    15 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x80,0x40,0x00,0xff }, /*    16 */
  {   -18,  -109,    45,0, 0x0000,0x0000, 0x94,0x4a,0x00,0xff }, /*    17 */
  {   -59,  -151,     7,0, 0x0000,0x0000, 0x53,0x29,0x00,0xff }, /*    18 */
  {   -65,  -151,    37,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    19 */
  {   -55,  -119,     8,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*    20 */
  {   -18,  -109,    45,0, 0x0000,0x0000, 0x94,0x4a,0x00,0xff }, /*    21 */
  {   -39,  -138,    62,0, 0x0000,0x0000, 0x51,0x28,0x00,0xff }, /*    22 */
  {   -13,  -151,    46,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    23 */
  {   -13,  -151,    46,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    24 */
  {   -39,  -138,    62,0, 0x0000,0x0000, 0x51,0x28,0x00,0xff }, /*    25 */
  {   -65,  -151,    37,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    26 */
  {   -65,  -151,    37,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    27 */
  {   -39,  -138,    62,0, 0x0000,0x0000, 0x51,0x28,0x00,0xff }, /*    28 */
  {   -52,  -109,    39,0, 0x0000,0x0000, 0xab,0x55,0x00,0xff }, /*    29 */
  {   -52,  -109,    39,0, 0x0000,0x0000, 0xab,0x55,0x00,0xff }, /*    30 */
  {   -39,  -138,    62,0, 0x0000,0x0000, 0x51,0x28,0x00,0xff }, /*    31 */
  {   -18,  -109,    45,0, 0x0000,0x0000, 0x94,0x4a,0x00,0xff }, /*    32 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x80,0x40,0x00,0xff }, /*    33 */
  {   -10,  -129,   -24,0, 0x0000,0x0000, 0x77,0x3b,0x00,0xff }, /*    34 */
  {   -55,  -119,     8,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*    35 */
  {    -5,  -151,    17,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*    36 */
  {    -5,  -151,   -23,0, 0x0000,0x0000, 0x59,0x2c,0x00,0xff }, /*    37 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x80,0x40,0x00,0xff }, /*    38 */
  {   -55,  -119,     8,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*    39 */
  {   -41,  -129,   -29,0, 0x0000,0x0000, 0x72,0x39,0x00,0xff }, /*    40 */
  {   -59,  -151,     7,0, 0x0000,0x0000, 0x53,0x29,0x00,0xff }, /*    41 */
  {   -13,  -151,    46,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    42 */
  {    -5,  -151,    17,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*    43 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x80,0x40,0x00,0xff }, /*    44 */
  {   -41,  -129,   -29,0, 0x0000,0x0000, 0x72,0x39,0x00,0xff }, /*    45 */
  {   -44,  -151,   -30,0, 0x0000,0x0000, 0x4b,0x25,0x00,0xff }, /*    46 */
  {   -59,  -151,     7,0, 0x0000,0x0000, 0x53,0x29,0x00,0xff }, /*    47 */
  {   -44,  -151,   -30,0, 0x0000,0x0000, 0x4b,0x25,0x00,0xff }, /*    48 */
  {    -5,  -151,   -23,0, 0x0000,0x0000, 0x59,0x2c,0x00,0xff }, /*    49 */
  {    -5,  -151,    17,0, 0x0000,0x0000, 0x6a,0x35,0x00,0xff }, /*    50 */
  {    -5,  -151,   -23,0, 0x0000,0x0000, 0x59,0x2c,0x00,0xff }, /*    51 */
  {   -10,  -129,   -24,0, 0x0000,0x0000, 0x77,0x3b,0x00,0xff }, /*    52 */
  {   -10,  -119,    16,0, 0x0000,0x0000, 0x80,0x40,0x00,0xff }, /*    53 */
  {   -10,  -129,   -24,0, 0x0000,0x0000, 0x77,0x3b,0x00,0xff }, /*    54 */
  {   -41,  -129,   -29,0, 0x0000,0x0000, 0x72,0x39,0x00,0xff }, /*    55 */
  {   -55,  -119,     8,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*    56 */
  {   -65,  -151,    37,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    57 */
  {   -52,  -109,    39,0, 0x0000,0x0000, 0xab,0x55,0x00,0xff }, /*    58 */
  {   -55,  -119,     8,0, 0x0000,0x0000, 0xa7,0x53,0x00,0xff }, /*    59 */
  {   -13,  -151,    46,0, 0x0000,0x0000, 0x60,0x30,0x00,0xff }, /*    60 */
  {   -65,  -151,    37,0, 0x0000,0x0000, 0x58,0x2c,0x00,0xff }, /*    61 */
  {   -59,  -151,     7,0, 0x0000,0x0000, 0x53,0x29,0x00,0xff }, /*    62 */
  {   -44,  -151,   -30,0, 0x0000,0x0000, 0x4b,0x25,0x00,0xff }, /*    63 */
  {   -41,  -129,   -29,0, 0x0000,0x0000, 0x72,0x39,0x00,0xff }, /*    64 */
  {   -10,  -129,   -24,0, 0x0000,0x0000, 0x77,0x3b,0x00,0xff }, /*    65 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_73_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP1Triangle(  0,  1,  2,  0),
  gsSP2Triangles(  3,  4,  5,  0,  6,  7,  8,  0),
  gsSP2Triangles(  9, 10, 11,  0, 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 30, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSP1Triangle(  4,  5,  6,  0),
  gsSP2Triangles(  7,  8,  9,  0, 10, 11, 12,  0),
  gsSP1Triangle( 13, 14, 15,  0),
  gsSP1Triangle( 16, 17, 18,  0),
  gsSP1Triangle( 19, 20, 21,  0),
  gsSP1Triangle( 22, 23, 24,  0),
  gsSP1Triangle( 25, 26, 27,  0),
  gsSPVertex( SEGADRS(5, 992), 2, 30 ),
  gsSP1Triangle( 28, 29, 30,  0),
  gsSPVertex( SEGADRS(5, 1024), 2, 0 ),
  gsSP1Triangle( 31,  0,  1,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_73_ = {
  gfx_Shape_of___POLYHEDRON_73_,
  vtx_Shape_of___POLYHEDRON_73_, 66, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

static Gfx _frake_run_aligner19[] = { gsSPEndDisplayList() };

/* ============================================================
          Vtx
 ============================================================ */
static Vtx vtx_Shape_of___POLYHEDRON_65_[] = {
  {   -38,   -53,   -22,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*     0 */
  {   -38,   -75,     3,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*     1 */
  {   -38,   -53,     3,0, 0x0000,0x0000, 0xff,0xb9,0x85,0xff }, /*     2 */
  {   -38,   -53,   -22,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*     3 */
  {   -38,   -75,   -22,0, 0x0000,0x0000, 0xb7,0x84,0x5f,0xff }, /*     4 */
  {   -38,   -75,     3,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*     5 */
  {   -13,   -53,   -22,0, 0x0000,0x0000, 0xbc,0x88,0x62,0xff }, /*     6 */
  {   -38,   -75,   -22,0, 0x0000,0x0000, 0xb7,0x84,0x5f,0xff }, /*     7 */
  {   -38,   -53,   -22,0, 0x0000,0x0000, 0xd5,0x9a,0x6f,0xff }, /*     8 */
  {   -13,   -53,   -22,0, 0x0000,0x0000, 0xbc,0x88,0x62,0xff }, /*     9 */
  {   -13,   -75,   -22,0, 0x0000,0x0000, 0xa0,0x73,0x53,0xff }, /*    10 */
  {   -38,   -75,   -22,0, 0x0000,0x0000, 0xb7,0x84,0x5f,0xff }, /*    11 */
  {   -13,   -53,     3,0, 0x0000,0x0000, 0xe7,0xa7,0x78,0xff }, /*    12 */
  {   -13,   -75,     3,0, 0x0000,0x0000, 0xa5,0x77,0x55,0xff }, /*    13 */
  {   -13,   -53,   -22,0, 0x0000,0x0000, 0xbc,0x88,0x62,0xff }, /*    14 */
  {   -38,   -53,     3,0, 0x0000,0x0000, 0xff,0xb9,0x85,0xff }, /*    15 */
  {   -13,   -75,     3,0, 0x0000,0x0000, 0xa5,0x77,0x55,0xff }, /*    16 */
  {   -13,   -53,     3,0, 0x0000,0x0000, 0xe7,0xa7,0x78,0xff }, /*    17 */
  {   -38,   -53,     3,0, 0x0000,0x0000, 0xff,0xb9,0x85,0xff }, /*    18 */
  {   -38,   -75,     3,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*    19 */
  {   -13,   -75,     3,0, 0x0000,0x0000, 0xa5,0x77,0x55,0xff }, /*    20 */
  {   -13,   -75,     3,0, 0x0000,0x0000, 0xa5,0x77,0x55,0xff }, /*    21 */
  {   -13,   -75,   -22,0, 0x0000,0x0000, 0xa0,0x73,0x53,0xff }, /*    22 */
  {   -13,   -53,   -22,0, 0x0000,0x0000, 0xbc,0x88,0x62,0xff }, /*    23 */
  {   -13,   -75,   -22,0, 0x0000,0x0000, 0xa0,0x73,0x53,0xff }, /*    24 */
  {   -26,   -86,    -9,0, 0x0000,0x0000, 0x81,0x38,0x28,0xff }, /*    25 */
  {   -38,   -75,   -22,0, 0x0000,0x0000, 0xb7,0x84,0x5f,0xff }, /*    26 */
  {   -38,   -75,   -22,0, 0x0000,0x0000, 0xb7,0x84,0x5f,0xff }, /*    27 */
  {   -26,   -86,    -9,0, 0x0000,0x0000, 0x81,0x38,0x28,0xff }, /*    28 */
  {   -38,   -75,     3,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*    29 */
  {   -38,   -75,     3,0, 0x0000,0x0000, 0x90,0x68,0x4b,0xff }, /*    30 */
  {   -26,   -86,    -9,0, 0x0000,0x0000, 0x81,0x38,0x28,0xff }, /*    31 */
  {   -13,   -75,     3,0, 0x0000,0x0000, 0xa5,0x77,0x55,0xff }, /*    32 */
  {   -13,   -75,     3,0, 0x0000,0x0000, 0xa5,0x77,0x55,0xff }, /*    33 */
  {   -26,   -86,    -9,0, 0x0000,0x0000, 0x81,0x38,0x28,0xff }, /*    34 */
  {   -13,   -75,   -22,0, 0x0000,0x0000, 0xa0,0x73,0x53,0xff }, /*    35 */
};

/* ============================================================
          Gfx
 ============================================================ */
static Gfx gfx_Shape_of___POLYHEDRON_65_[] = {
  gsSPTexture(0x8000, 0x8000, 0, G_TX_RENDERTILE, G_OFF),
  gsSPSetGeometryMode( G_SHADE|G_SHADING_SMOOTH ),
  gsDPSetCombineLERP(1, 0, SHADE, 0, 0, 0, 0, SHADE, 1, 0, SHADE, 0, 0, 0, 0, SHADE),
  gsSPVertex( SEGADRS(5, 0), 32, 0 ),
  gsSP2Triangles(  0,  1,  2,  0,  3,  4,  5,  0),
  gsSP2Triangles(  6,  7,  8,  0,  9, 10, 11,  0),
  gsSP1Triangle( 12, 13, 14,  0),
  gsSP2Triangles( 15, 16, 17,  0, 18, 19, 20,  0),
  gsSP1Triangle( 21, 22, 23,  0),
  gsSP1Triangle( 24, 25, 26,  0),
  gsSP1Triangle( 27, 28, 29,  0),
  gsSPVertex( SEGADRS(5, 512), 4, 0 ),
  gsSP1Triangle( 30, 31,  0,  0),
  gsSP1Triangle(  1,  2,  3,  0),
  gsSPEndDisplayList(),
};

/* ============================================================
          Shape Structure
 ============================================================ */
static NDR_SHAPE shape_Shape_of___POLYHEDRON_65_ = {
  gfx_Shape_of___POLYHEDRON_65_,
  vtx_Shape_of___POLYHEDRON_65_, 36, 5,
  bmp_frake_run,
  NULL,
  6, 7,
  NULL,
  NULL,
  NULL,
  NULL
};

/* ============================================================
          Geometric Animation Data Area
 ============================================================ */
static s16 Animation_for_body_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch0_val[]={
  0.0000, 1.6896, 3.3474, 4.9421, 5.7064, 4.1545, 2.5244, 0.8468, -0.6141,
  -1.8363, -3.0402, -4.2141, -3.6317, -2.4413, -1.2267, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_body_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch1_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_body_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch2_val[]={
  0.0000, -0.1162, -0.4627, -1.0328, -1.3986, -0.7204, -0.2610, -0.0291,
  -0.0153, -0.1374, -0.3804, -0.7418, -0.5465, -0.2439, -0.0611, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_body_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch3_val[]={
  0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_body_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch4_val[]={
  0.0000, 0.1373, 0.2747, 0.4120, 0.4807, 0.3434, 0.2060, 0.0687, -0.0498,
  -0.1494, -0.2489, -0.3485, -0.2987, -0.1991, -0.0996, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_body_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch5_val[]={
  0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_body_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_body_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_body_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_body_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_body_motion_ch0_key, Animation_for_body_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_body_motion_ch1_key, Animation_for_body_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_body_motion_ch2_key, Animation_for_body_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_body_motion_ch3_key, Animation_for_body_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_body_motion_ch4_key, Animation_for_body_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_body_motion_ch5_key, Animation_for_body_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_body_motion_ch6_key, Animation_for_body_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_body_motion_ch7_key, Animation_for_body_motion_ch7_val},
};

static NDR_ANIM Animation_for_body_motion={
  Animation_for_body_motion_chlist, 8, 30
};

static s16 Animation_for_R_uparm_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch0_val[]={
  25.1702, 21.6565, 18.2200, 15.1462, 13.5953, 14.5967, 14.4244, 12.5510,
  7.7307, 2.1350, -2.4030, -5.7108, -0.2392, 8.5153, 17.2441, 25.1702,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_uparm_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch1_val[]={
  13.8951, 12.5982, 13.0885, 15.2235, 15.6495, 9.3015, 6.6922, 7.9755,
  9.9484, 11.0013, 13.5808, 17.4875, 13.4046, 9.6057, 9.7497, 13.8951,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_uparm_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch2_val[]={
  -5.5650, 6.8178, 18.6519, 29.6714, 32.7202, 13.2020, -7.7993, -29.2207,
  -44.0737, -52.6937, -61.0712, -68.9817, -59.1627, -42.1000, -23.9841,
  -5.5651, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_uparm_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch3_val[]={
  0.0555, -0.0937, -0.2437, -0.3958, -0.4517, -0.2129, 0.0210, 0.2502,
  0.4240, 0.5492, 0.6792, 0.8146, 0.6904, 0.4776, 0.2658, 0.0555, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_uparm_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch4_val[]={
  0.0001, 0.0617, 0.1248, 0.1870, 0.2227, 0.1825, 0.1414, 0.1077, 0.0564,
  -0.0179, -0.0847, -0.1426, -0.1458, -0.1138, -0.0635, 0.0001, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_uparm_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch5_val[]={
  0.2684, 0.2209, 0.1593, 0.0834, 0.0386, 0.1066, 0.1440, 0.1524, 0.1049,
  0.0149, -0.0829, -0.1882, -0.0963, 0.0480, 0.1707, 0.2684, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_uparm_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_uparm_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_uparm_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_uparm_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_uparm_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_uparm_motion_ch0_key, Animation_for_R_uparm_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_uparm_motion_ch1_key, Animation_for_R_uparm_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_uparm_motion_ch2_key, Animation_for_R_uparm_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_uparm_motion_ch3_key, Animation_for_R_uparm_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_uparm_motion_ch4_key, Animation_for_R_uparm_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_uparm_motion_ch5_key, Animation_for_R_uparm_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_uparm_motion_ch6_key, Animation_for_R_uparm_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_uparm_motion_ch7_key, Animation_for_R_uparm_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_uparm_motion_ch8_key, Animation_for_R_uparm_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_uparm_motion={
  Animation_for_R_uparm_motion_chlist, 9, 30
};

static s16 Animation_for_R_lowarm_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch0_val[]={
  1.2748, 1.0096, 0.1823, -1.1590, -2.1937, -1.2217, -0.3899, 0.2861,
  0.6991, 0.9312, 1.1044, 1.2179, 1.2520, 1.2665, 1.2739, 1.2748, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_R_lowarm_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch1_val[]={
  1.7513, 1.3869, 0.2503, -1.5924, -3.0137, -1.6786, -0.5357, 0.3929,
  0.9605, 1.2791, 1.5172, 1.6732, 1.7201, 1.7397, 1.7501, 1.7513, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_lowarm_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch2_val[]={
  -8.1146, -12.1627, -15.9856, -19.3613, -21.1104, -19.4829, -17.6449,
  -15.6306, -13.9511, -12.6897, -11.4014, -10.0940, -9.4355, -8.9957,
  -8.5552, -8.1146, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
 };

static s16 Animation_for_R_lowarm_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch3_val[]={
  -0.4177, -0.6393, -0.8836, -1.1533, -1.3281, -1.1645, -1.0076, -0.8587,
  -0.7478, -0.6704, -0.5954, -0.5228, -0.4873, -0.4639, -0.4407, -0.4177,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_lowarm_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch4_val[]={
  0.2871, 0.4097, 0.5125, 0.5871, 0.6151, 0.5894, 0.5517, 0.5036, 0.4596,
  0.4247, 0.3876, 0.3486, 0.3285, 0.3148, 0.3010, 0.2871, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_lowarm_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch5_val[]={
  -0.0612, -0.1373, -0.2466, -0.3883, -0.4873, -0.3945, -0.3096, -0.2345,
  -0.1831, -0.1499, -0.1203, -0.0942, -0.0823, -0.0749, -0.0679, -0.0612,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_R_lowarm_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_lowarm_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_lowarm_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowarm_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_lowarm_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch0_key, Animation_for_R_lowarm_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch1_key, Animation_for_R_lowarm_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch2_key, Animation_for_R_lowarm_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch3_key, Animation_for_R_lowarm_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch4_key, Animation_for_R_lowarm_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch5_key, Animation_for_R_lowarm_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch6_key, Animation_for_R_lowarm_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch7_key, Animation_for_R_lowarm_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_lowarm_motion_ch8_key, Animation_for_R_lowarm_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_lowarm_motion={
  Animation_for_R_lowarm_motion_chlist, 9, 30
};

static s16 Animation_for_R_hand_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch0_val[]={
  -30.2555, -22.2586, -14.9626, -8.7616, -4.9218, -4.4865, -4.0685,
  -3.6683, -5.5156, -10.7332, -17.2305, -24.6688, -27.3499, -28.3137,
  -29.2824, -30.2555, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000,
};

static s16 Animation_for_R_hand_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch1_val[]={
  -40.4941, -29.7881, -20.0212, -11.7199, -6.5763, -5.9804, -5.4074,
  -4.8579, -7.3232, -14.3060, -23.0022, -32.9585, -36.5588, -37.8643,
  -39.1765, -40.4940, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
 };

static s16 Animation_for_R_hand_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch2_val[]={
  -67.1870, -63.6699, -57.1650, -48.0234, -39.4288, -38.2180, -36.9897,
  -35.7447, -41.0078, -51.4367, -59.6173, -65.1229, -66.3238, -66.6568,
  -66.9446, -67.1870, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000,
};

static s16 Animation_for_R_hand_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch3_val[]={
  -1.5708, -1.2830, -1.0107, -0.7621, -0.5865, -0.5659, -0.5456, -0.5255,
  -0.6218, -0.8506, -1.1033, -1.3773, -1.4725, -1.5052, -1.5380, -1.5708,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_hand_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch4_val[]={
  0.6415, 0.6213, 0.5638, 0.4756, 0.3912, 0.3793, 0.3672, 0.3549, 0.4067,
  0.5088, 0.5863, 0.6318, 0.6389, 0.6404, 0.6412, 0.6415, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_hand_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch5_val[]={
  -0.6407, -0.4701, -0.3173, -0.1933, -0.1192, -0.1109, -0.1029, -0.0952,
  -0.1307, -0.2321, -0.3642, -0.5215, -0.5788, -0.5993, -0.6200, -0.6407,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_hand_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_hand_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_hand_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_hand_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_hand_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_hand_motion_ch0_key, Animation_for_R_hand_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_hand_motion_ch1_key, Animation_for_R_hand_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_hand_motion_ch2_key, Animation_for_R_hand_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_hand_motion_ch3_key, Animation_for_R_hand_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_hand_motion_ch4_key, Animation_for_R_hand_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_hand_motion_ch5_key, Animation_for_R_hand_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_hand_motion_ch6_key, Animation_for_R_hand_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_hand_motion_ch7_key, Animation_for_R_hand_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_hand_motion_ch8_key, Animation_for_R_hand_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_hand_motion={
  Animation_for_R_hand_motion_chlist, 9, 30
};

static s16 Animation_for_R_sode_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch0_val[]={
  25.1702, 21.6565, 18.2200, 15.1462, 13.5953, 14.5967, 14.4244, 12.5510,
  7.7307, 2.1350, -2.4030, -5.7108, -0.2392, 8.5153, 17.2441, 25.1702,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_sode_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch1_val[]={
  13.8951, 12.5982, 13.0885, 15.2235, 15.6495, 9.3015, 6.6922, 7.9755,
  9.9484, 11.0013, 13.5808, 17.4875, 13.4046, 9.6057, 9.7497, 13.8951,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_sode_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch2_val[]={
  -5.5650, 6.8178, 18.6519, 29.6714, 32.7202, 13.2020, -7.7993, -29.2207,
  -44.0737, -52.6937, -61.0712, -68.9817, -59.1627, -42.1000, -23.9841,
  -5.5651, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_sode_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch3_val[]={
  0.0555, -0.0937, -0.2437, -0.3958, -0.4517, -0.2129, 0.0210, 0.2502,
  0.4240, 0.5492, 0.6792, 0.8146, 0.6904, 0.4776, 0.2658, 0.0555, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_sode_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch4_val[]={
  0.0001, 0.0617, 0.1248, 0.1870, 0.2227, 0.1825, 0.1414, 0.1077, 0.0564,
  -0.0179, -0.0847, -0.1426, -0.1458, -0.1138, -0.0635, 0.0001, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_sode_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch5_val[]={
  0.2684, 0.2209, 0.1593, 0.0834, 0.0386, 0.1066, 0.1440, 0.1524, 0.1049,
  0.0149, -0.0829, -0.1882, -0.0963, 0.0480, 0.1707, 0.2684, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_sode_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_sode_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_sode_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_sode_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_sode_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_sode_motion_ch0_key, Animation_for_R_sode_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_sode_motion_ch1_key, Animation_for_R_sode_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_sode_motion_ch2_key, Animation_for_R_sode_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_sode_motion_ch3_key, Animation_for_R_sode_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_sode_motion_ch4_key, Animation_for_R_sode_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_sode_motion_ch5_key, Animation_for_R_sode_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_sode_motion_ch6_key, Animation_for_R_sode_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_sode_motion_ch7_key, Animation_for_R_sode_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_sode_motion_ch8_key, Animation_for_R_sode_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_sode_motion={
  Animation_for_R_sode_motion_chlist, 9, 30
};

static s16 Animation_for_L_uparm_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch0_val[]={
  -6.6255, -5.8384, -6.0096, -7.1432, -6.8229, -3.1611, -5.4456, -13.3019,
  -19.1743, -19.5236, -19.7810, -20.0835, -13.8671, -7.9994, -5.5485,
  -6.6256, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_uparm_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch1_val[]={
  5.0496, 7.1672, 10.4890, 14.8885, 16.0425, 7.5225, 3.9365, 6.1265,
  10.0411, 12.0844, 15.3449, 19.6854, 12.5835, 4.8336, 2.0651, 5.0496,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_uparm_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch2_val[]={
  -30.1689, -39.0956, -47.5180, -55.2086, -57.5496, -43.6878, -27.6488,
  -12.1874, 0.3440, 11.6283, 22.1873, 31.8198, 25.2639, 9.2686, -9.9251,
  -30.1688, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_uparm_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch3_val[]={
  0.4145, 0.5573, 0.7084, 0.8684, 0.9046, 0.5808, 0.3033, 0.0549, -0.1365,
  -0.2832, -0.4343, -0.5906, -0.4119, -0.1396, 0.1298, 0.4145, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_uparm_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch4_val[]={
  0.2476, 0.3225, 0.3886, 0.4438, 0.4428, 0.2726, 0.0639, -0.1671, -0.3087,
  -0.3518, -0.3934, -0.4315, -0.3024, -0.1109, 0.0769, 0.2476, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_uparm_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch5_val[]={
  0.0018, 0.0728, 0.1551, 0.2481, 0.2717, 0.0915, -0.0425, -0.1378,
  -0.1534, -0.1057, -0.0445, 0.0304, -0.0170, -0.0600, -0.0535, 0.0018,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_uparm_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_uparm_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_uparm_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_uparm_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_uparm_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_uparm_motion_ch0_key, Animation_for_L_uparm_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_uparm_motion_ch1_key, Animation_for_L_uparm_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_uparm_motion_ch2_key, Animation_for_L_uparm_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_uparm_motion_ch3_key, Animation_for_L_uparm_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_uparm_motion_ch4_key, Animation_for_L_uparm_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_uparm_motion_ch5_key, Animation_for_L_uparm_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_uparm_motion_ch6_key, Animation_for_L_uparm_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_uparm_motion_ch7_key, Animation_for_L_uparm_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_uparm_motion_ch8_key, Animation_for_L_uparm_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_uparm_motion={
  Animation_for_L_uparm_motion_chlist, 9, 30
};

static s16 Animation_for_L_lowarm_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch0_val[]={
  -1.0019, -0.9891, -0.9763, -0.9633, -1.0293, -1.2318, -1.2690, -1.1401,
  -1.2569, -0.8973, 0.5425, 2.8997, 2.3027, 0.8275, -0.2908, -1.0018,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_lowarm_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch1_val[]={
  1.3762, 1.3591, 1.3415, 1.3233, 1.4140, 1.6923, 1.7432, 1.5663, 1.7267,
  1.2326, -0.7453, -3.9838, -3.1633, -1.1369, 0.3996, 1.3763, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_lowarm_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch2_val[]={
  -12.2184, -12.3067, -12.3949, -12.4830, -12.0181, -9.8609, -7.6774,
  -5.5040, -7.2521, -12.8983, -18.0236, -22.0498, -21.2669, -18.6768,
  -15.6143, -12.2184, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
 };

static s16 Animation_for_L_lowarm_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch3_val[]={
  -0.6425, -0.6477, -0.6529, -0.6581, -0.6308, -0.5102, -0.3950, -0.2843,
  -0.3732, -0.6829, -1.0380, -1.4383, -1.3455, -1.0927, -0.8576, -0.6425,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_lowarm_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch4_val[]={
  -0.4113, -0.4138, -0.4163, -0.4188, -0.4055, -0.3415, -0.2731, -0.2014,
  -0.2594, -0.4306, -0.5600, -0.6250, -0.6171, -0.5738, -0.5032, -0.4113,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_lowarm_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch5_val[]={
  0.1386, 0.1407, 0.1428, 0.1449, 0.1340, 0.0899, 0.0550, 0.0289, 0.0492,
  0.1551, 0.3256, 0.5514, 0.4974, 0.3550, 0.2340, 0.1386, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_L_lowarm_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_lowarm_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_lowarm_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowarm_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_lowarm_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch0_key, Animation_for_L_lowarm_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch1_key, Animation_for_L_lowarm_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch2_key, Animation_for_L_lowarm_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch3_key, Animation_for_L_lowarm_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch4_key, Animation_for_L_lowarm_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch5_key, Animation_for_L_lowarm_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch6_key, Animation_for_L_lowarm_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch7_key, Animation_for_L_lowarm_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_lowarm_motion_ch8_key, Animation_for_L_lowarm_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_lowarm_motion={
  Animation_for_L_lowarm_motion_chlist, 9, 30
};

static s16 Animation_for_L_hand_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch0_val[]={
  -0.1114, 1.0639, 7.5573, 18.3578, 25.8234, 16.5470, 8.7815, 3.1539,
  1.3703, 1.5319, 1.7008, 1.8767, 0.8888, -0.0077, -0.3430, -0.1114,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_hand_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch1_val[]={
  0.1148, -1.4577, -10.1495, -24.6080, -34.6015, -22.1800, -11.7805,
  -4.2437, -1.8527, -2.0653, -2.2872, -2.5187, -1.2011, -0.0081, 0.4327,
  0.1148, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
 };

static s16 Animation_for_L_hand_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch2_val[]={
  -1.4775, -24.8491, -45.6433, -60.6240, -65.6219, -58.8792, -48.0457,
  -33.9967, -26.5250, -27.3448, -28.1607, -28.9726, -23.8128, -16.4565,
  -8.9716, -1.4775, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
 };

static s16 Animation_for_L_hand_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch3_val[]={
  -0.0156, -0.3411, -0.7045, -1.1340, -1.4077, -1.0665, -0.7587, -0.4868,
  -0.3682, -0.3812, -0.3943, -0.4074, -0.3285, -0.2220, -0.1181, -0.0156,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_L_hand_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch4_val[]={
  -0.0148, -0.2475, -0.4524, -0.5958, -0.6354, -0.5799, -0.4759, -0.3377,
  -0.2641, -0.2722, -0.2802, -0.2882, -0.2373, -0.1646, -0.0902, -0.0148,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_L_hand_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch5_val[]={
  0.0001, 0.0434, 0.1698, 0.3876, 0.5463, 0.3499, 0.1936, 0.0851, 0.0498,
  0.0531, 0.0565, 0.0600, 0.0397, 0.0186, 0.0055, 0.0001, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_hand_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_hand_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_hand_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_hand_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_hand_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_hand_motion_ch0_key, Animation_for_L_hand_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_hand_motion_ch1_key, Animation_for_L_hand_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_hand_motion_ch2_key, Animation_for_L_hand_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_hand_motion_ch3_key, Animation_for_L_hand_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_hand_motion_ch4_key, Animation_for_L_hand_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_hand_motion_ch5_key, Animation_for_L_hand_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_hand_motion_ch6_key, Animation_for_L_hand_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_hand_motion_ch7_key, Animation_for_L_hand_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_hand_motion_ch8_key, Animation_for_L_hand_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_hand_motion={
  Animation_for_L_hand_motion_chlist, 9, 30
};

static s16 Animation_for_L_sode_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch0_val[]={
  -6.6255, -5.8384, -6.0096, -7.1432, -6.8229, -3.1611, -5.4456, -13.3019,
  -19.1743, -19.5236, -19.7810, -20.0835, -13.8671, -7.9994, -5.5485,
  -6.6256, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_sode_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch1_val[]={
  5.0496, 7.1672, 10.4890, 14.8885, 16.0425, 7.5225, 3.9365, 6.1265,
  10.0411, 12.0844, 15.3449, 19.6854, 12.5835, 4.8336, 2.0651, 5.0496,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_sode_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch2_val[]={
  -30.1689, -39.0956, -47.5180, -55.2086, -57.5496, -43.6878, -27.6488,
  -12.1874, 0.3440, 11.6283, 22.1873, 31.8198, 25.2639, 9.2686, -9.9251,
  -30.1688, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_sode_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch3_val[]={
  0.4145, 0.5573, 0.7084, 0.8684, 0.9046, 0.5808, 0.3033, 0.0549, -0.1365,
  -0.2832, -0.4343, -0.5906, -0.4119, -0.1396, 0.1298, 0.4145, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_sode_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch4_val[]={
  0.2476, 0.3225, 0.3886, 0.4438, 0.4428, 0.2726, 0.0639, -0.1671, -0.3087,
  -0.3518, -0.3934, -0.4315, -0.3024, -0.1109, 0.0769, 0.2476, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_sode_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch5_val[]={
  0.0018, 0.0728, 0.1551, 0.2481, 0.2717, 0.0915, -0.0425, -0.1378,
  -0.1534, -0.1057, -0.0445, 0.0304, -0.0170, -0.0600, -0.0535, 0.0018,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_sode_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_sode_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_sode_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_sode_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_sode_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_sode_motion_ch0_key, Animation_for_L_sode_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_sode_motion_ch1_key, Animation_for_L_sode_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_sode_motion_ch2_key, Animation_for_L_sode_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_sode_motion_ch3_key, Animation_for_L_sode_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_sode_motion_ch4_key, Animation_for_L_sode_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_sode_motion_ch5_key, Animation_for_L_sode_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_sode_motion_ch6_key, Animation_for_L_sode_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_sode_motion_ch7_key, Animation_for_L_sode_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_sode_motion_ch8_key, Animation_for_L_sode_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_sode_motion={
  Animation_for_L_sode_motion_chlist, 9, 30
};

static s16 Animation_for_head_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch0_val[]={
  0.0000, -1.7175, -3.4015, -5.0194, -5.7936, -4.2207, -2.5658, -0.8608,
  0.5647, 1.6895, 2.8001, 3.8873, 3.3472, 2.2472, 1.1283, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_head_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch1_val[]={
  0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_head_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch2_val[]={
  0.0000, -0.1201, -0.4781, -1.0669, -1.4445, -0.7442, -0.2697, -0.0301,
  -0.0129, -0.1162, -0.3219, -0.6282, -0.4626, -0.2063, -0.0517, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_head_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch3_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_head_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch4_val[]={
  0.0000, -0.1396, -0.2793, -0.4189, -0.4887, -0.3491, -0.2094, -0.0698,
  0.0458, 0.1373, 0.2289, 0.3205, 0.2747, 0.1831, 0.0916, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_head_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch5_val[]={
  0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_head_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_head_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_head_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_head_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_head_motion_ch0_key, Animation_for_head_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_head_motion_ch1_key, Animation_for_head_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_head_motion_ch2_key, Animation_for_head_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_head_motion_ch3_key, Animation_for_head_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_head_motion_ch4_key, Animation_for_head_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_head_motion_ch5_key, Animation_for_head_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_head_motion_ch6_key, Animation_for_head_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_head_motion_ch7_key, Animation_for_head_motion_ch7_val},
};

static NDR_ANIM Animation_for_head_motion={
  Animation_for_head_motion_chlist, 8, 30
};

static s16 Animation_for_L_upleg_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch0_val[]={
  -2.9680, -5.0261, -7.5551, -10.4911, -11.3874, -6.0016, -2.1563, -0.2081,
  0.0020, 0.0601, 0.1770, 0.3280, -0.2430, -1.1274, -2.0604, -2.9680,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_upleg_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch1_val[]={
  12.5350, 15.6551, 18.3928, 20.6629, 21.1871, 16.6361, 10.5328, 3.5687,
  -0.3787, -1.0719, -1.6133, -1.9305, 0.4426, 4.2748, 8.4533, 12.5350,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_upleg_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch2_val[]={
  -1.5498, -2.5060, -3.7278, -5.1960, -5.5272, -2.4806, -0.5641, 0.1223,
  -0.1008, -0.6469, -1.6303, -3.0121, -1.6729, -0.3129, -0.2467, -1.5498,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_L_upleg_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch3_val[]={
  -0.5236, -0.6279, -0.7316, -0.8347, -0.8519, -0.6116, -0.3681, -0.1228,
  0.0932, 0.2794, 0.4650, 0.6497, 0.4533, 0.1281, -0.1986, -0.5236,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_L_upleg_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch4_val[]={
  -0.0032, -0.0071, -0.0144, -0.0252, -0.0255, 0.0038, 0.0149, 0.0088,
  -0.0029, -0.0160, -0.0382, -0.0686, -0.0312, 0.0093, 0.0198, -0.0032,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_upleg_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch5_val[]={
  0.0826, 0.1102, 0.1378, 0.1659, 0.1739, 0.1240, 0.0772, 0.0275, 0.0367,
  0.1081, 0.1757, 0.2379, 0.2336, 0.1913, 0.1375, 0.0826, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_upleg_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_upleg_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_upleg_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_upleg_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_upleg_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_upleg_motion_ch0_key, Animation_for_L_upleg_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_upleg_motion_ch1_key, Animation_for_L_upleg_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_upleg_motion_ch2_key, Animation_for_L_upleg_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_upleg_motion_ch3_key, Animation_for_L_upleg_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_upleg_motion_ch4_key, Animation_for_L_upleg_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_upleg_motion_ch5_key, Animation_for_L_upleg_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_upleg_motion_ch6_key, Animation_for_L_upleg_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_upleg_motion_ch7_key, Animation_for_L_upleg_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_upleg_motion_ch8_key, Animation_for_L_upleg_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_upleg_motion={
  Animation_for_L_upleg_motion_chlist, 9, 30
};

static s16 Animation_for_L_lowleg_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch0_val[]={
  0.0000, 0.0000, 0.0001, 0.0001, 0.0001, 0.0000, 0.0000, -0.0000, -0.0000,
  0.0000, 0.0000, 0.0001, 0.0000, 0.0001, 0.0001, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_lowleg_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch1_val[]={
  -31.4106, -42.7692, -55.1223, -68.0920, -70.2244, -40.5234, -16.6349,
  -2.5519, -2.6238, -17.1466, -41.7244, -72.1075, -69.7027, -56.1628,
  -43.2563, -31.4106, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
 };

static s16 Animation_for_L_lowleg_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch2_val[]={
  56.6612, 63.3671, 67.9921, 70.3951, 70.5692, 62.2476, 42.7350, 15.2930,
  15.5526, 43.3608, 62.8578, 70.6725, 70.5322, 68.2729, 63.5977, 56.6612,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_lowleg_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch3_val[]={
  0.8875, 1.0626, 1.2377, 1.4127, 1.4411, 1.0293, 0.6176, 0.2059, 0.2094,
  0.6283, 1.0472, 1.4661, 1.4341, 1.2519, 1.0697, 0.8875, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_lowleg_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch4_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_L_lowleg_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch5_val[]={
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_L_lowleg_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_lowleg_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_lowleg_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_lowleg_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_lowleg_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch0_key, Animation_for_L_lowleg_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch1_key, Animation_for_L_lowleg_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch2_key, Animation_for_L_lowleg_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch3_key, Animation_for_L_lowleg_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch4_key, Animation_for_L_lowleg_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch5_key, Animation_for_L_lowleg_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch6_key, Animation_for_L_lowleg_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch7_key, Animation_for_L_lowleg_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_lowleg_motion_ch8_key, Animation_for_L_lowleg_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_lowleg_motion={
  Animation_for_L_lowleg_motion_chlist, 9, 30
};

static s16 Animation_for_L_shoe_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch0_val[]={
  2.9298, -1.1257, -5.1481, -9.0177, -11.0321, -8.0365, -4.8851, -1.6388,
  -0.9052, -2.7100, -4.4990, -6.2614, -4.8106, -2.2471, 0.3436, 2.9298,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_shoe_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch1_val[]={
  -1.0706, -0.1499, -3.2624, -10.3154, -15.7828, -8.1197, -2.9325, -0.3214,
  -0.0958, -0.8896, -2.4815, -4.8612, -2.8425, -0.6093, -0.0163, -1.0706,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_L_shoe_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch2_val[]={
  16.6166, -6.3838, -29.1978, -51.1429, -62.5673, -45.5788, -27.7051,
  -9.2949, -5.1333, -15.3699, -25.5157, -35.5111, -27.2827, -12.7438,
  1.9488, 16.6166, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_shoe_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch3_val[]={
  0.1231, -0.0472, -0.2175, -0.3876, -0.4807, -0.3436, -0.2062, -0.0688,
  -0.0379, -0.1138, -0.1897, -0.2655, -0.2030, -0.0943, 0.0144, 0.1231,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_L_shoe_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch4_val[]={
  0.0013, 0.0002, 0.0042, 0.0131, 0.0200, 0.0103, 0.0037, 0.0004, 0.0001,
  0.0011, 0.0032, 0.0062, 0.0036, 0.0008, 0.0000, 0.0013, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_shoe_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch5_val[]={
  -0.0217, 0.0083, 0.0381, 0.0667, 0.0816, 0.0594, 0.0361, 0.0121, 0.0067,
  0.0200, 0.0333, 0.0463, 0.0356, 0.0166, -0.0025, -0.0217, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_shoe_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_shoe_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_shoe_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_shoe_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_shoe_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_shoe_motion_ch0_key, Animation_for_L_shoe_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_shoe_motion_ch1_key, Animation_for_L_shoe_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_shoe_motion_ch2_key, Animation_for_L_shoe_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_shoe_motion_ch3_key, Animation_for_L_shoe_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_shoe_motion_ch4_key, Animation_for_L_shoe_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_shoe_motion_ch5_key, Animation_for_L_shoe_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_shoe_motion_ch6_key, Animation_for_L_shoe_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_shoe_motion_ch7_key, Animation_for_L_shoe_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_shoe_motion_ch8_key, Animation_for_L_shoe_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_shoe_motion={
  Animation_for_L_shoe_motion_chlist, 9, 30
};

static s16 Animation_for_L_leg_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch0_val[]={
  -2.9680, -5.0261, -7.5551, -10.4911, -11.3874, -6.0016, -2.1563, -0.2081,
  0.0020, 0.0601, 0.1770, 0.3280, -0.2430, -1.1274, -2.0604, -2.9680,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_leg_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch1_val[]={
  12.5350, 15.6551, 18.3928, 20.6629, 21.1871, 16.6361, 10.5328, 3.5687,
  -0.3787, -1.0719, -1.6133, -1.9305, 0.4426, 4.2748, 8.4533, 12.5350,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_leg_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch2_val[]={
  -1.5498, -2.5060, -3.7278, -5.1960, -5.5272, -2.4806, -0.5641, 0.1223,
  -0.1008, -0.6469, -1.6303, -3.0121, -1.6729, -0.3129, -0.2467, -1.5498,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_L_leg_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch3_val[]={
  -0.5236, -0.6279, -0.7316, -0.8347, -0.8519, -0.6116, -0.3681, -0.1228,
  0.0932, 0.2794, 0.4650, 0.6497, 0.4533, 0.1281, -0.1986, -0.5236,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_L_leg_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch4_val[]={
  -0.0032, -0.0071, -0.0144, -0.0252, -0.0255, 0.0038, 0.0149, 0.0088,
  -0.0029, -0.0160, -0.0382, -0.0686, -0.0312, 0.0093, 0.0198, -0.0032,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_leg_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch5_val[]={
  0.0826, 0.1102, 0.1378, 0.1659, 0.1739, 0.1240, 0.0772, 0.0275, 0.0367,
  0.1081, 0.1757, 0.2379, 0.2336, 0.1913, 0.1375, 0.0826, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_L_leg_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_leg_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_L_leg_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_L_leg_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_L_leg_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_L_leg_motion_ch0_key, Animation_for_L_leg_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_L_leg_motion_ch1_key, Animation_for_L_leg_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_L_leg_motion_ch2_key, Animation_for_L_leg_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_L_leg_motion_ch3_key, Animation_for_L_leg_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_L_leg_motion_ch4_key, Animation_for_L_leg_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_L_leg_motion_ch5_key, Animation_for_L_leg_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_L_leg_motion_ch6_key, Animation_for_L_leg_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_L_leg_motion_ch7_key, Animation_for_L_leg_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_L_leg_motion_ch8_key, Animation_for_L_leg_motion_ch8_val},
};

static NDR_ANIM Animation_for_L_leg_motion={
  Animation_for_L_leg_motion_chlist, 9, 30
};

static s16 Animation_for_hip_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch0_val[]={
  0.0000, -0.3194, -0.6251, -0.9157, -1.1122, -1.0686, -1.0249, -0.9810,
  -0.8856, -0.7223, -0.5385, -0.3363, -0.2857, -0.2406, -0.1438, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_hip_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch1_val[]={
  0.0000, 0.2649, 0.5503, 0.8551, 1.0605, 0.9195, 0.7784, 0.6375, 0.7285,
  1.0370, 1.3249, 1.5908, 1.2985, 0.8400, 0.4051, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_hip_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch2_val[]={
  -0.0000, -0.0070, -0.0281, -0.0638, -0.0961, -0.0808, -0.0673, -0.0556,
  -0.0534, -0.0649, -0.0831, -0.1076, -0.0718, -0.0310, -0.0075, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_hip_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch3_val[]={
  0.0000, -0.0207, -0.0416, -0.0627, -0.0758, -0.0646, -0.0534, -0.0423,
  -0.0475, -0.0688, -0.0897, -0.1101, -0.0929, -0.0625, -0.0314, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_hip_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch4_val[]={
  0.0000, -0.0265, -0.0532, -0.0802, -0.0993, -0.0946, -0.0898, -0.0850,
  -0.0800, -0.0761, -0.0738, -0.0732, -0.0550, -0.0335, -0.0151, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_hip_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch5_val[]={
  0.0000, -0.0294, -0.0582, -0.0866, -0.1079, -0.1099, -0.1118, -0.1137,
  -0.1518, -0.2262, -0.3008, -0.3757, -0.3167, -0.2121, -0.1065, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_hip_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_hip_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_hip_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_hip_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_hip_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_hip_motion_ch0_key, Animation_for_hip_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_hip_motion_ch1_key, Animation_for_hip_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_hip_motion_ch2_key, Animation_for_hip_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_hip_motion_ch3_key, Animation_for_hip_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_hip_motion_ch4_key, Animation_for_hip_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_hip_motion_ch5_key, Animation_for_hip_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_hip_motion_ch6_key, Animation_for_hip_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_hip_motion_ch7_key, Animation_for_hip_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_hip_motion_ch8_key, Animation_for_hip_motion_ch8_val},
};

static NDR_ANIM Animation_for_hip_motion={
  Animation_for_hip_motion_chlist, 9, 30
};

static s16 Animation_for_R_leg_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch0_val[]={
  0.0000, 0.0062, -0.2539, -0.7171, -0.9855, -0.3451, 0.1191, 0.2785,
  0.1405, -0.0542, -0.3219, -0.6600, -0.4831, -0.2194, -0.0606, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_leg_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch1_val[]={
  0.0000, -2.8331, -5.3756, -7.5817, -8.4826, -5.8569, -2.5506, 1.0520,
  3.5139, 4.8794, 6.2065, 7.4864, 6.6959, 4.7657, 2.4685, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_leg_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch2_val[]={
  0.0000, -0.9914, -2.1770, -3.4952, -3.9967, -2.3221, -1.6355, -2.1630,
  -3.1370, -3.6330, -4.1867, -4.7960, -3.2172, -1.4191, -0.3237, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_leg_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch3_val[]={
  0.0000, 0.1453, 0.2911, 0.4381, 0.4669, 0.1382, -0.1889, -0.5139,
  -0.7109, -0.7825, -0.8538, -0.9247, -0.7562, -0.5060, -0.2536, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_leg_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch4_val[]={
  -0.0000, 0.0361, 0.0756, 0.1164, 0.1332, 0.0869, 0.0594, 0.0595, 0.0764,
  0.0866, 0.0985, 0.1119, 0.0741, 0.0318, 0.0066, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_R_leg_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch5_val[]={
  0.0000, -0.0824, -0.1543, -0.2155, -0.2465, -0.1997, -0.1309, -0.0480,
  0.0169, 0.0598, 0.1024, 0.1445, 0.1338, 0.0956, 0.0495, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_leg_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_leg_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_leg_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_leg_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_leg_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_leg_motion_ch0_key, Animation_for_R_leg_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_leg_motion_ch1_key, Animation_for_R_leg_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_leg_motion_ch2_key, Animation_for_R_leg_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_leg_motion_ch3_key, Animation_for_R_leg_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_leg_motion_ch4_key, Animation_for_R_leg_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_leg_motion_ch5_key, Animation_for_R_leg_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_leg_motion_ch6_key, Animation_for_R_leg_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_leg_motion_ch7_key, Animation_for_R_leg_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_leg_motion_ch8_key, Animation_for_R_leg_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_leg_motion={
  Animation_for_R_leg_motion_chlist, 9, 30
};

static s16 Animation_for_R_lowleg_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch0_val[]={
  0.0000, 0.0000, 0.0001, 0.0001, 0.0000, 0.0001, 0.0001, 0.0001, 0.0001,
  0.0001, 0.0001, 0.0000, 0.0001, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_lowleg_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch1_val[]={
  0.0000, -6.7914, -22.8456, -45.9921, -64.3110, -57.0392, -49.9437,
  -43.0931, -38.3945, -35.6723, -33.0195, -30.4408, -20.1659, -9.8897,
  -3.0480, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_lowleg_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch2_val[]={
  0.0000, 26.8980, 49.5228, 64.8153, 69.9355, 68.4970, 66.3518, 63.5207,
  61.0980, 59.4955, 57.7805, 55.9558, 46.7960, 32.8937, 17.0171, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_lowleg_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch3_val[]={
  -0.0000, 0.3698, 0.7396, 1.1094, 1.3622, 1.2639, 1.1656, 1.0674, 0.9973,
  0.9554, 0.9135, 0.8717, 0.6889, 0.4593, 0.2296, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_R_lowleg_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch4_val[]={
  0.0000, -0.0000, -0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, -0.0000, 0.0000, 0.0000, -0.0000, -0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_lowleg_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch5_val[]={
  0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_lowleg_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_lowleg_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_lowleg_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_lowleg_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_lowleg_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch0_key, Animation_for_R_lowleg_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch1_key, Animation_for_R_lowleg_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch2_key, Animation_for_R_lowleg_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch3_key, Animation_for_R_lowleg_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch4_key, Animation_for_R_lowleg_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch5_key, Animation_for_R_lowleg_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch6_key, Animation_for_R_lowleg_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch7_key, Animation_for_R_lowleg_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_lowleg_motion_ch8_key, Animation_for_R_lowleg_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_lowleg_motion={
  Animation_for_R_lowleg_motion_chlist, 9, 30
};

static s16 Animation_for_R_shoe_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch0_val[]={
  0.0000, 1.0588, -3.6097, -13.6513, -21.5553, -11.9084, -4.5858, 0.0615,
  0.1263, -3.3637, -8.2351, -14.2986, -8.2237, -1.2175, 1.5910, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_shoe_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch1_val[]={
  0.0000, -1.3006, -5.2092, -11.6498, -15.4771, -7.0399, -1.8379, -0.0023,
  -0.0026, -1.8381, -7.0400, -15.4772, -11.6499, -5.2092, -1.3004, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_shoe_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch2_val[]={
  0.0000, -22.4867, -44.4477, -64.3962, -72.7334, -53.2392, -31.7938,
  -9.1319, -8.7318, -30.7802, -52.0170, -71.8649, -63.0086, -43.0007,
  -21.6488, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
 };

static s16 Animation_for_R_shoe_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch3_val[]={
  0.0000, -0.1409, -0.2886, -0.4434, -0.5204, -0.3459, -0.1747, -0.0072,
  -0.0072, -0.1732, -0.3409, -0.5099, -0.4360, -0.2855, -0.1401, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_shoe_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch4_val[]={
  -0.0000, 0.1379, 0.2723, 0.4030, 0.4859, 0.4324, 0.3744, 0.3117, 0.2964,
  0.3285, 0.3560, 0.3791, 0.3113, 0.2111, 0.1073, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_R_shoe_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch5_val[]={
  0.0000, -0.0244, -0.0497, -0.0769, -0.0915, -0.0611, -0.0308, -0.0013,
  -0.0013, -0.0303, -0.0592, -0.0871, -0.0743, -0.0490, -0.0243, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_shoe_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_shoe_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_shoe_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_shoe_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_shoe_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_shoe_motion_ch0_key, Animation_for_R_shoe_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_shoe_motion_ch1_key, Animation_for_R_shoe_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_shoe_motion_ch2_key, Animation_for_R_shoe_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_shoe_motion_ch3_key, Animation_for_R_shoe_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_shoe_motion_ch4_key, Animation_for_R_shoe_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_shoe_motion_ch5_key, Animation_for_R_shoe_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_shoe_motion_ch6_key, Animation_for_R_shoe_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_shoe_motion_ch7_key, Animation_for_R_shoe_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_shoe_motion_ch8_key, Animation_for_R_shoe_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_shoe_motion={
  Animation_for_R_shoe_motion_chlist, 9, 30
};

static s16 Animation_for_R_upleg_motion_ch0_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch0_val[]={
  0.0000, 0.0062, -0.2539, -0.7171, -0.9855, -0.3451, 0.1191, 0.2785,
  0.1405, -0.0542, -0.3219, -0.6600, -0.4831, -0.2194, -0.0606, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_upleg_motion_ch1_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch1_val[]={
  0.0000, -2.8331, -5.3756, -7.5817, -8.4826, -5.8569, -2.5506, 1.0520,
  3.5139, 4.8794, 6.2065, 7.4864, 6.6959, 4.7657, 2.4685, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_upleg_motion_ch2_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch2_val[]={
  0.0000, -0.9914, -2.1770, -3.4952, -3.9967, -2.3221, -1.6355, -2.1630,
  -3.1370, -3.6330, -4.1867, -4.7960, -3.2172, -1.4191, -0.3237, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_upleg_motion_ch3_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch3_val[]={
  0.0000, 0.1453, 0.2911, 0.4381, 0.4669, 0.1382, -0.1889, -0.5139,
  -0.7109, -0.7825, -0.8538, -0.9247, -0.7562, -0.5060, -0.2536, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_upleg_motion_ch4_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch4_val[]={
  -0.0000, 0.0361, 0.0756, 0.1164, 0.1332, 0.0869, 0.0594, 0.0595, 0.0764,
  0.0866, 0.0985, 0.1119, 0.0741, 0.0318, 0.0066, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
  -0.0000, -0.0000, -0.0000, -0.0000, -0.0000, -0.0000,
};

static s16 Animation_for_R_upleg_motion_ch5_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch5_val[]={
  0.0000, -0.0824, -0.1543, -0.2155, -0.2465, -0.1997, -0.1309, -0.0480,
  0.0169, 0.0598, 0.1024, 0.1445, 0.1338, 0.0956, 0.0495, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
  0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
};

static s16 Animation_for_R_upleg_motion_ch6_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch6_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_upleg_motion_ch7_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch7_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static s16 Animation_for_R_upleg_motion_ch8_key[]={
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
  21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
};
static float Animation_for_R_upleg_motion_ch8_val[]={
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000, 1.0000,
  1.0000, 1.0000, 1.0000, 1.0000,
};

static NDR_ANIM_CHANNEL Animation_for_R_upleg_motion_chlist[]={
  {NDC_ANIMKIND_TRS_X, 0, 30, 31,
   Animation_for_R_upleg_motion_ch0_key, Animation_for_R_upleg_motion_ch0_val},
  {NDC_ANIMKIND_TRS_Y, 0, 30, 31,
   Animation_for_R_upleg_motion_ch1_key, Animation_for_R_upleg_motion_ch1_val},
  {NDC_ANIMKIND_TRS_Z, 0, 30, 31,
   Animation_for_R_upleg_motion_ch2_key, Animation_for_R_upleg_motion_ch2_val},
  {NDC_ANIMKIND_ROT_X, 0, 30, 31,
   Animation_for_R_upleg_motion_ch3_key, Animation_for_R_upleg_motion_ch3_val},
  {NDC_ANIMKIND_ROT_Y, 0, 30, 31,
   Animation_for_R_upleg_motion_ch4_key, Animation_for_R_upleg_motion_ch4_val},
  {NDC_ANIMKIND_ROT_Z, 0, 30, 31,
   Animation_for_R_upleg_motion_ch5_key, Animation_for_R_upleg_motion_ch5_val},
  {NDC_ANIMKIND_SCL_X, 0, 30, 31,
   Animation_for_R_upleg_motion_ch6_key, Animation_for_R_upleg_motion_ch6_val},
  {NDC_ANIMKIND_SCL_Y, 0, 30, 31,
   Animation_for_R_upleg_motion_ch7_key, Animation_for_R_upleg_motion_ch7_val},
  {NDC_ANIMKIND_SCL_Z, 0, 30, 31,
   Animation_for_R_upleg_motion_ch8_key, Animation_for_R_upleg_motion_ch8_val},
};

static NDR_ANIM Animation_for_R_upleg_motion={
  Animation_for_R_upleg_motion_chlist, 9, 30
};

/* ============================================================
          Object Structure Area
 ============================================================ */
/* Object #0 */
static NDR_OBJ _dummy_obj={
  NDC_TYPE_NULL, 0, 0x00000000,
  {0,0,0, 0,0,0, 1,1,1},
  -1,
  0x0000, 0x00000000, 0x00000000, 0x00000000,
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  -1, 0, 0, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #1 */
static u16 _body_link[]={ 1, 4, 5, 8, 9, 10, 13, 14 };
static NDR_OBJ _body={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000000,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  0,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  0, 0, 2, 0x010203,
  {-1, -1}, 8, _body_link
};

/* Object #2 */
static u16 _R_uparm_link[]={ 1 };
static NDR_OBJ _R_uparm={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {25.170159,13.895111,-5.565050,
   0.055511,0.000101,0.268425,
   1.000000,1.000000,1.000000},
  1,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  1, 0, 2, 0x010203,
  {-1, -1}, 1, _R_uparm_link
};

/* Object #3 */
static u16 _R_lowarm_link[]={ 1 };
static NDR_OBJ _R_lowarm={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {1.274824,1.751316,-8.114614,
   -0.417670,0.287103,-0.061247,
   1.000000,1.000000,1.000000},
  2,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  2, 0, 2, 0x010203,
  {-1, -1}, 1, _R_lowarm_link
};

/* Object #4 */
static NDR_OBJ _R_hand={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {-30.255537,-40.494070,-67.186980,
   -1.570797,0.641539,-0.640677,
   1.000000,1.000000,1.000001},
  3,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  3, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #5 */
static NDR_OBJ _R_sode={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {25.170159,13.895111,-5.565050,
   0.055511,0.000101,0.268425,
   1.000000,1.000000,1.000000},
  4,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  4, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #6 */
static u16 _L_uparm_link[]={ 1 };
static NDR_OBJ _L_uparm={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {-6.625543,5.049620,-30.168858,
   0.414456,0.247633,0.001755,
   1.000000,1.000000,1.000000},
  5,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  5, 0, 2, 0x010203,
  {-1, -1}, 1, _L_uparm_link
};

/* Object #7 */
static u16 _L_lowarm_link[]={ 1 };
static NDR_OBJ _L_lowarm={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {-1.001889,1.376240,-12.218447,
   -0.642528,-0.411298,0.138618,
   1.000000,1.000001,1.000000},
  6,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  6, 0, 2, 0x010203,
  {-1, -1}, 1, _L_lowarm_link
};

/* Object #8 */
static NDR_OBJ _L_hand={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {-0.111365,0.114808,-1.477478,
   -0.015596,-0.014771,0.000142,
   1.000001,0.999999,1.000000},
  7,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  7, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #9 */
static NDR_OBJ _L_sode={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {-6.625543,5.049620,-30.168858,
   0.414456,0.247633,0.001755,
   1.000000,1.000000,1.000000},
  8,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  8, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #10 */
static NDR_OBJ _head={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000000,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  9,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  9, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #11 */
static u16 _L_upleg_link[]={ 1 };
static NDR_OBJ _L_upleg={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {-2.968035,12.535017,-1.549795,
   -0.523554,-0.003201,0.082629,
   1.000000,1.000000,1.000000},
  10,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  10, 0, 2, 0x010203,
  {-1, -1}, 1, _L_upleg_link
};

/* Object #12 */
static u16 _L_lowleg_link[]={ 1 };
static NDR_OBJ _L_lowleg={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000006,-31.410644,56.661229,
   0.887548,0.000002,0.000002,
   1.000000,1.000000,1.000000},
  11,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  11, 0, 2, 0x010203,
  {-1, -1}, 1, _L_lowleg_link
};

/* Object #13 */
static NDR_OBJ _L_shoe={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {2.929845,-1.070611,16.616549,
   0.123131,0.001335,-0.021658,
   1.000000,1.000001,1.000000},
  12,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  12, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #14 */
static NDR_OBJ _L_leg={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {-2.968035,12.535017,-1.549795,
   -0.523554,-0.003201,0.082629,
   1.000000,1.000000,1.000000},
  13,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  13, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #15 */
static u16 _hip_link[]={ 1, 4 };
static NDR_OBJ _hip={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,-0.000002,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  14,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  14, 0, 2, 0x010203,
  {-1, -1}, 2, _hip_link
};

/* Object #16 */
static u16 _R_leg_link[]={ 1 };
static NDR_OBJ _R_leg={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000002,
   0.000000,-0.000000,0.000000,
   1.000000,1.000000,1.000000},
  15,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  15, 0, 2, 0x010203,
  {-1, -1}, 1, _R_leg_link
};

/* Object #17 */
static u16 _R_lowleg_link[]={ 1 };
static NDR_OBJ _R_lowleg={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000000,
   -0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  16,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  16, 0, 2, 0x010203,
  {-1, -1}, 1, _R_lowleg_link
};

/* Object #18 */
static NDR_OBJ _R_shoe={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000000,
   0.000000,-0.000000,0.000000,
   1.000000,1.000000,1.000000},
  17,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  17, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #19 */
static NDR_OBJ _R_upleg={
  NDC_TYPE_3D, 0, 0xd00 + 0x00,
  {0.000000,0.000000,0.000002,
   0.000000,-0.000000,0.000000,
   1.000000,1.000000,1.000000},
  18,
  0x0000, 0x00002001, 0xf0800505, 0x00000000,
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  {0x0f,0x0f,0x0f,0x04,0x0f,0x0f,0x0f,0x04},
  18, 0, 2, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #20 */
static NDR_OBJ _NULL_OBJECT_1={
  NDC_TYPE_NULL, 0, 0x00000000,
  {0,0,0, 0,0,0, 1,1,1},
  -1,
  0x0000, 0x00000000, 0x00000000, 0x00000000,
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  -1, 0, 0, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #21 */
static NDR_OBJ _NULL_OBJECT_2={
  NDC_TYPE_NULL, 0, 0x00000000,
  {0,0,0, 0,0,0, 1,1,1},
  -1,
  0x0000, 0x00000000, 0x00000000, 0x00000000,
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  -1, 0, 0, 0x010203,
  {-1, -1}, 0, NULL
};

/* Object #22 */
static NDR_OBJ _NULL_OBJECT_3={
  NDC_TYPE_NULL, 0, 0x00000000,
  {0,0,0, 0,0,0, 1,1,1},
  -1,
  0x0000, 0x00000000, 0x00000000, 0x00000000,
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
  -1, 0, 0, 0x010203,
  {-1, -1}, 0, NULL
};

/* ============================================================
          Scene Structure Area
 ============================================================ */
static NDR_SCN_INST_OBJ run_scene_instlist[]={
 {
  {0,0,0, 0,0,0, 1,1,1},
  20 /* NULL_OBJECT_1 */
 },
 {
  {0,0,0, 0,0,0, 1,1,1},
  22 /* NULL_OBJECT_3 */
 },
 {
  {0,0,0, 0,0,0, 1,1,1},
  21 /* NULL_OBJECT_2 */
 },
 {
  {0.000000,0.000000,0.000000,
   0.000000,0.000000,0.000000,
   1.000000,1.000000,1.000000},
  1 /* body */
 },
};

static NDR_SCN_LIGHT _run_scene_light[]={
  { 0, 0xffffff00, 0.57735, 0.57735, 0.57735, 0, 0 }
};

static NDR_SCENE run_scene={
  {950.000000, 1000.000000, 0x4c, 0x4c, 0x4c, 0xff},
  {1, 45.841393, 1.333330, 1.000000, 0, 0, 0, 0, 1, 10000, 0, 1, 2},
  { 0x000000ff, 1, _run_scene_light },
  0x0000, 0xfffc,
  run_scene_instlist, 4,
};

/* ============================================================
          Shape Structure List
 ============================================================ */
static NDR_SHAPE *shapelist_frake_run[] = {
  &shape_Shape_of___POLYHEDRON_39_,
  &shape_Shape_of___POLYHEDRON_53_,
  &shape_Shape_of___POLYHEDRON_55_,
  &shape_Shape_of___POLYHEDRON_57_,
  &shape_Shape_of___POLYHEDRON_51_,
  &shape_Shape_of___POLYHEDRON_45_,
  &shape_Shape_of___POLYHEDRON_47_,
  &shape_Shape_of___POLYHEDRON_49_,
  &shape_Shape_of___POLYHEDRON_43_,
  &shape_Shape_of___POLYHEDRON_8_,
  &shape_Shape_of___POLYHEDRON_67_,
  &shape_Shape_of___POLYHEDRON_71_,
  &shape_Shape_of___POLYHEDRON_75_,
  &shape_Shape_of___POLYHEDRON_63_,
  &shape_Shape_of___POLYHEDRON_61_,
  &shape_Shape_of___POLYHEDRON_59_,
  &shape_Shape_of___POLYHEDRON_69_,
  &shape_Shape_of___POLYHEDRON_73_,
  &shape_Shape_of___POLYHEDRON_65_,
};

/* ============================================================
          Geometric Animation List
 ============================================================ */
static NDR_ANIM *animlist_frake_run[]={
  &Animation_for_body_motion,
  &Animation_for_R_uparm_motion,
  &Animation_for_R_lowarm_motion,
  &Animation_for_R_hand_motion,
  &Animation_for_R_sode_motion,
  &Animation_for_L_uparm_motion,
  &Animation_for_L_lowarm_motion,
  &Animation_for_L_hand_motion,
  &Animation_for_L_sode_motion,
  &Animation_for_head_motion,
  &Animation_for_L_upleg_motion,
  &Animation_for_L_lowleg_motion,
  &Animation_for_L_shoe_motion,
  &Animation_for_L_leg_motion,
  &Animation_for_hip_motion,
  &Animation_for_R_leg_motion,
  &Animation_for_R_lowleg_motion,
  &Animation_for_R_shoe_motion,
  &Animation_for_R_upleg_motion,
};

/* ============================================================
          Object Struct List
 ============================================================ */
static NDR_OBJ *objlist_frake_run[]={
  &_dummy_obj, /* 0 */
  &_body, /* 1 */
  &_R_uparm, /* 2 */
  &_R_lowarm, /* 3 */
  &_R_hand, /* 4 */
  &_R_sode, /* 5 */
  &_L_uparm, /* 6 */
  &_L_lowarm, /* 7 */
  &_L_hand, /* 8 */
  &_L_sode, /* 9 */
  &_head, /* 10 */
  &_L_upleg, /* 11 */
  &_L_lowleg, /* 12 */
  &_L_shoe, /* 13 */
  &_L_leg, /* 14 */
  &_hip, /* 15 */
  &_R_leg, /* 16 */
  &_R_lowleg, /* 17 */
  &_R_shoe, /* 18 */
  &_R_upleg, /* 19 */
  &_NULL_OBJECT_1, /* 20 */
  &_NULL_OBJECT_2, /* 21 */
  &_NULL_OBJECT_3, /* 22 */
};

/* ############################################################
          Resource Information
 ############################################################ */
NDR_RESINFO resinfo_frake_run={
  &run_scene,
  20.000000,
  objlist_frake_run,
  shapelist_frake_run,
  animlist_frake_run,
  NULL,
  NULL,
  NULL,
  23, 19, 19, 0, 0, 0
};

